<?php
/**
*
* @package arcade
* @version $Id: arcade.php 2024 2012-03-20 17:19:21Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_PHPBB_ARCADE', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);

// We use this to display the flash and image files for the game.  This is to hide the acutal path from the end user.
// However this will not work with IBProV3 or Arcadelib games so the real path must be used for the flash file for those type of games.
if (!empty($_GET['swf']) || !empty($_GET['img']))
{
	require($phpbb_root_path . 'arcade/includes/protect.' . $phpEx);
}

// implicit else: we are not in swf or img mode
include($phpbb_root_path . 'common.' . $phpEx);

$arcade_scorevar = (isset($_POST['sendscore'])) ? strtolower(request_var('sendscore', '')) : '';
if ($arcade_scorevar == 'verify' || $arcade_scorevar == 'save' || $arcade_scorevar == 'banned')
{
	$scoretype = PHPBBARCADE_GAME;
	include($phpbb_root_path . 'arcade/includes/scoretype.' . $phpEx);
}

// Handling of V3arcade games
// Only accept the var if its a POST
$v3arcade = (isset($_POST['sessdo'])) ? strtolower(request_var('sessdo', '')) : '';
if (!empty($v3arcade))
{
	$scoretype = V3ARCADE_GAME;
	include($phpbb_root_path . 'arcade/includes/scoretype.' . $phpEx);
}
// End Handling of V3arcade games

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

if (!isset($config['arcade_install']))
{
	redirect($phpbb_root_path . 'index.' . $phpEx);
}

include($phpbb_root_path . 'arcade/includes/common.' . $phpEx);

if ($user->data['user_type'] != USER_FOUNDER && arcade_ban_users())
{
	trigger_error('ARCADE_BANNED');
}

$mode			= request_var('mode', '');
$type			= request_var('type', '');

$cat_id			= request_var('c', 0);
$game_id		= request_var('g', 0);
$user_id		= request_var('u', 0);
$spop			= request_var('sp', false);
$start			= request_var('start', 0);
$search_id		= request_var('search_id', '');
$challenge_id	= ($user->data['is_registered']) ? request_var('cid', 0) : false;
$term			= utf8_normalize_nfc(request_var('term', '', true));
$chall = (strpos($mode, 'challenge') !== false) ? true : false;

$arcade_view = ($auth_arcade->acl_get('u_arcade')) ? true : false;
$acp_download_list_connection = ($arcade_config['download_list'] && $mode == 'download' && $user->data['user_id'] == ANONYMOUS && ($type == 'data' || $type == 'list')) ? true : false;

if (!$acp_download_list_connection && (!$arcade_view || ($chall && !$auth_arcade->acl_get('u_arcade_challenge'))))
{
	trigger_error((((!$arcade_view) ? 'ARCADE' : 'CHALLENGE') . '_NO_PERMISSION'));
}

// Is arcade or challenge disabled and user not an admin?
if (($arcade_config['arcade_disable'] || ($chall && $arcade_config['challenge_disable'])) && !$auth->acl_get('a_'))
{
	if ($user->data['is_bot'])
	{
		send_status_line(503, 'Service Unavailable');
	}

	if ($arcade_config['arcade_disable'])
	{
		$message = (!empty($arcade_config['arcade_disable_msg'])) ? $arcade_config['arcade_disable_msg'] : 'ARCADE_DISABLE';
	}
	else
	{
		$message = (!empty($arcade_config['challenge_disable_msg'])) ? $arcade_config['challenge_disable_msg'] : 'CHALLENGE_DISABLE';
	}

	trigger_error($message);
}

// Initialize arcade class
$arcade = new arcade(true, true, $chall);

if ($game_id && !$chall && !$arcade->get_game_field($game_id))
{
	trigger_error($user->lang['NO_GAME_ID'] . $arcade->back_link());
}

// default value
$page_title	= ($chall) ? 'ARCADE_CHALLENGE' : 'ARCADE_INDEX';
$page_html	= ($chall) ? 'challenge' : 'index';

if ($chall || ($type == 'challenge' && $challenge_id))
{
	include($phpbb_root_path . 'arcade/includes/class_challenge.' . $phpEx);
	$challenge = new challenge();
}

if ($chall)
{
	if (!$user->data['is_registered'] && in_array($mode, array('champion_challenge', 'rand_challenge')))
	{
		$mode = '';
	}

	$error1 = $error2 = array();
	$arrived_id		= request_var('arrived_id', array(0));
	$sent_id		= request_var('sent_id' , array(0));
	$bet			= request_var('bet', 0.000);
	$username		= utf8_normalize_nfc(request_var('username', '', true));
	$submit			= (isset($_POST['submit'])) ? true : false;
	$accept			= (isset($_POST['accept'])) ? true : false;
	$reject			= (isset($_POST['reject'])) ? true : false;
	$withdraw		= (isset($_POST['withdraw'])) ? true : false;
	$action			= ($accept) ? 'accept' : (($reject) ? 'reject' : (($withdraw) ? 'withdraw' : ''));
	$act_ids		= (sizeof($arrived_id)) ? $arrived_id : ((sizeof($sent_id)) ? $sent_id : array());

	switch ($mode)
	{
		case 'champion_challenge':
			$challenge->check('champion', $game_id, $user_id, $username, $bet, $error1);
		break;

		case 'rand_challenge':
			$challenge->random();
		break;

		case 'view_challenge_stats':
		break;

		default:
			if ($user->data['is_registered'])
			{
				if ($submit)
				{
					if (!check_form_key('challenge'))
					{
						$error[] = $user->lang['FORM_INVALID'];
					}

					$challenge->check('default', $game_id, $user_id, $username, $bet, $error1);
				}

				if ($action)
				{
					$challenge->user_action($action, $act_ids, $error2);
				}

				if ($arcade->optionget('challenge'))
				{
					$challenge->display_challenge($game_id, $user_id, $username, $bet, $error1, $error2);
				}
				else
				{
					$template->assign_var('S_CHALLENGE_USER_DISABLE', sprintf($user->lang['CHALLENGE_USER_DISABLE'],'<a href="' . $arcade->url('i=arcade&amp;mode=settings#tabs', 'ucp', $user->session_id) . '">' , '</a>'));
				}
			}

			display_arcade_header('challenge', $arcade_config['challenge_welcome_index'], false, $arcade_config['challenge_links_index']);
		break;
	}
}

switch ($mode)
{
	case 'download':
		if ($type == 'data' || $type == 'list')
		{
			if ($arcade_config['download_list'])
			{
				if ($type == 'data')
				{
					$arcade->display_download_data();
				}
				else
				{
					$sort_time	= request_var('st', 0);
					$sort_key	= request_var('sk', 'd');
					$sort_dir	= request_var('sd', 'd');
					$per_page	= request_var('per_page', 50);

					$arcade->display_download_list($cat_id, $start, $sort_key, $sort_dir, $sort_time, $per_page);
				}
			}
			else
			{
				$mode = $type = '';
			}
		}
	break;

	case 'random':
		if ($game_id = $arcade->get_random_game())
		{
			$mode = 'play';
		}
		else
		{
			trigger_error($user->lang['ARCADE_RANDOM_GAME_NO_MATCHES'] . $arcade->back_link());
		}
	break;
}

switch ($mode)
{
	case 'report':
		if (empty($game_id))
		{
			break;
		}

		include($phpbb_root_path . 'arcade/includes/reports.' . $phpEx);
	break;

	case 'stats':
	case 'view_challenge_stats':
		if (!$auth_arcade->acl_get('u_arcade_viewstats'))
		{
			trigger_error($user->lang['ARCADE_NO_PERMISSION_STATS'] . $arcade->back_link((($chall) ? 'mode=challenge' : '')));
		}

		$page_html = 'stats';
		include($phpbb_root_path . 'arcade/includes/stats.' . $phpEx);
	break;

	case 'download':
		if (!$auth_arcade->acl_get('u_arcade_download'))
		{
			trigger_error($user->lang['NO_PERMISSION_ARCADE_DOWNLOAD'] . $arcade->back_link());
		}

		include($phpbb_root_path . 'arcade/includes/download.' . $phpEx);
	break;

	case 'popup':
			$popup = $use_simple_header = true;

			if (!$auth_arcade->acl_get('u_arcade_popup'))
			{
				trigger_error($user->lang['NO_PERMISSION_ARCADE_PLAY_POPUP'] . $arcade->close_win());
			}
	case 'play':
		// We won't let guests play games in a new window, it would cause some other problems.
		if (empty($game_id))
		{
			break;
		}

		if ($mode == 'play')
		{
			$popup = $use_simple_header = false;
		}

		if (!$user->data['is_registered'] && !$auth_arcade->acl_getc_global('c_play'))
		{
			$total_games = $arcade->get_total('games');
			$message = sprintf($user->lang['ARCADE_REGISTER_MESSAGE_PLAY'], $total_games);

			if ($popup)
			{
				$message .= $arcade->close_win();
			}
			else
			{
				$message .= '<br />' . sprintf($user->lang['ARCADE_REGISTER_LOGIN_MESSAGE'], '<a href="' . append_sid("{$phpbb_root_path}ucp.$phpEx", "mode=register") . '">', '</a>', '<a href="' . append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=login&amp;redirect=' . urlencode("arcade.$phpEx?mode=play&g=$game_id")) . '">', '</a>');
			}

			trigger_error($message);
		}

		include($phpbb_root_path . 'arcade/includes/play.' . $phpEx);
	break;

	case 'score':
	case 'done':
		if (empty($game_id))
		{
			break;
		}

		// Well you should never get to this condition if you are not
		// logged in however just in case...
		if (!$user->data['is_registered'])
		{
			redirect(append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=login'));
		}

		// If playing the game in a popup window only use simple header when calling trigger_error()
		$popup = $use_simple_header = $spop;

		include($phpbb_root_path . 'arcade/includes/score.' . $phpEx);
	break;

	case 'cat':
	case 'view_cat_stats':
		// Break out and diplay main arcade page if the cat_id is not set
		if (empty($cat_id))
		{
			break;
		}

		if ($mode == 'view_cat_stats')
		{
			if (!$auth_arcade->acl_get('u_arcade_viewstats'))
			{
				trigger_error($user->lang['ARCADE_NO_PERMISSION_STATS'] . $arcade->back_link());
			}

			display_arcade_cat_stats($cat_id);
		}
	case 'fav':
		if ($mode == 'fav' && !$auth_arcade->acl_get('u_arcade_favorites'))
		{
			trigger_error($user->lang['ARCADE_NO_PERMISSION_FAV'] . $arcade->back_link());
		}
	case 'search':
		include($phpbb_root_path . 'arcade/includes/games.' . $phpEx);
	break;
}

if ($page_html == 'index')
{
	$template->assign_vars(array(
		'CAT_IMG'				=> $user->img('forum_read', 'NO_NEW_GAMES'),
		'CAT_NEW_IMG'			=> $user->img('forum_unread', 'NEW_GAMES'),
		'CAT_LOCKED_IMG'		=> $user->img('forum_read_locked', 'NEW_GAMES_LOCKED'),
		'CAT_NEW_LOCKED_IMG'	=> $user->img('forum_unread_locked', 'NO_NEW_GAMES_LOCKED'),
	));

	display_arcade_header('arcade', $arcade_config['welcome_index'], $arcade_config['links_index'], $arcade_config['search_index']);
	display_arcade();
}

if ($auth_arcade->acl_get('u_arcade_view_whoisplaying'))
{
	display_arcade_online();
}

page_header(((!empty($user->lang[$page_title])) ? $user->lang[$page_title] : $page_title), false);
$template->set_filenames(array(
	'body' => "arcade/{$page_html}_body.html")
);

page_footer();

?>