<?php
/**
*
* @package arcade
* @version $Id: arcade.php 1981 2012-03-14 01:33:30Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/*
* display_arcade_leaders()
* display_arcade_header()
* display_arcade_cat_stats()
* display_arcade()
* display_arcade_online()
* generate_arcade_nav()
* generate_cat_rules()
* get_arcade_parents()
* gen_arcade_auth_level()
* login_arcade_box()
* get_arcade_game_image()
* get_arcade_game_popup()
* get_arcade_game_name()
* get_arcade_user_name()
* get_arcade_user_avatar()
* make_arcade_jumpbox()
* arcade_game_update_install_file()
* arcade_game_create_install_file()
* arcade_msg_handler()
* arcade_fwrite()
* scandir()
* get_file_contents()
* arcade_debug()
* arcade_games_sort_order_select()
* arcade_games_sort_dir_select()
*/

/**
* Display Arcade Leaders
*/

function display_arcade_leaders($mode, $html_key)
{
	if (!in_array($mode, array('arcade', 'challenge')) || $html_key == '')
	{
		return;
	}

	global $user, $template, $arcade, $arcade_cache, $arcade_config;

	switch ($mode)
	{
		case 'arcade':
			$limit				= intval(($arcade_config['arcade_leaders'] > $arcade_config['arcade_leaders_header']) ? $arcade_config['arcade_leaders'] : $arcade_config['arcade_leaders_header']);
			$header_leaders		= $arcade_cache->obtain_arcade_leaders($limit);
			$real_leaders_count	= sizeof($header_leaders);
			$leaders_count		= ($real_leaders_count > $arcade_config['arcade_leaders_header']) ? $arcade_config['arcade_leaders_header'] : $real_leaders_count;
		break;

		case 'challenge':
			$limit				= intval(($arcade_config['challenge_leaders'] > $arcade_config['challenge_leaders_header']) ? $arcade_config['challenge_leaders'] : $arcade_config['challenge_leaders_header']);
			$header_leaders		= $arcade_cache->obtain_challenge_leaders($limit);
			$real_leaders_count	= sizeof($header_leaders);
			$leaders_count		= ($real_leaders_count > $arcade_config['challenge_leaders_header']) ? $arcade_config['challenge_leaders_header'] : $real_leaders_count;
		break;
	}

	$leaders_img = array(
		1 => $arcade->get_image('src', 'img', '1st.gif', ''),
		2 => $arcade->get_image('src', 'img', '2nd.gif', ''),
		3 => $arcade->get_image('src', 'img', '3rd.gif', ''),
	);

	$leaders_img_alt = array(
		1 => $user->lang['ARCADE_FIRST'],
		2 => $user->lang['ARCADE_SECOND'],
		3 => $user->lang['ARCADE_THIRD'],
	);

	$leaders_width = 0;
	if ($leaders_count > 0)
	{
		switch ($leaders_count)
		{
			case 1:
				$leaders_width = 100;
			break;

			case 2:
				$leaders_width = 49;
			break;

			default:
				$leaders_width = 33;
			break;
		}

		$rank = $actual_rank = $previous_wins = 0;
		for ($i = 0; $i < $leaders_count; $i++)
		{
			// This code is used to calculate the actual rank.
			// For example if there are ties...
			$actual_rank++;

			if ($previous_wins != $header_leaders[$i]['total_wins'])
			{
				$rank = $actual_rank;
			}

			$previous_wins	= $header_leaders[$i]['total_wins'];
			$user_link		= get_arcade_user_name('full', $header_leaders[$i]['user_id'], $header_leaders[$i]['username'], $header_leaders[$i]['user_colour'], $mode, 'x');

			$template->assign_block_vars($html_key, array(
				'ARCADE_LEADERS_RANK'			=> $rank,
				'ARCADE_LEADERS_ACTUAL_RANK'	=> $actual_rank,
				'ARCADE_LEADERS_AVATAR'			=> ($arcade->optionget('view_avatars')) ? get_arcade_user_avatar($header_leaders[$i]['user_avatar'], $header_leaders[$i]['user_avatar_type'], 30, 30, 'x', $mode, $header_leaders[$i]['user_id']) : false,
				'ARCADE_LEADERS'				=> $user_link,
				'ARCADE_LEADERS_IMAGE'			=> (isset($leaders_img[$rank])) ? $leaders_img[$rank] : $arcade->get_image('src', 'img', 'trophy.gif', ''),
				'ARCADE_LEADERS_IMAGE_ALT'		=> (isset($leaders_img_alt[$rank])) ? $leaders_img_alt[$rank] : sprintf($user->lang['ARCADE_PLACE'], $rank),
				'VICTORIES'						=> $header_leaders[$i]['total_wins']
			));

			$template->assign_var(strtoupper($html_key) . '_WIDTH', $leaders_width);
		}
	}
}

/**
* Display Arcade Header
*/
function display_arcade_header($mode = 'arcade', $show_welcome_box = true, $show_adv_search = false, $show_links = true)
{
	if (!in_array($mode, array('arcade', 'challenge')))
	{
		return;
	}

	global $arcade, $challenge, $arcade_cache, $arcade_config, $auth_arcade, $game_fav_data, $user, $template, $_SID;

	if (!$user->data['is_bot'] && $arcade_config['global_announce_enable'])
	{
		$arcade->global_announce();
	}

	$hc = ($mode == 'challenge') ? true : false;

	if ($show_welcome_box)
	{
		if ($hc)
		{
			// Start challenge top_games
			$arcade->obtain_data('top_games', $arcade_config['challenge_top_games_header']);
			// Start challenge leaders
		}
		else
		{
			// Start arcade newest_games
			$arcade->obtain_data('newest_games', $arcade_config['newest_games']);
			// Start arcade leaders
		}

		display_arcade_leaders($mode, 'header_leaders');

		// Start (arcade latest highscores or challenge latest winners)
		$latest_hiwi = ($hc) ? $challenge->obtain_challenge_latest_winners($arcade_config['challenge_latest_winners']) : $arcade->obtain_latest_highscores($arcade_config['latest_highscores']);

		$s_latest_score = false;
		foreach ($latest_hiwi as $latest)
		{
			$s_latest_score = true;
			if (!$auth_arcade->acl_get('c_view', $latest['cat_id']))
			{
				continue;
			}

			$game_popup = ($arcade->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $latest['cat_id'], $latest['game_id'], $latest['game_width'], $latest['game_height'], 'x') : '';

			if ($hc)
			{
				$winner_user		= get_arcade_user_name('full', $latest['winner_id'], $latest['winner_name'], $latest['winner_color'], 'challenge', 'x');
				$loser_user			= get_arcade_user_name('full', $latest['loser_id'], $latest['loser_name'], $latest['loser_color'], 'challenge', 'x');
				$champ_date			= sprintf($user->lang['CHALLENGE_WELCOME_DATE'], $user->format_date($latest['end_time']));
				$tooltip_msg		= sprintf($user->lang['CHALLENGE_WELCOME_SCORE'], $latest['winner_name'], $arcade->number_format($latest['winner_score']), $latest['loser_name'], $arcade->number_format($latest['loser_score']));
				$gn_tooltip			= get_arcade_game_name($latest['game_name'], true, 'play', $latest['cat_id'], $latest['game_id'], '', $tooltip_msg, true);
				$gn_date_tooltip	= get_arcade_game_name($latest['game_name'], true, 'play', $latest['cat_id'], $latest['game_id'], '', $tooltip_msg . '<br /><br />' . $champ_date , true);

				$champ_avatar		= ($arcade->optionget('view_avatars')) ? get_arcade_user_avatar($latest['winner_avatar'], $latest['winner_avatar_type'], 20, 20, 'x', 'challenge', $latest['winner_id']) : false;
				$champ_data			= sprintf($user->lang['CHALLENGE_WELCOME_CHAMP'], $winner_user, $game_popup . ' ' . $gn_tooltip, $loser_user);
				$champ_data_tooltip	= sprintf($user->lang['CHALLENGE_WELCOME_CHAMP'], $winner_user, $game_popup . ' ' . $gn_date_tooltip, $loser_user);
			}
			else
			{
				$high_score			= (!$arcade->hidden_score($latest['game_id'], $latest['game_highuser'])) ? $arcade->number_format($latest['game_highscore']) : $user->lang['HIDDEN'];
				$latest_scoreuser	= get_arcade_user_name('full', $latest['game_highuser'], $latest['username'], $latest['user_colour'], 'arcade', 'x');
				$champ_score		= sprintf($user->lang['ARCADE_WELCOME_SCORE'], $high_score);
				$champ_date			= sprintf($user->lang['ARCADE_WELCOME_SCORE_DATE'], $user->format_date($latest['game_highdate']));
				$game_name			= get_arcade_game_name($latest['game_name'], true, 'play', $latest['cat_id'], $latest['game_id'], 'x');
				$game_name_tooltip	= get_arcade_game_name($latest['game_name'], true, 'play', $latest['cat_id'], $latest['game_id'], '', $champ_score . '<br />' . $champ_date, true);

				$champ_avatar		= ($arcade->optionget('view_avatars')) ? get_arcade_user_avatar($latest['user_avatar'], $latest['user_avatar_type'], 20, 20, 'x', 'arcade', $latest['game_highuser']) : false;
				$champ_data			= sprintf($user->lang['ARCADE_WELCOME_CHAMP'], $latest_scoreuser, $game_popup . ' ' . $game_name, $arcade->number_format($latest['game_highscore'])) . ' ' . $champ_score;
				$champ_data_tooltip	= sprintf($user->lang['ARCADE_WELCOME_CHAMP'], $latest_scoreuser, $game_popup . ' ' . $game_name_tooltip);
			}

			$template->assign_block_vars('header_latest_chapm', array(
				'HEADING_CHAMP_AVATAR'	=> $champ_avatar,
				'HEADING_CHAMP'			=> $champ_data,
				'HEADING_CHAMP_DATE'	=> $champ_date,
				'HEADING_CHAMP_TOOLTIP'	=> $champ_data_tooltip,
			));
		}
		// End (arcade latest highscores or challenge latest winners)

		// Start user info
		$arcade->user_info('header', $mode);
		// End user info

		// Start total info
		$total_info = '';
		if ($hc)
		{
			$total_info = ($arcade_config['challenge_total_plays']) ? sprintf($user->lang['CHALLENGE_TOTAL_CHALLENGE' . (($arcade_config['challenge_total_plays'] > 1) ? 'S' : '')], $arcade->number_format($arcade_config['challenge_total_plays']), $arcade->time_format($arcade_config['challenge_total_plays_time'])) : '';
		}
		else
		{
			$total_games_header = sizeof($arcade->games);
			if ($total_games_header)
			{
				$total_games_header = sprintf($user->lang['ARCADE_TOTAL_GAME' . (($total_games_header > 1) ? 'S' : '')], $arcade->number_format($total_games_header));
				$total_downloads_header = ($arcade_config['total_downloads'] && $auth_arcade->acl_getc_global('c_download')) ? sprintf($user->lang['ARCADE_TOTAL_DOWNLOAD' . (($arcade_config['total_downloads'] > 1) ? 'S' : '')], $arcade->number_format($arcade_config['total_downloads'])) : '';
				$total_games_played = ($arcade_config['total_plays']) ? sprintf($user->lang['ARCADE_TOTAL_PLAY' . (($arcade_config['total_plays'] > 1) ? 'S' : '')], $arcade->number_format($arcade_config['total_plays']), $arcade->time_format($arcade_config['total_plays_time'])) : '';
				$total_info = "{$total_games_header}&nbsp;{$total_downloads_header}&nbsp;{$total_games_played}";
			}
		}
		// End total info

		$mode = strtoupper($mode);

		$template->assign_vars(array(
			'S_ARCADE_SHOW_WELCOME_BOX'		=> true,
			'S_LATEST_SCORE'				=> $s_latest_score,

			'L_ARCADE_WELCOME'				=> sprintf($user->lang[$mode . '_WELCOME'], get_arcade_user_name('full', $user->data['user_id'], $user->data['username'], '', strtolower($mode), 'x')),
			'L_ARCADE_GAMES_TITLE'			=> $user->lang[(($hc) ? 'CHALLENGE_TOP_GAMES' : 'ARCADE_NEWEST_GAMES')],
			'L_ARCADE_LEADERS_TITLE'		=> $user->lang[$mode . '_LEADERS'],
			'L_ARCADE_LATEST_CHAMP_TITLE'	=> $user->lang[(($hc) ? 'CHALLENGE_LATEST_WINNERS' : 'ARCADE_LATEST_HIGHSCORES')],
			'L_ARCADE_NO_LATEST_CHAMP'		=> $user->lang[(($hc) ? 'CHALLENGE_NO_LATEST_WINNERS' : 'ARCADE_NO_LATEST_HIGHSCORES')],

			'HEADER_TOTAL_INFO'				=> $total_info
		));
	}

	$s_search = ($auth_arcade->acl_get('u_arcade_search')) ? true : false;

	if (!$hc && $show_adv_search)
	{
		$arcade->games_list('quick_jump');

		$s_fav = ($auth_arcade->acl_get('u_arcade_favorites')) ? true : false;

		// Quick jump of your favorite games...
		if ($s_fav)
		{
			$game_fav_data = (!isset($game_fav_data)) ? $arcade->get_fav_data() : $game_fav_data;

			if (is_array($game_fav_data))
			{
				foreach ($game_fav_data as $gid => $game)
				{
					$template->assign_block_vars('fav', array(
						'GAME_ID'	=> $gid,
						'GAME_NAME'	=> $game['game_name'])
					);
				}
			}
		}

		$s_header_hidden_fields =  build_hidden_fields(array('mode' => 'play'));
		if ($_SID)
		{
			$s_header_hidden_fields .= build_hidden_fields(array('sid' =>  $_SID));
		}

		$template->assign_vars(array(
			'S_ACTION'						=> $arcade->url($arcade->gametop),
			'S_SHOW_ADV_SEARCH'				=> true,
			'S_HEADER_HIDDEN_FIELDS'		=> $s_header_hidden_fields,
			'S_ACTION_SEARCH'				=> ($s_search) ? $arcade->url('mode=search') : false,
			'S_HEADER_HIDDEN_FIELDS_SEARCH'	=> build_hidden_fields(array('mode' => 'search')),
			'S_ARCADE_SEARCH'				=> ($s_search) ? true : false,

			'ARCADE_BLOCK_WIDTH'			=> (($s_search && $s_fav) ? '33' : (($s_search || $s_fav) ? '49' : '99')) . '%'
		));
	}

	if ($show_links)
	{
		$gc_play = ($auth_arcade->acl_getc_global('c_play')) ? true : false;

		$template->assign_vars(array(
			'S_SHOW_LINKS'				=> (!$hc || ($hc && $arcade->optionget('challenge'))) ? true : false,
			'S_HEADER_GAME_RANDOM'		=> (($hc && $user->data['is_registered'] && $gc_play) || ($gc_play && !$hc)) ? true : false,
			'S_HEADER_NEW_GAMES'		=> (!$hc && $s_search && $gc_play) ? true : false,

			'L_HEADER_GAME_RANDOM'		=> $user->lang[(($hc) ? 'CHALLENGE_RANDOM' : 'ARCADE_FEELING_LUCKY')],

			'U_HEADER_GAME_RANDOM'		=> $arcade->url((($hc) ? 'mode=rand_challenge' : 'mode=random')) . $arcade->gametop,
			'U_HEADER_NEW_GAMES'		=> $arcade->url('mode=search&amp;search_id=newgames'),
			'U_HEADER_MY_FAV'			=> (!$hc) ? $arcade->url('mode=fav') : false,
			'U_HEADER_STATS'			=> $arcade->url((($hc) ? 'mode=view_challenge_stats' : 'mode=stats')),
			'U_HEADER_MY_STATS'			=> $arcade->url((($hc) ? 'mode=view_challenge_stats' : 'mode=stats') . "&amp;u={$user->data['user_id']}")
		));
	}

	$template->assign_vars(array(
		'S_ARCADE_STATISTICS'	=> ($auth_arcade->acl_get('u_arcade_viewstats')) ? true : false,
		'S_ARCADE_FAVORITE'		=> ($user->data['is_registered'] && $auth_arcade->acl_get('u_arcade_favorites')) ? true : false,
	));
}

function display_arcade_cat_stats($cat_id)
{
	global $user, $arcade, $arcade_config, $template;

	$arcade->obtain_data('longest_highscores', $arcade_config['cat_longest_held_scores'], 'least', $cat_id, false);
	$arcade->obtain_data('leaders', $arcade_config['cat_leaders'], 'most', $cat_id, false);
	$arcade->obtain_data('popular_games', $arcade_config['cat_most_popular'], 'most', $cat_id, false);
	$arcade->obtain_data('popular_games', $arcade_config['cat_least_popular'], 'least', $cat_id, false);
	$arcade->obtain_data('played_users', $arcade_config['cat_most_played_users'], 'most', $cat_id, false);
	$arcade->obtain_data('played_users', $arcade_config['cat_least_played_users'], 'least', $cat_id, false);
	$arcade->obtain_data('downloaded_games', $arcade_config['cat_most_downloaded'], 'most', $cat_id, false);
	$arcade->obtain_data('downloaded_games', $arcade_config['cat_least_downloaded'], 'least', $cat_id, false);

	$cat_name = $arcade->get_cat_field($cat_id, 'cat_name');

	$template->assign_vars(array(
		'S_CAT_STATS'			=> true,

		'L_ARCADE_STATS_TITLE'	=> sprintf($user->lang['ARCADE_CAT_STATS'], $cat_name),
		'L_ARCADE_LEADERS'		=> sprintf($user->lang['ARCADE_CAT_LEADERS'], $cat_name)
	));
}

/**
* Display Arcade
*/
function display_arcade($root_data = '')
{
	global $db, $auth, $auth_arcade, $user, $template;
	global $config, $arcade, $arcade_config;

	$cat_rows = $subcats = $cat_ids = array();
	$parent_id = $visible_cats = 0;

	if (!$root_data)
	{
		$root_data = array('cat_id' => 0);
		$sql_where = '';
	}
	else
	{
		$sql_where = ' WHERE left_id > ' . (int) $root_data['left_id'] . ' AND left_id < ' . (int) $root_data['right_id'];
	}

	$sql = 'SELECT *
		FROM ' . ARCADE_CATS_TABLE . "
		$sql_where
		ORDER BY left_id";
	$result = $db->sql_query($sql, $arcade_config['cache_time'] * 3600);

	$branch_root_id = $root_data['cat_id'];
	while ($row = $db->sql_fetchrow($result))
	{
		$cat_id = $row['cat_id'];

		// Category with no members
		if ($row['cat_type'] == ARCADE_CAT && ($row['left_id'] + 1 == $row['right_id']))
		{
			continue;
		}

		// Skip branch
		if (isset($right_id))
		{
			if ($row['left_id'] < $right_id)
			{
				continue;
			}
			unset($right_id);
		}

		if (!$auth_arcade->acl_get('c_list', $cat_id))
		{
			// if the user does not have permissions to list this category, skip everything until next branch
			$right_id = $row['right_id'];
			continue;
		}

		$cat_ids[] = $cat_id;

		if ($row['parent_id'] == $root_data['cat_id'] || $row['parent_id'] == $branch_root_id)
		{
			// Direct child of current branch
			$parent_id = $cat_id;
			$cat_rows[$cat_id] = $row;

			if ($row['cat_type'] == ARCADE_CAT && $row['parent_id'] == $root_data['cat_id'])
			{
				$branch_root_id = $cat_id;
			}
			$cat_rows[$parent_id]['orig_cat_last_play_time'] = $row['cat_last_play_time'];
		}
		else if ($row['cat_type'] != ARCADE_CAT)
		{
			$subcats[$parent_id][$cat_id]['display'] = ($row['display_on_index']) ? true : false;
			$subcats[$parent_id][$cat_id]['name'] = $row['cat_name'];
			$subcats[$parent_id][$cat_id]['orig_cat_last_play_time'] = $row['cat_last_play_time'];
			$subcats[$parent_id][$cat_id]['cat_last_game_installdate'] = $row['cat_last_game_installdate'];

			$cat_rows[$parent_id]['cat_games'] += $row['cat_games'];

			// Do not list redirects in LINK Categorys as Posts.
			if ($row['cat_type'] != ARCADE_LINK)
			{
				$cat_rows[$parent_id]['cat_plays'] += $row['cat_plays'];
			}

			if ($row['cat_last_play_time'] > $cat_rows[$parent_id]['cat_last_play_time'])
			{
				$cat_rows[$parent_id]['cat_last_play_game_id'] = $row['cat_last_play_game_id'];
				$cat_rows[$parent_id]['cat_last_play_game_name'] = $row['cat_last_play_game_name'];
				$cat_rows[$parent_id]['cat_last_play_time'] = $row['cat_last_play_time'];
				$cat_rows[$parent_id]['cat_last_play_user_id'] = $row['cat_last_play_user_id'];
				$cat_rows[$parent_id]['cat_last_play_username'] = $row['cat_last_play_username'];
				$cat_rows[$parent_id]['cat_last_play_user_colour'] = $row['cat_last_play_user_colour'];
				$cat_rows[$parent_id]['cat_last_play_score'] = $row['cat_last_play_score'];
			}
		}
	}
	$db->sql_freeresult($result);

	// Used to tell whatever we have to create a dummy category or not.
	$last_catless = true;
	foreach ($cat_rows as $row)
	{
		// Empty category
		if ($row['parent_id'] == $root_data['cat_id'] && $row['cat_type'] == ARCADE_CAT)
		{
			$template->assign_block_vars('catrow', array(
				'S_IS_CAT'				=> true,
				'CAT_ID'				=> $row['cat_id'],
				'CAT_NAME'				=> $row['cat_name'],
				'CAT_DESC'				=> generate_text_for_display($row['cat_desc'], $row['cat_desc_uid'], $row['cat_desc_bitfield'], $row['cat_desc_options']),
				'CAT_FOLDER_IMG'		=> '',
				'CAT_FOLDER_IMG_SRC'	=> '',
				'CAT_IMAGE'				=> $arcade->get_image('full', 'cat', $row['cat_image'], $row['cat_name']),
				'CAT_IMAGE_SRC'			=> $arcade->get_image('src', 'cat', $row['cat_image'], $row['cat_name']),
				'U_VIEWCAT'				=> $arcade->url('mode=cat&amp;c=' . $row['cat_id'])));

			continue;
		}

		$visible_cats++;
		$cat_id = $row['cat_id'];

		$cat_new_games = ((time() - $row['cat_last_game_installdate']) <= ($arcade_config['new_games_delay'] * 86400)) ? true : false;

		$folder_image = $folder_alt = $l_subcats = '';
		$subcats_list = array();

		// Generate list of subcategorys if we need to
		if (isset($subcats[$cat_id]))
		{
			foreach ($subcats[$cat_id] as $subcat_id => $subcat_row)
			{
				$subcat_new_games =  ( (time() - $subcat_row['cat_last_game_installdate']) <= ($arcade_config['new_games_delay'] * 86400))? true : false;

				if ($subcat_row['display'] && $subcat_row['name'])
				{
					$subcats_list[] = array(
						'link'		=> $arcade->url('mode=cat&amp;c=' . $subcat_id),
						'name'		=> $subcat_row['name'],
						'unread'	=> $subcat_new_games,
					);
				}
				else
				{
					unset($subcats[$cat_id][$subcat_id]);
				}

				// If one subcat has new games the cat gets new games too...
				if ($subcat_new_games)
				{
					$cat_new_games = true;
				}
			}

			$l_subcats = (sizeof($subcats[$cat_id]) == 1) ? $user->lang['ARCADE_SUBCAT'] . ': ' : $user->lang['ARCADE_SUBCATS'] . ': ';
			$folder_image = ($cat_new_games) ? 'forum_unread_subforum' : 'forum_read_subforum';
		}
		else
		{
			switch ($row['cat_type'])
			{
				case ARCADE_CAT_GAMES:
					$folder_image = ($cat_new_games) ? 'forum_unread' : 'forum_read';
				break;

				case ARCADE_LINK:
					$folder_image = 'forum_link';
				break;
			}
		}

		// Which folder should we display?
		if ($row['cat_status'] == ITEM_LOCKED)
		{
			$folder_image = ($cat_new_games) ? 'forum_unread_locked' : 'forum_read_locked';
			$folder_alt = 'ARCADE_CAT_LOCKED';
		}
		else
		{
			$folder_alt = ($cat_new_games) ? 'ARCADE_NEW_GAMES' : 'ARCADE_NO_NEW_GAMES';
		}

		// Create last play link information, if appropriate
		if ($row['cat_last_play_game_id'] && $auth_arcade->acl_get('c_view', $cat_id))
		{
			$last_play_time = $user->format_date($row['cat_last_play_time']);
			$style_color = 'style="color: #' . $row['cat_last_play_user_colour'] . '; font-weight: bold;"';

			$data = array(
				'cat_id'	=> $cat_id,
				'game_id'	=> $row['cat_last_play_game_id'],
				'game_name'	=> $row['cat_last_play_game_name']
			);

			$last_score		= (!$arcade->hidden_score($row['cat_last_play_game_id'], $row['cat_last_play_user_id'])) ? $arcade->number_format($row['cat_last_play_score']) : $user->lang['HIDDEN'];
			$game_link		= get_arcade_game_name($data['game_name'], true, 'play', $data['cat_id'], $data['game_id'], 'x');
			$player_link	= get_arcade_user_name('full', $row['cat_last_play_user_id'], $row['cat_last_play_username'], $row['cat_last_play_user_colour'], 'arcade', 'x');
			$last_play		= sprintf($user->lang['ARCADE_LAST_PLAY'], $player_link, $last_score, $game_link);
		}
		else
		{
			$last_play_time = '';
			$last_play = '';
		}

		if ($row['cat_type'] != ARCADE_LINK)
		{
			$u_viewcat = $arcade->url('mode=cat&amp;c=' . $row['cat_id']);
		}
		else
		{
			// If the category is a link and we count redirects we need to visit it
			// If the category is having a password or no read access we do not expose the link, but instead handle it in viewforum
			if (($row['cat_flags'] & ARCADE_FLAG_LINK_TRACK) || $row['cat_password'] || !$auth_arcade->acl_get('c_view', $cat_id))
			{
				$u_viewcat = $arcade->url('mode=cat&amp;c=' . $row['cat_id']);
			}
			else
			{
				$u_viewcat = $row['cat_link'];
			}
		}

		$s_subcats_list = array();
		foreach ($subcats_list as $subcat)
		{
			$s_subcats_list[] = '<a href="' . $subcat['link'] . '" class="subforum ' . (($subcat['unread']) ? 'unread' : 'read') . '">' . $subcat['name'] . '</a>';
		}
		$s_subcats_list = (string) implode(', ', $s_subcats_list);
		$catless = ($row['parent_id'] == $root_data['cat_id']) ? true : false;

		$l_play_click_count = ($row['cat_type'] == ARCADE_LINK) ? 'CLICKS' : 'PLAYS';
		$play_click_count = ($row['cat_type'] != ARCADE_LINK || $row['cat_flags'] & ARCADE_FLAG_LINK_TRACK) ? $arcade->number_format($row['cat_plays']) : '';

		$template->assign_block_vars('catrow', array(
			'S_IS_CAT'				=> false,
			'S_NO_CAT'				=> $catless && !$last_catless,
			'S_IS_LINK'				=> ($row['cat_type'] == ARCADE_LINK) ? true : false,
			'S_CAT_NEW_GAMES'		=> $cat_new_games,
			'S_LOCKED_CAT'			=> ($row['cat_status'] == ITEM_LOCKED) ? true : false,
			'S_LIST_SUBCATS'		=> ($row['display_subcat_list']) ? true : false,
			'S_SUBCATS'				=> (sizeof($subcats_list)) ? true : false,

			'CAT_ID'				=> $row['cat_id'],
			'CAT_NAME'				=> $row['cat_name'],
			'CAT_DESC'				=> generate_text_for_display($row['cat_desc'], $row['cat_desc_uid'], $row['cat_desc_bitfield'], $row['cat_desc_options']),
			'GAMES'					=> $arcade->number_format($row['cat_games']),
			$l_play_click_count		=> $play_click_count,
			'CAT_FOLDER_IMG'		=> $user->img($folder_image, $folder_alt),
			'CAT_FOLDER_IMG_SRC'	=> $user->img($folder_image, $folder_alt, false, '', 'src'),
			'CAT_FOLDER_IMG_ALT'	=> isset($user->lang[$folder_alt]) ? $user->lang[$folder_alt] : '',
			'CAT_IMAGE'				=> $arcade->get_image('full', 'cat', $row['cat_image'], $row['cat_name']),
			'CAT_IMAGE_SRC'			=> $arcade->get_image('src', 'cat', $row['cat_image'], $row['cat_name']),
			'CAT_DISPLAY'			=> $row['cat_display'],
			'SUBCATS'				=> $s_subcats_list,
			'LAST_PLAY_TIME'		=> $last_play_time,
			'LAST_PLAY'				=> $last_play,

			'L_SUBCAT_STR'			=> $l_subcats,
			'L_CAT_FOLDER_ALT'		=> $folder_alt,

			'U_VIEWCAT'				=> $u_viewcat,
		));

		// Assign subforums loop for style authors
		foreach ($subcats_list as $subcat)
		{
			$template->assign_block_vars('catrow.subcat', array(
				'U_SUBCAT'		=> $subcat['link'],
				'SUBCAT_NAME'	=> $subcat['name'],
				'S_UNREAD'		=> $subcat['unread'])
			);
		}
		$last_catless = $catless;
	}

	$template->assign_vars(array(
		'S_HAS_SUBCAT'	=> ($visible_cats) ? true : false,
		'L_SUBCAT'		=> ($visible_cats == 1) ? $user->lang['ARCADE_SUBCAT'] : $user->lang['ARCADE_SUBCATS']
	));

	return;
}

/**
* Display Arcade Online
*/
function display_arcade_online()
{
	global $user, $template, $arcade, $arcade_config, $auth_arcade;

	if ($auth_arcade->acl_get('u_arcade_view_whoisplaying'))
	{
		$online_users = array(
			'games_names'	=> array(),
			'games_players'	=> array(),
			'total_online'	=> 0,
			'visible_online'=> 0,
			'hidden_online'	=> 0,
			'guests_online'	=> 0,
			'online_time'	=> 0
		);

		$arcade->obtain_users_online($online_users);

		$count = 1;
		if ($online_users['total_online'] > 0)
		{
			foreach ($online_users['games_names'] as $key => $game_name)
			{
				$template->assign_block_vars('arcade_online_row', array(
					'GAME'			=> $game_name,
					'PLAYER_LIST'	=> $online_users['games_players'][$key]
				));

				$count++;
			}
		}

		$vars_online = array(
			'ARCADE_ONLINE'	=> array('total_online', 'l_t_user_s', 0),
			'REG'			=> array('visible_online', 'l_r_user_s', 1),
			'HIDDEN'		=> array('hidden_online', 'l_h_user_s', 1),
			'GUEST'			=> array('guests_online', 'l_g_user_s', 0)
		);

		foreach ($vars_online as $l_prefix => $var_ary)
		{
			if ($var_ary[2])
			{
				$l_suffix = '_AND';
			}
			else
			{
				$l_suffix = '';
			}
			switch ($online_users[$var_ary[0]])
			{
				case 0:
					${$var_ary[1]} = $user->lang[$l_prefix . '_USERS_ZERO_TOTAL' . $l_suffix];
				break;

				case 1:
					${$var_ary[1]} = $user->lang[$l_prefix . '_USER_TOTAL' . $l_suffix];
				break;

				default:
					${$var_ary[1]} = $user->lang[$l_prefix . '_USERS_TOTAL' . $l_suffix];
				break;
			}
		}
		unset($vars_online);

		$l_online_users  = sprintf($l_t_user_s, $online_users['total_online']);
		$l_online_users .= sprintf($l_r_user_s, $online_users['visible_online']);
		$l_online_users .= sprintf($l_h_user_s, $online_users['hidden_online']);
		$l_online_users .= sprintf($l_g_user_s, $online_users['guests_online']);
		$l_online_time   = sprintf($user->lang['VIEW_ONLINE_TIME' . (($online_users['online_time'] > 1) ? 'S' : '')], $online_users['online_time']);

		if ($online_users['total_online'] > $arcade_config['record_playing_users'])
		{
			$arcade_config['record_playing_users']	= $online_users['total_online'];
			$arcade_config['record_playing_date']	= time();

			$arcade->set_config('record_playing_users', $arcade_config['record_playing_users'], true);
			$arcade->set_config('record_playing_date', $arcade_config['record_playing_date'], true);
		}

		$template->assign_vars(array(
			'S_CAN_VIEW_WHOISPLAYING'		=> true,

			'ARCADE_TOTAL_USERS_PLAYING'	=> $l_online_users . ' (' . $l_online_time . ')',
			'ARCADE_RECORD_USERS_PLAYING'	=> sprintf($user->lang['ARCADE_ONLINE_USERS_RECORD'], $arcade_config['record_playing_users'], $user->format_date($arcade_config['record_playing_date'], false, true)),
			'GAMES_COUNT'					=> $count
		));
	}
}

/**
* Create forum navigation links for given category, create parent
* list if currently null, assign basic category info to template
*/
function generate_arcade_nav(&$cat_data, $in_game = false, $game_stats = false)
{
	global $db, $user, $template, $auth, $arcade, $auth_arcade;

	if (!$auth_arcade->acl_get('c_list', $cat_data['cat_id']))
	{
		return;
	}

	// Get category parents
	$cat_parents = get_arcade_parents($cat_data);

	if (!empty($cat_data))
	{
		// Build navigation links
		foreach ($cat_parents as $parent_cat_id => $parent_data)
		{
			list($parent_name, $parent_type) = array_values($parent_data);

			// Skip this parent if the user does not have the permission to view it
			if (!$auth_arcade->acl_get('c_list', $parent_cat_id))
			{
				continue;
			}

			$template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $parent_name,
				'FORUM_ID'		=> $parent_cat_id,
				'U_VIEW_FORUM'	=> $arcade->url('mode=cat&amp;c=' . $parent_cat_id)
			));
		}
	}

	$template->assign_block_vars('navlinks', array(
		'FORUM_NAME'	=> $cat_data['cat_name'],
		'FORUM_ID'		=> $cat_data['cat_id'],
		'U_VIEW_FORUM'	=> $arcade->url('mode=cat&amp;c=' . $cat_data['cat_id'])
	));

	if ($in_game)
	{
		$mode = ($game_stats) ? $game_stats : 'play';

		$template->assign_block_vars('navlinks', array(
			'FORUM_NAME'	=> $cat_data['game_name'],
			'U_VIEW_FORUM'	=> str_replace($arcade->gametop, '', $arcade->url('mode=' . $mode . '&amp;g=' . $cat_data['game_id']))
		));
	}

	if (!empty($cat_data['view_stats']))
	{
		$template->assign_block_vars('navlinks', array(
			'FORUM_NAME'	=> $user->lang['ARCADE_STATS'],
			'FORUM_ID'		=> $cat_data['cat_id'],
			'U_VIEW_FORUM'	=> $arcade->url('mode=view_cat_stats&amp;c=' . $cat_data['cat_id'])
		));
	}

	$template->assign_vars(array(
		'CAT_NAME'		=> $cat_data['cat_name'],
		'U_VIEW_FORUM'	=> $arcade->url('mode=cat&amp;c=' . $cat_data['cat_id']),
	));
}

/**
* Create category rules for given category
*/
function generate_cat_rules(&$cat_data)
{
	if (!$cat_data['cat_rules'] && !$cat_data['cat_rules_link'])
	{
		return;
	}

	global $template;

	if ($cat_data['cat_rules'])
	{
		$cat_data['cat_rules'] = generate_text_for_display($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options']);
	}

	$template->assign_vars(array(
		'S_CAT_RULES'	=> true,
		'U_CAT_RULES'	=> $cat_data['cat_rules_link'],
		'CAT_RULES'		=> $cat_data['cat_rules']
	));
}

/**
* Returns forum parents as an array. Get them from forum_data if available, or update the database otherwise
*/
function get_arcade_parents(&$cat_data)
{
	global $db;

	$cat_parents = array();

	if ($cat_data['parent_id'] > 0)
	{
		if ($cat_data['cat_parents'] == '')
		{
			$sql = 'SELECT cat_id, cat_name, cat_type
				FROM ' . ARCADE_CATS_TABLE . '
				WHERE left_id < ' . (int) $cat_data['left_id'] . '
					AND right_id > ' . (int) $cat_data['right_id'] . '
				ORDER BY left_id ASC';
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$cat_parents[$row['cat_id']] = array($row['cat_name'], (int) $row['cat_type']);
			}
			$db->sql_freeresult($result);

			$cat_data['cat_parents'] = serialize($cat_parents);

			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET cat_parents = '" . $db->sql_escape($cat_data['cat_parents']) . "'
				WHERE parent_id = " . (int) $cat_data['parent_id'];
			$db->sql_query($sql);
		}
		else
		{
			$cat_parents = unserialize($cat_data['cat_parents']);
		}
	}

	return $cat_parents;
}

function gen_arcade_auth_level($cat_id, $cat_status)
{
	global $template, $auth, $auth_arcade, $arcade, $user, $config;

	$locked = ($cat_status == ITEM_LOCKED && !$auth->acl_get('a_arcade_cat')) ? true : false;

	$rules = array_filter(array(
		($auth_arcade->acl_get('c_play', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_PLAY_CAN'] : $user->lang['ARCADE_RULES_PLAY_CANNOT'],
		($arcade->points['show']) ? ($auth_arcade->acl_get('c_playfree', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_PLAY_FREE_CAN'] : $user->lang['ARCADE_RULES_PLAY_FREE_CANNOT'] : '',
		($user->data['is_registered'] && $auth_arcade->acl_get('c_score', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_SCORE_CAN'] : $user->lang['ARCADE_RULES_SCORE_CANNOT'],
		($user->data['is_registered'] && $auth_arcade->acl_get('c_comment', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_COMMENT_CAN'] : $user->lang['ARCADE_RULES_COMMENT_CANNOT'],
		($auth_arcade->acl_get('c_report', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_REPORT_CAN'] : $user->lang['ARCADE_RULES_REPORT_CANNOT'],
		($user->data['is_registered'] && $auth_arcade->acl_get('c_rate', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_RATE_CAN'] : $user->lang['ARCADE_RULES_RATE_CANNOT'],
		($user->data['is_registered'] && $auth_arcade->acl_get('c_re_rate', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_RE_RATE_CAN'] : $user->lang['ARCADE_RULES_RE_RATE_CANNOT'],
		($auth_arcade->acl_get('u_arcade_download') && $auth_arcade->acl_get('c_download', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_DOWNLOAD_CAN'] : $user->lang['ARCADE_RULES_DOWNLOAD_CANNOT'],
		($auth_arcade->acl_get('c_downloadfree') && !$locked) ? $user->lang['ARCADE_RULES_DOWNLOAD_FREE_CAN'] : $user->lang['ARCADE_RULES_DOWNLOAD_FREE_CANNOT'],
		($auth_arcade->acl_get('c_ignorecontrol', $cat_id) && !$locked) ? $user->lang['ARCADE_RULES_IGNORE_CONTROL_CAN'] : $user->lang['ARCADE_RULES_IGNORE_CONTROL_CANNOT'],
		($user->data['is_registered']) ? (($auth_arcade->acl_get('c_challenge_playing_games', $cat_id) && !$locked) ? $user->lang['CHALLENGE_RULES_PLAYING_GAMES_CAN'] : $user->lang['CHALLENGE_RULES_PLAYING_GAMES_CANNOT']) : '',
		($user->data['is_registered']) ? (($auth_arcade->acl_get('c_challenge_start_game_warning', $cat_id) && !$locked) ? $user->lang['CHALLENGE_RULES_START_GAME_WARNING_CAN'] : $user->lang['CHALLENGE_RULES_START_GAME_WARNING_CANNOT']) : '',
	));

	foreach ($rules as $rule)
	{
		if (!$rule)
		{
			continue;
		}

		$template->assign_block_vars('rules', array('RULE' => $rule));
	}
}

function login_arcade_box($cat_data)
{
	global $db, $config, $arcade_config, $user, $template, $arcade;

	// Founder ignore the password.
	if ($arcade_config['founder_exempt'] && $user->data['user_type'] == USER_FOUNDER)
	{
		return true;
	}

	$password = request_var('password', '', true);

	$sql = 'SELECT cat_id
		FROM ' . ARCADE_ACCESS_TABLE . '
		WHERE cat_id = ' . $cat_data['cat_id'] . '
			AND user_id = ' . $user->data['user_id'] . "
			AND session_id = '" . $db->sql_escape($user->session_id) . "'";
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);

	if ($row)
	{
		return true;
	}

	if ($password)
	{
		// Remove expired authorised sessions
		$sql = 'SELECT session_id
			FROM ' . SESSIONS_TABLE;
		$result = $db->sql_query($sql);

		if ($row = $db->sql_fetchrow($result))
		{
			$sql_in = array();
			do
			{
				$sql_in[] = (string) $row['session_id'];
			}
			while ($row = $db->sql_fetchrow($result));

			// Remove expired sessions
			$sql = 'DELETE FROM ' . ARCADE_ACCESS_TABLE . '
				WHERE ' . $db->sql_in_set('session_id', $sql_in, true);
			$db->sql_query($sql);
		}
		$db->sql_freeresult($result);

		if (phpbb_check_hash($password, $cat_data['cat_password']))
		{
			$sql_ary = array(
				'cat_id'		=> (int) $cat_data['cat_id'],
				'user_id'		=> (int) $user->data['user_id'],
				'session_id'	=> (string) $user->session_id,
			);

			$db->sql_query('INSERT INTO ' . ARCADE_ACCESS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));

			return true;
		}

		$template->assign_var('LOGIN_ERROR', $user->lang['WRONG_PASSWORD']);
	}

	page_header($user->lang['LOGIN']);

	$template->assign_vars(array(
		'CAT_NAME'			=> $cat_data['cat_name'],
		'S_LOGIN_ACTION'	=> build_url(),
	));

	$template->set_filenames(array(
		'body' => 'arcade/login_cat.html')
	);

	page_footer();
}

function get_arcade_game_image($image, $width = 50, $height = 50, $title = '', $mode = false, $cat_id = false, $game_id = false, $user_id = false)
{
	global $arcade, $arcade_config;

	$game_image = '';

	if ($arcade_config['display_game_image'] && $image && $arcade->optionget('view_game_image'))
	{
		$game_image = '<img src="' . $arcade->url("img={$image}") . '" alt="" title="" width="' . $width . '" height="' . $height . '" style="vertical-align: middle;" />';

		if (!in_array($mode, array('play', 'stats', 'download', 'challenge')))
		{
			$mode = false;
		}

		if ($mode && $arcade_config['enable_image_link'] && $cat_id && $game_id)
		{
			$active_image = false;
			$arcade->active_check($mode, $active_image, $title, $cat_id);

			if ($active_image)
			{
				if ($mode == 'challenge')
				{
					$mode = 'view_challenge_stats';
				}

				$game_image = '<a href="' . $arcade->url("mode=$mode&amp;g=$game_id" . (($user_id) ? "&amp;u=$user_id" : '')) . '">' . (($title) ? str_replace('alt="" title=""', 'alt="' . $title . '" title="' . $title . '"', $game_image) : $game_image) . '</a>';
			}
		}
	}

	return $game_image;
}

function get_arcade_game_popup($mode, $cat_id, $game_id, $width, $height, $title = '', $size = 11)
{
	global $arcade, $arcade_config;

	$game_popup = '';

	if (!in_array($mode, array('icon', 'link')))
	{
		$mode = false;
	}

	if ($mode && $cat_id && $game_id && $width && $height && (($mode == 'icon' && $arcade_config['display_game_popup_icon'] && $arcade->data('popup_icon') && $arcade->optionget('view_popup_icon')) || ($mode != 'icon')))
	{
		$active_popup = false;
		$arcade->active_check($mode, $active_popup, $title, $cat_id);

		if ($active_popup)
		{
			switch ($mode)
			{
				case 'icon':
					$game_popup = '<a href="#" onclick="arcade_popup_game(\'' . $arcade->url("mode=popup&amp;g={$game_id}") . '\', \'' . $width . '\', \'' . $height . '\'); return false;"><img style="vertical-align: middle;" width="' . $size . '" height="' . $size . '" src="' . $arcade->data['popup_icon'] . '" alt="' . $title . '" title="' . $title . '" /></a>';
				break;

				case 'link':
					global $user;
					$game_popup = '<a href="#" onclick="arcade_popup_game(\'' . $arcade->url("mode=popup&amp;g={$game_id}") . '\', \'' . $width . '\', \'' . $height . '\'); return false;">' . $user->lang['ARCADE_PLAYING_NEW_WINDOW'] . '</a>';
					$game_popup = ($title) ? str_replace('<a', '<a title="' . $title . '"', $game_popup) : $game_popup;
				break;
			}
		}
	}

	return $game_popup;
}

function get_arcade_game_name($gname, $chars_limit = true, $mode = false, $cat_id = false, $game_id = false, $title = '', $tooltip = false, $header = false, $user_id = false)
{
	$game_name = '';

	if ($gname)
	{
		global $arcade_config;

		$gname = ($chars_limit && $arcade_config['gamename_maxchars']) ? ((strlen($gname) > $arcade_config['gamename_maxchars']) ? substr($gname, 0, $arcade_config['gamename_maxchars']) . '...' : $gname) : $gname;

		if (!in_array($mode, array('play', 'stats', 'download', 'challenge')))
		{
			$mode = false;
		}

		if ($mode && $cat_id && $game_id)
		{
			global $arcade;

			$active_game_name = false;
			$arcade->active_check($mode, $active_game_name, $title, $cat_id);

			if ($active_game_name)
			{
				if ($mode == 'challenge')
				{
					$mode = 'view_challenge_stats';
				}

				$game_name = '<a href="' . $arcade->url("mode=$mode&amp;g=$game_id" . (($user_id) ? "&amp;u=$user_id" : '')) . '">' . $gname . '</a>';

				if (!$tooltip && $title)
				{
					$game_name = str_replace('<a', '<a title="' . $title . '"', $game_name);
				}

				if ($tooltip)
				{
					$game_name = str_replace(">$gname</a>", ' class="genmed arcade_tooltip"' . (($header) ? ' style="font-weight: normal;"' : '') . '>' . $gname . '<span' . (($header) ? ' class="arcade_header"' : '') . '>' . $tooltip . '</span></a>', $game_name);
				}
			}
		}

		$game_name = ($game_name) ? $game_name : $gname;
	}

	return $game_name;
}

function get_arcade_user_name($mode, $user_id, $username, $user_colour, $type = false, $title = '', $custom_url = false)
{
	global $arcade;

	if (!in_array($type, array('arcade', 'challenge', 'pm', 'profile')))
	{
		$type = false;
	}

	if ($type && $user_id && $user_id != ANONYMOUS)
	{
		$active_link = false;
		$arcade->active_check("{$type}_user", $active_link, $title);

		if ($active_link)
		{
			switch ($type)
			{
				case 'arcade':
					$custom_url = $arcade->url("mode=stats");
				break;

				case 'challenge':
					$custom_url = $arcade->url("mode=view_challenge_stats");
				break;

				case 'pm':
					global $user;
					$custom_url = $arcade->url("i=pm&amp;mode=compose", 'ucp', $user->session_id);
				break;
			}
		}
	}
	else
	{
		$title = false;
	}

	$username = get_username_string($mode, $user_id, $username, $user_colour, false, $custom_url);

	return (($title) ? str_replace('<a', '<a title="' . $title . '"', $username) : $username);
}

function get_arcade_user_avatar($avatar, $avatar_type, $width, $height, $title = '', $mode = false, $user_id = false)
{
	global $user, $arcade, $arcade_config;

	$user_avatar = '';

	if (!in_array($mode, array('arcade', 'challenge', 'profile')))
	{
		$mode = false;
	}

	if ($arcade_config['display_user_avatar'] && $arcade->optionget('view_avatars'))
	{
		$user_avatar = str_replace('<img' , '<img style="vertical-align: middle;"', get_user_avatar($avatar, $avatar_type, '{WIDTH}', '{HEIGHT}', '{TITLE}'));

		if (!$user_avatar && $arcade->data('no_avatar'))
		{
			$user_avatar = '<img style="vertical-align: middle;" width="{WIDTH}" height="{HEIGHT}" src="' . $arcade->data['no_avatar'] . '" alt="{TITLE}" title="{TITLE}" />';
		}

		if ($user_avatar)
		{
			$active_avatar = false;

			if ($mode && $arcade_config['enable_user_avatar_link'] && $user_id && $user_id != ANONYMOUS)
			{
				$arcade->active_check("{$mode}_user", $active_avatar, $title);

				if ($active_avatar)
				{
					switch ($mode)
					{
						case 'arcade':
							$user_avatar = '<a href="' . $arcade->url("mode=stats&amp;u={$user_id}") . '">' . $user_avatar . '</a>';
						break;

						case 'challenge':
							$user_avatar = '<a href="' . $arcade->url("mode=view_challenge_stats&amp;u={$user_id}") . '">' . $user_avatar . '</a>';
						break;

						case 'profile':
							global $user;
							$user_avatar = '<a href="' . $arcade->url("mode=viewprofile&amp;u={$user_id}", 'memberlist', $user->session_id) . '">' . $user_avatar . '</a>';
						break;
					}
				}
			}
			else
			{
				$arcade->active_check('def', $active_avatar, $title);
			}

			if (!$width || !$height)
			{
				$width = $height = 100;
			}

			if (strpos($user_avatar, 'title="{TITLE}"') === false)
			{
				$user_avatar = str_replace('alt="{TITLE}"', 'alt="{TITLE}" title="{TITLE}"', $user_avatar);
			}

			$user_avatar = str_replace(array('{WIDTH}', '{HEIGHT}', '{TITLE}'), array($width, $height, $title), $user_avatar);
		}
	}

	return $user_avatar;
}

/**
* Generate Jumpbox
*/
function make_arcade_jumpbox($action, $cat_id = false, $select_all = false, $acl_list = false)
{
	global $config, $auth, $auth_arcade, $template, $user, $db, $arcade;

	if (!$config['load_jumpbox'])
	{
		return;
	}

	$sql = 'SELECT cat_id, cat_name, parent_id, cat_type, left_id, right_id
			FROM ' . ARCADE_CATS_TABLE . '
			ORDER BY left_id ASC';
	$result = $db->sql_query($sql, 600);

	$right = $padding = 0;
	$padding_store = array('0' => 0);
	$display_jumpbox = false;
	$iteration = 0;

	// Sometimes it could happen that forums will be displayed here not be displayed within the index page
	// This is the result of forums not displayed at index, having list permissions and a parent of a forum with no permissions.
	// If this happens, the padding could be "broken"

	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['left_id'] < $right)
		{
			$padding++;
			$padding_store[$row['parent_id']] = $padding;
		}
		else if ($row['left_id'] > $right + 1)
		{
			// Ok, if the $padding_store for this parent is empty there is something wrong. For now we will skip over it.
			// @todo digging deep to find out "how" this can happen.
			$padding = (isset($padding_store[$row['parent_id']])) ? $padding_store[$row['parent_id']] : $padding;
		}

		$right = $row['right_id'];

		if ($row['cat_type'] == ARCADE_CAT && ($row['left_id'] + 1 == $row['right_id']))
		{
			// Non-postable forum with no subforums, don't display
			continue;
		}

		if (!$auth_arcade->acl_get('c_list', $row['cat_id']))
		{
			// if the user does not have permissions to list this forum skip
			continue;
		}

		if ($acl_list && !$auth_arcade->acl_gets($acl_list, $row['cat_id']))
		{
			continue;
		}
		if (!$display_jumpbox)
		{
			$template->assign_block_vars('jumpbox_arcade', array(
				'CAT_ID'		=> ($select_all) ? 0 : -1,
				'CAT_NAME'		=> ($select_all) ? $user->lang['ALL_FORUMS'] : $user->lang['ARCADE_SELECT_CATEGORY'],
				'S_CAT_COUNT'	=> $iteration)
			);

			$iteration++;
			$display_jumpbox = true;
		}

		$template->assign_block_vars('jumpbox_arcade', array(
			'CAT_ID'		=> $row['cat_id'],
			'CAT_NAME'		=> $row['cat_name'],
			'SELECTED'		=> ($row['cat_id'] == $cat_id) ? ' selected="selected"' : '',
			'S_CAT_COUNT'	=> $iteration,
		));

		for ($i = 0; $i < $padding; $i++)
		{
			$template->assign_block_vars('jumpbox_arcade.level', array());
		}
		$iteration++;
	}
	$db->sql_freeresult($result);
	unset($padding_store);

	$template->assign_vars(array(
		'S_DISPLAY_JUMPBOX'	=> $display_jumpbox,
		'S_JUMPBOX_ACTION'	=> $action)
	);

	return;
}

/**
* Updates install file from the information in the db
*/
function arcade_game_update_install_file($game_id)
{
	global $arcade;

	$game_info = $arcade->get_game_data($game_id);
	$install_file = arcade_game_create_install_file($game_info);
	$updated_file = $arcade->set_path($game_info['game_swf'], 'install');
	return arcade_fwrite($updated_file, $install_file);
}

/**
* Creates install file for the game
*/
function arcade_game_create_install_file(&$game_info)
{
	global $file_functions, $phpbb_root_path, $phpEx;

	$game_type = '';
	$game_info['game_type'] = (int) $game_info['game_type'];
	switch ($game_info['game_type'])
	{
		case AMOD_GAME:
			$game_type = 'AMOD_GAME';
		break;

		case AR_GAME:
			$game_type = 'AR_GAME';
		break;

		case IBPRO_GAME:
			$game_type = 'IBPRO_GAME';
		break;

		case ARCADELIB_GAME:
			$game_type = 'ARCADELIB_GAME';
		break;

		case V3ARCADE_GAME:
			$game_type = 'V3ARCADE_GAME';
		break;

		case IBPROV3_GAME:
			$game_type = 'IBPROV3_GAME';
		break;

		case PHPBBARCADE_GAME:
			$game_type = 'PHPBBARCADE_GAME';
		break;

		case NOSCORE_GAME:
			$game_type = 'NOSCORE_GAME';
		break;

		default:
			$game_type = 'UNKNOWN_GAME';
		break;
	}

	$game_scoretype = '';
	$game_info['game_width'] = (int) $game_info['game_width'];
	$game_info['game_height'] = (int) $game_info['game_height'];
	$game_info['game_scoretype'] = (int) $game_info['game_scoretype'];

	switch ($game_info['game_scoretype'])
	{
		case SCORETYPE_HIGH:
			$game_scoretype = 'SCORETYPE_HIGH';
		break;

		case SCORETYPE_LOW:
			$game_scoretype = 'SCORETYPE_LOW';
		break;

		default:
			$game_scoretype = 'SCORETYPE_HIGH';
		break;
	}

	$game_image = '$game_file.\'.gif\'';
	if (isset($game_info['game_image']) && isset($game_info['game_swf']))
	{
		if ($file_functions->remove_extension($game_info['game_image']) == $file_functions->remove_extension($game_info['game_swf']))
		{
			$game_image = '$game_file . \'' . ((strrchr($game_info['game_image'], '.')) ? strrchr($game_info['game_image'], '.') : '.gif') . '\'';
		}
		else
		{
			$game_image = '\'' . addslashes($game_info['game_image']) . '\'';
		}
	}

	$game_scorevar = '$game_file';
	if (isset($game_info['game_swf']) && isset($game_info['game_scorevar']))
	{
		if ($file_functions->remove_extension($game_info['game_swf']) != $game_info['game_scorevar'])
		{
			$game_scorevar = '\'' . $game_info['game_scorevar'] . '\'';
		}
	}

	$default_install_file = get_file_contents($phpbb_root_path . 'arcade/includes/default_install_file.' . $phpEx);
	$data = array(addslashes($game_info['game_name']), $game_scorevar, addslashes($game_info['game_desc']), $game_image, $game_type, $game_info['game_width'], $game_info['game_height'], $game_scoretype);
	return str_replace(array('{GAME_NAME}', '{GAME_SCOREVAR}', '{GAME_DESC}', '{GAME_IMAGE}', '{GAME_TYPE}', '{GAME_WIDTH}', '{GAME_HEIGHT}', '{GAME_SCORETYPE}'), $data, $default_install_file);
}

/**
* Error and message handler, call with trigger_error if reqd
* Slightly modified phpBB function to include ability to
* display simple header and footers
*/
function arcade_msg_handler($errno, $msg_text, $errfile, $errline)
{
	global $cache, $db, $auth, $template, $config, $user;
	global $phpEx, $phpbb_root_path, $msg_title, $msg_long_text, $use_simple_header;

	// Do not display notices if we suppress them via @
	if (error_reporting() == 0 && $errno != E_USER_ERROR && $errno != E_USER_WARNING && $errno != E_USER_NOTICE)
	{
		return;
	}

	// Message handler is stripping text. In case we need it, we are possible to define long text...
	if (isset($msg_long_text) && $msg_long_text && !$msg_text)
	{
		$msg_text = $msg_long_text;
	}

	if (!defined('E_DEPRECATED'))
	{
		define('E_DEPRECATED', 8192);
	}

	switch ($errno)
	{
		case E_NOTICE:
		case E_WARNING:

			// Check the error reporting level and return if the error level does not match
			// If DEBUG is defined the default level is E_ALL
			if (($errno & ((defined('DEBUG')) ? E_ALL : error_reporting())) == 0)
			{
				return;
			}

			if (strpos($errfile, 'cache') === false && strpos($errfile, 'template.') === false)
			{
				$errfile = phpbb_filter_root_path($errfile);
				$msg_text = phpbb_filter_root_path($msg_text);
				$error_name = ($errno === E_WARNING) ? 'PHP Warning' : 'PHP Notice';
				echo '<b>[phpBB Debug] ' . $error_name . '</b>: in file <b>' . $errfile . '</b> on line <b>' . $errline . '</b>: <b>' . $msg_text . '</b><br />' . "\n";

				// we are writing an image - the user won't see the debug, so let's place it in the log
				if (defined('IMAGE_OUTPUT') || defined('IN_CRON'))
				{
					add_log('critical', 'LOG_IMAGE_GENERATION_ERROR', $errfile, $errline, $msg_text);
				}
				// echo '<br /><br />BACKTRACE<br />' . get_backtrace() . '<br />' . "\n";
			}

			return;

		break;

		case E_USER_ERROR:

			if (!empty($user) && !empty($user->lang))
			{
				$msg_text = (!empty($user->lang[$msg_text])) ? $user->lang[$msg_text] : $msg_text;
				$msg_title = (!isset($msg_title)) ? $user->lang['GENERAL_ERROR'] : ((!empty($user->lang[$msg_title])) ? $user->lang[$msg_title] : $msg_title);

				$l_return_index = sprintf($user->lang['RETURN_INDEX'], '<a href="' . $phpbb_root_path . '">', '</a>');
				$l_notify = '';

				if (!empty($config['board_contact']))
				{
					$l_notify = '<p>' . sprintf($user->lang['NOTIFY_ADMIN_EMAIL'], $config['board_contact']) . '</p>';
				}
			}
			else
			{
				$msg_title = 'General Error';
				$l_return_index = '<a href="' . $phpbb_root_path . '">Return to index page</a>';
				$l_notify = '';

				if (!empty($config['board_contact']))
				{
					$l_notify = '<p>Please notify the board administrator or webmaster: <a href="mailto:' . $config['board_contact'] . '">' . $config['board_contact'] . '</a></p>';
				}
			}

			if ((defined('DEBUG') || defined('IN_CRON') || defined('IMAGE_OUTPUT')) && isset($db))
			{
				// let's avoid loops
				$db->sql_return_on_error(true);
				add_log('critical', 'LOG_GENERAL_ERROR', $msg_title, $msg_text);
				$db->sql_return_on_error(false);
			}

			// Do not send 200 OK, but service unavailable on errors
			send_status_line(503, 'Service Unavailable');

			garbage_collection();

			// Try to not call the adm page data...

			echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
			echo '<html xmlns="http://www.w3.org/1999/xhtml" dir="ltr">';
			echo '<head>';
			echo '<meta http-equiv="content-type" content="text/html; charset=utf-8" />';
			echo '<title>' . $msg_title . '</title>';
			echo '<style type="text/css">' . "\n" . '/* <![CDATA[ */' . "\n";
			echo '* { margin: 0; padding: 0; } html { font-size: 100%; height: 100%; margin-bottom: 1px; background-color: #E4EDF0; } body { font-family: "Lucida Grande", Verdana, Helvetica, Arial, sans-serif; color: #536482; background: #E4EDF0; font-size: 62.5%; margin: 0; } ';
			echo 'a:link, a:active, a:visited { color: #006699; text-decoration: none; } a:hover { color: #DD6900; text-decoration: underline; } ';
			echo '#wrap { padding: 0 20px 15px 20px; min-width: 615px; } #page-header { text-align: right; height: 40px; } #page-footer { clear: both; font-size: 1em; text-align: center; } ';
			echo '.panel { margin: 4px 0; background-color: #FFFFFF; border: solid 1px  #A9B8C2; } ';
			echo '#errorpage #page-header a { font-weight: bold; line-height: 6em; } #errorpage #content { padding: 10px; } #errorpage #content h1 { line-height: 1.2em; margin-bottom: 0; color: #DF075C; } ';
			echo '#errorpage #content div { margin-top: 20px; margin-bottom: 5px; border-bottom: 1px solid #CCCCCC; padding-bottom: 5px; color: #333333; font: bold 1.2em "Lucida Grande", Arial, Helvetica, sans-serif; text-decoration: none; line-height: 120%; text-align: left; } ';
			echo "\n" . '/* ]]> */' . "\n";
			echo '</style>';
			echo '</head>';
			echo '<body id="errorpage">';
			echo '<div id="wrap">';
			echo '	<div id="page-header">';
			echo '		' . $l_return_index;
			echo '	</div>';
			echo '	<div id="acp">';
			echo '	<div class="panel">';
			echo '		<div id="content">';
			echo '			<h1>' . $msg_title . '</h1>';

			echo '			<div>' . $msg_text . '</div>';

			echo $l_notify;

			echo '		</div>';
			echo '	</div>';
			echo '	</div>';
			echo '	<div id="page-footer">';
			echo '		Powered by <a href="http://www.phpbb.com/">phpBB</a>&reg; Forum Software &copy; phpBB Group';
			echo '	</div>';
			echo '</div>';
			echo '</body>';
			echo '</html>';

			exit_handler();

			// On a fatal error (and E_USER_ERROR *is* fatal) we never want other scripts to continue and force an exit here.
			exit;
		break;

		case E_USER_WARNING:
		case E_USER_NOTICE:

			define('IN_ERROR_HANDLER', true);

			if (empty($user->data))
			{
				$user->session_begin();
			}

			// We re-init the auth array to get correct results on login/logout
			$auth->acl($user->data);

			if (empty($user->lang))
			{
				$user->setup();
			}

			if ($msg_text == 'ERROR_NO_ATTACHMENT' || $msg_text == 'NO_FORUM' || $msg_text == 'NO_TOPIC' || $msg_text == 'NO_USER')
			{
				send_status_line(404, 'Not Found');
			}

			$msg_text = (!empty($user->lang[$msg_text])) ? $user->lang[$msg_text] : $msg_text;
			$msg_title = (!isset($msg_title)) ? $user->lang['INFORMATION'] : ((!empty($user->lang[$msg_title])) ? $user->lang[$msg_title] : $msg_title);

			if (!defined('HEADER_INC'))
			{
				if (defined('IN_ADMIN') && isset($user->data['session_admin']) && $user->data['session_admin'])
				{
					adm_page_header($msg_title);
				}
				else
				{
					page_header($msg_title, false);
				}
			}

			$template->set_filenames(array(
				'body' => 'arcade/message_body.html')
			);

			$template->assign_vars(array(
				'MESSAGE_TITLE'			=> $msg_title,
				'MESSAGE_TEXT'			=> $msg_text,
				'S_USE_SIMPLE_HEADER'	=> (!empty($use_simple_header)) ? true : false,
				'S_USER_WARNING'		=> ($errno == E_USER_WARNING) ? true : false,
				'S_USER_NOTICE'			=> ($errno == E_USER_NOTICE) ? true : false)
			);

			// We do not want the cron script to be called on error messages
			define('IN_CRON', true);

			if (defined('IN_ADMIN') && isset($user->data['session_admin']) && $user->data['session_admin'])
			{
				adm_page_footer();
			}
			else
			{
				page_footer();
			}

			exit_handler();
		break;

		// PHP4 compatibility
		case E_DEPRECATED:
			return true;
		break;
	}

	// If we notice an error not handled here we pass this back to PHP by returning false
	// This may not work for all php versions
	return false;
}

function arcade_fwrite($file_path, $data, $empty_return = true)
{
	if (!$data && $empty_return)
	{
		return false;
	}

	if ($fp = @fopen($file_path, 'wb'))
	{
		fwrite($fp, $data);
		fclose($fp);

		return true;
	}
	else
	{
		return false;
	}
}

// These are needed to ensure compatability with php4
if (!function_exists('scandir'))
{
	function scandir($directory, $sorting_order = 0)
	{
		$list = array();

		$dh = opendir($directory);
		while (($file = readdir($dh)) !== false)
		{
			$list[] = $file;
		}
		closedir($dh);

		(!$sorting_order) ? sort($list) : rsort($list);

		return $list;
	}
}

if (!function_exists('get_file_contents'))
{
	function get_file_contents($filename)
	{
		if (function_exists('file_get_contents'))
		{
			return file_get_contents($filename);
		}
		else
		{
			$f = @fopen($filename, 'r');
			if (!$f)
			{
				return false;
			}
			else
			{
				$fcontents = fread($f, filesize($filename));
				fclose($f);
				return $fcontents;
			}
		}
	}
}

/**
* Function used to quickly show the contents of variables
*/
function arcade_debug($var = false)
{
	$return = '';
	if ($var !== false)
	{
		$return .= '<pre>';
		$return .= var_export($var, true);
		$return .= '</pre>';
	}
	else
	{
		$debug = array(
			'POST'		=> $_POST,
			'GET'		=> $_GET,
			'REQUEST'	=> $_REQUEST,
			'COOKIE'	=> $_COOKIE,
			'SERVER'	=> $_SERVER,
		);

		foreach ($debug as $key => $value)
		{
			$return .= '<p><b>' . $key . '</b>';
			$return .= '<pre>';
			$return .= var_export($value, true);
			$return .= '</pre>';
			$return .= '</p>';
		}
	}

	return $return;
}

function arcade_games_sort_order_select($value, $key = '')
{
	global $user;

	return '<option value="' . ARCADE_ORDER_ACP .'"'. (($value == ARCADE_ORDER_ACP) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_DEFAULT'] . '</option><option value="' . ARCADE_ORDER_FIXED .'"'. (($value == ARCADE_ORDER_FIXED) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_FIXED'] . '</option><option value="' . ARCADE_ORDER_INSTALLDATE .'"'. (($value == ARCADE_ORDER_INSTALLDATE) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_INSTALLDATE'] . '</option><option value="' . ARCADE_ORDER_NAME . '"' . (($value == ARCADE_ORDER_NAME) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_NAME'] . '</option><option value="' . ARCADE_ORDER_PLAYS . '"' . (($value == ARCADE_ORDER_PLAYS) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_PLAYS'] . '</option><option value="' . ARCADE_ORDER_RATING . '"' . (($value == ARCADE_ORDER_RATING) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_RATING'] . '</option>';
}

function arcade_games_sort_dir_select($value, $key = '')
{
	global $user;

	return '<option value="' . ARCADE_ORDER_ACP .'"'. (($value == ARCADE_ORDER_ACP) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_DEFAULT'] . '</option><option value="' . ARCADE_ORDER_ASC . '"' . (($value == ARCADE_ORDER_ASC) ? ' selected="selected"' : '') . '>' . $user->lang['ASCENDING'] . '</option><option value="' . ARCADE_ORDER_DESC . '"' . (($value == ARCADE_ORDER_DESC) ? ' selected="selected"' : '') . '>' . $user->lang['DESCENDING'] . '</option>';
}

?>