<?php
/**
*
* @package arcade
* @version $Id: class.php 2006 2012-03-17 00:35:04Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Class for handling arcade
* @package arcade
*/
class arcade
{
	var $gametop = '#gametop';
	var $data = array();
	var $cats = array();
	var $games = array();
	var $users = array();
	var $totals = array();
	var $points = array();
	var $hidden_users = array();
	var $keyoptions = array('arcade_pm' => 0, 'challenge' => 1, 'view_avatars' => 2, 'view_game_image' => 3, 'view_popup_icon' => 4, 'notifypm' => 5, 'bbcode' => 6, 'smilies' => 7);

	/**
	* Constructor used to setup arcade
	* $in_arcade can be set to false
	* to not alter board navigation links
	*/
	function arcade($in_arcade = true, $set_data = true, $chall = false)
	{
		global $user, $auth, $template, $phpbb_root_path, $phpEx;
		global $arcade_config, $arcade_cache;

		$template->assign_vars(array(
			'S_IN_ARCADE'				=> $in_arcade,
			'S_ARCADE_CLASS'			=> true,
			'S_ARCADE_DISABLED'			=> ($auth->acl_get('a_') && $arcade_config['arcade_disable']) ? true : false,
			'S_CHALLENGE_DISABLED'		=> ($auth->acl_get('a_') && $arcade_config['challenge_disable']) ? true : false,

			'GAMETOP'					=> str_replace('#', '', $this->gametop),
			'PHPEX'						=> $phpEx,
			'LOADING_IMG1'				=> $this->get_image('src', 'img', 'loading1.gif'),
			'LOADING_IMG2'				=> $this->get_image('src', 'img', 'loading2.gif'),
			'STAR_IMG'					=> $this->get_image('src', 'img', 'star.png'),
			'ARCADE_VERSION_INFO'		=> $arcade_config['copyright']
		));

		if ($in_arcade)
		{
			$template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $user->lang['ARCADE_INDEX'],
				'U_VIEW_FORUM'	=> $this->url()
			));
		}

		if ($arcade_config['display_game_popup_icon'] && $popup_icon = $this->get_image('src', 'img', 'popup.png'))
		{
			$this->data['popup_icon'] = $popup_icon;
		}

		if ($this->optionget('view_avatars') && $no_avatar = $this->get_image('src', 'img', 'noavatar.gif'))
		{
			$this->data['no_avatar'] = $no_avatar;
		}

		if ($set_data)
		{
			$this->init_points();
			$this->set_disabled();

			if (!$chall)
			{
				$this->obtain_hidden_scores();

				if (!defined('IN_ADMIN') && !defined('USE_ARCADE_ADMIN'))
				{
					$this->games = $this->obtain_games();
				}
			}

			$this->cats = $arcade_cache->obtain_arcade_cats();
		}
	}

	/**
	* Generate the correct arcade url
	*
	* @param mixed $params String or array of additional url parameters
	* @param string $page The name of page
	* @param string $session_id Possibility to use a custom session id instead of the global one
	*/
	function url($params = false, $page = 'arcade', $sid = false)
	{
		global $phpbb_root_path, $phpEx;

		if ($params)
		{
			if ((strpos($params, 'img=') !== false) || (strpos($params, 'swf=') !== false))
			{
				return "{$phpbb_root_path}{$page}.{$phpEx}?{$params}";
			}
		}
		return append_sid($phpbb_root_path . $page . '.' . $phpEx, $params, true, $sid) . (($params && strpos($params, 'mode=play') !== false) ? $this->gametop : '');
	}

	/**
	* Generate return links for a game, category and the arcade
	*/
	function return_links($game_data, $br = true, $popup = false)
	{
		global $user;

		$s_content_flow_begin = ($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right';

		return ($br ? '<br /><br />' : '') . (($popup) ? sprintf($user->lang['ARCADE_RETURN_LINKS_POPUP'], '<a class="'. $s_content_flow_begin .'" href="'. $this->url("mode=popup&amp;g={$game_data['game_id']}") .'">', $game_data['game_name'], '</a>') . $this->close_win() : sprintf($user->lang['ARCADE_RETURN_LINKS'], '<a class="'. $s_content_flow_begin .'" href="'. $this->url("mode=play&amp;g={$game_data['game_id']}") .'">', $game_data['game_name'], '</a>', '<a class="'. $s_content_flow_begin .'" href="'. $this->url("mode=cat&amp;c={$game_data['cat_id']}&amp;g={$game_data['game_id']}#g{$game_data['game_id']}") .'">', $game_data['cat_name'], '</a>', '<a class="'. $s_content_flow_begin .'" href="'. $this->url() .'">', '</a>'));
	}

	/**
	* Check if the arcade should be closed down
	*/
	function set_disabled()
	{
		global $arcade_config;

		// Return if auto disable is not turned on
		if (!$arcade_config['auto_disable'])
		{
			return;
		}

		$current_date = getdate();
		$current_time = time();

		// Return if auto disable start time is not valid
		$time_array = $this->validate_time($arcade_config['auto_disable_start']);
		if (!sizeof($time_array))
		{
			return;
		}
		$auto_disable_start = mktime($time_array['hour'], $time_array['min'], 0, $current_date['mon'], $current_date['mday'], $current_date['year']);

		// Return if auto disable end time is not valid
		$time_array = $this->validate_time($arcade_config['auto_disable_end']);
		if (!sizeof($time_array))
		{
			return;
		}
		$auto_disable_end = mktime($time_array['hour'], $time_array['min'], 0, $current_date['mon'], $current_date['mday'], $current_date['year']);

		// Return if auto disable start time is not set eariler than end time
		if ($auto_disable_start >= $auto_disable_end)
		{
			return;
		}

		// If everything checks out let set the correct value for arcade_disable
		if ($current_time >= $auto_disable_start && $current_time < $auto_disable_end)
		{
			if ($arcade_config['arcade_disable'] == false)
			{
				$this->set_config('arcade_disable', true);
			}
		}
		else
		{
			if ($arcade_config['arcade_disable'] == true)
			{
				$this->set_config('arcade_disable', false);
			}
		}
	}

	/**
	* Returns an array of ids for the specified type
	* This is for the local arcade permissions
	*/
	function get_permissions($type)
	{
		global $auth_arcade;

		return array_unique(array_keys($auth_arcade->acl_getc($type, true)));
	}

	/**
	* Set config value. Creates missing config entry.
	*/
	function set_config($config_name, $config_value, $is_dynamic = false)
	{
		global $db, $arcade_cache, $arcade_config;

		$sql = 'UPDATE ' . ARCADE_CONFIG_TABLE . "
				SET config_value = '" . $db->sql_escape($config_value) . "'
				WHERE config_name = '" . $db->sql_escape($config_name) . "'";
		$db->sql_query($sql);

		if (!$db->sql_affectedrows() && !isset($arcade_config[$config_name]))
		{
			$sql = 'INSERT INTO ' . ARCADE_CONFIG_TABLE . ' ' . $db->sql_build_array('INSERT', array(
					'config_name'	=> $config_name,
					'config_value'	=> $config_value,
					'is_dynamic'	=> ($is_dynamic) ? 1 : 0));
			$db->sql_query($sql);
		}

		$arcade_config[$config_name] = $config_value;

		if (!$is_dynamic)
		{
			$arcade_cache->destroy('_arcade');
		}
	}

	/**
	* Set various file/path name based off of passed file name
	*
	* Types
	* - install				Returns filename with path of install file
	* - path				Returns game path
	* - gamedata 			Returns gamedata path
	* - v3arcade_gamedata	Returns v3arcade games gamedata path
	* - 					Returns filename with path
	*/
	function set_path($file, $type = '', $use_phpbb_root = true)
	{
		global $phpbb_root_path, $phpEx, $file_functions, $arcade_config;
		$path = $file_functions->remove_extension($file);

		$return = '';
		switch ($type)
		{
			case 'install':
				$return = $arcade_config['game_path'] . $path . '/' . $path . '.' . $phpEx;
			break;

			case 'path':
				$return = $arcade_config['game_path'] . $path . '/';
			break;

			case 'gamedata':
				$return =  'arcade/gamedata/' . $path . '/';
			break;

			case 'v3arcade_gamedata':
				$return = 'games/' . $path . '/';
			break;

			default:
				$return = $arcade_config['game_path'] . $path . '/' . $file;
			break;
		}

		return ($use_phpbb_root) ? $phpbb_root_path . $return : $return;
	}

	function get_gamedata($file, $use_phpbb_root = true)
	{
		if (file_exists($this->set_path($file, 'gamedata')))
		{
			return $this->set_path($file, 'gamedata', $use_phpbb_root);
		}
		else if (file_exists($this->set_path($file, 'v3arcade_gamedata')))
		{
			return $this->set_path($file, 'v3arcade_gamedata', $use_phpbb_root);
		}
		else
		{
			return false;
		}
	}

	function download_game($game, $use_method, $methods, $update = true, $download = true)
	{
		global $user, $arcade_cache, $arcade_config, $auth_arcade, $file_functions, $phpbb_root_path, $phpEx;

		if (!class_exists('compress'))
		{
			include($phpbb_root_path . 'includes/functions_compress.' . $phpEx);
		}

		$backup_path = $phpbb_root_path . $arcade_config['cat_backup_path'] . $this->characters_encoding($this->get_cat_field($game['cat_id'], 'cat_name')) . '/';
		$path = ($download && $arcade_config['download_on_demand']) ? $phpbb_root_path . 'store/' : $backup_path;

		// If we are not in download mode or we are but we are not serving games on demand
		// we check for and setup the backup directory category folder
		if (!$download || ($download && !$arcade_config['download_on_demand']))
		{
			if (!phpbb_is_writable($phpbb_root_path . $arcade_config['cat_backup_path']))
			{
				trigger_error(sprintf($user->lang['ARCADE_ERROR_DIR_WRITABLE'], $arcade_config['cat_backup_path']), E_USER_WARNING);
			}

			if (!file_exists($path))
			{
				@mkdir($path, 0777);
				@chmod($path, 0777);
			}

			if (!file_exists($path . 'index.htm'))
			{
				arcade_fwrite($path . 'index.htm', '', false);
			}

			if (!phpbb_is_writable($path))
			{
				trigger_error(sprintf($user->lang['ARCADE_ERROR_DIR_WRITABLE'], str_replace($phpbb_root_path, '', $path)), E_USER_WARNING);
			}
		}

		if ($update)
		{
			$this->update_download_total($game['game_id']);
			$this->set_config('total_downloads', intval($arcade_config['total_downloads'] + 1));
			$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
		}

		$filename = $file_functions->remove_extension($game['game_swf']);

		if (!in_array($use_method, $methods))
		{
			$use_method = '.tar';
		}

		// If the file exists in the backup directory and if the file modification time of the game install file is less than or equal to
		// the file modification time of the compressed folder in the backup directory send the file in the backup directory instead
		// of creating the compressed file again.
		if ($download && file_exists($backup_path . $filename . $use_method) && (@filemtime($this->set_path($game['game_swf'], 'install')) <= @filemtime($backup_path . $filename . $use_method)))
		{
			$this->send_download_to_browser($backup_path, $filename, $use_method);
			garbage_collection();
			exit_handler();
		}

		$game_file = $this->set_path($game['game_swf']);
		$game_path = $this->set_path($game['game_swf'], 'path');
		$gamedata = $this->get_gamedata($game['game_swf']);

		$error = array();
		if (!file_exists($game_file))
		{
			$error[] = $game_file;
		}

		if (sizeof($error))
		{
			$this->add_log(array(
				'user_id'		=> $user->data['user_id'],
				'game_id'		=> $game['game_id'],
				'error_date'	=> time(),
				'error_type'	=> ARCADE_ERROR_FILEMISSING,
				'error_ip'		=> $user->ip,
			));

			if ($auth->acl_get('a_') && defined('DEBUG_EXTRA'))
			{
				trigger_error(sprintf($user->lang['ARCADE_DOWNLOAD_MISSING_FILES_DEBUG'], implode('<br />', $error)) . $this->back_link());
			}
			else
			{
				trigger_error(sprintf($user->lang['ARCADE_DOWNLOAD_MISSING_FILES']) . $this->back_link());
			}
		}

		if ($use_method == '.zip')
		{
			$compress = new compress_zip('w', $path . $filename . $use_method);
		}
		else
		{
			$compress = new compress_tar('w', $path . $filename . $use_method, $use_method);
		}

		$file_list = $file_functions->filelist('', $game_path);
		if ($gamedata)
		{
			$file_list = array_merge($file_list, $file_functions->filelist('', $gamedata));
		}

		// Correct path locations before adding files
		$dir_list = array();
		$search = array($game_path, $phpbb_root_path . 'arcade/gamedata', $phpbb_root_path . 'games');
		$replace = array('', 'gamedata', 'games');

		// Add all the game files
		$file_list = array_unique($file_list);
		foreach ($file_list as $file)
		{
			$dir_list[] = dirname($file);
			$compress->add_custom_file($file, str_replace($search, $replace, $file));
		}
		unset($file_list);

		// Add blank index.htm files
		$dir_list = array_unique($dir_list);
		foreach ($dir_list as $dir)
		{
			$file_functions->append_slash($dir);
			$compress->add_data('', str_replace($search, $replace, $dir) . 'index.htm');
		}
		unset($dir_list);

		// Close file
		$compress->close();

		if ($download)
		{
			$this->send_download_to_browser($path, $filename, $use_method);
			// Delete file from the store once user downloads it if serving download on demand
			if ($arcade_config['download_on_demand'])
			{
				@unlink($path . $filename . $use_method);
			}

			if ($this->points['show'])
			{
				$game_download_cost = $this->get_download_cost($game);

				if ($game_download_cost != ARCADE_FREE && $game_download_cost > 0 && !$auth_arcade->acl_get('c_downloadfree', $game['cat_id']))
				{
					$this->set_points('subtract', $user->data['user_id'], $game_download_cost);
				}
			}

			garbage_collection();
			exit_handler();
		}
	}

	function send_download_to_browser($path, $filename, $use_method, $download_name = false)
	{

		if ($download_name === false)
		{
			$download_name = $filename;
		}

		switch ($use_method)
		{
			case '.tar':
				$mimetype = 'application/x-tar';
			break;

			case '.tar.gz':
				$mimetype = 'application/x-gzip';
			break;

			case '.tar.bz2':
				$mimetype = 'application/x-bzip2';
			break;

			case '.zip':
				$mimetype = 'application/zip';
			break;

			default:
				$mimetype = 'application/octet-stream';
			break;
		}

		header('Pragma: no-cache');
		header("Content-Type: $mimetype; name=\"$download_name$use_method\"");
		header("Content-disposition: attachment; filename=$download_name$use_method");

		$fp = @fopen("$path$filename$use_method", 'rb');
		if ($fp)
		{
			while ($buffer = fread($fp, 1024))
			{
				echo $buffer;
			}
			fclose($fp);
		}
	}

	function characters_encoding($string)
	{
		$string = str_replace(array("\xC3\x80", "\xC3\x81", "\xC3\x82", "\xC3\x83", "\xC3\x84", "\xC3\x85")				, "\x41", $string); // A
		$string = str_replace(array("\xC3\x88", "\xC3\x89", "\xC3\x8a", "\xC3\x8b")										, "\x45", $string); // E
		$string = str_replace(array("\xC3\x8c", "\xC3\x8d", "\xC3\x8e", "\xC3\x8f")										, "\x49", $string); // I
		$string = str_replace(array("\xC3\x92", "\xC3\x93", "\xC3\x94", "\xC3\x95", "\xC3\x96", "\xC5\x90")				, "\x4F", $string); // O
		$string = str_replace(array("\xC3\x99", "\xC3\x9a", "\xC3\x9b", "\xC3\x9c", "\xC5\xB0")							, "\x55", $string); // U
		$string = str_replace(array("\xC3\x9d")																			, "\x59", $string); // Y
		$string = str_replace(array("\xC3\xa0", "\xC3\xa1", "\xC3\xa2", "\xC3\xa3", "\xC3\xa4", "\xC3\xa5")				, "\x61", $string); // a
		$string = str_replace(array("\xC3\xa8", "\xC3\xa9", "\xC3\xaa", "\xC3\xab")										, "\x65", $string); // e
		$string = str_replace(array("\xC3\xac", "\xC3\xad", "\xC3\xae", "\xC3\xaf")										, "\x69", $string); // i
		$string = str_replace(array("\xC3\xb1")																			, "\x6e", $string); // n
		$string = str_replace(array("\xC3\xb2", "\xC3\xb3", "\xC3\xb4", "\xC3\xb5", "\xC3\xb6", "\xC3\xb0", "\xC5\x91")	, "\x6F", $string); // o
		$string = str_replace(array("\xC3\xb9", "\xC3\xba", "\xC3\xbb", "\xC3\xbc", "\xC5\xB1")							, "\x75", $string); // u
		$string = str_replace(array("\xC3\xbd", "\xC3\xbf")																, "\x79", $string); // y
		$string = str_replace(" "																						, "_"	, $string); // space

		return trim(preg_replace('/[^\w -]+/', '', htmlspecialchars_decode($string)));
	}

	/*
	* Set filesize for game this
	* includes all extra files
	*/
	function set_filesize($game_id)
	{
		global $arcade_cache, $db, $file_functions;

		$sql = 'SELECT game_id, game_swf
				FROM ' . ARCADE_GAMES_TABLE . '
				WHERE game_id = ' . (int) $game_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$filesize = $file_functions->filesize(array_filter(array($this->set_path($row['game_swf']), $this->get_gamedata($row['game_swf']))));

		if ($filesize)
		{
			$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
					SET game_filesize = '. (int) $filesize . '
					WHERE game_id = ' . (int) $game_id;
			$db->sql_query($sql);
		}

		$arcade_cache->destroy('_arcade_games_filesize');

		return $filesize;
	}

	/*
	* Set game display size
	*/
	function set_game_size(&$new_game_width, &$new_game_height, $game_width, $game_height, $game_swf)
	{
		global $arcade_config;

		if ($arcade_config['game_autosize'])
		{
			if (list($width, $height) = getimagesize($this->set_path($game_swf)))
			{
				$new_game_width = $width;
				$new_game_height = $height;
			}
		}

		if (empty($new_game_width) || empty($new_game_height))
		{
			if (!empty($arcade_config['game_width']))
			{
				$new_game_width = $arcade_config['game_width'];
			}
			else
			{
				$new_game_width = $game_width;
			}


			if (!empty($arcade_config['game_height']))
			{
				$new_game_height = $arcade_config['game_height'];
			}
			else
			{
				$new_game_height = $game_height;
			}
		}
	}

	function back_link($params = false, $lang = 'arcade', $br2 = true)
	{
		global $user;

		$s_content_flow_begin = ($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right';
		return (($br2) ? '<br /><br />' : '<br />') . '<a class="'. $s_content_flow_begin .'" href="' . $this->url($params) . '">' . $user->lang['ARCADE_BACK_TO_' . strtoupper($lang)] . '</a>';
	}

	/**
	* Displays html code for images including
	* dimensions if possible
	*
	* Thanks easygo
	*/
	function set_image($path, $alt = '', $style = '')
	{
		$alt	= (!empty($alt)) ? 'alt="' . $alt . '" title="' . $alt . '"' : 'alt=""';
		$style	= (!empty($style)) ? 'style="' . $style . '"' : '';

		if (list($width, $height) = getimagesize($path))
		{
			$img = '<img src="' . $path . '" width="' . $width . '" height="' . $height . '" ' . $alt . $style . ' />';
		}
		else
		{
			$img = '<img src="' . $path . '" ' . $alt . $style . ' />';
		}

		return $img;
	}

	/**
	* This function checks to see if there are style specific
	* category images first
	*/
	function get_image($mode, $type, $image, $alt = '', $default = false, $style = '')
	{
		if (!$image)
		{
			return;
		}

		global $user, $phpbb_root_path, $arcade_config;

		$theme_path = $image_path = '';
		switch (strtolower($type))
		{
			case 'cat':
				$theme_path = "{$phpbb_root_path}styles/" . $user->theme['theme_path'] . '/theme/images/arcade/cats/';
				$image_path = $phpbb_root_path . $arcade_config['cat_image_path'];
			break;

			case 'img':
				$theme_path = "{$phpbb_root_path}styles/" . $user->theme['theme_path'] . '/theme/images/arcade/';
				$image_path = $phpbb_root_path . $arcade_config['image_path'];
			break;

			default:
				return;
			break;
		}

		if (file_exists($theme_path . $image) && $default === false)
		{
			$path = $theme_path . $image;
		}
		else if (file_exists($image_path . $image))
		{
			$path = $image_path . $image;
		}
		else
		{
			return;
		}

		switch (strtolower($mode))
		{
			case 'full':
				return $this->set_image($path, $alt, $style);
			break;

			case 'src':
				return $path;
			break;

			default:
				return;
			break;
		}
	}

	/**
	* Get all the categories the user has played games in
	* Follows the permissions systems so only categories
	* the viewing user can see are returned
	*/
	function get_user_categories($user_id = false)
	{
		global $user, $db;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'		=> 'c.cat_id, c.cat_name',
			'FROM'			=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
					'ON'	=> 'g.game_id = s.game_id'
				),
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),
			'WHERE'			=> 's.user_id = ' . (int) $user_id . ' AND ' . $db->sql_in_set('c.cat_id', $this->get_permissions('c_view'), false, true),
			'ORDER_BY'		=> 'c.cat_name ASC',
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);

		$cats = array();
		$cats[] = $user->lang['ARCADE_ALL_CATEGORIES'];
		while ($row = $db->sql_fetchrow($result))
		{
			$cats[$row['cat_id']] = $row['cat_name'];
		}
		$db->sql_freeresult($result);

		return $cats;
	}

	/**
	* Get one of the following fields from the games array (games user has permission for):
	* game_id, game_name, game_swf, game_scorevar, game_highuser, game_highdate, cat_id
	*
	* To add more field modify the obtain_games() method of this class.
	*
	* This can be used to check if a user has permission to view a game by just passing
	* the game id
	* @returns requested field or boolean
	*/
	function get_game_field($game_id, $field = false, $new_load = false)
	{
		if (!empty($this->games[$game_id]) || ($field && $new_load))
		{
			if (empty($field))
			{
				return true;
			}

			if (!isset($this->games[$game_id][$field]))
			{
				if (!$new_load)
				{
					trigger_error('ARCADE_INVALID_FIELD', E_USER_ERROR);
				}
				else
				{
					$game = $this->get_game_data($game_id);
					return (isset($game[$field])) ? $game[$field] : false;
				}
			}

			return $this->games[$game_id][$field];
		}

		return false;
	}

	/**
	* Get game data of passed game id or ids
	*/
	function get_game_data($game_id, $order = false, $limit = false, $start = 0)
	{
		global $db, $user;

		$sql_array = array(
			'SELECT'		=> 'g.*',
			'FROM'			=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'WHERE'			=> $db->sql_in_set('g.game_id', $game_id),
		);

		if ($order)
		{
			$sql_array['ORDER_BY'] = $order;
		}

		$sql = $db->sql_build_query('SELECT', $sql_array);

		if ($limit)
		{
			$result = $db->sql_query_limit($sql, $limit, $start);
		}
		else
		{
			$result = $db->sql_query($sql);
		}

		if (is_array($game_id))
		{
			$row = $db->sql_fetchrowset($result);
		}
		else
		{
			$row = $db->sql_fetchrow($result);
		}

		$db->sql_freeresult($result);

		return ($row) ? $row : false;
	}

	/**
	* Get game and category data of passed game id
	*/
	function get_game_cat_data($game_id)
	{
		global $db, $user;

		$sql_array = array(
			'SELECT'		=> 'g.*,
								c.cat_name, c.cat_desc, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.cat_parents, c.left_id, c.right_id, c.cat_download, c.cat_download_cost, c.cat_cost, c.cat_reward, c.cat_use_jackpot, c.cat_age,
								r.game_rating',
			'FROM'			=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_RATING_TABLE => 'r'),
					'ON'	=> 'g.game_id = r.game_id AND r.user_id = ' . (int) $user->data['user_id']
				),
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),
			'WHERE'			=> 'g.game_id = ' . (int) $game_id,
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		return ($row) ? $row : false;
	}

	function add_log($sql_ary)
	{
		global $db;

		$db->sql_query('INSERT INTO ' . ARCADE_ERRORS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
	}

	function update_game_type($game_id, $scoretype)
	{
		global $db, $user;

		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET game_type = ' . (int) $scoretype . '
				WHERE game_id = ' . (int) $game_id;
		$db->sql_query($sql);

		if (!arcade_game_update_install_file($game_id))
		{
			$this->add_log(array(
				'user_id'				=> (int) $user->data['user_id'],
				'game_id'				=> (int) $game_id,
				'error_date'			=> time(),
				'error_type'			=> ARCADE_ERROR_UPDATE_FILE,
				'error_ip'				=> $user->ip,
			));
		}
	}

	/**
	* Gets any category field
	*/
	function get_cat_field($cat_id, $field)
	{
		if (!$cat_id || $field == '')
		{
			return false;
		}

		global $user;

		if (isset($this->cats[$cat_id]))
		{
			if (!isset($this->cats[$cat_id][$field]))
			{
				trigger_error(sprintf($user->lang['ARCADE_INVALID_FIELD'], $field), E_USER_ERROR);
			}

			return $this->cats[$cat_id][$field];
		}

		trigger_error(sprintf($user->lang['ARCADE_INVALID_CAT_ID'], $cat_id), E_USER_ERROR);
	}

	/**
	* Get category data for passed
	* cat id or ids
	*/
	function get_cat_data($cat_id)
	{
		global $db;

		$sql_array = array(
			'SELECT'	=> 'c.cat_id, c.cat_name, c.cat_desc, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.cat_parents, c.left_id, c.right_id',
			'FROM'		=> array(
				ARCADE_CATS_TABLE	=> 'c',
			),
			'WHERE'		=> $db->sql_in_set('cat_id', $cat_id),
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		if (is_array($cat_id))
		{
			$row = $db->sql_fetchrowset($result);
		}
		else
		{
			$row = $db->sql_fetchrow($result);
		}
		$db->sql_freeresult($result);

		return ($row) ? $row : false;
	}

	function get_game_cost_notplay($cost = 0)
	{
		$game_ids = array();

		if ($this->points['show'])
		{
			global $db, $arcade_config;

			$sql = 'SELECT g.game_id, g.game_cost, c.cat_cost, c.cat_id
					FROM ' . ARCADE_GAMES_TABLE . ' g
					LEFT JOIN ' . ARCADE_CATS_TABLE . ' c ON g.cat_id = c.cat_id
					WHERE	((g.game_cost > 0) OR
							 (g.game_cost > ' . ARCADE_FREE . ' AND c.cat_cost > 0) OR
							 (g.game_cost > ' . ARCADE_FREE . ' AND c.cat_cost > ' . ARCADE_FREE . ' AND ' . (float) $arcade_config['game_cost'] . ' > 0)
							)
					AND ' . $db->sql_in_set('c.cat_id', $this->get_permissions('c_playfree'), true, true);
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				if (($this->get_cost($row) + $cost) > $this->points['total'])
				{
					// can not play game ids
					$game_ids[] = $row['game_id'];
				}
			}
			$db->sql_freeresult($result);
		}

		return $game_ids;
	}

	function close_win()
	{
		global $user;
		return '<br /><br /><a href="#" onclick="window.close(); return false;">' . $user->lang['CLOSE_WINDOW'] . '</a>';
	}

	/**
	* Get a random game_id from the arcade
	* This is done like this to support all db types
	* since some do not have a RAND() function
	*/
	function get_random_game()
	{
		global $db;

		$sql_array = array(
			'SELECT'		=> 'g.game_id',
			'FROM'			=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),
			'WHERE'			=> 'c.cat_status <> ' . ITEM_LOCKED . ' AND ' . $db->sql_in_set('c.cat_id', $this->get_permissions('c_play'), false, true) . '
				AND ' . $db->sql_in_set('g.game_id', array_map('intval', $this->get_game_cost_notplay()), true, true),
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		// Return false if no games are found
		$game_id = false;
		if (sizeof($row))
		{
			$game_id = $row[mt_rand(0, sizeof($row) - 1)]['game_id'];
		}

		return $game_id;
	}

	/**
	* Display a users favorite games, if no user id
	* is passed it default to viewing user
	*/
	function get_fav_data($user_id = false)
	{
		global $db, $user, $auth_arcade;

		if (!$auth_arcade->acl_get('u_arcade_favorites'))
		{
			return false;
		}

		$fav = array();

		if (!$user->data['is_registered'])
		{
			return $fav;
		}

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'		=> 'g.game_id, g.game_name, g.game_name_clean',
			'FROM'			=> array(
				ARCADE_FAVS_TABLE => 'f',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
					'ON'	=> 'f.game_id = g.game_id'
				),
			),
			'WHERE'			=> 'f.user_id = ' . (int) $user_id . ' AND ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true),
			'ORDER_BY'		=> 'g.game_name_clean ASC',
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$fav[$row['game_id']] = array(
				'game_name' => $row['game_name'],
			);
		}
		$db->sql_freeresult($result);

		return $fav;
	}

	/**
	* Return an array contain all the game ids and names
	* of the games the user has played, if no user id is
	* passed it defaults to viewing user
	*/
	function get_played_games($user_id = false)
	{
		global $arcade_config;

		if (!$arcade_config['played_colour'])
		{
			return array();
		}

		global $db, $user;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'	=> 'g.game_id',
			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
					'ON'	=> 'g.game_id = s.game_id'
				),
			),
			'WHERE'		=> 's.user_id = ' . (int) $user_id,
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);

		$played = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$played[$row['game_id']] = true;
		}
		$db->sql_freeresult($result);
		return $played;
	}

	/**
	* Display correct game type based on constant passed type
	*/
	function display_game_type($game_type)
	{
		global $user;

		switch ($game_type)
		{
			case AMOD_GAME:
				$type = $user->lang['AMOD_GAME'];
			break;

			case AR_GAME:
				$type = $user->lang['AR_GAME'];
			break;

			case IBPRO_GAME:
				$type = $user->lang['IBPRO_GAME'];
			break;

			case V3ARCADE_GAME:
				$type = $user->lang['V3ARCADE_GAME'];
			break;

			case IBPROV3_GAME:
				$type = $user->lang['IBPROV3_GAME'];
			break;

			case ARCADELIB_GAME:
				$type = $user->lang['ARCADELIB_GAME'];
			break;

			case PHPBBARCADE_GAME:
				$type = $user->lang['PHPBBARCADE_GAME'];
			break;

			case NOSCORE_GAME:
				$type = $user->lang['NOSCORE_GAME'];
			break;

			default:
				$type = $user->lang['ARCADE_UNKNOWN'];
			break;
		}
		return $type;
	}

	/*
	* This function displays the site name, categories and download methods
	* available from the server you are download from
	*/
	function display_download_data()
	{
		global $arcade_config;

		if ($arcade_config['download_list'])
		{
			global $config;

			$categories = array();
			foreach ($this->cats as $cat_id => $cats)
			{
				if ($cats['cat_download'] == true)
				{
					$categories[$cat_id] = array(
						'cat_id'	=> $cat_id,
						'cat_name'	=> $cats['cat_name'],
					);
				}
			}

			$download_data = array(
				'sitename'		=> $config['sitename'],
				'message'		=> $arcade_config['download_list_message'],
				'file_type'		=> ($arcade_config['download_file_type'] !== '') ? true : false,
				'methods'		=> ($arcade_config['download_file_type'] !== '') ? array($arcade_config['download_file_type']) : $this->compress_methods(),
				'categories'	=> $categories,
			);

			$download_data = gzcompress(serialize($download_data), 9);
			echo $download_data;
		}

		garbage_collection();
		exit_handler();
	}

	/**
	* This function displays all the games for download.  One issue occurs though,
	* since the server is connecting to the site to get the list and the server is most likely
	* not ever going to be logged in we must display every game where the category and/or game
	* is able to be downloaded based on the cat_download and game_download setting.  Once the
	* list is displayed in the ACP the permissions system will take over for downloading.
	*/
	function display_download_list($cat_id, $start, $sort_key, $sort_dir, $sort_time, $per_page)
	{
		global $arcade_config;

		if ($arcade_config['download_list'])
		{
			global $db;

			$where = ($cat_id) ? ' AND g.cat_id = ' . (int) $cat_id : '';
			$install_time = ($sort_time) ? time() - ($sort_time * 86400) : false;
			$where = ($install_time) ? $where . ' AND g.game_installdate > ' . $install_time : $where;

			$sort = ($sort_key == 'n') ? 'g.game_name_clean' : 'g.game_installdate';
			$sort_dir = ($sort_dir == 'a') ? 'ASC' : 'DESC';

			// Only allow 50, 100 or 200 or 400 games per page
			// This will make sure that there are not tons of different settings out there
			$per_page = (in_array($per_page, array(50, 100, 200, 400))) ? $per_page : 50;

			$download_list = array();

			$sql_array = array(
				'SELECT'		=> 'COUNT(g.game_id) as total',
				'FROM'			=> array(
					ARCADE_GAMES_TABLE	=> 'g',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
						'ON'	=> 'g.cat_id = c.cat_id'
					),
				),
				'WHERE'			=> 'c.cat_download = 1 AND g.game_download = 1 ' . $where,
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql, $arcade_config['cache_time'] * 3600);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$download_list['total'] = $row['total'];
			unset($row);

			$show_points = ($this->points['show']) ? true : false;

			$sql_array = array(
				'SELECT'		=> 'g.game_id, g.game_swf, g.game_name, g.game_name_clean, g.game_download_cost, g.game_desc, g.game_installdate, g.game_filesize, c.cat_id, c.cat_download_cost',
				'FROM'			=> array(
					ARCADE_GAMES_TABLE	=> 'g',
				),
				'LEFT_JOIN'		=> array(
					array(
						'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
						'ON'	=> 'g.cat_id = c.cat_id'
					),
				),
				'WHERE'			=> 'c.cat_download = 1 AND g.game_download = 1 ' . $where,
				'ORDER_BY'		=> $sort . ' ' . $sort_dir,
			);

			$result = $db->sql_query($sql);
			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query_limit($sql, $per_page, $start, $arcade_config['cache_time'] * 3600);
			$games = array();
			while ($row = $db->sql_fetchrow($result))
			{
				$download_cost = 0;

				if ($show_points)
				{
					$download_cost = $this->get_download_cost($row);
					$download_cost = ($download_cost != ARCADE_FREE && $download_cost > 0) ? $download_cost : 0;
				}

				$games[$row['game_id']] = array(
					'game_id'			=> $row['game_id'],
					'game_name'			=> $row['game_name'],
					'game_name_clean'	=> $row['game_name_clean'],
					'game_swf'			=> $row['game_swf'],
					'game_desc'			=> $row['game_desc'],
					'download_cost'		=> $this->number_format($download_cost),
					'game_installdate'	=> $row['game_installdate'],
					'game_filesize'		=> $row['game_filesize'],
					'cat_id'			=> $row['cat_id'],
				);
			}
			$db->sql_freeresult($result);
			$download_list['games'] = $games;
			unset($games);

			// Try to save some bandwidth and allows us to still have an array on the other side
			$download_list = gzcompress(serialize($download_list), 9);
			echo $download_list;
		}

		garbage_collection();
		exit_handler();
	}

	/**
	* Updates a games download total
	*/
	function update_download_total($game_id)
	{
		global $db, $user;

		// Update download count
		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET game_download_total = game_download_total + 1
				WHERE game_id = ' . (int) $game_id;
		$db->sql_query($sql);

		$sql = 'UPDATE ' . ARCADE_DOWNLOAD_TABLE. '
				SET total = total + 1, download_time = ' . time() . '
				WHERE game_id = ' . (int) $game_id . ' AND user_id = ' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		if (!$db->sql_affectedrows())
		{
			$sql = 'INSERT INTO ' . ARCADE_DOWNLOAD_TABLE . ' ' . $db->sql_build_array('INSERT', array(
					'game_id'		=> (int) $game_id,
					'user_id'		=> (int) $user->data['user_id'],
					'total'			=> 1,
					'download_time'	=> time(),
			));
			$db->sql_query($sql);
		}
	}

	/**
	* Take seconds and convert to readable format
	* Example usage of filter:
	* To return days, hours, minutes and seconds
	* $filter = 'day|hour|minute|second';
	* If $filter is set to false it returns full result
	*/
	function time_format($secs, $full_label = false, $filter = false)
	{
		global $user;

		$output = '';
		$filter = ($filter) ? explode('|', strtolower($filter)) : false;

		$minute = ($full_label) ? 'minute' : 'min';
		$second = ($full_label) ? 'second' : 'sec';

		$time_array = array(
			'year'		=> 60 * 60 * 24 * 365,
			'month'		=> 60 * 60 * 24 * 30,
			'week'		=> 60 * 60 * 24 * 7,
			'day'		=> 60 * 60 * 24,
			'hour'		=> 60 * 60,
			$minute		=> 60,
			$second		=> 0,
		);

		foreach ($time_array as $key => $value)
		{
			if ($filter && !in_array($key, $filter))
			{
				continue;
			}

			$item = ($value) ? intval(intval($secs) / $value) : intval($secs);
			if ($item > 0)
			{
				$secs = $secs - ($item * $value);
				$output .= ' ' . $item . ' ' . (($item > 1) ? $user->lang['TIME_' . strtoupper($key) . 'S'] : $user->lang['TIME_' . strtoupper($key)]);
			}
		}

		return $output;
	}

	/**
	* Wrapper function for php number format
	* Displays scores in users local language format
	*/
	function number_format($num, $default = false)
	{
		if ($default)
		{
			return number_format($num, 3, '.', '');
		}

		if (!$num)
		{
			return 0;
		}

		global $user;

		// This function will format the the passed number to be displayed by the arcade.
		// This will fix the problem that was happening with some games
		// using decimal numbers for scores.  It will only display decimal
		// places if necessary.

		// Anytime that a number is displayed in the arcade for any reason it should
		// go through this function.  This includes portal blocks and such...

		// Lets see if the number has a decimal point
		$decimals = explode('.', $num);
		if (isset($decimals[1]))
		{
			// Ok so we have a decimal point remove all trailing zeroes and get the length
			$decimals = strlen(rtrim($decimals[1], '0'));
		}
		else
		{
			// No decimal point so set it to zero
			$decimals = 0;
		}

		return number_format($num, $decimals, $user->lang['SEPARATOR_DECIMAL'], $user->lang['SEPARATOR_THOUSANDS']);
	}

	function global_announce()
	{
		global $db, $user, $arcade_config, $template;

		$sql = 'SELECT * FROM ' . ARCADE_ANNOUNCE_TABLE;
		$result = $db->sql_query($sql, $arcade_config['cache_time'] * 3600);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if ($row)
		{
			if ($row['announce_anonym'] || $user->data['is_registered'])
			{
				$template->assign_vars(array(
					'S_ARCADE_GLOBAL_ANNOUNCE'	=> true,
					'S_GLOBAL_ANNOUNCE_SCROLL'	=> ($row['announce_scroll']) ? true : false,
					'GLOBAL_ANNOUNCE_TITLE'		=> censor_text($row['announce_title']),
					'GLOBAL_ANNOUNCE_AUTHOR'	=> ($row['announce_author']) ? $user->lang['POST_BY_AUTHOR'] . ' ' . get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'arcade', 'x') . ' - ' . $user->lang['POST_TIME'] . ': ' . $user->format_date($row['announce_date'], false, true) : false,
					'GLOBAL_ANNOUNCE_TEXT'		=> generate_text_for_display($row['announce_text'], $row['announce_uid'], $row['announce_bitfield'], $row['announce_options']),
				));
			}
		}
	}

	function hidden_score($game_id, $user_id)
	{
		global $user;
		return ($user->data['user_id'] != $user_id && !empty($this->hidden_users[$game_id][$user_id])) ? true : false;
	}

	/**
	* Tries to make sure the time is set correctly in
	* the settings page
	*
	* We expect military time, for example 14:23
	* Value must be a hour between 1 and 23 and a minute between 0 and 59
	* these must be separated with a colon ':'
	*/
	function validate_time($value)
	{
		$value = explode(':', $value);
		if (sizeof($value) < 2)
		{
			return array();
		}

		list($hour, $min) = array_map('intval', $value);
		if (($hour < 1 || $hour > 23) || ($min < 0 || $min > 59))
		{
			return array();
		}
		else
		{
			return array(
				'hour'	=> $hour,
				'min'	=> $min,
			);
		}
	}

	function user_info($type = 'header', $mode = 'arcade', $user_id = false, $profile = false)
	{
		global $db, $user, $auth_arcade, $arcade_config, $template;

		$type = strtoupper($type);

		if (!in_array($mode, array('arcade', 'challenge')))
		{
			$mode = 'arcade';
		}

		if ($user_id)
		{
			$user_info = $this->userdata('user_info', $user_id);
		}
		else
		{
			$user_info = $user->data;
		}

		if ($user_info['user_type'] == USER_IGNORE)
		{
			$userdata = array(
				'user_id'			=> $user_info['user_id'],
				'full_username'		=> get_username_string('no_profile', $user_info['user_id'], $user_info['username'], $user_info['user_colour']),
				'username'			=> ($user_info['user_id'] == ANONYMOUS) ? $user->lang['GUEST'] : $user_info['username'],
				'avatar'			=> ($this->optionget('view_avatars')) ? get_arcade_user_avatar($user_info['user_avatar'], $user_info['user_avatar_type'], $user_info['user_avatar_width'], $user_info['user_avatar_height']) : '',
				'rank_title'		=> '',
				'rank_image'		=> '',
				'rank_image_src'	=> '',
				'total_wins'		=> 0,
				'total_plays'		=> 0,
				'total_times'		=> 0,
				'total_ties'		=> 0,
				'total_loses'		=> 0,
				'last_play'			=> 0
			);
		}
		else
		{
			$rank_title = $rank_img = $rank_img_src = '';
			$total_wins = $total_ties = $total_plays = $total_times = $last_challenge = $last_play = 0;
			get_user_rank($user_info['user_rank'], $user_info['user_posts'], $rank_title, $rank_img, $rank_img_src);

			if ($mode == 'arcade')
			{
				// Total plays and times and Calculates the users total number of arcade victories
				$sql = 'SELECT SUM(p.total_plays) AS total_played, SUM(p.total_time) AS total_times, COUNT(g.game_highuser) AS total_wins
						FROM ' . ARCADE_PLAYS_TABLE . ' p
						LEFT JOIN ' . ARCADE_GAMES_TABLE . ' g ON g.game_highuser = ' . (int) $user_info['user_id'] . ' AND p.game_id = g.game_id
						WHERE p.user_id = ' . (int) $user_info['user_id'];
				$result = $db->sql_query($sql);
				$playing = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$total_plays	= (int) $playing['total_played'];
				$total_times	= (int) $playing['total_times'];
				$total_wins		= (int) $playing['total_wins'];
			}
			else if ($mode == 'challenge')
			{
				$sql = 'SELECT champ_challenger_id, champ_challenger_time, champ_opponent_time, champ_winner, champ_end_time
						FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
						WHERE champ_close = ' . CHALLENGE_CLOSE . '
						AND (champ_challenger_id = ' . (int) $user_info['user_id'] . ' OR champ_opponent_id = ' . (int) $user_info['user_id'] . ')
						ORDER BY champ_end_time ASC';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					if ($row['champ_winner'] == $user_info['user_id'])
					{
						$total_wins++;
					}

					if ($row['champ_winner'] == CHALLENGE_TIE)
					{
						$total_ties++;
					}

					$total_plays++;

					$total_times += ($user_info['user_id'] == $row['champ_challenger_id']) ? $row['champ_challenger_time'] : $row['champ_opponent_time'];
					$last_play = $row['champ_end_time'];
				}
				$db->sql_freeresult($result);

				$cp_data = array(
					'total_ties'		=> $total_ties,
					'total_loses'		=> intval($total_plays - $total_wins - $total_ties),
					'last_play'			=> $last_play
				);
			}

			$userdata = array(
				'user_id'			=> $user_info['user_id'],
				'full_username'		=> get_arcade_user_name('full', $user_info['user_id'], $user_info['username'], $user_info['user_colour'], (($profile) ? 'profile' : $mode), 'x'),
				'username'			=> $user_info['username'],
				'avatar'			=> ($this->optionget('view_avatars')) ? get_arcade_user_avatar($user_info['user_avatar'], $user_info['user_avatar_type'], $user_info['user_avatar_width'], $user_info['user_avatar_height'], 'x', (($profile) ? 'profile' : $mode), $user_info['user_id']) : '',
				'rank_title'		=> $rank_title,
				'rank_image'		=> $rank_img,
				'rank_image_src'	=> $rank_img_src,
				'total_wins'		=> $total_wins,
				'total_plays'		=> $total_plays,
				'total_times'		=> $total_times
			);

			if ($mode == 'challenge')
			{
				$userdata = array_merge($userdata, $cp_data);
			}
		}

		$template->assign_vars(array(
			"{$type}_AVATAR"			=> $userdata['avatar'],
			"{$type}_RANK_TITLE"		=> $userdata['rank_title'],
			"{$type}_RANK_IMAGE"		=> $userdata['rank_image'],
			"{$type}_USERNAME"			=> $userdata['full_username'],
		));

		if ($user_info['user_type'] <> USER_IGNORE)
		{
			if ($mode == 'challenge')
			{
				$template->assign_vars(array(
					"{$type}_CHALLENGE_TIES"			=> $this->number_format($userdata['total_ties']),
					"{$type}_CHALLENGE_LOSES"			=> $this->number_format($userdata['total_loses']),
					"{$type}_LAST_WINNING_CHALLENGE"	=> ($userdata['last_play'] > 0) ? $user->format_date($userdata['last_play']) : '-',
				));
			}

			$template->assign_vars(array(
				"{$type}_ARCADE_WINS"	=> $this->number_format($userdata['total_wins']),
				"{$type}_ARCADE_PLAYS"	=> $this->number_format($userdata['total_plays']),
				"{$type}_ARCADE_TIMES"	=> ($userdata['total_times'] > 0) ? $this->time_format($userdata['total_times']) : 0,
			));

			if ($this->points['show'])
			{
				$template->assign_vars(array(
					"S_SHOW_POINTS"			=> true,
					"{$type}_USER_POINTS"	=> $this->number_format($this->points['total']) . ' ' . $this->points['name'],
				));
			}

			$mode = strtoupper($mode);

			$template->assign_vars(array(
				'S_HEADER_USER_DATA'		=> ($type == 'HEADER') ? true : false,
				'L_ARCADE_WELCOME_PLAYS'	=> $user->lang[$mode . '_WELCOME_PLAYS'],
				'L_ARCADE_WELCOME_WINS'		=> $user->lang[$mode . '_WELCOME_WINS'],
				'L_ARCADE_WELCOME_TIME'		=> $user->lang[$mode . '_WELCOME_TIME'],
			));
		}

		unset($user_info);

		return $userdata;
	}

	function obtain_users_online(&$online_users)
	{
		global $db, $user, $auth, $config, $arcade_config;

		$prew_id = array();
		$prew_ip = array();
		// a little discrete magic to cache this for 30 seconds
		$online_users['online_time'] = (int) (($arcade_config['online_time'] > 0) ? $arcade_config['online_time'] : $config['load_online_time']);
		$online_time = intval(time() - ($online_users['online_time'] * 60));

		$sql = 'SELECT s.session_viewonline, s.session_page, s.session_ip, u.user_type, u.user_id, u.username, u.user_colour, g.game_id, g.game_name, g.cat_id
			FROM		' . ARCADE_SESSIONS_TABLE . '	a
			LEFT JOIN	' . SESSIONS_TABLE . '			s ON a.phpbb_session_id = s.session_id
			LEFT JOIN	' . USERS_TABLE . '				u ON a.user_id = u.user_id
			LEFT JOIN	' . ARCADE_GAMES_TABLE . '		g ON a.game_id = g.game_id
			WHERE a.phpbb_session_id = s.session_id
			AND s.session_user_id = u.user_id
			AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . '
			AND (a.user_id <> ' . ANONYMOUS . ' AND a.start_time >= ' . ($online_time - ((int) ($online_time % 30))) . ')
				OR
				(a.user_id =  ' . ANONYMOUS . ' AND a.start_time >= ' . ($online_time - ((int) ($online_time % 60))) . ')
			ORDER BY a.start_time DESC';
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			if ($this->playing_page($row['session_page']) === false)
			{
				continue;
			}

			$username	= ($row['user_id'] == ANONYMOUS) ? $user->lang['GUEST'] : $row['username'];
			$username	= get_arcade_user_name(($row['user_type'] <> USER_IGNORE) ? 'full' : 'no_profile', $row['user_id'], $username, $row['user_colour'], 'arcade', 'x');

			if ($row['user_id'] != ANONYMOUS && !isset($prew_id[$row['user_id']]))
			{
				$prew_id[$row['user_id']] = 1;

				if (!$row['session_viewonline'])
				{
					$player_link = '<em>' . $username . '</em>';
					$online_users['hidden_online']++;
				}
				else
				{
					$player_link = $username;
					$online_users['visible_online']++;
				}
			}
			else if ($row['user_id'] == ANONYMOUS && !isset($prew_ip[$row['session_ip']]))
			{
				$prew_ip[$row['session_ip']] = 1;
				$player_link = $username;
				$online_users['guests_online']++;
			}
			else
			{
				continue;
			}

			if ($row['session_viewonline'] || $auth->acl_get('u_viewonline'))
			{
				// Build a list of users playing a game
				if (!isset($online_users['games_names'][$row['game_id']]))
				{
					$online_users['games_names'][$row['game_id']] = get_arcade_game_name($row['game_name'], false, 'play', $row['cat_id'], $row['game_id'], 'x');
					$online_users['games_players'][$row['game_id']] = $player_link;
				}
				else
				{
					$online_users['games_players'][$row['game_id']] .=  ', ' . $player_link;
				}
			}
		}
		$db->sql_freeresult($result);

		$online_users['total_online'] = intval($online_users['guests_online'] + $online_users['visible_online'] + $online_users['hidden_online']);
	}

	function playing_page($page)
	{
		$playing_page = false;

		if (strpos($page, 'arcade') !== false && (strpos($page, 'play') !== false || strpos($page, 'popup') !== false || strpos($page, 'random') !== false))
		{
			$playing_page = true;
		}

		return $playing_page;
	}

	/**
	* All-encompasing sync function
	*
	* Modes:
	* - category		Sync category data
	* - game			Sync game data
	* - rating			Sync rating data
	* - total_data		Sync games, plays, downloads, users, reports data
	*/
	function sync($mode, $id = '')
	{
		global $db, $user, $arcade_cache;

		if ($id == '' && ($mode == 'rating' || $mode == 'game'))
		{
			trigger_error('ARCADE_SYNC_MODE_NOT_SUPPORTED');
		}

		switch($mode)
		{
			case 'category':
				$sql = 'SELECT cat_id
					FROM ' . ARCADE_CATS_TABLE;

				if ($id != '')
				{
					if (!is_array($id))
					{
						$id = array((int) $id);
					}

					$sql .= ' WHERE ' . $db->sql_in_set('cat_id', $id) . ' GROUP BY cat_id';
				}

				$cat_result = $db->sql_query($sql);

				while ($cat_row = $db->sql_fetchrow($cat_result))
				{
					$cat_id = $cat_row['cat_id'];

					$sql = 'SELECT game_installdate
						FROM ' . ARCADE_GAMES_TABLE . '
						WHERE cat_id = ' . (int) $cat_id . '
						ORDER BY game_installdate DESC';
					$result = $db->sql_query_limit($sql, 1);
					$cat_last_game_installdate = $db->sql_fetchfield('game_installdate');
					$db->sql_freeresult($result);

					$sql = 'SELECT COUNT(*) AS cat_games
							FROM ' . ARCADE_GAMES_TABLE . '
							WHERE cat_id = ' . (int) $cat_id;
					$result = $db->sql_query($sql);
					$cat_games = $db->sql_fetchfield('cat_games');
					$db->sql_freeresult($result);

					$sql_array = array(
						'SELECT'		=> 'SUM(p.total_plays) AS cat_plays',
						'FROM'			=> array(
							ARCADE_GAMES_TABLE	=> 'g',
						),
						'LEFT_JOIN'		=> array(
							array(
								'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
								'ON'	=> 'g.game_id = p.game_id'
							),
						),
						'WHERE'			=> 'g.cat_id = ' . (int) $cat_id . '
							AND g.game_id = p.game_id',
					);

					$sql = $db->sql_build_query('SELECT', $sql_array);
					$result = $db->sql_query($sql);
					$cat_plays = (int) $db->sql_fetchfield('cat_plays');
					$db->sql_freeresult($result);

					$sql_ary = array(
						'cat_games'					=> (int) $cat_games,
						'cat_plays'					=> (int) $cat_plays,
						'cat_last_game_installdate'	=> (int) $cat_last_game_installdate,
					);

					$sql = 'UPDATE ' . ARCADE_CATS_TABLE  . '
							SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
							WHERE cat_id = ' . (int) $cat_id;
					$db->sql_query($sql);
				}
				$db->sql_freeresult($cat_result);
			break;

			case 'game':
				if (!is_array($id))
				{
					$id = ($id) ? array((int) $id) : array();
				}

				$sql = 'SELECT SUM(total_plays) AS game_plays, game_id
						FROM ' . ARCADE_PLAYS_TABLE . '
						WHERE ' . $db->sql_in_set('game_id', $id) . '
						GROUP BY game_id';
				$result = $db->sql_query($sql);

				$plays_id = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$plays_id[] = $row['game_id'];
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_plays = ' . (int) $row['game_plays'] . '
							WHERE game_id = ' . (int) $row['game_id'];
					$db->sql_query($sql);
				}
				$db->sql_freeresult($result);

				$no_plays = array();
				if (!empty($plays_id))
				{
					foreach ($id as $game_id)
					{
						if (!in_array($game_id, $plays_id))
						{
							$no_plays[] = $game_id;
						}
					}
				}
				else
				{
					$no_plays = $id;
				}

				if (!empty($no_plays))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_plays = 0
							WHERE ' . $db->sql_in_set('game_id', $no_plays);
					$db->sql_query($sql);
				}
			break;

			case 'rating':
				if (!is_array($id))
				{
					$id = ($id) ? array((int) $id) : array();
				}

				$sql = 'SELECT game_id, COUNT(*) as game_votetotal, SUM(game_rating) as game_votesum
						FROM ' . ARCADE_RATING_TABLE . '
						WHERE ' . $db->sql_in_set('game_id', $id) . '
						GROUP BY game_id';
				$result = $db->sql_query($sql);

				$plays_id = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$plays_id[] = $row['game_id'];
					$returned_results = true;
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_votetotal	= ' . (int) $row['game_votetotal'] . ',
								game_votesum	= ' . (int) $row['game_votesum'] . '
							WHERE game_id = ' . (int) $row['game_id'];
					$db->sql_query($sql);
				}
				$db->sql_freeresult($result);

				$no_plays = array();
				if (!empty($plays_id))
				{
					foreach ($id as $game_id)
					{
						if (!in_array($game_id, $plays_id))
						{
							$no_plays[] = $game_id;
						}
					}
				}
				else
				{
					$no_plays = $id;
				}

				if (!empty($no_plays))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_votetotal = 0, game_votesum = 0
							WHERE ' . $db->sql_in_set('game_id', $no_plays);
					$db->sql_query($sql);
				}
			break;

			case 'total_data':
				if (!is_array($id))
				{
					$id = array($id);
				}

				if (in_array('plays' , $id) || in_array('all' , $id))
				{
					$play_data = $this->get_total('plays');
					$this->set_config('total_plays', intval($play_data['games_played']), true);
					$this->set_config('total_plays_time', intval($play_data['games_time']), true);
				}

				if (in_array('downloads' , $id) || in_array('all' , $id))
				{
					$total_downloads = (int) $this->get_total('downloads');
					$this->set_config('total_downloads', $total_downloads, true);
				}

				if (in_array('reports' , $id) || in_array('all' , $id))
				{
					$reports_open = (int) $this->get_total('reports_open');
					$this->set_config('reports_open', $reports_open);
					$arcade_cache->destroy('sql', ARCADE_REPORTS_TABLE);
				}

				if (in_array('challenge' , $id) || in_array('all' , $id))
				{
					$challenge_total = $this->get_total('challenge_total');
					$this->set_config('challenge_total_plays', intval($challenge_total['total_plays']), true);
					$this->set_config('challenge_total_plays_time', intval($challenge_total['total_times']), true);
				}
			break;

			default:
				trigger_error('NO_MODE');
			break;
		}
		return;
	}

	/**
	* Gets a total number of games based on the parameters it
	* was sent.
	*
	* $type can be set to fav, games or search
	*/
	function get_total($type = '', $id = 0)
	{
		global $db, $user;

		$type = strtolower($type);
		$sql_where = '';

		switch ($type)
		{
			case 'fav':
				if ($id)
				{
					$sql_array = array(
						'SELECT'		=> 'COUNT(f.game_id) as total',
						'FROM'			=> array(
							ARCADE_FAVS_TABLE	=> 'f',
						),
						'LEFT_JOIN'		=> array(
							array(
								'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
								'ON'	=> 'f.game_id = g.game_id'
							),
						),
						'WHERE'			=> 'f.user_id = ' . (int) $id . ' AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					trigger_error('ARCADE_NO_ID_ERROR');
				}
			break;

			case 'games':
				$sql_array = array(
					'SELECT'	=> 'COUNT(game_id) as total',
					'FROM'		=> array(
						ARCADE_GAMES_TABLE => 'g',
					),
				);

				if ($id)
				{
					$sql_array['WHERE'] = 'cat_id = ' . (int) $id;
				}
				$sql = $db->sql_build_query('SELECT', $sql_array);
			break;

			case 'scores':
				if ($id)
				{
					$sql_array = array(
						'SELECT'		=> 'COUNT(score) as total',
						'FROM'			=> array(
							USERS_TABLE	=> 'u',
						),
						'LEFT_JOIN'		=> array(
							array(
								'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
								'ON'	=> 'u.user_id = s.user_id'
							),
						),
						'WHERE'			=> 's.game_id = ' . (int) $id,
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					$sql_array = array(
						'SELECT'		=> 'g.game_id',
						'FROM'			=> array(
							ARCADE_GAMES_TABLE	=> 'g',
						),
						'LEFT_JOIN'		=> array(
							array(
								'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
								'ON'	=> 'g.game_id = s.game_id'
							),
						),
						'WHERE'			=> 'g.game_id = s.game_id',
					);

					$sql = $db->sql_build_query('SELECT', $sql_array);
					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					return sizeof($row);
				}
			break;

			case 'search_newgames':
				if ($id)
				{
					// Return all the newest games
					$sql_array = array(
						'SELECT'	=> 'COUNT(g.game_id) as total',
						'FROM'		=> array(
							ARCADE_GAMES_TABLE	=> 'g',
						),
						'WHERE'		=> 'g.game_installdate >= ' . (int) $id . '
										AND ' . $db->sql_in_set('g.cat_id', $this->games_search_protection(), true, true) . '
										AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					trigger_error('ARCADE_NO_ID_ERROR');
				}
			break;

			case 'search':
				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(g.game_id) as total',
						'FROM'		=> array(
							ARCADE_GAMES_TABLE => 'g',
						),
						'WHERE'		=> '(g.game_name_clean ' . $db->sql_like_expression($id) . '
										OR LOWER(g.game_desc) ' . $db->sql_like_expression($id) . ')
										AND ' . $db->sql_in_set('g.cat_id', $this->games_search_protection(), true, true) . '
										AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					trigger_error('ARCADE_NO_ID_ERROR');
				}
			break;

			case 'download_stats':
				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(game_id) as total',
						'FROM'		=> array(
							ARCADE_DOWNLOAD_TABLE	=> 'd',
						),
						'WHERE'		=> 'user_id = ' . (int) $id,
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					$sql_array = array(
						'SELECT'	=> 'user_id',

						'FROM'		=> array(
							ARCADE_DOWNLOAD_TABLE	=> 'd',
						),

						'GROUP_BY'		=> 'user_id',
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);

					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					return sizeof($row);
				}
			break;

			case 'plays':
				$sql = 'SELECT SUM(total_plays) AS games_played, SUM(total_time) AS games_time
						FROM ' . ARCADE_PLAYS_TABLE;
				$result = $db->sql_query($sql);
				$play_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				return $play_data;
			break;

			case 'downloads':
				$sql = 'SELECT SUM(total) as total
						FROM ' . ARCADE_DOWNLOAD_TABLE;
			break;

			case 'reports_open':
				$sql = 'SELECT COUNT(report_id) as total
						FROM ' . ARCADE_REPORTS_TABLE . '
						WHERE report_closed = ' . ARCADE_REPORT_OPEN;
			break;

			case 'users_played':
				if ($db->sql_layer === 'sqlite')
				{
					$sql = 'SELECT COUNT(user_id) as total
						FROM (
							SELECT DISTINCT user_id
							FROM ' . ARCADE_PLAYS_TABLE . '
						)';
				}
				else
				{
					$sql = 'SELECT COUNT(DISTINCT user_id) as total
							FROM ' . ARCADE_PLAYS_TABLE;
				}
			break;

			case 'challenge_users_played':
					$sql = 'SELECT COUNT(*) as total
						FROM (
							SELECT DISTINCT c1.champ_challenger_id FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' c1
							UNION
							SELECT DISTINCT c2.champ_opponent_id FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' c2
						) c';
			break;

			case 'challenge_total':
				$sql = 'SELECT SUM(champ_challenger_time + champ_opponent_time) AS total_times, COUNT(champ_id) AS total_plays
						FROM '.ARCADE_CHALLENGE_CHAMP_TABLE . '
						WHERE champ_close = ' . CHALLENGE_CLOSE;
				$result = $db->sql_query($sql);
				$total = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				return $total;
			break;

			case 'error':
				$sql = 'SELECT COUNT(*) as total FROM ' . ARCADE_ERRORS_TABLE;
			break;

			case 'banned':
				$sql = 'SELECT COUNT(*) as total FROM ' . ARCADE_USERS_BANNED_TABLE;
			break;

			default:
				trigger_error('ARCADE_NO_TOTAL_TYPE_ERROR', E_USER_ERROR);
			break;
		}

		$result = $db->sql_query($sql);
		$total = (int) $db->sql_fetchfield('total');
		$db->sql_freeresult($result);

		return $total;
	}

	/**
	* This is never used for IBProV3 or Arcadelib games because it will cause scoring errors with these types.
	* If a game is having scoring errors, try shutting this off for the specific game type.
	*/
	function get_protection($type)
	{
		global $arcade_config;

		$use_protection = false;
		switch($type)
		{
			case AMOD_GAME:
				$use_protection = ($arcade_config['protect_amod']) ? true : false;
			break;

			case IBPRO_GAME:
				$use_protection = ($arcade_config['protect_ibpro']) ? true : false;
			break;

			case V3ARCADE_GAME:
				$use_protection = ($arcade_config['protect_v3arcade']) ? true : false;
			break;

			case PHPBBARCADE_GAME:
				$use_protection = ($arcade_config['protect_phpbbarcade']) ? true : false;
			break;

			default:
			break;
		}

		return $use_protection;
	}

	function banned($type, $user_id = false, $game_id = false, $score = 0)
	{
		global $db, $user, $arcade_cache;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		if ($user_id == ANONYMOUS)
		{
			return;
		}

		$sql_ary = array(
			'user_id'		=> (int) $user_id,
			'banned_type'	=> $type,
			'banned_ip'		=> $user->ip,
			'banned_date'	=> time(),
			'game_id'		=> (int) $game_id,
			'score'			=> (float) $score
		);

		$db->sql_query('INSERT INTO ' . ARCADE_USERS_BANNED_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
		$arcade_cache->destroy('sql', ARCADE_USERS_BANNED_TABLE);
	}

	/**
	* Set the rating image for the game
	*/
	function set_rating_image($data, $mode = '')
	{
		global $user, $auth_arcade, $template, $phpbb_root_path;

		$rate_block	= (!$user->data['is_registered']) ? true : false;
		$c_rate		= $auth_arcade->acl_get('c_rate', $data['cat_id']) ? true : false;
		$c_new_rate = $auth_arcade->acl_get('c_re_rate', $data['cat_id']) ? true : false;

		if ($data['game_votetotal'] > 0)
		{
			$star_width = (int) (($data['game_votesum'] / $data['game_votetotal']) * 16);
		}
		else
		{
			$star_width = 0;
		}

		$title = '';

		if (!$c_rate && !$rate_block)
		{
			$title.= $user->lang['ARCADE_RATING_NO_PERMISSION'];
		}
		else if ($c_rate && !$c_new_rate && (!empty($data['game_rating'])) && !$rate_block)
		{
			$title.= sprintf($user->lang['ARCADE_RATING_ALREADY'], $data['game_rating']);
		}

		$star_image = "<div style='height:16px; ". (!$mode ? "width:100px; margin-right:auto; margin-left:auto;" : '') ."' id='star_{$data['game_id']}'>
						<ul class='arcade-rate'" . ($title ? "title='{$title}'" : "") . ">
							<li class='arcade-current-rate' style='width:{$star_width}px;'>.</li>
							<li class='arcade-rating-num'><span title='{$user->lang['ARCADE_RATING_NUM']}'>({$data['game_votetotal']})</span></li>";

		if (($c_rate && (empty($data['game_rating'])) && !$rate_block) || ($c_rate && $c_new_rate && !$rate_block))
		{
			for ($x = 1; $x <= 5; $x++)
			{
				$star_image.= "<li><a onclick='arcade(\"rating\", \"{$data['game_id']}\", \"{$x}\", \"{$data['cat_id']}\", \"{$mode}\");return false;' title='" . sprintf($user->lang['ARCADE_RATING_VALUE'], $x) . "' class='arcade-rate-{$x}' rel='nofollow'>{$x}</a></li>";
			}
		}

		$star_image.= "</ul></div>";

		return $star_image;
	}

	/**
	* Set the favorites image for the game
	*/
	function set_fav_image($game_data, $game_id, $mode = '')
	{
		global $user;

		if (!$user->data['is_registered'])
		{
			return;
		}
		else
		{
			if (!empty($game_data[$game_id]))
			{
				$image = "<span id='fav_{$game_id}'><a onclick='arcade(\"fav\", \"{$game_id}\", \"" . ARCADE_FAV_DEL . "\", \"\", \"{$mode}\"); return false;'><img class='arcade_fav' src='" . $this->get_image('src', 'img', 'remove_favorite.png') . "' title='{$user->lang['ARCADE_REMOVE_FAV']}' alt='{$user->lang['ARCADE_REMOVE_FAV']}' /></a></span>";
			}
			else
			{
				$image = "<span id='fav_{$game_id}'><a onclick='arcade(\"fav\", \"{$game_id}\", \"" . ARCADE_FAV_ADD . "\", \"\", \"{$mode}\"); return false;'><img class='arcade_fav' src='" . $this->get_image('src', 'img', 'add_favorite.png')	  . "' title='{$user->lang['ARCADE_ADD_FAV']}'	  alt='{$user->lang['ARCADE_ADD_FAV']}'	/></a></span>";
			}

			return $image;
		}
	}

	function games_list($type)
	{
		global $user, $template, $arcade_config, $auth_arcade;

		$play_games = false;

		if ($arcade_config['load_list'])
		{
			$played_game_color = str_replace('#', '', $arcade_config['played_colour']);

			// Quick jump of all the games in the arcade...
			if ($user->data['is_registered'] && $type == 'quick_jump' && $played_game_color !== '')
			{
				$played_games = $this->get_played_games();
				$template->assign_vars(array(
					'PLAYED_GAME_TITLE'		=> ' title="' . $user->lang['ARCADE_PLAYED_GAMES_HIGHLIGHT'] . '"',
					'PLAYED_GAME_COLOUR'	=> $played_game_color
				));
			}
		}

		$user_age = $this->user_age();
		$founder_exemption = ($arcade_config['founder_exempt'] && $user->data['user_type'] == USER_FOUNDER) ? true : false;

		foreach ($this->games as $gid => $game)
		{
			if ($arcade_config['load_list'])
			{
				$cat_age = $this->get_cat_field($game['cat_id'], 'cat_age');

				if ($type == 'quick_jump' && ($this->get_cat_field($game['cat_id'], 'cat_status') == ITEM_LOCKED || !$auth_arcade->acl_get('c_play', $game['cat_id']) || ($cat_age && !$founder_exemption && ($user_age === '' || ($user_age < $cat_age)))))
				{
					continue;
				}

				$played = true;
				if (!isset($played_games[$gid]))
				{
					$played = false;
				}

				// Only show a game in the games statistics drop down if its been played
				if (($type == 'game_jump') && ($game['game_highuser'] == 0 && $game['game_highdate'] == 0))
				{
					continue;
				}

				$template->assign_block_vars($type, array(
					'S_PLAYED_GAME'	=> ($type == 'quick_jump') ? $played : false,
					'GAME_ID' 		=> $gid,
					'GAME_NAME'		=> $game['game_name'],
				));
			}
			else
			{
				if (($type == 'quick_jump') && ($this->get_cat_field($game['cat_id'], 'cat_status') == ITEM_LOCKED || !$auth_arcade->acl_get('c_play', $game['cat_id'])))
				{
					continue;
				}

				$play_games = true;
				break;
			}
		}

		if (!$arcade_config['load_list'])
		{
			$template->assign_var((($type == 'quick_jump') ? 'HEADER_' : 'STATS_') . 'GAMES_LIST_LOADING', ($play_games) ? '<a style="cursor:pointer; font-weight: bold;" onclick="arcade(\'games_list\', \'' . (($type == 'quick_jump') ? 1 : 2) . '\'); return false;">' . $user->lang['ARCADE_GAMES_LIST_LOAD'] .'</a>' : $user->lang['ARCADE_NO_GAMES']);
		}
	}

	function users_list()
	{
		global $template, $arcade_config;

		if ($arcade_config['load_list'])
		{
			foreach ($this->obtain_arcade_users() as $_uid => $_user)
			{
				$template->assign_block_vars('stat_user_jump', array(
					'USER_ID'	=> $_uid,
					'USERNAME'	=> $_user['username'])
				);
			}
		}
		else
		{
			global $user;
			$template->assign_var('USERS_LIST_LOADING', (sizeof($this->obtain_arcade_users())) ? '<a style="cursor:pointer; font-weight: bold;" onclick="arcade(\'users_list\', \'2\'); return false;">' . $user->lang['ARCADE_USERS_LIST_LOAD'] .'</a>' : $user->lang['ARCADE_NO_PLAYS']);
		}
	}

	/**
	* Replace the placeholder inside the private message
	*/
	function prepare_pm($type, &$text, $ac_data)
	{
		global $user, $arcade_config, $lang, $phpEx;
		// Used in conjuction with the send_pm function.  This is
		// used to replace the place holders with the data...
		$arcade_url = generate_board_url() . '/arcade.' . $phpEx;
		switch ($type)
		{
			case 'arcade':
				$old_user_link	= ($ac_data['view_stats']) ? '[url=' . $arcade_url . '?mode=stats&amp;u=' . $ac_data['other_user_id'] . ']' . $lang['ARCADE_HERE'] . '[/url]' : '';

				$text = str_replace('[game_id]'			, $ac_data['game_id']		, $text);
				$text = str_replace('[game_name]'		, $ac_data['game_name']		, $text);
				$text = str_replace('[old_user_id]'		, $ac_data['other_user_id']	, $text);
				$text = str_replace('[old_username]'	, $ac_data['other_username'], $text);
				$text = str_replace('[new_user_id]'		, $user->data['user_id']	, $text);
				$text = str_replace('[new_username]'	, $user->data['username']	, $text);
				$text = str_replace('[old_score]'		, $ac_data['old_score']		, $text);
				$text = str_replace('[new_score]'		, $ac_data['new_score']		, $text);
				$text = str_replace('[old_user_link]'	, $old_user_link, $text);
			break;

			case 'challenge_result':
				if (!empty($ac_data['winner_score']))
				{
					$text = str_replace('[winner_username]'	, $ac_data['winner_username']	, $text);
					$text = str_replace('[loser_username]'	, $ac_data['loser_username']	, $text);
					$text = str_replace('[winner_user_link]', $ac_data['winner_user_link']	, $text);
					$text = str_replace('[loser_user_link]'	, $ac_data['loser_user_link']	, $text);
					$text = str_replace('[winner_score]'	, $ac_data['winner_score']		, $text);
					$text = str_replace('[loser_score]'		, $ac_data['loser_score']		, $text);
				}
				else
				{
					$text = str_replace('[user_score]'			, $ac_data['user_score']		, $text);
					$text = str_replace('[opponent_username]'	, $ac_data['opponent_username']	, $text);
					$text = str_replace('[opponent_user_link]'	, $ac_data['opponent_user_link']	, $text);
				}
			break;
		}

		switch ($type)
		{
			case 'arcade':
			case 'challenge':
				$username			= ($user->data['user_colour']) ? '[color=#' .  $user->data['user_colour'] . ']' . $user->data['username'] . '[/color]' : $user->data['username'];
				$new_user_link		= ($ac_data['view_stats']) ? '[url=' . $arcade_url . '?' . (($type == 'challenge') ? 'mode=view_challenge_stats' : 'mode=stats') . '&amp;u=' . $user->data['user_id'] . ']' . $username . '[/url]' : $username;

				$text = str_replace(array('[new_user_link]', '[challenger_username]'), $new_user_link, $text);
			case 'challenge_result':
				$game_link			= ($ac_data['play_game']) ? '[url=' . $arcade_url . '?mode=play&amp;g='  . $ac_data['game_id'] . $this->gametop . ']' . $ac_data['game_name'] . '[/url]' : $ac_data['game_name'];
				$game_image			= ($ac_data['view_image']) ? '[img]' . $arcade_url . '?img=' . $ac_data['game_image'] . '[/img]' : '';
				$game_image			= ($ac_data['play_game'] && $game_image) ? '[url=' . $arcade_url . '?mode=play&amp;g='  . $ac_data['game_id'] . $this->gametop . ']' . $game_image . '[/url]' : $game_image;
				$game_image			= ($game_image !== '') ? $game_image . '<br />' : '';
				$game_stats_link	= ($ac_data['view_stats']) ? '[url=' . $arcade_url . '?mode=stats&amp;g=' . $ac_data['game_id'] . ']' . $lang['ARCADE_GAME_STATS'] . '[/url]<br />' : '';
				$game_popup_link	= ($ac_data['play_game'] && $ac_data['view_popup_link']) ? '[popup-game=' . $arcade_url . '?mode=popup&amp;g=' . $ac_data['game_id'] . ', ' . $ac_data['game_width'] . ', ' . $ac_data['game_height'] . ']' . $lang['ARCADE_PLAYING_NEW_WINDOW'] . '[/popup-game]<br />' : '';
				$game_popup_link   .= ($game_image || $game_stats_link || $game_popup_link) ? '<br />' : '';

				$text = str_replace('[game_link]'		, $game_link		, $text);
				$text = str_replace('[game_image]'		, $game_image		, $text);
				$text = str_replace('[game_stats_link]'	, $game_stats_link	, $text);
				$text = str_replace('[game_popup_link]'	, $game_popup_link	, $text);
			break;
		}
	}

	/**
	* Send a private message with the
	* message set in the acp
	*/
	function send_pm($type, $data, $other = true)
	{
		global $db, $user, $lang, $phpbb_root_path, $phpEx;

		if (!$user->data['is_registered'] || !isset($data['other_user_id']) || $data['other_user_id'] <= ANONYMOUS)
		{
			return;
		}

		$this->set_other_user_permissions($data, $other);
		$lang = $this->message_language($data['other_user_lang']);

		if (!isset($lang[$data['PM_SUBJECT']]) || !isset($lang[$data['PM_MESSAGE']]))
		{
			return;
		}

		$subject = $lang[$data['PM_SUBJECT']];
		$message = $lang[$data['PM_MESSAGE']];

		if ($type != 'arcade')
		{
			$challenge_view	= '<br /><br />[url=' . generate_board_url() . '/arcade.' . $phpEx . '?mode=challenge#boxtop]' . $lang['CHALLENGE_VIEW'] . '[/url]';
		}

		switch ($type)
		{
			case 'arcade':
				$message = $message . (($data['view_stats']) ? '<br /><br />' . $lang['ARCADE_PM_VIEW_YOUR_STAT'] : '');
			break;

			case 'challenge':
				$tour_bet = ($data['bet'] > 0) ? '<br />' . sprintf($lang['CHALLENGE_MSG_TOURNAMENT_BET'], $this->number_format($data['bet']) . ' ' . $this->points['name']) : '';
				$deduction = ($data['total_cost'] > 0) ? '<br />' . sprintf($lang['CHALLENGE_MSG_DEDUCTION_' . (($data['bet'] > 0) ? (($data['free_play']) ? 'BET' : 'BET_COST') : 'COST')], $this->number_format($data['total_cost']) . ' ' . $this->points['name']) . '<br />' . $lang['CHALLENGE_MSG_REJECT_REFUND'] : '';
				$message .= $tour_bet . $deduction . $challenge_view;
			break;

			case 'challenge_user_action':
				$refund = ($data['total_cost'] > 0) ? '<br />' . sprintf($lang['CHALLENGE_MSG_REFUND' . (($data['tc_num'] > 1) ? 'S' : '')], $this->number_format($data['total_cost']) . ' ' . $this->points['name']) : '';
				$message = sprintf($message, $data['username'], $data['games_name']) . $refund . $challenge_view;
			break;

			case 'challenge_result':
				$prize = ($data['prize'] > 0) ? '<br />' . sprintf($lang[$data['BET_MSG']], $this->number_format($data['prize']) . ' ' . $this->points['name']) : '';
				$game_cost_refund = ($data['refund_game_cost'] > 0) ? '<br />' . sprintf($lang['CHALLENGE_MSG_GAME_COST_REFUND'], $this->number_format($data['refund_game_cost']) . ' ' . $this->points['name']) : '';
				$message .= $prize . $game_cost_refund . ((!empty($data['winner_score'])) ? $challenge_view : '');
			break;

			default:
				return;
			break;
		}

		$message = $lang['ARCADE_MESSAGE_GENERATED'] . '<br /><br />' . $message;

		if ($type != 'challenge_user_action')
		{
			$this->prepare_pm($type, $subject, $data);
			$this->prepare_pm($type, $message, $data);
		}

		$user->add_lang('ucp');

		$subject = utf8_normalize_nfc($subject);
		$message = utf8_normalize_nfc($message);

		$uid = $bitfield = $options = '';
		$allow_bbcode = $allow_urls = $allow_smilies = true;

		generate_text_for_storage($message, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);

		$pm_data = array(
			'address_list'			=> array('u' => array($data['other_user_id'] => 'to')),
			'from_user_id'			=> ($other) ? $user->data['user_id'] : $data['user_id'],
			'from_user_ip'			=> ($other) ? $user->ip : $data['user_ip'],
			'from_username'			=> ($other) ? $user->data['username'] : $data['username'],
			'reply_from_root_level'	=> 0,
			'reply_from_msg_id'		=> 0,
			'icon_id'				=> 0,
			'enable_sig'			=> false,
			'enable_bbcode'			=> $allow_bbcode,
			'enable_smilies'		=> $allow_smilies,
			'enable_urls'			=> $allow_urls,
			'bbcode_bitfield'		=> $bitfield,
			'bbcode_uid'			=> $uid,
			'message'				=> $message
		);

		if (!function_exists('submit_pm'))
		{
			include($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);
		}

		submit_pm('post', $subject, $pm_data);
	}

	function notifypm_check($user_id, &$user_notify_pm)
	{
		global $arcade_notifypm;

		if (isset($arcade_notifypm) && $user_id == $arcade_notifypm['user_id'])
		{
			$user_notify_pm = $arcade_notifypm['value'];
		}
	}

	function set_other_user_permissions(&$data, $other)
	{
		global $config, $arcade_config, $arcade_notifypm;

		if ($other)
		{
			global $auth_arcade2;

			$set = array(
				'view_image'		=> (!empty($data['game_image']) && $arcade_config['display_game_image'] && $this->optionget('view_game_image', $data['other_user_arcade_options'], true)) ? true : false,
				'view_popup_link'	=> ($auth_arcade2->acl_get('u_arcade_popup')) ? true : false,
				'play_game'			=> ($auth_arcade2->acl_get('c_play', $data['cat_id'])) ? true : false,
				'view_stats'		=> ($auth_arcade2->acl_get('u_arcade_viewstats')) ? true : false
			);

			$arcade_notifypm = array(
				'user_id'	=> $data['other_user_id'],
				'value'		=> ($this->optionget('notifypm', $data['other_user_arcade_options'], true)) ? 1 : 0
			);
		}
		else
		{
			global $user, $auth_arcade;

			$set = array(
				'view_image'		=> (!empty($data['game_image']) && $arcade_config['display_game_image'] && $this->optionget('view_game_image')) ? true : false,
				'view_popup_link'	=> ($auth_arcade->acl_get('u_arcade_popup')) ? true : false,
				'play_game'			=> ($auth_arcade->acl_get('c_play', $data['cat_id'])) ? true : false,
				'view_stats'		=> ($auth_arcade->acl_get('u_arcade_viewstats')) ? true : false
			);

			$arcade_notifypm = array(
				'user_id'	=> $user->data['user_id'],
				'value'		=> ($this->optionget('notifypm')) ? 1 : 0
			);
		}

		$data = array_merge($data, $set);
	}

	function message_language($lang_dir)
	{
		global $user, $phpbb_root_path, $phpEx;

		if($user->data['user_lang'] != $lang_dir && file_exists($phpbb_root_path . 'language/' . $lang_dir . "/mods/arcade.{$phpEx}"))
		{
			$lang = array();
			include($phpbb_root_path . 'language/' . $lang_dir . "/mods/arcade.{$phpEx}");
		}
		else
		{
			$lang = $user->lang;
		}

		return $lang;
	}

	function score_done($game_id, $popup, $game_info, $score_desc = '')
	{
		global $user, $template, $auth_arcade, $arcade_session;

		$s_nav = (!$popup && !$game_info) ? true : false;

		if (!$game_info)
		{
			$game_info = $this->get_game_cat_data($game_id);
		}

		if ($s_nav)
		{
			generate_arcade_nav($game_info, true);
		}

		if ($popup)
		{
			$message  = sprintf($user->lang['ARCADE_POPUP_DONE'], $game_info['game_name'], '<a href="' . $this->url("mode=" . (($popup) ? 'popup' : 'play') . "&amp;g={$game_id}") . '">', $game_info['game_name'], '</a>');
			$message .= '<br /><br /><a href="#" onclick="arcade_refresh_page(\''. $this->url("mode=cat&amp;c={$game_info['cat_id']}&amp;g={$game_info['game_id']}#g{$game_info['game_id']}") .'\'); return false;">' . $user->lang['CLOSE_WINDOW'] . '</a>';
		}
		else
		{
			$message  = sprintf($user->lang['ARCADE_FULL_DONE'], $game_info['game_name']) . $this->return_links($game_info);
		}

		$arcade_session->set_cookie();

		$template->assign_vars(array(
			'S_USE_SIMPLE_HEADER'	=> $popup,
			'S_SCORE_DONE'			=> true,
			'S_ARCADE_SCORE_DESC'	=> $score_desc,

			'L_ARCADE_DONE'			=> $message,

			'GAME_RATING_IMG'		=> $this->set_rating_image($game_info),
			'GAME_IMAGE'			=> get_arcade_game_image($game_info['game_image'])
		));

		if (!$popup && $auth_arcade->acl_get('u_arcade_view_whoisplaying'))
		{
			display_arcade_online();
		}

		// Output page
		page_header($user->lang['INDEX'], false);

		$template->set_filenames(array(
			'body' => 'arcade/score_body.html'
		));

		page_footer();
	}

	/**
	* Limits access to play games based on parameters set in the ACP
	*/
	function limit_play()
	{
		global $user, $db, $arcade_config;

		if ($arcade_config['limit_play'] != LIMIT_PLAY_TYPE_NONE)
		{
			$total_posts = intval($arcade_config['limit_play_total_posts']);
			$days = intval($arcade_config['limit_play_days']);
			$posts = intval($arcade_config['limit_play_posts']);
			$current_time = time();
			$old_time = $current_time - (60 * 60 * 24 * $days);
			$error = array();

			if ($arcade_config['limit_play'] == LIMIT_PLAY_TYPE_POSTS || $arcade_config['limit_play'] == LIMIT_PLAY_TYPE_BOTH)
			{
				$sql = 'SELECT COUNT(post_id) as total_posts
						FROM ' . POSTS_TABLE . '
						WHERE poster_id = ' . (int) $user->data['user_id'] . '
						AND post_postcount > 0';
				$result = $db->sql_query($sql);
				$actual_total_posts = (int) $db->sql_fetchfield('total_posts');
				$db->sql_freeresult($result);

				if ($actual_total_posts < $total_posts)
				{
					$error[] = sprintf($user->lang['ARCADE_LIMIT_PLAY_TYPE_POSTS'], $total_posts, $total_posts - $actual_total_posts);
				}
			}

			if ($arcade_config['limit_play'] == LIMIT_PLAY_TYPE_DAYS || $arcade_config['limit_play'] == LIMIT_PLAY_TYPE_BOTH)
			{
				$sql = 'SELECT COUNT(post_id) as total_posts
						FROM ' . POSTS_TABLE . '
						WHERE poster_id = ' . (int) $user->data['user_id'] . '
						AND post_time
						BETWEEN ' . (int) $old_time . ' AND ' . (int) $current_time . '
						AND post_postcount > 0';
				$result = $db->sql_query($sql);
				$actual_posts_per_day = (int) $db->sql_fetchfield('total_posts');
				$db->sql_freeresult($result);

				if ($actual_posts_per_day < $posts)
				{
					$error[] = sprintf($user->lang['ARCADE_LIMIT_PLAY_TYPE_DAYS'], $posts, $days, $posts - $actual_posts_per_day);
				}
			}

			if (sizeof($error))
			{
				trigger_error(implode('<br /><br />', $error) . $this->back_link());
			}
		}
	}

	/**
	* Set the headers to notify the browser not to cache
	* If this isn't here the IBPro v3 games will not work
	*/
	function set_header_no_cache()
	{
		@header("HTTP/1.0 200 OK");
		@header("HTTP/1.1 200 OK");
		@header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		@header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
		@header('Pragma: no-cache');
	}

	function update_last_play_time($game_id, $cat_id)
	{
		global $db, $user, $auth_arcade, $arcade_config;

		$time			= time();
		$interval		= ($user->data['user_id'] == ANONYMOUS) ? $arcade_config['play_anonymous_interval'] : $arcade_config['play_interval'];
		$remaining_time	= ($time - $user->data['user_arcade_last_play'] > $interval) ? true : false;

		if ($interval && !$auth_arcade->acl_get('c_ignoreflood_play', $cat_id) && $remaining_time)
		{
			$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
					SET user_arcade_last_play = ' . $time . '
					WHERE user_id = ' . (int) $user->data['user_id'];
			$db->sql_query($sql);
		}
	}

	/**
	* Find available compress methods on the server
	*/
	function compress_methods($remove_dot = false)
	{
		if ($remove_dot)
		{
			$methods = array('tar');
			$available_methods = array('gz' => 'zlib', 'bz2' => 'bz2', 'zip' => 'zlib');
		}
		else
		{
			$methods = array('.tar');
			$available_methods = array('.tar.gz' => 'zlib', '.tar.bz2' => 'bz2', '.zip' => 'zlib');
		}

		foreach ($available_methods as $type => $module)
		{
			if (!@extension_loaded($module))
			{
				continue;
			}
			$methods[] = $type;
		}

		return $methods;
	}

	/**
	* Cache and return users who have played the arcade based on viewing users permission
	*/
	function obtain_arcade_users()
	{
		global $db, $arcade_config;

		// This is used to allow the user to see every user in a drop
		// so they can select one to see his/her scores...
		$sql_array = array(
			'SELECT'		=> 'u.user_id, u.username, u.username_clean',
			'FROM'			=> array(
				USERS_TABLE	=> 'u',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
					'ON'	=> 'u.user_id = s.user_id'
				),
				array(
					'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
					'ON'	=> 's.game_id = g.game_id'
				),
			),
			'WHERE'			=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
				AND u.user_id = s.user_id
				AND s.game_id = g.game_id
				AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
			'ORDER_BY'		=> 'u.username_clean ASC',
		);

		$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
		$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));

		$arcade_users = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$arcade_users[$row['user_id']] = array(
				'username' => $row['username']
			);
		}
		$db->sql_freeresult($result);

		return $arcade_users;
	}

	function obtain_report_games()
	{
		global $db, $arcade_config;

		$sql = 'SELECT game_id FROM ' . ARCADE_REPORTS_TABLE;
		$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));
		$report_gameids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$report_gameids[] = $row['game_id'];
		}
		$db->sql_freeresult($result);

		return array_unique($report_gameids);
	}

	/**
	* Cache and return the latest highscores based on viewing users permission
	*/
	function obtain_latest_highscores($limit)
	{
		global $db, $arcade_config;

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_width, g.game_height, g.game_highuser, g.game_highscore, g.game_highdate, g.cat_id,
							u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'g.game_highuser = u.user_id'
				),
			),
			'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
				AND g.game_highuser = u.user_id',
			'ORDER_BY'	=> 'g.game_highdate DESC'
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($sql, $limit, 0, intval($arcade_config['cache_time'] * 3600));

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}

	function active_check($mode, &$active, &$title, $cat_id = false)
	{
		global $user, $auth, $auth_arcade, $arcade_config;

		$url_disabled = false;
		if ($cat_id)
		{
			$cat_age = $this->get_cat_field($cat_id, 'cat_age');
			$founder_exemption = ($arcade_config['founder_exempt'] && $user->data['user_type'] == USER_FOUNDER) ? true : false;
			$user_age = $this->user_age();
			$url_disabled = ($cat_age && !$founder_exemption && ($user_age === '' || ($user_age < $cat_age))) ? true : false;
		}

		switch ($mode)
		{
			case 'play':
				$active	= (!$url_disabled && $cat_id && $this->get_cat_field($cat_id, 'cat_status') == ITEM_UNLOCKED && $auth_arcade->acl_get('c_play', $cat_id)) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'ARCADE_CLICK_PLAY' : $title) : '';
			break;

			case 'stats':
			case 'challenge':
				$active	= ($auth_arcade->acl_get('u_arcade_viewstats')) ? true : false;
				$title	= ($active) ? (($title == 'x') ? (($mode == 'stats') ? 'ARCADE_GAME_STATS' : 'CHALLENGE_GAME_STATS') : $title) : '';
			break;

			case 'download':
				$active	= ($cat_id && $auth_arcade->acl_get('u_arcade_download') && $auth_arcade->acl_get('c_download', $cat_id)) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'ARCADE_DOWNLOAD_GAME' : $title) : '';
			break;

			case 'icon':
				$active	= (!$url_disabled && $cat_id && $this->get_cat_field($cat_id, 'cat_status') == ITEM_UNLOCKED && $auth_arcade->acl_get('u_arcade_popup') && $auth_arcade->acl_get('c_play', $cat_id)) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'ARCADE_PLAYING_NEW_WINDOW' : $title) : '';
			break;

			case 'link':
				$active	= (!$url_disabled && $cat_id && $this->get_cat_field($cat_id, 'cat_status') == ITEM_UNLOCKED && $auth_arcade->acl_get('u_arcade_popup') && $auth_arcade->acl_get('c_play', $cat_id)) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'ARCADE_PLAYING_NEW_WINDOW' : $title) : '';
			break;

			case 'arcade_user':
			case 'challenge_user':
				$active	= ($auth_arcade->acl_get('u_arcade_viewstats')) ? true : false;
				$title	= ($active) ? (($title == 'x') ? (($mode == 'arcade_user') ? 'ARCADE_USER_STATS' : 'CHALLENGE_USER_STATS') : $title) : '';
			break;

			case 'profile_user':
				$active	= ($auth->acl_get('u_viewprofile')) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'VIEWING_MEMBER_PROFILE' : $title) : '';
			break;

			case 'pm_user':
				$active	= ($auth->acl_get('u_sendpm')) ? true : false;
				$title	= ($active) ? (($title == 'x') ? 'SEND_PRIVATE_MESSAGE' : $title) : '';
			break;

			default:
				$title	= '';
				$active	= false;
			break;
		}

		$title = (!empty($user->lang[$title])) ? $user->lang[$title] : $title;
	}

	/**
	* Cache and return the games stats based on viewing users permission
	*/
	function obtain_data($action, $limit = 0, $type = 'most', $cat_id = false, $cache_ttl = true, $extra_hkey = '', $extra_limit = 0)
	{
		$limit = intval($limit);

		if (!$limit)
		{
			return;
		}

		global $db, $user, $template, $auth_arcade, $arcade_config, $arcade_cache;

		if (!in_array($type, array('most', 'least')))
		{
			$type = 'most';
		}

		switch ($type)
		{
			case 'most':
				$order = 'DESC';
			break;

			default:
				$order = 'ASC';
			break;
		}

		$count		= 0;
		$cat_stats	= ($cat_id) ? "AND g.cat_id = $cat_id" : '';
		$cache_ttl	= ($cache_ttl) ? intval($arcade_config['cache_time'] * 3600) : 0;
		$html_key	= ($extra_hkey !== '') ? $extra_hkey : '';

		switch ($action)
		{
			case 'leaders':
				$limit = intval(($arcade_config['arcade_leaders'] > $arcade_config['arcade_leaders_header']) ? $arcade_config['arcade_leaders'] : $arcade_config['arcade_leaders_header']);
				$leaders = $arcade_cache->obtain_arcade_leaders($limit, $cat_id);
				$real_leaders_count = sizeof($leaders);
				$leaders_count = ($arcade_config['arcade_leaders'] > $real_leaders_count) ? $real_leaders_count : $arcade_config['arcade_leaders'];
			case 'challenge_leaders':
				$mode = ($action == 'leaders') ? 'arcade' : 'challenge';
				$rank = $actual_rank = $previous_score = 0;

				if ($action == 'challenge_leaders')
				{
					$limit = intval(($arcade_config['challenge_leaders'] > $arcade_config['challenge_leaders_header']) ? $arcade_config['challenge_leaders'] : $arcade_config['challenge_leaders_header']);
					$leaders = $arcade_cache->obtain_challenge_leaders($limit);
					$real_leaders_count = sizeof($leaders);
					$leaders_count = ($arcade_config['challenge_leaders'] > $real_leaders_count) ? $real_leaders_count : $arcade_config['challenge_leaders'];
				}

				if ($extra_limit > 0)
				{
					$leaders_count = ($extra_limit > $leaders_count) ? $leaders_count : $extra_limit;
				}

				for ($i = 0, $leaders_count; $i < $leaders_count; $i++)
				{
					// This code is used to calculate the actual rank.
					// For example if there are ties...
					$actual_rank++;
					if ($previous_score != $leaders[$i]['total_wins'])
					{
						$rank = $actual_rank;
					}
					$previous_score = $leaders[$i]['total_wins'];

					$total_wins = $this->number_format($leaders[$i]['total_wins']);
					$username =  get_arcade_user_name('full', $leaders[$i]['user_id'], $leaders[$i]['username'], $leaders[$i]['user_colour'], $mode, 'x');

					$template->assign_block_vars("{$html_key}leaders_users", array(
						'RANK'				=> $rank,
						'AVATAR'			=> ($this->optionget('view_avatars')) ? get_arcade_user_avatar($leaders[$i]['user_avatar'], $leaders[$i]['user_avatar_type'], 20, 20, 'x', $mode, $leaders[$i]['user_id']) : false,
						'USER_WINS'			=> sprintf($user->lang['ARCADE_STATS_WIN' . (($leaders[$i]['total_wins'] > 1) ? 'S' : '')], $username, "($total_wins)"),
						'USER_WINS_TITLE'	=> sprintf($user->lang['ARCADE_STATS_WIN' . (($leaders[$i]['total_wins'] > 1) ? 'S' : '')], $leaders[$i]['username'], "($total_wins)"),
						'USERNAME'			=> $username,
						'TOTAL_WINS'		=> $total_wins,
					));
				}
			break;

			case 'longest_highscores':
				$rank = $actual_rank = $previous_time = 0;

				$sql_array = array(
					'SELECT'	=> 'g.game_id, g.game_name, g.game_highscore, g.game_highdate, g.game_width, g.game_height, g.cat_id, u.user_id, u.username, u.user_colour',
					'FROM'		=> array(
						ARCADE_GAMES_TABLE	=> 'g',
					),
					'LEFT_JOIN'	=> array(
						array(
							'FROM'	=> array(USERS_TABLE => 'u'),
							'ON'	=> 'g.game_highuser = u.user_id'
						),
					),
					'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
						AND g.game_highuser = u.user_id
						AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . "
						$cat_stats",
					'ORDER_BY'	=> 'g.game_highdate ASC'
				);

				$sql = $db->sql_build_query('SELECT', $sql_array);
				$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
				while ($row = $db->sql_fetchrow($result))
				{
					if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
					{
						break;
					}

					// This code is used to calculate the actual rank.
					// For example if there are ties...
					$actual_rank++;

					if ($previous_time != $row['game_highdate'])
					{
						$rank = $actual_rank;
					}

					$previous_time	= $row['game_highdate'];
					$time_held		= (time() - $row['game_highdate']);
					$time_held		= $this->time_format($time_held);

					$template->assign_block_vars("{$html_key}record_row", array(
						'GAME_POPUP'		=> ($this->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $row['cat_id'], $row['game_id'], $row['game_width'], $row['game_height'], 'x') : false,
						'GAME_NAME'			=> get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x'),
						'USERNAME'			=> get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'arcade', 'x'),
						'HIGHSCORE'			=> (!$this->hidden_score($row['game_id'], $row['user_id'])) ? $this->number_format($row['game_highscore']) : $user->lang['HIDDEN'],
						'HIGHSCORE_DATE'	=> $user->format_date($row['game_highdate']),
						'RANK'				=> $rank,
						'TIME_HELD'			=> $time_held
					));

					$count++;
				}
				$db->sql_freeresult($result);
			break;

			case 'challenge_popular_games':
					$sql = 'SELECT COUNT(g.game_id) as game_plays, g.game_id, g.game_name, g.game_image, g.game_width, g.game_height, g.cat_id
							FROM ' . ARCADE_GAMES_TABLE . ' g, ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' c
							WHERE g.game_id = c.game_id
							AND c.champ_close = ' . CHALLENGE_CLOSE . '
							AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . '
							GROUP BY g.game_id, g.game_name, g.game_image, g.game_width, g.game_height, g.cat_id
							ORDER BY game_plays ' . $order;
			case 'popular_games':
				if ($action == 'popular_games')
				{
					$sql = 'SELECT g.game_id, g.game_name, g.game_image, g.game_plays, g.game_width, g.game_height, g.cat_id
							FROM ' . ARCADE_GAMES_TABLE . ' g
							WHERE ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . '
							' . $cat_stats . '
							ORDER BY g.game_plays ' . $order;
				}

				$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
				$html_key .= ($type == 'most') ? 'pop_games' : 'lpop_games';
				while ($row = $db->sql_fetchrow($result))
				{
					if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
					{
						break;
					}

					$template->assign_block_vars($html_key, array(
						'GAME_IMAGE'	=> ($this->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 20, 20, 'x', 'play', $row['cat_id'], $row['game_id']) : false,
						'GAME_POPUP'	=> ($this->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $row['cat_id'], $row['game_id'], $row['game_width'], $row['game_height'], 'x') : false,
						'GAME_DATA'		=> sprintf($user->lang['ARCADE_STATS_PLAY' . (($row['game_plays'] > 1) ? 'S' : '')], get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x'), $this->number_format($row['game_plays'])),

						'GAME_NAME'		=> get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x'),
						'GAME_PLAYS'	=> $this->number_format($row['game_plays'])
					));

					$count++;
				}
				$db->sql_freeresult($result);
			break;

			case 'downloaded_games':
				if (!$user->data['is_bot'] && $auth_arcade->acl_get('u_arcade_download') && $auth_arcade->acl_getc_global('c_download'))
				{
					$sql = 'SELECT g.game_id, g.game_image, g.game_name, g.game_installdate, g.game_download_total, g.cat_id
							FROM ' . ARCADE_GAMES_TABLE . ' g
							WHERE ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . '
							' . $cat_stats . '
							ORDER BY g.game_download_total ' . $order;
					$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
					$html_key .= ($type == 'most') ? 'downloaded_games' : 'ldownloaded_games';
					while ($row = $db->sql_fetchrow($result))
					{
						if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
						{
							break;
						}

						$template->assign_block_vars($html_key, array(
							'GAME_IMAGE'	=> ($this->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 20, 20, 'x', 'download', $row['cat_id'], $row['game_id']) : false,
							'GAME_DATA'		=> sprintf($user->lang['ARCADE_GAME_DOWNLOAD_TOTAL' . (($row['game_download_total'] > 1) ? 'S' : '')], get_arcade_game_name($row['game_name'], true, 'download', $row['cat_id'], $row['game_id'], 'x'), $this->number_format($row['game_download_total']))
						));

						$count++;
					}
					$db->sql_freeresult($result);
				}
			break;

			case 'top_games':
				$sql = 'SELECT COUNT(cc.game_id) as total_played, g.game_id, g.game_image, g.game_name, g.game_width, g.game_height, g.cat_id
						FROM ' . ARCADE_GAMES_TABLE . ' g
						LEFT JOIN ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' cc ON g.game_id = cc.game_id
						WHERE cc.champ_close = ' . CHALLENGE_CLOSE . '
						AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true) . '
						AND g.game_id = cc.game_id
						GROUP BY g.game_id, g.game_image, g.game_name, g.game_width, g.game_height, g.cat_id
						ORDER BY total_played DESC';
			case 'newest_games':
				if ($action == 'newest_games')
				{
					$sql = 'SELECT game_id, game_image, game_name, game_width, game_height, game_installdate, cat_id
							FROM ' . ARCADE_GAMES_TABLE . '
							WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
							ORDER BY game_installdate DESC';
				}

				$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
				while ($row = $db->sql_fetchrow($result))
				{
					if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
					{
						break;
					}

					$template->assign_block_vars("{$html_key}header_games", array(
						'GAME_IMAGE'	=> ($this->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 20, 20, 'x', 'play', $row['cat_id'], $row['game_id']) : false,
						'GAME_POPUP'	=> ($this->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $row['cat_id'], $row['game_id'], $row['game_width'], $row['game_height'], 'x') : false,
						'GAME_NAME'		=> get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x'),
						'GAME_PLAYED'	=> ($action == 'top_games') ? $this->number_format($row['total_played']) : false
					));

					$count++;
				}
				$db->sql_freeresult($result);
			break;

			case 'challenge_played_users':
					$sql = 'SELECT COUNT(c.champ_id) as total_plays, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
							FROM ' . USERS_TABLE . ' u, ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' c
							WHERE u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
							AND c.champ_close = ' . CHALLENGE_CLOSE . '
							AND (u.user_id = c.champ_challenger_id OR u.user_id = c.champ_opponent_id)
							GROUP BY u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
							ORDER BY total_plays ' . $order;
			case 'played_users':
				if ($action == 'played_users')
				{
					$sql = 'SELECT SUM(p.total_plays) as total_plays, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
							FROM ' . USERS_TABLE . ' u, ' . ARCADE_PLAYS_TABLE . ' p' . (($cat_stats !== '') ? ', ' . ARCADE_GAMES_TABLE . ' g' : '') . '
							WHERE u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
							AND u.user_id = p.user_id
							' . (($cat_stats !== '') ? "AND p.game_id = g.game_id $cat_stats" : '') . '
							GROUP BY u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
							ORDER BY total_plays ' . $order;
				}

				$mode = ($action == 'played_users') ? 'arcade' : 'challenge';
				$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
				$html_key .= ($type == 'most') ? 'pop_users' : 'lpop_users';

				while ($row = $db->sql_fetchrow($result))
				{
					if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
					{
						break;
					}

					$username = get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], $mode, 'x');

					$template->assign_block_vars($html_key, array(
						'AVATAR'		=> ($this->optionget('view_avatars')) ? get_arcade_user_avatar($row['user_avatar'], $row['user_avatar_type'], 20, 20, 'x', $mode, $row['user_id']) : false,
						'USER_DATA'		=> sprintf($user->lang['ARCADE_STATS_PLAY' . (($row['total_plays'] > 1) ? 'S' : '')], $username, $this->number_format($row['total_plays'])),
						'USER_NAME'		=> $username,
						'USER_PLAYS'	=> $this->number_format($row['total_plays'])
					));

					$count++;
				}
				$db->sql_freeresult($result);
			break;

			case 'games_jackpot':
				if ($this->points['show'])
				{
					$sql = 'SELECT game_id, game_name, game_image, game_jackpot, game_width, game_height, cat_id
							FROM ' . ARCADE_GAMES_TABLE . '
							WHERE game_jackpot > 0
							AND ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
							AND ' . $db->sql_in_set('game_id', $this->get_use_jackpot(true), true, true) . '
							ORDER BY game_jackpot ' . $order;
					$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
					$html_key .= ($type == 'most') ? 'games_jackpot' : 'lgames_jackpot';
					while ($row = $db->sql_fetchrow($result))
					{
						if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
						{
							break;
						}

						$template->assign_block_vars($html_key, array(
							'GAME_POPUP'	=> ($this->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $row['cat_id'], $row['game_id'], $row['game_width'], $row['game_height'], 'x') : false,
							'GAME_IMAGE'	=> ($this->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 20, 20, 'x', 'play', $row['cat_id'], $row['game_id']) : false,
							'GAME_NAME'		=> get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x'),
							'GAME_JACKPOT'	=> $this->number_format($row['game_jackpot']),
						));

						$count++;
					}
					$db->sql_freeresult($result);
				}
			break;

			case 'users_playtime':
				$sql = 'SELECT SUM(s.total_time) AS total_time, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
					FROM ' . USERS_TABLE . ' u
					LEFT JOIN ' . ARCADE_PLAYS_TABLE . ' s ON u.user_id = s.user_id
					WHERE u.user_id = s.user_id
					AND u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
					GROUP BY u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type
					ORDER BY total_time ' . $order;
				$result = $db->sql_query_limit($sql, $limit, 0, $cache_ttl);
				$html_key .= ($type == 'most') ? 'playtime' : 'lplaytime';
				while ($row = $db->sql_fetchrow($result))
				{
					if ($cache_ttl > 0 && $extra_limit > 0 && $count == $extra_limit)
					{
						break;
					}

					$template->assign_block_vars($html_key, array(
						'AVATAR'	=> ($this->optionget('view_avatars')) ? get_arcade_user_avatar($row['user_avatar'], $row['user_avatar_type'], 20, 20, 'x', 'arcade', $row['user_id']) : false,
						'USERNAME'	=> get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'arcade', 'x'),
						'PLAYTIME'	=> $this->number_format(round($row['total_time']/3600, 2)),
					));

					$count++;
				}
				$db->sql_freeresult($result);
			break;
		}
	}

	function get_use_jackpot($empty = false)
	{
		$game_ids = array();

		if ($this->points['show'])
		{
			foreach ($this->games as $game_id => $row)
			{
				$row['cat_use_jackpot'] = $this->get_cat_field($row['cat_id'], 'cat_use_jackpot');

				if (($empty && !$this->use_jackpot($row)) || (!$empty && $this->use_jackpot($row)))
				{
					$game_ids[] = $game_id;
				}
			}
		}

		return $game_ids;
	}

	function user_online($user_id)
	{
		global $db, $user, $config, $auth, $template;

		$online = false;

		if ($config['load_onlinetrack'])
		{
			$sql = 'SELECT MAX(session_time) AS session_time, MIN(session_viewonline) AS session_viewonline
					FROM ' . SESSIONS_TABLE . '
					WHERE session_user_id = ' . (int) $user_id;
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$session_time = (isset($row['session_time'])) ? $row['session_time'] : 0;
			$session_viewonline = (isset($row['session_viewonline'])) ? $row['session_viewonline'] : 0;
			unset($row);

			$update_time = $config['load_online_time'] * 60;
			$online = (time() - $update_time < $session_time && ((isset($session_viewonline) && $session_viewonline) || $auth->acl_get('u_viewonline'))) ? true : false;
		}

		$template->assign_vars(array(
			'ONLINE_IMG'	=> (!$config['load_onlinetrack']) ? '' : (($online) ? $user->img('icon_user_online', 'ONLINE') : $user->img('icon_user_offline', 'OFFLINE')),
			'S_ONLINE'		=> ($config['load_onlinetrack'] && $online) ? true : false
		));
	}

	function obtain_games()
	{
		global $db, $arcade_config;

		$sql = 'SELECT game_id, game_name, game_name_clean, game_swf, game_scorevar, game_highuser, game_highdate, game_use_jackpot, cat_id
				FROM ' . ARCADE_GAMES_TABLE . '
				WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
				ORDER BY game_name_clean ASC';
		$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));

		$games = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$games[$row['game_id']] = array(
				'game_name'			=> $row['game_name'],
				'game_swf'			=> $row['game_swf'],
				'game_scorevar'		=> $row['game_scorevar'],
				'game_highuser'		=> $row['game_highuser'],
				'game_highdate'		=> $row['game_highdate'],
				'game_use_jackpot'	=> $row['game_use_jackpot'],
				'cat_id'			=> $row['cat_id']
			);
		}
		$db->sql_freeresult($result);

		return $games;
	}

	/**
	* Initialize the points system integration
	*/
	function init_points()
	{
		global $config, $db, $phpbb_root_path, $phpEx, $arcade_config;

		$this->points = array(
			'installed'	=> false,
			'show'		=> false,
			'type'		=> false,
			'enabled'	=> true,
			'name'		=> '',
			'total'		=> 0,
		);

		if (defined('IN_ULTIMATE_POINTS') || defined('IN_JV_POINTS_SYSTEM'))
		{
			$this->points['type'] = (defined('IN_ULTIMATE_POINTS')) ? ARCADE_ULTIMATE_POINTS_SYSTEM : ARCADE_JV_POINTS_SYSTEM;
			$this->points['installed'] = true;
			$this->points['enabled'] = $config['points_enable'];
			define('USER_POINTS', 'user_points');
		}
		else if (file_exists($phpbb_root_path . 'includes/mods/functions_points.' . $phpEx))
		{
			if (!function_exists('add_points'))
			{
				include($phpbb_root_path . 'includes/mods/functions_points.' . $phpEx);
			}

			if (!function_exists('set_bank'))
			{
				$this->points['type'] = ARCADE_SIMPLE_POINTS_SYSTEM;
				$this->points['installed'] = true;
				$this->points['enabled'] = $config['points_enable'];
				define('USER_POINTS', 'user_points');
			}
			else
			{
				$this->points['type'] = ARCADE_POINTS_SYSTEM;
				$this->points['installed'] = true;

				$sql = 'SELECT points_enable FROM ' . POINTS_CONFIG_TABLE;
				$result = $db->sql_query($sql);
				$this->points['enabled'] = $db->sql_fetchfield('points_enable');
				$db->sql_freeresult($result);
			}
		}
		else if (file_exists($phpbb_root_path . 'includes/mods/cash/cash_class.' . $phpEx))
		{
			$this->points['type'] = ARCADE_CASH_MOD;
			$this->points['installed'] = true;
		}
		else if (file_exists($phpbb_root_path . 'includes/functions_points.' . $phpEx))
		{
			$this->points['type'] = ARCADE_POINTS_MOD;
			$this->points['installed'] = true;
			$this->points['enabled'] = $config['points_enable'];
			define('USER_POINTS', 'user_points');
		}

		$this->points['show'] = ($this->points['installed'] && $this->points['enabled'] && $arcade_config['use_points']) ? true : false;

		if ($this->points['show'])
		{
			$points_array = $this->get_points();
			$this->points['name'] = $points_array['name'];
			$this->points['total'] = $points_array['total'];
		}
	}

	/**
	* Get points total and points name for a specified user
	* If no user is specified get the data for the current user
	*/
	function get_points($user_id = false)
	{
		global $user, $db, $config, $arcade_config;

		$return = array(
			'total'	=> 0,
			'name'	=> '',
		);

		if ($this->points['type'] == ARCADE_CASH_MOD)
		{
			$user_id = ($user_id) ? $user_id : $user->data['user_id'];

			$sql = $db->sql_build_query('SELECT', array(
				'SELECT'	=> 'c.cash_name, ca.cash_amt',
				'FROM'		=> array(
					CASH_TABLE			=> 'c',
					CASH_AMOUNT_TABLE	=> 'ca',
				),
				'WHERE'		=> 'ca.user_id = ' . (int) $user_id . ' AND c.cash_id = ' . (int) $arcade_config['cm_currency_id'] . ' AND ca.cash_id = c.cash_id',
			));

			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$return = array(
				'total'	=> $row['cash_amt'],
				'name'	=> $row['cash_name'],
			);
		}
		else
		{
			if (in_array($this->points['type'], array(ARCADE_SIMPLE_POINTS_SYSTEM, ARCADE_ULTIMATE_POINTS_SYSTEM, ARCADE_POINTS_MOD)))
			{
				$return['name'] = $config['points_name'];
			}
			else if ($this->points['type'] == ARCADE_JV_POINTS_SYSTEM)
			{
				$return['name'] = $user->lang['JV_POINTS'];
			}
			else
			{
				$sql = 'SELECT points_name
						FROM ' . POINTS_CONFIG_TABLE;
				$result = $db->sql_query($sql);
				$return['name'] = $db->sql_fetchfield('points_name');
				$db->sql_freeresult($result);
			}

			if (!$user_id)
			{
				$return['total'] = $user->data[USER_POINTS];
			}
			else
			{
				$sql = 'SELECT ' . USER_POINTS . '
						FROM ' . USERS_TABLE . '
						WHERE user_id = ' . (int) $user_id;
				$result = $db->sql_query($sql);
				$return['total'] = $db->sql_fetchfield(USER_POINTS);
				$db->sql_freeresult($result);
			}
		}

		return $return;
	}

	function nl2br_plus($text)
	{
		return nl2br(str_replace(array("\n", "&quot;", '"'), array("<br />", "'", "'"), $text));
	}

	/**
	* Function to add or subtact points from a user
	*/
	function set_points($mode, $user_id, $amount, $update = true)
	{
		global $user, $db, $arcade_config;

		// Return true if the amount is 0 or free (-1)
		if ($amount <= 0)
		{
			return true;
		}

		$result = false;
		$mode = strtolower($mode);
		switch ($mode)
		{
			case 'add':
				if ($this->points['type'] == ARCADE_CASH_MOD)
				{
					global $cash;
					$result = $cash->give_cash($user_id, $amount, $arcade_config['cm_currency_id']);
				}
				else
				{
					$sql = 'UPDATE ' . USERS_TABLE . '
							SET ' . USER_POINTS . ' = ' . USER_POINTS . ' + ' . $amount . '
							WHERE user_id = ' . (int) $user_id;
					$db->sql_query($sql);

					$result = true;
				}

			break;

			case 'subtract':
				if ($this->points['type'] == ARCADE_CASH_MOD)
				{
					global $cash;
					$result = $cash->take_cash($user_id, $amount, $arcade_config['cm_currency_id']);
				}
				else
				{
					// The user does not have enough points
					if ($user->data[USER_POINTS] < $amount)
					{
						$result = false;
					}
					else
					{
						$sql = 'UPDATE ' . USERS_TABLE . '
								SET ' . USER_POINTS . ' = ' . USER_POINTS . ' - ' . $amount . '
								WHERE user_id = ' . (int) $user_id;
						$db->sql_query($sql);

						$result = true;
					}
				}

			break;

			default:
			break;
		}

		// If setting points was successful, update the users current total for display as well
		if ($result && $user->data['user_id'] == $user_id)
		{
			if ($mode == 'add')
			{
				$this->points['total'] = $this->points['total'] + $amount;
			}
			else
			{
				$this->points['total'] = $this->points['total'] - $amount;
			}

			if ($update && defined('USER_POINTS'))
			{
				$user->data[USER_POINTS] = $this->points['total'];
			}
		}

		return $result;
	}

	/**
	* Return download cost infomation for a game
	*/
	function get_download_cost($data)
	{
		global $arcade_config;

		$cost = 0;
		$game_download_cost = (float) $data['game_download_cost'];
		$cat_download_cost = (float) $data['cat_download_cost'];

		if (!empty($game_download_cost))
		{
			$cost = $game_download_cost;
		}
		else if (!empty($cat_download_cost))
		{
			$cost = $cat_download_cost;
		}
		else
		{
			$cost = (float) $arcade_config['game_download_cost'];
		}

		return $cost;
	}

	/**
	* Return cost infomation for a game
	*/
	function get_cost($data)
	{
		global $arcade_config;

		$cost = 0;
		$game_cost = (float) $data['game_cost'];
		$cat_cost = (float) $data['cat_cost'];

		if (!empty($game_cost))
		{
			$cost = $game_cost;
		}
		else if (!empty($cat_cost))
		{
			$cost = $cat_cost;
		}
		else
		{
			$cost = (float) $arcade_config['game_cost'];
		}

		return $cost;
	}

	/**
	* Return reward infomation for a game
	*/
	function get_reward($data)
	{
		global $db, $arcade_config;

		$reward = 0;
		if ($this->use_jackpot($data))
		{
			$reward = (float) $data['game_jackpot'];
		}
		else
		{
			$game_reward = (float) $data['game_reward'];
			$cat_reward = (float) $data['cat_reward'];

			if (!empty($game_reward))
			{
				$reward = $game_reward;
			}
			else if (!empty($cat_reward))
			{
				$reward = $cat_reward;
			}
			else
			{
				$reward = (float) $arcade_config['game_reward'];
			}
		}

		return $reward;
	}

	/**
	* Find out if the game should be using the jackpot setting
	*/
	function use_jackpot($data)
	{
		global $arcade_config;

		$result = false;

		if (!empty($data['game_use_jackpot']))
		{
			$result = true;
		}
		else if (!empty($data['cat_use_jackpot']))
		{
			$result = true;
		}
		else
		{
			$result = $arcade_config['use_jackpot'];
		}

		return $result;
	}

	/**
	* Function to add or clear jackpot
	* $mode can be 'add' or 'clear'
	*/
	function set_jackpot($mode, $data)
	{
		global $db, $arcade_config;

		$mode = strtolower($mode);
		switch ($mode)
		{
			case 'add':
				$cost = $this->get_cost($data);

				// Return if the amount is 0 or free (-1)
				if ($cost <= 0)
				{
					return false;
				}

				// Make sure jackpot is not less than the minimum
				$jackpot = ($data['game_jackpot'] < $arcade_config['jackpot_Minimum']) ? $arcade_config['jackpot_Minimum'] : $data['game_jackpot'];
				// Add to jackpot
				$jackpot += $cost;
				// Make sure jackpot is not more than the maximum if a maximum is set
				if ($arcade_config['jackpot_Maximum'] && $jackpot > $arcade_config['jackpot_Maximum'])
				{
					$jackpot = $arcade_config['jackpot_Maximum'];
				}

				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_jackpot = ' . (float) $jackpot . '
						WHERE game_id = ' . (int) $data['game_id'];
				$db->sql_query($sql);

				return $jackpot;
			break;

			case 'clear':
				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_jackpot = ' . (float) $arcade_config['jackpot_Minimum'] . '
						WHERE game_id = ' . (int) $data['game_id'];
				$db->sql_query($sql);
			break;

			default:
			break;
		}
		return false;
	}

	/**
	* Returns list of categories to exclude from search based
	* on ACP settings and whether or not the category has an age limit
	* and/or is password protected
	*/
	function games_search_protection()
	{
		global $db, $user, $arcade_config;

		$cat_ids = array();

		// Bots exemption age limit
		$bots_exemption	= (!$user->data['is_registered'] && $user->data['is_bot']) ? true : false;
		$pass_check		= (($arcade_config['search_filter_password'] == ARCADE_CHECK_EVERYONE) || ($arcade_config['search_filter_password'] == ARCADE_CHECK_USER_NORMAL && $user->data['user_type'] != USER_FOUNDER)) ? true : false;
		$age_check		= (!$bots_exemption && (($arcade_config['search_filter_age'] == ARCADE_CHECK_EVERYONE) || ($arcade_config['search_filter_age'] == ARCADE_CHECK_USER_NORMAL && $user->data['user_type'] != USER_FOUNDER))) ? true : false;

		if (!$pass_check && !$age_check)
		{
			return $cat_ids;
		}

		$sql = 'SELECT c.cat_id, c.cat_age, c.cat_password, a.user_id
				FROM ' . ARCADE_CATS_TABLE   . ' c
				LEFT JOIN ' . ARCADE_ACCESS_TABLE . " a ON (c.cat_id = a.cat_id
				AND a.session_id = '" . $db->sql_escape($user->session_id) . "')";
		$result = $db->sql_query($sql);

		$user_age = $this->user_age();

		while($row = $db->sql_fetchrow($result))
		{
			if (($user_age === '' && $row['cat_age'] && $age_check) || ($row['cat_age'] && $user_age < $row['cat_age'] && $age_check) || ($row['cat_password'] && $row['user_id'] != $user->data['user_id'] && $pass_check))
			{
				// forbidden categories
				$cat_ids[] = $row['cat_id'];
			}
		}
		$db->sql_freeresult($result);

		return $cat_ids;
	}

	function user_age($user_birthday = false)
	{
		global $user;

		if ($user_birthday === false)
		{
			$user_birthday = $user->data['user_birthday'];
		}

		$age = '';

		if ($user_birthday)
		{
			list($bday_day, $bday_month, $bday_year) = array_map('intval', explode('-', $user_birthday));

			if ($bday_year)
			{
				$now = phpbb_gmgetdate(time() + $user->timezone + $user->dst);

				$diff = $now['mon'] - $bday_month;
				if ($diff == 0)
				{
					$diff = ($now['mday'] - $bday_day < 0) ? 1 : 0;
				}
				else
				{
					$diff = ($diff < 0) ? 1 : 0;
				}

				$age = (int) ($now['year'] - $bday_year - $diff);
			}
		}

		return $age;
	}

	/**
	* Verify a users age against category age limit
	*/
	function verify_age($type, $age)
	{
		global $user, $arcade_config;

		// Founder check and Bots exemption age limit
		if (($arcade_config['founder_exempt'] && $user->data['user_type'] == USER_FOUNDER) || (!$user->data['is_registered'] && $user->data['is_bot']))
		{
			return;
		}

		if (!$user->data['is_registered'])
		{
			$message = ($type == 'cat') ? 'ARCADE_BLOCKED_CAT_VIEW' : 'ARCADE_BLOCKED_GAME_PLAY';
			trigger_error($message);
		}

		$user_age = $this->user_age();

		if ($user_age === '')
		{
			$message = ($type == 'cat') ? 'ARCADE_BLOCKED_NO_AGE_CAT' : 'ARCADE_BLOCKED_NO_AGE_PLAY';
			trigger_error($message);
		}

		if ($user_age < $age)
		{
			$message = ($type == 'cat') ? 'ARCADE_BLOCKED_CAT_AGE' : 'ARCADE_BLOCKED_PLAY_AGE';
			trigger_error(sprintf($user->lang[$message], $age));
		}
	}

	function confirm($lang_ary, $popup, $game_id = false)
	{
		global $user, $template;

		$template->assign_vars(array(
			'S_USE_SIMPLE_HEADER'	=> $popup,

			'MESSAGE_TITLE'			=> $lang_ary['MESSAGE_TITLE'],
			'MESSAGE_TEXT'			=> $lang_ary['MESSAGE_TEXT'],
		));

		if (!empty($lang_ary['CONFIRM_BUTTON1']))
		{
			$template->assign_vars(array(
				'S_RESTART_GAME'		=> true,
				'S_HIDDEN_FIELDS'		=> (!empty($lang_ary['HIDDEN_KEY'])) ? build_hidden_fields($lang_ary['HIDDEN_KEY']) : '',

				'U_ACTION1'				=> $this->url('mode=' . (($popup) ? 'popup' : 'play') . '&amp;g=' . $game_id),
				'U_ACTION2'				=> $this->url('mode=challenge#boxtop'),

				'CONFIRM_BUTTON1'		=> $lang_ary['CONFIRM_BUTTON1'],
				'CONFIRM_BUTTON2'		=> (!empty($lang_ary['CONFIRM_BUTTON2'])) ? $lang_ary['CONFIRM_BUTTON2'] : false
			));
		}

		page_header($lang_ary['PAGE_TITLE'], false);

		$template->set_filenames(array(
			'body' => 'arcade/message_body.html')
		);

		page_footer();
	}

	function challenge_check($gdata, $popup)
	{
		global $db, $user, $auth_arcade;

		if (!isset($_POST['start']))
		{
			$sql = 'SELECT c.challenge_id, c.challenger_id, c.challenger_game_cost, c.opponent_id, c.opponent_game_cost
					FROM ' . ARCADE_CHALLENGE_TABLE . ' c, ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' cc
					WHERE c.challenge_accept = ' . CHALLENGE_ACCEPTED . '
					AND cc.champ_close <> ' . CHALLENGE_CLOSE . '
					AND c.game_id = cc.game_id
					AND c.challenger_id = cc.champ_challenger_id
					AND c.opponent_id = cc.champ_opponent_id
					AND c.game_id = ' . (int) $gdata['game_id'] . '
					AND ((c.challenger_id	= ' . (int) $user->data['user_id'] . ' AND cc.champ_challenger_score = 0 AND cc.champ_challenger_time = 0) OR
						 (c.opponent_id		= ' . (int) $user->data['user_id'] . ' AND cc.champ_opponent_score = 0 AND cc.champ_opponent_time = 0))';
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if ($row)
			{
				add_form_key('challenge_play');
				$column = ($user->data['user_id'] == $row['challenger_id']) ? 'challenger' : 'opponent';

				$game_cost_active = false;
				if ($this->points['show'] && $row[$column . '_game_cost'] > 0)
				{
					$game_cost_active = true;

					$game_cost = $this->get_cost($gdata);

					if ($game_cost > $row[$column . '_game_cost'])
					{
						$row[$column . '_game_cost'] = $game_cost;
					}
				}

				if ($game_cost_active)
				{
					$sql = 'UPDATE ' . ARCADE_CHALLENGE_TABLE . "
							SET {$column}_game_cost = 0
							WHERE challenge_id = " . (int) $row['challenge_id'];
					$db->sql_query($sql);

					$this->set_points('add', $user->data['user_id'], $row[$column . '_game_cost'], false);
				}

				$lang_ary = array(
					'MESSAGE_TITLE'		=> $user->lang['ARCADE_CHALLENGE'],
					'MESSAGE_TEXT'		=> $user->lang['ARCADE_CHALLENGE_PLAYING_GAME'] . (($game_cost_active) ? '<br />' . $user->lang['ARCADE_CHALLENGE_PLAYING_GAME_COST'] : ''),
					'PAGE_TITLE'		=> $user->lang['ARCADE_CHALLENGE'] . ' ' . $user->lang['INFORMATION'],
					'CONFIRM_BUTTON1'	=> $user->lang['ARCADE_PLAY_LINK'],
					'HIDDEN_KEY'		=> array('challenge_play' => true)
				);

				$this->confirm($lang_ary, $popup, $gdata['game_id']);
			}
		}
		else if (!empty($_POST['challenge_play']) && (check_form_key('challenge_play')))
		{
			return;
		}

		if (!$auth_arcade->acl_get('c_challenge_playing_games', $gdata['cat_id']))
		{
			$sql = 'SELECT c.champ_challenger_id, c.champ_opponent_id, g.game_id, g.game_name, g.game_name_clean, g.cat_id
					FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '	c
					LEFT JOIN ' . ARCADE_GAMES_TABLE . '		g ON c.game_id = g.game_id
					LEFT JOIN ' . ARCADE_CATS_TABLE . '			ac ON g.cat_id = ac.cat_id
					WHERE c.champ_close <> ' . CHALLENGE_CLOSE . '
					AND ac.cat_test <> ' . ARCADE_CAT_TEST . '
					AND ac.cat_status = ' . ITEM_UNLOCKED . '
					AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_play'), false, true) . '
					AND ((c.champ_challenger_id = '. (int) $user->data['user_id'] .' AND c.champ_challenger_score = 0 AND c.champ_challenger_time = 0) OR
						(c.champ_opponent_id   = '. (int) $user->data['user_id'] .' AND c.champ_opponent_score = 0 AND c.champ_opponent_time = 0))
					ORDER BY g.game_name_clean ASC';
			$result = $db->sql_query($sql);

			$data = array();
			$game_links = '';

			while ($row = $db->sql_fetchrow($result))
			{
				if (!empty($data[$row['game_id']]))
				{
					continue;
				}

				if ($row['game_id'] == $gdata['game_id'])
				{
					$game_links = '';
					break;
				}

				$data[$row['game_id']] = 1;
				$game_links .= ((!$game_links) ? '' : ', ') . '<a href="' . $this->url('mode=' . (($popup) ? 'popup' : 'play') . '&amp;g=' . $row['game_id']) . '"><b>' . $row['game_name'] . '</b></a>';
			}
			$db->sql_freeresult($result);

			if ($game_links)
			{
				$message  = $user->lang['ARCADE_CHALLENGE_PLAYING_STOP_INFORMATION'] . '<br />';
				$message .= $user->lang['CHALLENGE_ONGOING_CHALLENGE' . ((sizeof($data) > 1) ? 'S' : '')] . ': ';
				$message .= $game_links;

				$lang_ary = array(
					'MESSAGE_TITLE'	=> $user->lang['INFORMATION'],
					'MESSAGE_TEXT'	=> $message,
					'PAGE_TITLE'	=> $user->lang['ARCADE_CHALLENGE'] . ' ' . $user->lang['INFORMATION']
				);

				$this->confirm($lang_ary, $popup);
			}
		}

		if (!isset($_POST['start']) && !$auth_arcade->acl_get('c_challenge_start_game_warning', $gdata['cat_id']))
		{
			$sql = 'SELECT c.game_id
					FROM ' . ARCADE_CHALLENGE_TABLE . ' c
					LEFT JOIN ' . ARCADE_GAMES_TABLE . ' g ON c.game_id = g.game_id
					LEFT JOIN ' . ARCADE_CATS_TABLE . ' ac ON g.cat_id = ac.cat_id
					WHERE c.game_id = g.game_id
					AND ac.cat_test <> ' . ARCADE_CAT_TEST . '
					AND ac.cat_status = ' . ITEM_UNLOCKED . '
					AND ' . $db->sql_in_set('ac.cat_id', $this->get_permissions('c_play'), false, true) . '
					AND c.challenge_accept = ' . CHALLENGE_UNACCEPTED . '
					AND c.opponent_id = ' . (int) $user->data['user_id'];
			$result = $db->sql_query($sql);
			$new_challenge = (int) $db->sql_fetchfield('c.game_id');
			$db->sql_freeresult($result);

			if ($new_challenge)
			{
				$lang_ary = array(
					'MESSAGE_TITLE'		=> $user->lang['INFORMATION'],
					'MESSAGE_TEXT'		=> $user->lang['ARCADE_NEW_CHALLENGE_INFORMATION'],
					'CONFIRM_BUTTON1'	=> $user->lang['ARCADE_PLAY_LINK'],
					'CONFIRM_BUTTON2'	=> $user->lang['CHALLENGE_VIEW'],
					'PAGE_TITLE'		=> $user->lang['ARCADE_CHALLENGE'] . ' ' . $user->lang['INFORMATION']
				);

				$this->confirm($lang_ary, $popup, $gdata['game_id']);
			}
		}
	}

	function challenge($data, $score, $send_pm)
	{
		global $db, $user, $arcade_config, $arcade_cache, $auth_arcade, $phpEx;

		$sql = 'SELECT c.challenge_id, c.challenger_id, c.opponent_id, c.challenge_points, c.challenger_game_cost, c.opponent_game_cost, cc.champ_id, cc.champ_challenger_score, cc.champ_challenger_time, cc.champ_opponent_score, cc.champ_opponent_time
				FROM ' . ARCADE_CHALLENGE_TABLE . ' c
				LEFT JOIN ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' cc ON cc.champ_close <> ' . CHALLENGE_CLOSE . ' AND c.game_id = cc.game_id AND c.challenger_id = cc.champ_challenger_id AND c.opponent_id = cc.champ_opponent_id
				WHERE c.game_id = ' . (int) $data['game_id'] . '
				AND c.challenge_accept = ' . CHALLENGE_ACCEPTED . '
				AND ((cc.champ_challenger_id = '. (int) $user->data['user_id'] .' AND cc.champ_challenger_score = 0 AND cc.champ_challenger_time = 0) OR
					(cc.champ_opponent_id   = '. (int) $user->data['user_id'] .' AND cc.champ_opponent_score = 0 AND cc.champ_opponent_time = 0))';
		$result = $db->sql_query($sql);
		$champs = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$champs[$row['champ_id']] = array(
				'cat_id'				=> $data['cat_id'],
				'challenge_id'			=> $row['challenge_id'],
				'challenger_id'			=> $row['challenger_id'],
				'opponent_id'			=> $row['opponent_id'],
				'challenge_bet'			=> $row['challenge_points'],
				'challenger_score'		=> $row['champ_challenger_score'],
				'challenger_time'		=> $row['champ_challenger_time'],
				'opponent_score'		=> $row['champ_opponent_score'],
				'opponent_time'			=> $row['champ_opponent_time'],
				'challenger_game_cost'	=> $row['challenger_game_cost'],
				'opponent_game_cost'	=> $row['opponent_game_cost']
			);
		}
		$db->sql_freeresult($result);

		if (!sizeof($champs))
		{
			return;
		}

		$delete_challenges = array();
		$total_plays = $total_times = 0;
		$high_score_type = ($data['game_scoretype'] == SCORETYPE_HIGH) ? true : false;

		foreach ($champs as $champ_id => $champ)
		{
			$co_col = ($champ['challenger_id'] == $user->data['user_id']) ? 'challenger' : (($champ['opponent_id'] == $user->data['user_id']) ? 'opponent' : false);

			if (!$co_col)
			{
				continue;
			}

			$my_challenger = ($co_col == 'challenger') ? true : false;
			$other_id = $champ[(($my_challenger) ? 'opponent' : 'challenger') . '_id'];
			$champ["{$co_col}_score"] = (float) $score;
			$champ["{$co_col}_time"] = (int) $data['total_time'];

			if (($champ['opponent_score'] > 0 && $champ['challenger_score'] > 0) || ($champ['challenger_time'] && $champ['opponent_time']))
			{
				$winner = $c_refund = $o_refund = 0;
				$total_plays++;
				$total_times += intval($champ['challenger_time'] + $champ['opponent_time']);

				if ($champ['challenger_score'] == $champ['opponent_score'])
				{
					$challenger_message = $opponent_message = 'CHALLENGE_PM_MESSAGE_FINAL_TIE';

					if ($champ['challenge_bet'] > 0)
					{
						$prize			= true;
						$challenger_bet = $opponent_bet = 'CHALLENGE_MSG_FINAL_BET_TIE';
						$c_refund += $champ['challenge_bet'];
						$o_refund += $champ['challenge_bet'];
					}
				}
				else
				{
					$challenger_win = (($high_score_type && $champ['challenger_score'] > $champ['opponent_score']) || (!$high_score_type && $champ['challenger_score'] < $champ['opponent_score'])) ? true : false;

					$challenger_message	= 'CHALLENGE_PM_MESSAGE_FINAL_' . (($challenger_win) ? 'WINNER' : 'LOSER');
					$opponent_message	= 'CHALLENGE_PM_MESSAGE_FINAL_' . ((!$challenger_win) ? 'WINNER' : 'LOSER');
					$winner				= $champ[(($challenger_win) ? 'challenger' : 'opponent') . '_id'];

					if ($champ['challenge_bet'] > 0)
					{
						$challenger_bet	= 'CHALLENGE_MSG_FINAL_BET_' . (($challenger_win) ? 'WINNER' : 'LOSER');
						$opponent_bet	= 'CHALLENGE_MSG_FINAL_BET_' . ((!$challenger_win) ? 'WINNER' : 'LOSER');
						$prize			= ($champ['challenge_bet'] * 2);
						$c_refund += ($challenger_win) ? $prize : 0;
						$o_refund += (!$challenger_win) ? $prize : 0;
					}
				}

				$c_refund += ($champ['challenger_game_cost'] > 0) ? $champ['challenger_game_cost'] : 0;
				$o_refund += ($champ['opponent_game_cost'] > 0) ? $champ['opponent_game_cost'] : 0;

				if ($this->points['installed'])
				{
					if ($c_refund > 0)
					{
						$this->set_points('add', $champ['challenger_id'], $c_refund);
					}

					if ($o_refund > 0)
					{
						$this->set_points('add', $champ['opponent_id'], $o_refund);
					}
				}

				if ($arcade_config['send_arcade_pm'])
				{
					global $auth_arcade2;

					$other_user = $this->userdata('set_auth', $other_id);

					$auth_arcade2 = new auth_arcade();
					$auth_arcade2->acl($other_user);

					$winner_score	= ($winner) ? (($challenger_win) ? $champ['challenger_score'] : $champ['opponent_score']) : 0;
					$loser_score	= ($winner) ? (($challenger_win) ? $champ['opponent_score'] : $champ['challenger_score']) : 0;

					if (($winner && $winner == $user->data['user_id']) || (!$winner && $my_challenger))
					{
						$winner_username	= ($user->data['user_colour']) ? '[color=#' .  $user->data['user_colour'] . ']' . $user->data['username'] . '[/color]' : $user->data['username'];
						$loser_username		= ($other_user['user_colour']) ? '[color=#' .  $other_user['user_colour'] . ']' . $other_user['username'] . '[/color]' : $other_user['username'];

						$winner_user_link	= '[url=' . generate_board_url() . '/arcade.' . $phpEx . '?mode=view_challenge_stats&amp;u=' . $user->data['user_id'] . ']' . $winner_username . '[/url]';
						$loser_user_link	= '[url=' . generate_board_url() . '/arcade.' . $phpEx . '?mode=view_challenge_stats&amp;u=' . $other_user['user_id'] . ']' . $loser_username . '[/url]';
					}
					else
					{
						$winner_username	= ($other_user['user_colour']) ? '[color=#' .  $other_user['user_colour'] . ']' . $other_user['username'] . '[/color]' : $other_user['username'];
						$loser_username		= ($user->data['user_colour']) ? '[color=#' .  $user->data['user_colour'] . ']' . $user->data['username'] . '[/color]' : $user->data['username'];

						$winner_user_link	= '[url=' . generate_board_url() . '/arcade.' . $phpEx . '?mode=view_challenge_stats&amp;u=' . $other_user['user_id'] . ']' . $winner_username . '[/url]';
						$loser_user_link	= '[url=' . generate_board_url() . '/arcade.' . $phpEx . '?mode=view_challenge_stats&amp;u=' . $user->data['user_id'] . ']' . $loser_username . '[/url]';
					}

					$pm_data = array(
						'PM_SUBJECT'		=> 'CHALLENGE_PM_SUBJECT_FINAL',
						'cat_id'			=> $data['cat_id'],
						'game_id'			=> $data['game_id'],
						'game_name'			=> $data['game_name'],
						'game_image'		=> $data['game_image'],
						'game_width'		=> $data['game_width'],
						'game_height'		=> $data['game_height'],
						'prize'				=> (!empty($prize)) ? $champ['challenge_bet'] : 0
					);

					$challenger_msg = array(
						'PM_MESSAGE'				=> $challenger_message,
						'BET_MSG'					=> (!empty($challenger_bet)) ? $challenger_bet : '',
						'user_id'					=> ($my_challenger) ? $other_user['user_id'] : $user->data['user_id'],
						'user_ip'					=> ($my_challenger) ? $other_user['user_ip'] : $user->ip,
						'username'					=> ($my_challenger) ? $other_user['username'] : $user->data['username'],
						'refund_game_cost'			=> ($this->points['installed'] && ($champ['challenger_game_cost'] > 0)) ? $champ['challenger_game_cost'] : 0,

						'other_user_id'				=> ($my_challenger) ? $user->data['user_id'] : $other_user['user_id'],
						'other_user_lang'			=> ($my_challenger) ? $user->data['user_lang'] : $other_user['user_lang'],
						'other_user_arcade_options'	=> ($my_challenger) ? $user->data['user_arcade_options'] : $other_user['user_arcade_options'],
					);

					$opponent_msg = array(
						'PM_MESSAGE'				=> $opponent_message,
						'BET_MSG'					=> (!empty($opponent_bet)) ? $opponent_bet : '',
						'user_id'					=> ($my_challenger) ? $user->data['user_id'] : $other_user['user_id'],
						'user_ip'					=> ($my_challenger) ? $user->ip : $other_user['user_ip'],
						'username'					=> ($my_challenger) ? $user->data['username'] : $other_user['username'],
						'refund_game_cost'			=> ($this->points['installed'] && ($champ['opponent_game_cost'] > 0)) ? $champ['opponent_game_cost'] : 0,

						'other_user_id'				=> ($my_challenger) ? $other_user['user_id'] : $user->data['user_id'],
						'other_user_lang'			=> ($my_challenger) ? $other_user['user_lang'] : $user->data['user_lang'],
						'other_user_arcade_options'	=> ($my_challenger) ? $other_user['user_arcade_options'] : $user->data['user_arcade_options'],
					);

					if ($winner)
					{
						$pm_data += array(
							'winner_score'		=> $this->number_format($winner_score),
							'loser_score'		=> $this->number_format($loser_score),
							'winner_user_link'	=> $winner_user_link,
							'loser_user_link'	=> $loser_user_link,
							'winner_username'	=> $winner_username,
							'loser_username'	=> $loser_username
						);
					}
					else
					{
						$challenger_msg += array(
							'opponent_user_link'	=> $loser_user_link,
							'opponent_username'		=> $loser_username,
							'user_score'			=> $this->number_format($score)
						);

						$opponent_msg += array(
							'opponent_user_link'	=> $winner_user_link,
							'opponent_username'		=> $winner_username,
							'user_score'			=> $this->number_format($score)
						);
					}

					$ch_pm = ($my_challenger)  ? $this->optionget('arcade_pm') : $this->optionget('arcade_pm', $other_user['user_arcade_options'], true);
					$op_pm = (!$my_challenger) ? $this->optionget('arcade_pm') : $this->optionget('arcade_pm', $other_user['user_arcade_options'], true);

					if ($ch_pm)
					{
						$other		= (!$my_challenger) ? true : false;
						$pm_datas	= array_merge($pm_data, $challenger_msg);
						$view_stats = ($my_challenger) ? $auth_arcade->acl_get('u_arcade_viewstats') : $auth_arcade2->acl_get('u_arcade_viewstats');

						if (!$view_stats)
						{
							if ($winner)
							{
								$pm_datas['winner_user_link']	= $winner_username;
								$pm_datas['loser_user_link']	= $loser_username;
							}
							else
							{
								$pm_datas['opponent_user_link']	= $loser_username;
							}
						}

						$this->send_pm('challenge_result', $pm_datas, $other);
					}

					if ($op_pm)
					{
						$other		= ($my_challenger) ? true : false;
						$pm_datas	= array_merge($pm_data, $opponent_msg);
						$view_stats = (!$my_challenger) ? $auth_arcade->acl_get('u_arcade_viewstats') : $auth_arcade2->acl_get('u_arcade_viewstats');

						if (!$view_stats)
						{
							if ($winner)
							{
								$pm_datas['winner_user_link']	= $winner_username;
								$pm_datas['loser_user_link']	= $loser_username;
							}
							else
							{
								$pm_datas['opponent_user_link']	= $winner_username;
							}
						}

						$this->send_pm('challenge_result', $pm_datas, $other);
					}

					unset($auth_arcade2);
				}

				$sql_ary = array(
					"champ_{$co_col}_time"	=> (int) $data['total_time'],
					"champ_{$co_col}_score"	=> (float) $score,
					'champ_winner'			=> (int) $winner,
					'champ_end_time'		=> time(),
					'champ_close'			=> CHALLENGE_CLOSE,
				);

				$delete_challenges[] = $champ['challenge_id'];
			}
			else
			{
				$sql_ary = array(
					"champ_{$co_col}_time"	=> (int) $data['total_time'],
					"champ_{$co_col}_score"	=> (float) $score
				);
			}

			$sql = 'UPDATE ' . ARCADE_CHALLENGE_CHAMP_TABLE  . '
					SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
					WHERE champ_id = ' . (int) $champ_id;
			$db->sql_query($sql);
		}

		if (sizeof($delete_challenges))
		{
			$sql = 'DELETE FROM ' . ARCADE_CHALLENGE_TABLE . '
					WHERE ' . $db->sql_in_set('challenge_id', $delete_challenges);
			$db->sql_query($sql);

			$this->set_config('challenge_total_plays', intval($arcade_config['challenge_total_plays'] + $total_plays), true);
			$this->set_config('challenge_total_plays_time', intval($arcade_config['challenge_total_plays_time'] + $total_times), true);

			$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);
			$arcade_cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);
			$arcade_cache->destroy('_arcade_challenge_leaders');
			$arcade_cache->destroy('_arcade_challenge_leaders_all');
			$arcade_cache->destroy('_arcade_challenge_users');

			if ($send_pm)
			{
				$this->hidden_users = array();
				$this->obtain_hidden_scores();
			}
		}
	}

	function obtain_hidden_scores()
	{
		global $db, $arcade_config;

		$sql = 'SELECT game_id, challenger_id, opponent_id
				FROM ' . ARCADE_CHALLENGE_TABLE . '
				WHERE challenge_accept = ' . CHALLENGE_ACCEPTED;
		$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));
		while ($row = $db->sql_fetchrow($result))
		{
			$this->hidden_users[$row['game_id']][$row['challenger_id']] = 1;
			$this->hidden_users[$row['game_id']][$row['opponent_id']] = 1;
		}
		$db->sql_freeresult($result);
	}

	function data($key)
	{
		if (!isset($this->data[$key]))
		{
			$this->data[$key] = false;
		}

		return ($this->data[$key]) ? true : false;
	}

	function userdata($mode, $user_id = false, $username = false)
	{
		global $db, $user;

		$userdata = array();
		$one_column = false;
		$arcade_data = false;

		switch ($mode)
		{
			case 'username':
				$one_column = true;
				$columns = 'u.username';
			break;

			case 'user_type':
				$one_column = true;
				$columns = 'u.user_type';
			break;

			case 'full_username':
				$columns = 'u.user_id, u.username, u.user_colour';
			break;

			case 'user_info':
				$columns = 'u.user_type, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height, u.user_rank, u.user_posts';
			break;

			case 'set_auth':
				$arcade_data = true;
				$columns = 'u.user_type, u.user_id, u.username, u.user_colour, u.user_lang, u.user_ip, u.user_options, u.user_permissions, au.user_arcade_options, au.user_arcade_permissions';
			break;

			default:
				$columns = false;
			break;
		}

		if ($columns && ($user_id || $username))
		{
			$left_join = ($arcade_data) ? 'LEFT JOIN ' . ARCADE_USERS_TABLE . ' au ON u.user_id = au.user_id' : '';

			$sql = "SELECT $columns
					FROM " . USERS_TABLE . " u
					$left_join
					WHERE " . (($username) ? "u.username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'" : 'u.user_id = ' . (int) $user_id);
			$result = $db->sql_query($sql);

			if ($one_column)
			{
				$userdata = $db->sql_fetchfield(str_replace(array('u.', 'au.'), '', $columns));
			}
			else
			{
				$userdata = $db->sql_fetchrow($result);
			}

			$db->sql_freeresult($result);

			if ($arcade_data && !isset($userdata['user_arcade_options']))
			{
				set_arcade_userdata($userdata);
			}
		}

		return $userdata;
	}

	/**
	* Get option bit field from user arcade options
	*/
	function optionget($key, $data = false, $other = false)
	{
		global $user;

		$var = ($other) ? $data : $user->data['user_arcade_options'];
		return ($var & 1 << $this->keyoptions[$key]) ? true : false;
	}

	/**
	* Set option bit field for user arcade options
	*/
	function optionset($key, $value, $data = false, $other = false)
	{
		global $user;

		$var = ($other) ? $data : $user->data['user_arcade_options'];

		if ($value && !($var & 1 << $this->keyoptions[$key]))
		{
			$var += 1 << $this->keyoptions[$key];
		}
		else if (!$value && ($var & 1 << $this->keyoptions[$key]))
		{
			$var -= 1 << $this->keyoptions[$key];
		}
		else
		{
			return ($other) ? $var : false;
		}

		if (!$other)
		{
			$user->data['user_arcade_options'] = $var;
			return true;
		}
		else
		{
			return $var;
		}
	}
}

?>