<?php
/**
*
* @package arcade challenge
* @version $Id: class_challenge.php 1970 2012-03-12 20:53:56Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/

if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

class challenge
{
	function challenge($in_challenge = true)
	{
		global $user, $template, $arcade, $arcade_config;

		if ((time() - $arcade_config['gc_time']) > $arcade_config['challenge_exp_last_gc'])
		{
			$this->check_expiry_date();
		}

		$template->assign_vars(array(
			'S_CHALLENGE'		=> true,
			'S_IN_CHALLENGE'	=> $in_challenge,
		));

		if ($in_challenge)
		{
			$template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $user->lang['CHALLENGE'],
				'U_VIEW_FORUM'	=> $arcade->url('mode=challenge')
			));
		}
	}

	/**
	* Return the games based on playing users
	*/
	function play_games($extra_data)
	{
		global $db, $user, $arcade, $arcade_config;

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_name_clean, g.cat_id, cc.champ_id',
			'FROM'		=> array(
				ARCADE_GAMES_TABLE => 'g',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
				array(
					'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'cc'),
					'ON'	=> 'g.game_id = cc.game_id AND cc.champ_close = ' . CHALLENGE_CLOSE
				),
			),
			'WHERE'			=> $db->sql_in_set('g.cat_id', $arcade->get_permissions('c_play'), false, true) . '
				AND c.cat_test <> ' . ARCADE_CAT_TEST . '
				AND c.cat_status = ' . ITEM_UNLOCKED,
			'GROUP_BY'		=> 'g.game_id, g.game_name, g.game_name_clean, g.cat_id',
			'ORDER_BY'		=> 'g.game_name_clean ASC'
		);

		if ($extra_data)
		{
			$sql_array['SELECT'] .= ', r.report_id, f.user_id';
			$sql_array['LEFT_JOIN'][] = array('FROM' => array(ARCADE_REPORTS_TABLE => 'r'), 'ON' => 'g.game_id = r.game_id');
			$sql_array['LEFT_JOIN'][] = array('FROM' => array(ARCADE_FAVS_TABLE => 'f'), 'ON' => 'g.game_id = f.game_id AND f.user_id = ' . (int) $user->data['user_id']);
		}

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$games = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if (!empty($row['report_id']))
			{
				continue;
			}

			$games[$row['game_id']] = array(
				'game_name'	=> $row['game_name'],
				'game_fav'	=> (!empty($row['user_id'])) ? true : false,
				'game_play'	=> (!empty($row['champ_id'])) ? true : false,
				'cat_id'	=> $row['cat_id']
			);
		}
		$db->sql_freeresult($result);

		return $games;
	}

	function friends($user_id = false)
	{
		global $db, $user, $template;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql = 'SELECT zebra_id
				FROM ' . ZEBRA_TABLE . '
				WHERE user_id = ' . (int) $user_id . '
				AND friend = 1';
		$result = $db->sql_query($sql);
		$friends = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$friends[$row['zebra_id']] = $row['zebra_id'];
		}
		$db->sql_freeresult($result);

		return $friends;
	}

	function obtain_challenge_latest_winners($limit)
	{
		global $db, $arcade, $arcade_config;

		$sql_array = array(
			'SELECT'	=> 'c.champ_id, c.champ_challenger_id, c.champ_challenger_score, c.champ_opponent_id, c.champ_opponent_score, c.champ_winner, c.champ_end_time, g.game_id, g.game_name, g.game_width, g.game_height, g.cat_id, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
			'FROM'		=> array(
				USERS_TABLE => 'u',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'c'),
					'ON'	=> '(u.user_id = c.champ_challenger_id OR u.user_id = c.champ_opponent_id)'
				),
				array(
					'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
					'ON'	=> 'c.game_id = g.game_id'
				),
			),
			'WHERE'		=> 'c.champ_close = ' . CHALLENGE_CLOSE . '
				AND (u.user_id = c.champ_challenger_id OR u.user_id = c.champ_opponent_id)
				AND c.champ_winner <> ' . CHALLENGE_TIE,
			'ORDER_BY'	=> 'c.champ_end_time DESC'
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($sql, ($limit * 2), 0, intval($arcade_config['cache_time'] * 3600));
		$latest_winners = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if (!isset($latest_winners[$row['champ_id']]))
			{
				$latest_winners[$row['champ_id']] = array(
					'end_time'				=> $row['champ_end_time'],
					'game_id'				=> $row['game_id'],
					'game_name'				=> $row['game_name'],
					'game_width'			=> $row['game_width'],
					'game_height'			=> $row['game_height'],
					'cat_id'				=> $row['cat_id'],
				);
			}

			if ($row['champ_winner'] == $row['user_id'])
			{
				$latest_winners[$row['champ_id']] += array(
					'winner_id'				=> $row['user_id'],
					'winner_name'			=> $row['username'],
					'winner_color'			=> $row['user_colour'],
					'winner_score'			=> ($row['champ_winner'] == $row['champ_challenger_id']) ? $row['champ_challenger_score'] : $row['champ_opponent_score'],
					'winner_avatar'			=> $row['user_avatar'],
					'winner_avatar_type'	=> $row['user_avatar_type'],
				);
			}
			else
			{
				$latest_winners[$row['champ_id']] += array(
					'loser_id'				=> $row['user_id'],
					'loser_name'			=> $row['username'],
					'loser_color'			=> $row['user_colour'],
					'loser_score'			=> ($row['champ_winner'] == $row['champ_challenger_id']) ? $row['champ_opponent_score'] : $row['champ_challenger_score'],
				);
			}
		}
		$db->sql_freeresult($result);

		return $latest_winners;
	}

	function display_challenge($game_id, $user_id, $username, $bet, $error1, $error2)
	{
		global $db, $user, $template, $arcade, $arcade_config, $auth_arcade;

		$load_list = (sizeof($error1)) ? true : false;

		if ($load_list)
		{
			$this->games_list(true);
			$this->users_list(true);
		}

		add_form_key('challenge');

		if ($arcade->points['show'])
		{
			$template->assign_vars(array(
				'S_CHALLENGE_BET_FIX'	=> ($arcade_config['challenge_bet_fix']) ? true : false,

				'CHALLENGE_BET_MIN'		=> ($arcade_config['challenge_bet_Minimum']) ? $arcade->number_format($arcade_config['challenge_bet_Minimum']) . ' ' . $arcade->points['name'] : false,
				'CHALLENGE_BET_MAX'		=> ($arcade_config['challenge_bet_Maximum']) ? $arcade->number_format($arcade_config['challenge_bet_Maximum']) . ' ' . $arcade->points['name'] : false,
				'CHALLENGE_BET'			=> $arcade->number_format((($arcade_config['challenge_bet_fix']) ? $arcade_config['challenge_bet_fix'] : (($bet) ? $bet : $arcade_config['challenge_bet_Minimum'])), true),
				'POINTS_NAME'			=> $arcade->points['name'],
			));
		}

		$s_hidden_fields = array('mode' => 'challenge');

		$template->assign_vars(array(
			'U_ACTION'			=> $arcade->url('#challenge'),
			'U_FIND_USERNAME'	=> $arcade->url('mode=searchuser&amp;form=challenge&amp;field=username&amp;select_single=true', 'memberlist'),

			'S_HIDDEN_FIELDS'	=> build_hidden_fields($s_hidden_fields),

			'S_ERROR1'			=> (sizeof($error1)) ? true : false,
			'ERROR_MSG1'		=> implode('<br />', $error1),

			'S_ERROR2'			=> (sizeof($error2)) ? true : false,
			'ERROR_MSG2'		=> implode('<br />', $error2),

			'OPPONENT_GAME_ID'	=> $game_id,
			'OPPONENT_USER_ID'	=> $user_id,
			'OPPONENT_USERNAME'	=> $username
		));

		if (!$load_list)
		{
			$sql_array = array(
				'SELECT'	=> 'c.challenge_id, c.challenge_time, c.challenge_points, c.challenger_id, c.opponent_id, cc.champ_challenger_score, cc.champ_opponent_score, cc.champ_challenger_time, cc.champ_opponent_time, g.game_id, g.game_name, g.game_image, g.game_width, g.game_height, g.cat_id, u.user_id, u.username, u.username_clean, u.user_colour',
				'FROM'		=> array(
					ARCADE_CHALLENGE_TABLE	=> 'c',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'cc'),
						'ON'	=> 'c.game_id = cc.game_id AND c.challenger_id = cc.champ_challenger_id AND c.opponent_id = cc.champ_opponent_id'
					),
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> '(c.challenger_id = u.user_id OR c.opponent_id = u.user_id)'
					),
					array(
						'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
						'ON'	=> 'c.game_id = g.game_id'
					),
					array(
						'FROM'	=> array(ARCADE_CATS_TABLE => 'ac'),
						'ON'	=> 'g.cat_id = ac.cat_id'
					),
				),
				'WHERE'		=> 'ac.cat_test <> ' . ARCADE_CAT_TEST . '
					AND ac.cat_status = ' . ITEM_UNLOCKED . '
					AND c.challenge_accept = ' . CHALLENGE_ACCEPTED . '
					AND cc.champ_close <> ' . CHALLENGE_CLOSE . '
					AND ' . $db->sql_in_set('ac.cat_id', $arcade->get_permissions('c_play'), false, true) . '
					AND (c.challenger_id = ' . (int) $user->data['user_id'] . ' OR c.opponent_id = ' . (int) $user->data['user_id'] . ')',
				'ORDER_BY'	=> 'c.challenge_time, u.username_clean, c.game_id ASC'
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql);
			$ongoings = array();
			while ($row = $db->sql_fetchrow($result))
			{
				if (time() >= $row['challenge_time'])
				{
					continue;
				}

				$score		= ($row['user_id'] == $row['challenger_id']) ? $row['champ_challenger_score'] : $row['champ_opponent_score'];
				$user_time	= ($row['user_id'] == $row['challenger_id']) ? $row['champ_challenger_time'] : $row['champ_opponent_time'];

				if (!isset($ongoings[$row['challenge_id']]))
				{
					$ongoings[$row['challenge_id']] = array(
						'game_id'			=> $row['game_id'],
						'game_name'			=> $row['game_name'],
						'game_image'		=> $row['game_image'],
						'game_width'		=> $row['game_width'],
						'game_height'		=> $row['game_height'],
						'cat_id'			=> $row['cat_id'],
						'bet'				=> $row['challenge_points'],
						'expiration_date'	=> $row['challenge_time']
					);
				}

				if ($user->data['user_id'] == $row['user_id'])
				{
					$ongoings[$row['challenge_id']] += array(
						'ascore'		=> $score,
						'atime'			=> $user_time
					);
				}
				else
				{
					$ongoings[$row['challenge_id']] += array(
						'user_id'		=> $row['user_id'],
						'username'		=> $row['username'],
						'user_colour'	=> $row['user_colour'],
						'bscore'		=> $score,
						'btime'			=> $user_time
					);
				}
			}
			$db->sql_freeresult($result);

			$s_report = false;

			foreach ($ongoings as $challenge_id => $ongoing)
			{
				$cat_report			= ($auth_arcade->acl_get('c_report', $ongoing['cat_id'])) ? true : false;
				$own_played			= ($ongoing['ascore'] == 0 && !$ongoing['atime']) ? false : true;
				$opponent_played	= ($ongoing['bscore'] == 0 && !$ongoing['btime']) ? false : true;
				$title				= ($own_played) ? '' : $user->lang['ARCADE_PLAY_LINK'];
				$game_image			= ($arcade->optionget('view_game_image')) ? (($own_played) ? get_arcade_game_image($ongoing['game_image'], 30, 30) : get_arcade_game_image($ongoing['game_image'], 30, 30, 'x', 'play', $ongoing['cat_id'], $ongoing['game_id'])) : false;
				$popup_icon			= ($arcade->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $ongoing['cat_id'], $ongoing['game_id'], $ongoing['game_width'], $ongoing['game_height'], 'x', 13) . ' ' : '';
				$game_name			= ($own_played) ? $ongoing['game_name'] : $popup_icon . get_arcade_game_name($ongoing['game_name'], true, 'play', $ongoing['cat_id'], $ongoing['game_id'], 'x');
				$opponent_username	= get_arcade_user_name('full', $ongoing['user_id'], $ongoing['username'], $ongoing['user_colour'], 'challenge', 'x');
				$bet				= ($ongoing['bet'] > 0) ? ' ' . sprintf($user->lang['CHALLENGE_TOURNAMENT_BET'], $arcade->number_format($ongoing['bet']) . ' ' . $arcade->points['name']) : '';
				$ongoing_info		= sprintf($user->lang['CHALLENGE_ONGOING_INFO'], sprintf($user->lang['CHALLENGE_OWN' . (($own_played) ? '' : '_NOT') . '_PLAYED'], $game_name), $opponent_username, $user->lang['CHALLENGE_OPPONENT' . (($opponent_played) ? '' : '_NOT') . '_PLAYED'] . $bet);

				if ($cat_report)
				{
					$s_report = true;
				}

				$template->assign_block_vars('ongoing', array(
					'S_REPORT_GAME'	=> $cat_report,

					'GAME_IMAGE'	=> $game_image,
					'CHALLENGE'		=> $ongoing_info,
					'EXP_DATE'		=> $user->format_date($ongoing['expiration_date'], false, true),
					'GAME_REPORT'	=> '<a href="' . $arcade->url('mode=report&amp;type=challenge&amp;g=' . $ongoing['game_id'] . "&amp;cid=$challenge_id") . '#report_game">' . $user->lang['ARCADE_REPORT_GAME'] . '</a>'
				));
			}

			$arriveds	= $this->user_challenges('arrived');
			$sents		= $this->user_challenges('sent');

			$template->assign_vars(array(
				'S_REPORT_GAME'			=> $s_report,
				'S_CURRENT_CHALLENGES'	=> true,

				'ONGOING_CHALLENGE'		=> $user->lang['CHALLENGE_ONGOING_CHALLENGE' . ((sizeof($ongoings) > 1) ? 'S' : '')],
				'ARRIVED_CHALLENGE'		=> $user->lang['CHALLENGE_ARRIVED_CHALLENGE' . (($arriveds > 1) ? 'S' : '')],
				'SENT_CHALLENGE'		=> $user->lang['CHALLENGE_SENT_CHALLENGE' . (($sents > 1) ? 'S' : '')],
			));
		}
	}

	function user_challenges($type)
	{
		global $db, $user, $arcade, $arcade_config, $template;

		$sql_array = array(
			'SELECT'		=> 'c.challenge_id, c.challenge_points, c.challenge_time, g.game_id, g.game_name, g.game_image, g.game_width, g.game_height, g.cat_id, u.user_id, u.username, u.username_clean, u.user_colour',
			'FROM'			=> array(
				ARCADE_CHALLENGE_TABLE => 'c',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'c.' . (($type == 'arrived') ? 'challenger_id' : 'opponent_id') . ' = u.user_id'
				),
				array(
					'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
					'ON'	=> 'c.game_id = g.game_id'
				),
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'ac'),
					'ON'	=> 'g.cat_id = ac.cat_id'
				),
			),
			'WHERE'			=> 'ac.cat_test <> ' . ARCADE_CAT_TEST . '
				AND ac.cat_status = ' . ITEM_UNLOCKED . '
				AND ' . $db->sql_in_set('ac.cat_id', $arcade->get_permissions('c_play'), false, true) . '
				AND c.' . (($type == 'arrived') ? 'opponent_id' : 'challenger_id') . ' = ' . (int) $user->data['user_id'] . ' AND c.challenge_accept = ' . CHALLENGE_UNACCEPTED,
			'ORDER_BY'		=> 'c.challenge_time, u.username_clean, c.game_id ASC'
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$count = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if(time() >= $row['challenge_time'])
			{
				continue;
			}

			$count[]	= $row['challenge_id'];
			$popup_icon	= ($arcade->optionget('view_popup_icon')) ? get_arcade_game_popup('icon', $row['cat_id'], $row['game_id'], $row['game_width'], $row['game_height'], 'x', 13) . ' ' : '';
			$game_name	= $popup_icon . get_arcade_game_name($row['game_name'], true, 'play', $row['cat_id'], $row['game_id'], 'x');
			$username	= get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'challenge', 'x');
			$bet		= ($row['challenge_points'] > 0) ? ' ' . sprintf($user->lang['CHALLENGE_' . strtoupper($type) . '_BET'], $arcade->number_format($row['challenge_points']) . ' ' . $arcade->points['name']) : '';
			$info		= sprintf($user->lang['CHALLENGE_' . strtoupper($type) . '_INFO'], $username, $game_name) . $bet;

			$template->assign_block_vars($type, array(
				'GAME_IMAGE'	=> ($arcade->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 30, 30, 'x', 'play', $row['cat_id'], $row['game_id']) : false,
				'CHALLENGE'		=> $info,
				'EXP_DATE'		=> $user->format_date($row['challenge_time'], false, true),
				'ID'			=> $row['challenge_id']
			));
		}
		$db->sql_freeresult($result);

		return count($count);
	}

	function games_list($load = false)
	{
		global $user, $template, $arcade, $auth_arcade, $arcade_config;

		$user_fav = $play_games = false;

		if ($load)
		{
			$played_game_color = str_replace('#', '', $arcade_config['played_colour']);

			// Quick jump of all the games in the arcade challenge...
			if ($played_game_color !== '')
			{
				$played_games = $this->get_played_games();
				$template->assign_vars(array(
					'PLAYED_GAME_TITLE'		=> ' title="' . $user->lang['CHALLENGE_PLAYED_GAMES_HIGHLIGHT'] . '"',
					'PLAYED_GAME_COLOUR'	=> $played_game_color
				));
			}
		}

		$user_age = $arcade->user_age();
		$founder_exemption = ($arcade_config['founder_exempt'] && $user->data['user_type'] == USER_FOUNDER) ? true : false;

		foreach ($this->play_games($load) as $gid => $game)
		{
			$cat_age = $arcade->get_cat_field($game['cat_id'], 'cat_age');

			if ($load && $cat_age && !$founder_exemption && (!$user_age || ($user_age < $cat_age)))
			{
				continue;
			}

			$play_games = true;

			if (!$load && !$arcade_config['load_list'])
			{
				break;
			}

			if ($arcade_config['load_list'] || $load)
			{
				$played = true;
				if (!isset($played_games[$gid]))
				{
					$played = false;
				}

				// Only show a game in the games statistics drop down if its been played challenge
				if (!$load && !$game['game_play'])
				{
					continue;
				}

				$template->assign_block_vars((($load) ? 'quick_jump' : 'game_jump'), array(
					'S_PLAYED_GAME'	=> ($load) ? $played : false,
					'GAME_ID'		=> $gid,
					'GAME_NAME'		=> $game['game_name'],
				));
			}

			if ($load && $game['game_fav'] && $auth_arcade->acl_get('u_arcade_favorites'))
			{
				$user_fav = true;
				$template->assign_block_vars('quick_fav',array(
					'GAME_ID'	=> $gid,
					'GAME_NAME'	=> $game['game_name']
				));
			}
		}

		if ($load && !$user_fav)
		{
			$template->assign_var('S_FAV_DISABLED_RADIO', true);
		}

		if (!$load && !$arcade_config['load_list'])
		{
			$template->assign_var('STATS_GAMES_LIST_LOADING', (($play_games) ? '<a style="cursor:pointer; font-weight: bold;" onclick="arcade(\'games_list\', \'2\', \'chall\'); return false;">' . $user->lang['ARCADE_GAMES_LIST_LOAD'] .'</a>' : $user->lang['ARCADE_NO_GAMES']));
		}
	}

	function obtain_all_users($no_cache = false)
	{
		global $db, $arcade, $arcade_config;

		$sql_array = array(
			'SELECT'		=> 'u.user_id, u.username, u.username_clean, au.user_arcade_options, c.champ_id',

			'FROM'			=> array(
				USERS_TABLE	=> 'u',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'c'),
					'ON'	=> 'c.champ_close = ' . CHALLENGE_CLOSE . ' AND (c.champ_challenger_id = u.user_id OR c.champ_opponent_id = u.user_id)'
				),
				array(
					'FROM'	=> array(ARCADE_USERS_TABLE => 'au'),
					'ON'	=> 'u.user_id = au.user_id'
				),
			),
			'WHERE'			=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')',
			'GROUP_BY'		=> 'u.user_id, u.username, u.username_clean, au.user_arcade_options',
			'ORDER_BY'		=> 'u.username_clean ASC',
		);

		if ($no_cache)
		{
			$sql_array['SELECT'] .= ', u.user_type, au.user_arcade_permissions';
			$sql_array['GROUP_BY'] .= ', u.user_type, au.user_arcade_permissions';
			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql);
		}
		else
		{
			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));
		}

		$challenge_users = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if (!isset($row['user_arcade_options']))
			{
				set_arcade_userdata($row);
			}

			if (!$arcade->optionget('challenge', $row['user_arcade_options'], true))
			{
				continue;
			}

			$challenge_users[$row['user_id']] = array(
				'username'	=> $row['username'],
				'user_play'	=> (!empty($row['champ_id'])) ? true : false
			);

			if ($no_cache)
			{
				$challenge_users[$row['user_id']]['user_type']					= $row['user_type'];
				$challenge_users[$row['user_id']]['user_arcade_permissions']	= $row['user_arcade_permissions'];
			}
		}
		$db->sql_freeresult($result);

		return $challenge_users;
	}

	function users_list($load = false)
	{
		global $user, $template, $arcade, $auth_arcade, $arcade_config, $arcade_cache;

		$user_friends = $challenge_users = false;
		$played_user_color = str_replace('#', '', $arcade_config['played_colour']);

		// Quick jump of all the users in the challenge...
		if ($load)
		{
			$friend = $this->friends();

			if ($played_user_color !== '' && $arcade_config['challenge_users_select'])
			{
				$played_users = $this->get_played_users();

				$template->assign_vars(array(
					'PLAYED_USER_TITLE'		=> ' title="' . $user->lang['CHALLENGE_PLAYED_USERS_HIGHLIGHT'] . '"',
					'PLAYED_USER_COLOUR'	=> $played_user_color
				));
			}
		}

		foreach ($this->obtain_all_users() as $uid => $_user)
		{
			if (($load && ($uid == $user->data['user_id'])) || (!$load && !$_user['user_play']))
			{
				continue;
			}

			$challenge_users = true;

			if (!$arcade_config['challenge_users_select'] && !$load)
			{
				break;
			}

			if ($load && !empty($friend[$uid]) && $uid == $friend[$uid])
			{
				$user_friends = true;
				$template->assign_block_vars('friends', array(
					'USER_ID'	=> $uid,
					'USERNAME'	=> $_user['username'],
				));
			}

			if ($arcade_config['challenge_users_select'] && ($arcade_config['load_list'] || $load))
			{
				$played = true;
				if (!isset($played_users[$uid]))
				{
					$played = false;
				}

				$template->assign_block_vars((($load) ? 'user_jump' : 'stat_user_jump'), array(
					'S_PLAYED_USER'	=> ($load) ? $played : false,
					'USER_ID'		=> $uid,
					'USERNAME'		=> $_user['username'],
				));
			}
		}

		if ($load && !$user_friends)
		{
			$template->assign_var('S_FRIEND_DISABLED_RADIO', true);
		}

		if ($arcade_config['challenge_users_select'] && $load)
		{
			$template->assign_var('S_USER_SELECT', true);
		}
		else if (!$arcade_config['load_list'] && !$load)
		{
			$template->assign_var('USERS_LIST_LOADING', (($challenge_users) ? '<a style="cursor:pointer; font-weight: bold;" onclick="arcade(\'users_list\', \'2\', \'chall\'); return false;">' . $user->lang['ARCADE_USERS_LIST_LOAD'] .'</a>' : $user->lang['CHALLENGE_NO_USERS']));
		}
		else if($load)
		{
			$template->assign_var('S_USER_TEXT', ($challenge_users) ? true : false);
		}
	}

	/**
	* Return an array contain all the game ids and names
	* of the games the user has challenge played, if no user id is
	* passed it defaults to viewing user
	*/
	function get_played_games($user_id = false)
	{
		global $arcade, $arcade_config;

		if (!$arcade_config['played_colour'])
		{
			return array();
		}

		global $db, $user;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'	=> 'g.game_id',
			'FROM'		=> array(
				ARCADE_GAMES_TABLE => 'g',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'c'),
					'ON'	=> 'g.game_id = c.game_id'
				),
			),
			'WHERE'		=> 'champ_close = ' . CHALLENGE_CLOSE . '
				AND ' . $db->sql_in_set('g.cat_id', $arcade->get_permissions('c_view'), false, true) . '
				AND (c.champ_challenger_id = ' . (int) $user_id . ' OR c.champ_opponent_id = ' . (int) $user_id . ')',
		);
		$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
		$result = $db->sql_query($sql);

		$played = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$played[$row['game_id']] = true;
		}
		$db->sql_freeresult($result);

		return $played;
	}

	function get_played_users($user_id = false)
	{
		global $arcade_config;

		if (!$arcade_config['played_colour'])
		{
			return array();
		}

		global $db, $user;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'	=> 'u.user_id',
			'FROM'		=> array(
				USERS_TABLE => 'u',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CHALLENGE_CHAMP_TABLE => 'c'),
					'ON'	=> '(u.user_id = c.champ_challenger_id OR u.user_id = c.champ_opponent_id)'
				),
			),
			'WHERE'		=> 'c.champ_close = ' . CHALLENGE_CLOSE . '
				AND (c.champ_challenger_id = ' . (int) $user_id . ' OR c.champ_opponent_id = ' . (int) $user_id . ')',
		);

		$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
		$result = $db->sql_query($sql);

		$played = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$played[$row['user_id']] = true;
		}
		$db->sql_freeresult($result);

		return $played;
	}

	function check($mode, $game_id, $user_id, &$username, $bet, &$error)
	{
		global $db, $user, $auth_arcade, $arcade, $arcade_config;

		$game_opponent_data = $arcade->get_game_cat_data($game_id);

		if ($game_opponent_data)
		{
			$game_opponent_data = array_merge($game_opponent_data, $arcade->userdata('set_auth', $user_id, $username));
		}

		if (!isset($game_opponent_data['game_id']) || !$game_opponent_data['game_id'])
		{
			$error[] = $user->lang['NO_GAME_ID'];
		}

		if (!isset($game_opponent_data['user_id']) || !$game_opponent_data['user_id'])
		{
			$error[] = $user->lang['NO_USER_ID'];
		}

		if (!empty($game_opponent_data['user_id']) && ($game_opponent_data['user_id'] == $user->data['user_id']))
		{
			$error[] = $user->lang['CHALLENGE_OPPONENT_MYSELF_ERROR'];
		}

		if (!sizeof($error))
		{
			if (!$arcade->optionget('challenge', $game_opponent_data['user_arcade_options'], true))
			{
				$error[] = sprintf($user->lang['CHALLENGE_OPPONENT_USER_DISADLED_ERROR'], $game_opponent_data['username']);
			}
		}

		if (!sizeof($error))
		{
			global $auth_arcade2;

			$auth_arcade2 = new auth_arcade();
			$auth_arcade2->acl($game_opponent_data);

			$username = $game_opponent_data['username'];

			if (!$auth_arcade2->acl_get('u_arcade'))
			{
				$error[] = sprintf($user->lang['ARCADE_OPPONENT_USER_NO_PERMISSION'], $game_opponent_data['username']);
			}

			if (!$auth_arcade2->acl_get('u_arcade_challenge'))
			{
				$error[] = sprintf($user->lang['CHALLENGE_OPPONENT_USER_NO_PERMISSION'], $game_opponent_data['username']);
			}

			if (!$auth_arcade2->acl_get('c_play', $game_opponent_data['cat_id']))
			{
				$error[] = sprintf($user->lang['CHALLENGE_NO_PERMISSION_ARCADE_PLAY'], $game_opponent_data['username'], $game_opponent_data['game_name']);
			}

			if ($this->user_game_opponent('def', $game_opponent_data['game_id'], $game_opponent_data['user_id']))
			{
				$error[] = sprintf($user->lang['CHALLENGE_DUPLICATION'], $game_opponent_data['username'], $game_opponent_data['game_name']);
			}

			$challenger_free_play = $opponent_free_play = true;
			$challenger_game_cost = $opponent_game_cost = $challenger_total_cost = $opponent_total_cost = 0;

			if (!sizeof($error) && $arcade->points['show'])
			{
				$bet					= ($bet > 0) ? $bet : $arcade_config['challenge_bet_fix'];
				$challenger_points		= $arcade->points['total'];
				$opponent_points		= $arcade->get_points($game_opponent_data['user_id']);
				$game_cost				= $arcade->get_cost($game_opponent_data);
				$game_cost				= ($game_cost > 0) ? $game_cost : 0;
				$challenger_free_play	= (!$game_cost || $auth_arcade->acl_get('c_playfree', $game_opponent_data['cat_id'])) ? true : false;
				$opponent_free_play		= (!$game_cost || $auth_arcade2->acl_get('c_playfree', $game_opponent_data['cat_id'])) ? true : false;
				$challenger_total_cost	= ($challenger_free_play) ? $bet : ($bet + $game_cost);
				$opponent_total_cost	= ($opponent_free_play) ? $bet : ($bet + $game_cost);
				$challenger_game_cost	= ($challenger_free_play) ? 0 : $game_cost;
				$opponent_game_cost		= ($opponent_free_play) ? 0 : $game_cost;

				if (!$arcade_config['challenge_bet_fix'] && $arcade_config['challenge_bet_Minimum'] && ($arcade_config['challenge_bet_Minimum'] > $bet))
				{
					$error[] = $user->lang['CHALLENGE_BET_TOO_SMALL'];
				}
				else if (!$arcade_config['challenge_bet_fix'] && $arcade_config['challenge_bet_Maximum'] && ($arcade_config['challenge_bet_Maximum'] < $bet))
				{
					$error[] = $user->lang['CHALLENGE_BET_TOO_LARGE'];
				}
				else
				{
					if (!$challenger_free_play && $game_cost > 0 && $challenger_points < $game_cost)
					{
						$error[] = sprintf($user->lang['CHALLENGE_CHALLENGER_USER_NO_POINTS'], $arcade->points['name'], $arcade->number_format($game_cost) . ' ' . $arcade->points['name'], $arcade>number_format($challenger_points) . ' ' . $arcade->points['name']);
					}
					else
					{
						if (!$challenger_free_play && $challenger_total_cost > 0 && $challenger_points < $challenger_total_cost)
						{
							$error[] = sprintf($user->lang['CHALLENGE_CHALLENGER_BET_COST_NO_POINTS'], $arcade->number_format($challenger_total_cost) . ' ' . $arcade->points['name'], $arcade->number_format($challenger_points) . ' ' . $arcade->points['name']);
						}
						else if ($bet > 0 && $challenger_points < $bet)
						{
							$error[] = sprintf($user->lang['CHALLENGE_CHALLENGER_BET_NO_POINTS'], $arcade->number_format($challenger_points) . ' ' . $arcade->points['name']);
						}
					}

					if (!$opponent_free_play && $game_cost > 0 && $opponent_points['total'] < $game_cost)
					{
						$error[] = sprintf($user->lang['CHALLENGE_OPPONENT_USER_NO_POINTS'], $game_opponent_data['username'], $arcade->points['name'], $arcade->number_format($game_cost) . ' ' . $arcade->points['name'], $game_opponent_data['username'], $arcade->number_format($opponent_points['total']) . ' ' . $arcade->points['name']);
					}
					else
					{
						if (!$opponent_free_play && $opponent_total_cost > 0 && $opponent_points['total'] < $opponent_total_cost)
						{
							$error[] = sprintf($user->lang['CHALLENGE_OPPONENT_BET_COST_NO_POINTS'], $arcade->number_format($opponent_total_cost) . ' ' . $arcade->points['name'], $game_opponent_data['username'], $arcade->number_format($opponent_points['total']) . ' ' . $arcade->points['name']);
						}
						else if ($bet > 0 && $opponent_points['total'] < $bet)
						{
							$error[] = sprintf($user->lang['CHALLENGE_OPPONENT_BET_NO_POINTS'], $game_opponent_data['username'], $arcade->number_format($opponent_points['total']) . ' ' . $arcade->points['name']);
						}
					}
				}

				if(!sizeof($error))
				{
					if ($challenger_total_cost > 0)
					{
						$arcade->set_points('subtract', $user->data['user_id'], $challenger_total_cost);
					}

					if ($opponent_total_cost > 0)
					{
						$arcade->set_points('subtract', $game_opponent_data['user_id'], $opponent_total_cost);
					}
				}
			}
			else
			{
				$bet = 0;
			}
		}

		if (!sizeof($error))
		{
			$data = array(
				'challenge_points'		=> (float) $bet,
				'challenger_id'			=> (int) $user->data['user_id'],
				'opponent_id'			=> (int) $game_opponent_data['user_id'],
				'game_id'				=> (int) $game_opponent_data['game_id'],
				'challenger_game_cost'	=> (float) $challenger_game_cost,
				'opponent_game_cost'	=> (float) $opponent_game_cost,
				'challenge_time'		=> intval(time() + $arcade_config['challenge_acc_exp_date'])
			);

			$db->sql_query('INSERT INTO ' . ARCADE_CHALLENGE_TABLE . ' ' . $db->sql_build_array('INSERT', $data));

			$send_user_pm = ($arcade->optionget('arcade_pm', $game_opponent_data['user_arcade_options'], true)) ? true : false;

			if ($arcade_config['send_arcade_pm'] && $send_user_pm)
			{
				$challenge_data = array(
					'PM_SUBJECT'				=> 'CHALLENGE_PM_SUBJECT',
					'PM_MESSAGE'				=> 'CHALLENGE_PM_MESSAGE',
					'game_id'					=> $game_opponent_data['game_id'],
					'game_name'					=> $game_opponent_data['game_name'],
					'game_image'				=> $game_opponent_data['game_image'],
					'game_width'				=> $game_opponent_data['game_width'],
					'game_height'				=> $game_opponent_data['game_height'],
					'cat_id'					=> $game_opponent_data['cat_id'],
					'bet'						=> $bet,
					'free_play'					=> $opponent_free_play,
					'total_cost'				=> $opponent_total_cost,

					'other_user_id'				=> $game_opponent_data['user_id'],
					'other_user_lang'			=> $game_opponent_data['user_lang'],
					'other_user_arcade_options'	=> $game_opponent_data['user_arcade_options']
				);

				$arcade->send_pm('challenge', $challenge_data);
			}

			unset($auth_arcade2);

			$deduction	= ($challenger_total_cost > 0) ? '<br />' . sprintf($user->lang['CHALLENGE_MSG_DEDUCTION_' . (($bet > 0) ? (($challenger_free_play) ? 'BET' : 'BET_COST') : 'COST')], $arcade->number_format($challenger_total_cost) . ' ' . $arcade->points['name']) : '';
			$pm_ed		= ($arcade_config['send_arcade_pm']) ? ' ' . sprintf($user->lang['CHALLENGE_OTHER_USER_PM_' . (($send_user_pm) ? 'ENABLED' : 'DISABLED')], $game_opponent_data['username']) : '';
			$message	= $user->lang['CHALLENGE_SUCCESSFUL'] . '<br />' . $deduction . $pm_ed . (($mode == 'default') ? $arcade->back_link('mode=challenge', 'challenge') : '');
		}

		switch ($mode)
		{
			case 'champion':
				$lang_ary = array(
					'PAGE_TITLE'	=> $user->lang['ARCADE_CHALLENGE'] . ' ' . $user->lang['INFORMATION'],
					'MESSAGE_TITLE'	=> $user->lang['INFORMATION'],
					'MESSAGE_TEXT'	=> ((!sizeof($error)) ? $message : implode('<br />', $error)) . $arcade->close_win()
				);

				$arcade->confirm($lang_ary, true);
			break;

			case 'random':
				if (sizeof($error))
				{
					trigger_error(implode('<br />', $error));
				}

				$challenge_again = '<a class="' . (($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right') . '" href="' . $arcade->url('mode=rand_challenge') . '">' . $user->lang['CHALLENGE_RANDOM_AGAIN'] . '</a>';
				$message .= '<br ><br />' . $challenge_again . $arcade->back_link('mode=challenge', 'challenge', false);
			default:
				if (!sizeof($error))
				{
					trigger_error($message);
				}
			break;
		}
	}

	function report($challenge_id)
	{
		global $arcade;

		$error = array();
		$msg = $this->user_action('report', $challenge_id, $error);

		if (sizeof($error))
		{
			trigger_error(implode('<br />', $error) . $arcade->back_link('mode=challenge', 'challenge'));
		}

		return $msg;
	}

	function user_action($action, $ids, &$error)
	{
		global $db, $user, $arcade, $arcade_config, $arcade_cache, $auth_arcade2;

		if (!$ids)
		{
			$error[] = $user->lang['NO_CHALLENGE_ID'];
		}
		else
		{
			$sql_arry = array(
				'SELECT'		=> 'c.challenger_id, c.opponent_id , c.challenge_points, c.challenger_game_cost, c.opponent_game_cost,
									g.game_id, g.game_name, g.cat_id,
									u.user_type, u.user_id , u.username, u.user_lang,
									au.user_arcade_options, au.user_arcade_permissions',
				'FROM'			=> array(
					ARCADE_CHALLENGE_TABLE => 'c',
				),
				'LEFT_JOIN'		=> array(
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> ($action == 'report') ? '(c.opponent_id = ' . (int) $user->data['user_id'] . ' AND c.challenger_id = u.user_id) OR (c.challenger_id = ' . (int) $user->data['user_id'] . ' AND c.opponent_id = u.user_id)' : 'c.' . (($action == 'withdraw') ? 'opponent_id' : 'challenger_id') . ' = u.user_id AND c.' . (($action == 'withdraw') ? 'challenger_id' : 'opponent_id') . ' = ' . (int) $user->data['user_id']
					),
					array(
						'FROM'	=> array(ARCADE_USERS_TABLE => 'au'),
						'ON'	=> 'u.user_id = au.user_id'
					),
					array(
						'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
						'ON'	=> 'c.game_id = g.game_id'
					),
				),
				'WHERE'			=> $db->sql_in_set('c.challenge_id', $ids) . '
					AND ' . $db->sql_in_set('g.cat_id', $arcade->get_permissions('c_play'), false, true) . '
					AND c.challenge_accept = ' . (($action == 'report') ? CHALLENGE_ACCEPTED : CHALLENGE_UNACCEPTED)
			);

			if ($action == 'report')
			{
				$sql_arry['SELECT'] .= ', cc.champ_id';
				$sql_arry['LEFT_JOIN'][] = array('FROM' => array(ARCADE_CHALLENGE_CHAMP_TABLE => 'cc'), 'ON' => 'cc.champ_close <> ' . CHALLENGE_CLOSE . ' AND c.game_id = cc.game_id AND c.challenger_id = cc.champ_challenger_id AND c.opponent_id = cc.champ_opponent_id');
			}

			$sql = $db->sql_build_query('SELECT', $sql_arry);
			$result = $db->sql_query($sql);

			$more_chall = false;
			$datas = $champ_ids = $message = array();

			while ($row = $db->sql_fetchrow($result))
			{
				if ($action == 'report')
				{
					$champ_ids[] = $row['champ_id'];
				}

				if (!isset($datas[$row['user_id']]))
				{
					$datas[$row['user_id']] = array(
						'cat_id'					=> $row['cat_id'],
						'game_id'					=> $row['game_id'],
						'games_name'				=> array($row['game_name']),
						'username'					=> $row['username'],

						'user_type'					=> $row['user_type'],
						'user_lang'					=> $row['user_lang'],
						'user_arcade_options'		=> $row['user_arcade_options'],
						'user_arcade_permissions'	=> $row['user_arcade_permissions'],
					);
				}
				else
				{
					$datas[$row['user_id']]['games_name'][] = $row['game_name'];
				}

				switch ($action)
				{
					case 'accept':
						$sql_array = array(
							'champ_challenger_id'	=> (int) $row['challenger_id'],
							'champ_opponent_id'		=> (int) $row['opponent_id'],
							'game_id'				=> (int) $row['game_id'],
							'champ_winner'			=> 0
						);

						$db->sql_query('INSERT INTO ' . ARCADE_CHALLENGE_CHAMP_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_array));
					break;

					case 'reject':
					case 'withdraw':
					case 'report':
						$challenger_cost	= floatval($row['challenge_points'] + $row['challenger_game_cost']);
						$opponent_cost		= floatval($row['challenge_points'] + $row['opponent_game_cost']);

						$datas[$row['user_id']]['own_key'][$row['game_id']] = ($action == 'report') ? (($user->data['user_id'] == $row['challenger_id']) ? 'withdraw' : 'reject') : false;
						$datas[$row['user_id']]['ctc_count'] = (!isset($datas[$row['user_id']]['ctc_count'])) ? (($challenger_cost > 0) ? 1 : 0) : ($datas[$row['user_id']]['ctc_count'] + (($challenger_cost > 0) ? 1 : 0));
						$datas[$row['user_id']]['otc_count'] = (!isset($datas[$row['user_id']]['otc_count'])) ? (($opponent_cost > 0) ? 1 : 0) : ($datas[$row['user_id']]['otc_count'] + (($opponent_cost > 0) ? 1 : 0));

						$datas[$row['user_id']]['challenger_total_cost'] = (!isset($datas[$row['user_id']]['challenger_total_cost'])) ? $challenger_cost : ($datas[$row['user_id']]['challenger_total_cost'] + $challenger_cost);
						$datas[$row['user_id']]['opponent_total_cost'] = (!isset($datas[$row['user_id']]['opponent_total_cost'])) ? $opponent_cost : ($datas[$row['user_id']]['opponent_total_cost'] + $opponent_cost);
					break;
				}
			}
			$db->sql_freeresult($result);

			if ($arcade_config['send_arcade_pm'])
			{
				$auth_arcade2 = new auth_arcade();
			}

			foreach ($datas as $user_id => $row)
			{
				$type = (!empty($row['own_key'][$row['game_id']])) ? $row['own_key'][$row['game_id']] : $action;

				$data = array(
					'PM_SUBJECT'				=> 'CHALLENGE_PM_SUBJECT_' . strtoupper($action),
					'PM_MESSAGE'				=> 'CHALLENGE_PM_MESSAGE_' . strtoupper($action) . '_GAME' . (($type != 'report' && sizeof($row['games_name']) > 1) ? 'S' : ''),
					'username'					=> $user->data['username'],
					'games_name'				=> implode(', ', $row['games_name']),
					'total_cost'				=> ($type == 'reject') ? $row['challenger_total_cost'] : (($type == 'withdraw') ? $row['opponent_total_cost'] : 0),
					'tc_num'					=> ($type == 'reject') ? $row['ctc_count'] : (($type == 'withdraw') ? $row['otc_count'] : 0),

					'cat_id'					=> $row['cat_id'],
					'other_user_lang'			=> $row['user_lang'],
					'other_user_id'				=> $user_id,
					'other_user_arcade_options'	=> $row['user_arcade_options']
				);

				$total_cost		= ($type == 'reject') ? $row['opponent_total_cost'] : (($type == 'withdraw') ? $row['challenger_total_cost'] : 0);
				$tc_num			= ($type == 'reject') ? $row['otc_count'] : (($type == 'withdraw') ? $row['ctc_count'] : 0);
				$games_name		= sprintf($user->lang['CHALLENGE_GAME' . ((sizeof($row['games_name']) > 1) ? 'S' : '')], $data['games_name']);
				$refund			= ($type != 'accept' && $total_cost > 0) ? ' ' . sprintf($user->lang['CHALLENGE_MSG_REFUND' . (($tc_num > 1) ? 'S' : '')], $arcade->number_format($total_cost) . ' ' . $arcade->points['name']) : '';
				$send_user_pm	= ($arcade->optionget('arcade_pm', $row['user_arcade_options'], true)) ? true : false;
				$pm_ed			= ($arcade_config['send_arcade_pm']) ? ' ' . sprintf($user->lang['CHALLENGE_OTHER_USER_PM_' . (($send_user_pm) ? 'ENABLED' : 'DISABLED')], $row['username']) : '';

				if ($type != 'accept')
				{
					$cid = ($type == 'reject') ? $user_id : $user->data['user_id'];
					$oid = ($type == 'reject') ? $user->data['user_id'] : $user_id;

					if ($row['challenger_total_cost'] > 0)
					{
						$arcade->set_points('add', $cid, $row['challenger_total_cost']);
					}

					if ($row['opponent_total_cost'] > 0)
					{
						$arcade->set_points('add', $oid, $row['opponent_total_cost']);
					}
				}

				if (sizeof($row['games_name']) > 1)
				{
					$more_chall = true;
				}

				$message[] = $games_name . $refund . $pm_ed;

				if ($arcade_config['send_arcade_pm'] && $send_user_pm)
				{
					$row['user_id'] = $user_id;
					$auth_arcade2->acl($row);

					$arcade->send_pm('challenge_user_action', $data);
				}

				unset($data, $row);
			}

			unset($auth_arcade2, $datas);

			if (!sizeof($message))
			{
				$error[] = $user->lang['NO_CHALLENGE'];
				return;
			}

			switch ($action)
			{
				case 'accept':
					$sql = 'UPDATE ' . ARCADE_CHALLENGE_TABLE . '
							SET challenge_accept = ' . CHALLENGE_ACCEPTED . ', challenge_time = ' . intval(time() + $arcade_config['challenge_ong_exp_date']) . '
							WHERE ' . $db->sql_in_set('challenge_id', $ids);
					$db->sql_query($sql);
				break;

				case 'report':
					$sql = 'DELETE FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
							WHERE ' . $db->sql_in_set('champ_id', $champ_ids);
					$db->sql_query($sql);
				case 'reject':
				case 'withdraw':
					$sql = 'DELETE FROM ' . ARCADE_CHALLENGE_TABLE . '
							WHERE ' . $db->sql_in_set('challenge_id', $ids);
					$db->sql_query($sql);
				break;
			}

			$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);

			if ($action != 'report')
			{
				$success_msg = $user->lang['CHALLENGE' . (($more_chall || sizeof($message) > 1) ? 'S' : '') . '_' . strtoupper($action) . '_SUCCESSFUL'] . '<br />';
				trigger_error($success_msg . implode('<br />', $message) . $arcade->back_link('mode=challenge', 'challenge'));
			}

			return $message;
		}
	}

	function user_game_opponent($mode, $game_id, $user_id = false)
	{
		global $db, $user;

		switch ($mode)
		{
			case 'all':
				if (!$user_id)
				{
					$user_id = $user->data['user_id'];
				}

				$user_ids = array();

				$sql = 'SELECT challenger_id, opponent_id
						FROM ' . ARCADE_CHALLENGE_TABLE . '
						WHERE game_id = ' . (int) $game_id . '
						AND (
							 (challenger_id	= ' . (int) $user_id . ') OR
							 (opponent_id	= ' . (int) $user_id . ')
							)';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$user_ids[] = ($user_id == $row['challenger_id']) ? $row['opponent_id'] : $row['challenger_id'];
				}
				$db->sql_freeresult($result);

				return $user_ids;
			break;

			default:
				if (!$user_id)
				{
					return false;
				}

				$sql = 'SELECT challenge_id
						FROM ' . ARCADE_CHALLENGE_TABLE . '
						WHERE game_id = ' . (int) $game_id . '
						AND ((challenger_id = ' . (int) $user_id . ' AND opponent_id = ' . (int) $user->data['user_id'] . ') OR
							(challenger_id = ' . (int) $user->data['user_id'] . ' AND opponent_id = ' . (int) $user_id . '))';
				$result = $db->sql_query_limit($sql, 1);
				$challenge_id = (int) $db->sql_fetchfield('challenge_id');
				$db->sql_freeresult($result);

				return $challenge_id;
			break;
		}

		return $challenge_id;
	}

	function random()
	{
		global $db, $user, $arcade, $arcade_config;

		$stop_rand	= false;
		$error		= $game_data = $user_data = array();
		$random = array(
			'bet' => ($arcade->points['show']) ? (($arcade_config['challenge_bet_fix'] > 0) ? $arcade_config['challenge_bet_fix'] : (($arcade_config['challenge_bet_Minimum'] > 0) ? $arcade_config['challenge_bet_Minimum'] : 0)) : 0
		);

		if ($random['bet'] > 0 && $arcade->points['total'] < $random['bet'])
		{
			$stop_rand = true;
		}

		if (!$stop_rand)
		{
			$sql = 'SELECT g.game_id, g.game_cost, c.cat_cost, c.cat_id
					FROM ' . ARCADE_GAMES_TABLE . ' g
					LEFT JOIN ' . ARCADE_CATS_TABLE . ' c	 ON g.cat_id = c.cat_id
					WHERE	' . $db->sql_in_set('g.game_id', array_map('intval', $arcade->obtain_report_games()), true, true) . '
					AND		' . $db->sql_in_set('g.game_id', array_map('intval', $arcade->get_game_cost_notplay($random['bet'])), true, true) . '
					AND c.cat_status = ' . ITEM_UNLOCKED . '
					AND c.cat_test <> ' . ARCADE_CAT_TEST . '
					AND ' . $db->sql_in_set('c.cat_id', $arcade->get_permissions('c_play'), false, true) . '
					AND ' . $db->sql_in_set('c.cat_id', $arcade->games_search_protection(), true, true) . '
					ORDER BY c.cat_id ASC';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$game_data[] = array(
					'game_id'	=> $row['game_id'],
					'game_cost'	=> $row['game_cost'],
					'cat_cost'	=> $row['cat_cost'],
					'cat_id'	=> $row['cat_id']
				);
			}
			$db->sql_freeresult($result);

			if (sizeof($game_data))
			{
				$random = array_merge($random, $game_data[mt_rand(0, sizeof($game_data) - 1)]);
			}
			else
			{
				$stop_rand = true;
			}

			unset($game_data);

			if (!$stop_rand)
			{
				$no_auth_count = 0;
				$auth_arcade2 = new auth_arcade();
				$dup_users = $this->user_game_opponent('all', $random['game_id']);

				foreach ($this->obtain_all_users(true) as $user_id => $cuser)
				{
					if ($user_id == $user->data['user_id'] || in_array($user_id, $dup_users))
					{
						continue;
					}

					if (!$cuser['user_arcade_permissions'])
					{
						$no_auth_count++;
					}

					$cuser['user_id'] = $user_id;
					$auth_arcade2->acl($cuser);

					if ($auth_arcade2->acl_get('u_arcade') && $auth_arcade2->acl_get('u_arcade_challenge') && $auth_arcade2->acl_get('c_play', $random['cat_id']))
					{
						$points_action = false;
						if ($arcade->points['show'])
						{
							$cost = $arcade->get_cost($random);

							if ($cost > 0 || $random['bet'] > 0)
							{
								$points_action = true;
								$user_points = $arcade->get_points($user_id);
							}
						}

						if ((!$points_action) || ($auth_arcade2->acl_get('c_playfree', $random['cat_id']) && ($random['bet'] <= $user_points['total'])) || (($cost + $random['bet']) <= $user_points['total']))
						{
							$user_data[] = array(
								'user_id'	=> $user_id,
								'username'	=> $cuser['username']
							);
						}
					}

					if ($no_auth_count >= 50)
					{
						break;
					}
				}

				unset($auth_arcade2);

				if (sizeof($user_data))
				{
					$random = array_merge($random, $user_data[mt_rand(0, sizeof($user_data) - 1)]);
					$this->check('random', $random['game_id'], $random['user_id'], $random['username'], $random['bet'], $error);
				}
			}
		}

		$challenge_again = '<br /><br /><a class="' . (($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right') . '" href="' . $arcade->url('mode=rand_challenge') . '">' . $user->lang['CHALLENGE_RANDOM_AGAIN'] . '</a>';
		trigger_error($user->lang['CHALLENGE_NO_RANDOM'] . $challenge_again . $arcade->back_link('mode=challenge', 'challenge', false));
	}

	function check_expiry_date()
	{
		global $db, $arcade, $arcade_config;

		$time = time();

		$sql = 'SELECT * FROM ' . ARCADE_CHALLENGE_TABLE . "
				WHERE challenge_time <= $time";
		$result = $db->sql_query($sql);

		$cids = array();
		while ($del = $db->sql_fetchrow($result))
		{
			$cids[] = $del['challenge_id'];

			$sql = 'DELETE FROM '. ARCADE_CHALLENGE_CHAMP_TABLE .'
					WHERE game_id = '. (int) $del['game_id'] .'
					AND champ_opponent_id = '. (int) $del['opponent_id'] .'
					AND champ_challenger_id = '. (int) $del['challenger_id'] .'
					AND champ_close <> ' . CHALLENGE_CLOSE;
			$db->sql_query($sql);

			$challenger_total_cost	= floatval($del['challenge_points'] + $del['challenger_game_cost']);
			$opponent_total_cost	= floatval($del['challenge_points'] + $del['opponent_game_cost']);

			if ($challenger_total_cost > 0)
			{
				$arcade->set_points('add', $del['challenger_id'], $challenger_total_cost);
			}

			if ($opponent_total_cost > 0)
			{
				$arcade->set_points('add', $del['opponent_id'], $opponent_total_cost);
			}
		}
		$db->sql_freeresult($result);

		if (sizeof($cids))
		{
			$db->sql_query('DELETE FROM ' . ARCADE_CHALLENGE_TABLE . ' WHERE ' . $db->sql_in_set('challenge_id', $cids));
		}

		$arcade->set_config('challenge_exp_last_gc', $time, true);
	}
}

?>