<?php
/**
*
* @package arcade
* @version $Id: functions.php 2024 2012-03-20 17:19:21Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/* Common arcade functions */
/*---------------------------
* arcade_display_data()
* display_arcade_menu()
* display_arcade_highscores()
* set_arcade_userdata()
* arcade_user_add()
* arcade_ghost_permissions()
* arcade_ban_users()
* arcade_default_set()
---------------------------*/
/* Common arcade functions */

function arcade_display_data($function = false, $datas = array(), $arcade_class = false)
{
	global $config, $arcade_config;

	if (!isset($config['arcade_install']))
	{
		return false;
	}

	if (!isset($arcade_config))
	{
		arcade_default_set($arcade_class);
	}

	if ($function && sizeof($datas))
	{
		return call_user_func_array('display_arcade_' . $function, $datas);
	}
	else if ($function)
	{
		call_user_func('display_arcade_' . $function);
	}
}

function display_arcade_menu()
{
	if (defined('USE_ARCADE_MENU'))
	{
		return;
	}

	global $user, $auth, $template, $phpbb_root_path, $phpEx, $auth_arcade, $arcade_config;

	define('USE_ARCADE_MENU', true);

	$template->assign_vars(array(
		'S_DISPLAY_ARCADE'				=> ($auth_arcade->acl_get('u_arcade')) ? true : false,
		'S_DISPLAY_ARCADE_CHALLENGE'	=> ($auth_arcade->acl_get('u_arcade_challenge')) ? true : false,
		'S_ARCADE_FLASH_INTRO'			=> ($arcade_config['arcade_flash_intro']) ? 'yes' : 'no',

		'T_ARCADE_JS_PATH'				=> "{$phpbb_root_path}arcade/js",

		'U_ARCADE'						=> append_sid("{$phpbb_root_path}arcade.$phpEx"),
		'U_ARCADE_CHALLENGE'			=> append_sid("{$phpbb_root_path}arcade.$phpEx", 'mode=challenge'),

		'S_ARCADE_REPORTS'				=> ($auth->acl_get('a_arcade_utilities')) ? true : false,
		'S_ARCADE_REPORTS_OPEN'			=> ($arcade_config['reports_open']) ? true : false,
		'U_ARCADE_REPORTS_OPEN'			=> append_sid("{$phpbb_root_path}adm/index.$phpEx", 'i=arcade_utilities&amp;mode=reports', true, $user->session_id),

		'ARCADE_REPORTS_OPEN'			=> ($arcade_config['reports_open'] > 1) ? sprintf($user->lang['ARCADE_REPORTS_OPEN'], $arcade_config['reports_open']) : sprintf($user->lang['ARCADE_REPORT_OPEN'], $arcade_config['reports_open'])
	));
}

function display_arcade_highscores($page, $user_id, $highscore = false)
{
	global $user, $arcade_cache, $auth_arcade, $arcade_config, $arcade_highscore_data, $challenge_wins_data, $phpbb_root_path, $phpEx;

	if (($page == 'memberlist' && !$arcade_config['display_memberlist']) || ($page == 'viewtopic' && !$arcade_config['display_viewtopic']))
	{
		return false;
	}

	if (!isset($arcade_highscore_data))
	{
		$arcade_highscore_data = $arcade_cache->obtain_arcade_leaders_all();
	}

	if (!isset($challenge_wins_data))
	{
		$challenge_wins_data = $arcade_cache->obtain_challenge_leaders_all();
	}

	$total_highscores = $a_leader_id = 0;
	$total_wins = $c_leader_id = 0;

	if (!empty($arcade_highscore_data[$user_id]))
	{
		$total_highscores = $arcade_highscore_data[$user_id];
		$a_leader_id = array_shift(array_keys($arcade_highscore_data));
	}

	if (!empty($challenge_wins_data[$user_id]))
	{
		$total_wins = $challenge_wins_data[$user_id];
		$c_leader_id = array_shift(array_keys($challenge_wins_data));
	}

	if ($highscore == 'arcade')
	{
		return $total_highscores;
	}
	else if ($highscore == 'challenge')
	{
		return $total_wins;
	}
	else
	{
		return array(
			'S_HAS_ARCADE_DISPLAY'		=> true,
			'S_IS_ARCADE_LEADER'		=> ($a_leader_id > 0 && $a_leader_id == $user_id) ? true : false,
			'S_ARCADE_STATS'			=> ($total_highscores && $auth_arcade->acl_get('u_arcade_viewstats')) ? true : false,
			'U_ARCADE_STATS'			=> append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=stats&amp;u={$user_id}"),
			'ARCADE_TOTAL_HIGHSCORES'	=> $total_highscores,

			'S_IS_CHALLENGE_LEADER'		=> ($c_leader_id > 0 && $c_leader_id == $user_id) ? true : false,
			'S_CHALLENGE_STATS'			=> ($total_wins && $auth_arcade->acl_get('u_arcade_viewstats')) ? true : false,
			'U_CHALLENGE_STATS'			=> append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=view_challenge_stats&amp;u={$user_id}"),
			'TOTAL_CHALLENGE_WINS'		=> $total_wins
		);
	}
}

function set_arcade_userdata(&$userdata)
{
	if (!$userdata['user_id'])
	{
		return;
	}

	global $db;

	// Add arcade user data to $user->data
	$sql = 'SELECT *
			FROM ' . ARCADE_USERS_TABLE . '
			WHERE user_id = ' . (int) $userdata['user_id'];
	$result = $db->sql_query($sql);
	$array = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);

	if (!$db->sql_affectedrows())
	{
		$array = arcade_user_add($userdata['user_id'], false);
	}

	$userdata = array_merge($userdata, $array);
}

function arcade_user_add($user_id, $new_user = true)
{
	global $db, $config;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	$sql_ary = array(
		'user_id'						=> (int) $user_id,
		'user_arcade_permissions'		=> '',
		'user_arcade_perm_from'			=> 0,
		'user_arcade_last_download'		=> 0,
		'user_arcade_last_play'			=> 0,
		'user_arcade_last_search'		=> 0,
		'user_arcade_last_search_term'	=> '',
		'user_arcade_options'			=> ($user_id == ANONYMOUS) ? 220 : 223,
		'games_per_page'				=> 0,
		'games_sort_dir'				=> 'd',
		'games_sort_order'				=> 'd'
	);

	$db->sql_query('INSERT INTO ' . ARCADE_USERS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));

	if ($new_user)
	{
		global $cache;

		$cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);
		return;
	}

	return $sql_ary;
}

function arcade_ghost_permissions($user_id, $username)
{
	global $user, $config, $phpbb_root_path, $phpEx, $auth_arcade;

	if (!isset($config['arcade_install']))
	{
		return '';
	}

	if (!isset($auth_arcade))
	{
		arcade_default_set();
	}

	include($phpbb_root_path . 'arcade/includes/auth_admin.' . $phpEx);
	$auth_arcade_admin = new auth_arcade_admin();
	$massage = 'ARCADE_SWITCH_' . (($auth_arcade_admin->ghost_permissions($user_id, $user->data['user_id'])) ? 'PERMISSION' : 'NO_PERMISSION');
	return (!empty($user->lang[$massage])) ? '<br />' . sprintf($user->lang[$massage], $username) : '';
}

function arcade_restore_perm()
{
	global $user, $config, $auth_arcade;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	if (!isset($auth_arcade))
	{
		arcade_default_set();
	}

	$auth_arcade->acl_cache($user->data);
}

function arcade_ban_users()
{
	global $db, $user;

	$cache_ttl = 3600;

	$sql = 'SELECT user_id
			FROM ' . ARCADE_USERS_BANNED_TABLE . '
			WHERE	(user_id <> ' . ANONYMOUS . ' AND user_id = ' . (int) $user->data['user_id'] . ") OR
					(banned_ip = '" . $db->sql_escape($user->ip) . "')";
	$result = $db->sql_query($sql, $cache_ttl);
	$banned = (int) $db->sql_fetchfield('user_id');
	$db->sql_freeresult($result);

	return ($banned) ? true : false;
}

function arcade_default_set($arcade_class = false)
{
	global $user, $phpbb_root_path, $phpEx, $arcade, $auth_arcade, $arcade_cache, $arcade_config;

	if (!class_exists('auth_arcade'))
	{
		include($phpbb_root_path . 'arcade/includes/auth.' . $phpEx);
	}

	if (!class_exists('arcade_cache'))
	{
		include($phpbb_root_path . 'arcade/includes/cache.' . $phpEx);
	}

	if (!isset($auth_arcade))
	{
		$user->add_lang('mods/arcade');
		set_arcade_userdata($user->data);
		$auth_arcade = new auth_arcade();
		$auth_arcade->acl($user->data);
	}

	if (!isset($arcade_cache))
	{
		$arcade_cache	= new arcade_cache();
		$arcade_config	= $arcade_cache->obtain_arcade_config();
	}

	if ($arcade_class && !isset($arcade))
	{
		include($phpbb_root_path . 'arcade/includes/common.' . $phpEx);
	}
}

function arcade_cache_purge()
{
	global $cache, $config;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	$cache->destroy('sql', ARCADE_CATS_TABLE);
	$cache->destroy('sql', ARCADE_GAMES_TABLE);
	$cache->destroy('sql', ARCADE_PLAYS_TABLE);
	$cache->destroy('sql', ARCADE_SCORES_TABLE);
	$cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);
	$cache->destroy('_arcade_leaders');
	$cache->destroy('_arcade_challenge_leaders');
}

function arcade_acl_clear_prefetch($user_id)
{
	global $config, $phpbb_root_path, $phpEx;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	if (!class_exists('auth_arcade'))
	{
		include($phpbb_root_path . 'arcade/includes/auth.' . $phpEx);
	}

	$auth_arcade = new auth_arcade();
	$auth_arcade->acl_clear_prefetch($user_id);
}

function arcade_delete_user($user_id)
{
	global $config, $arcade, $auth_arcade, $arcade_cache, $arcade_config, $phpbb_root_path, $phpEx;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	if (!class_exists('arcade_admin'))
	{
		if (!defined('USE_ARCADE_ADMIN'))
		{
			define('USE_ARCADE_ADMIN', true);
		}

		include($phpbb_root_path . 'arcade/includes/common.' . $phpEx);
	}

	$arcade->delete_user($user_id);
}

function arcade_update_user_colour($user_id_ary, $user_colour)
{
	global $db, $config;

	if (!isset($config['arcade_install']))
	{
		return;
	}

	$sql = 'UPDATE ' . ARCADE_CATS_TABLE . " SET cat_last_play_user_colour = '" . $db->sql_escape($user_colour) . "'
			WHERE " . $db->sql_in_set('cat_last_play_user_id', $user_id_ary);
	$db->sql_query($sql);

	arcade_cache_purge();
}

?>