<?php
/**
*
* @package arcade
* @version $Id: play.php 1957 2012-03-10 19:26:47Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

// Grab the data for game and navlinks
$sql_array = array(
	'SELECT'	=> 'g.game_id, g.game_name, g.game_desc, g.game_image, g.game_swf, g.game_width, g.game_height, g.game_scorevar, g.game_scoretype, g.game_type, game_download, g.game_filesize, g.game_votetotal, g.game_votesum, g.game_cost, g.game_reward, g.game_jackpot, g.game_use_jackpot, g.game_plays, g.game_download_cost,
					r.game_rating,
					c.*',
	'FROM'		=> array(
		ARCADE_GAMES_TABLE	=> 'g',
	),
	'LEFT_JOIN'	=> array(
		array(
			'FROM'	=> array(ARCADE_RATING_TABLE => 'r'),
			'ON'	=> 'g.game_id = r.game_id AND r.user_id = ' . (int) $user->data['user_id']
		),
		array(
			'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
			'ON'	=> 'g.cat_id = c.cat_id'
		),
	),
	'WHERE'		=> 'g.game_id = ' . (int) $game_id,
);

// This is only needed if we are playing in a popup
if ($popup)
{
	$sql_array['SELECT'] .= ', g.game_highuser, g.game_highscore, u.username';
	$sql_array['LEFT_JOIN'][] = array('FROM' => array(USERS_TABLE => 'u'), 'ON' => 'g.game_highuser = u.user_id');
}

$sql = $db->sql_build_query('SELECT', $sql_array);
$result = $db->sql_query($sql);
$row = $db->sql_fetchrow($result);
$db->sql_freeresult($result);

if (!$row)
{
	trigger_error($user->lang['NO_GAME_ID'] . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
}

if (!$popup && $row['cat_style'] && ($user->data['user_style'] != $row['cat_style']))
{
	$user->setup('', $row['cat_style']);
}

if (!$auth_arcade->acl_get('c_play', $row['cat_id']))
{
	trigger_error($user->lang['NO_PERMISSION_ARCADE_PLAY'] . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
}

if ($row['cat_status'] == ITEM_LOCKED)
{
	trigger_error($user->lang['ARCADE_CAT_LOCKED_ERROR'] . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
}

// Check flood control
$interval = ($user->data['user_id'] == ANONYMOUS) ? $arcade_config['play_anonymous_interval'] : $arcade_config['play_interval'];
if ($interval && !$auth_arcade->acl_get('c_ignoreflood_play', $row['cat_id']) && strpos($user->data['session_browser'] , 'www.facebook') === false)
{
	$time = time() - $interval;
	if ($user->data['user_arcade_last_play'] > $time)
	{
		trigger_error(sprintf($user->lang['ARCADE_NO_PLAY_TIME'], $arcade->time_format($interval, true), $arcade->time_format($user->data['user_arcade_last_play'] - $time, true)) . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
	}
}

// Check if the limt play is set in ACP
if ($arcade_config['limit_play'] != LIMIT_PLAY_TYPE_NONE && !$auth_arcade->acl_get('c_ignorecontrol', $row['cat_id']))
{
	$arcade->limit_play();
}

$page_title = $user->lang['ARCADE'] . ' - ' . $row['cat_name'] . ' - ' . $row['game_name'];

// age limit verify
if ($row['cat_age'])
{
	$arcade->verify_age('play', $row['cat_age']);
}

// Category is passworded ... check whether access has been granted to this
// user this session, if not show login box before playing the game
if ($row['cat_password'])
{
	login_arcade_box($row);
}

if ($row['cat_test'])
{
	$template->assign_var('S_ARCADE_CAT_TEST_MODE', true);
}

$challenge_enabled = (!$row['cat_test'] && !$arcade_config['challenge_disable']) ? true : false;

if ($challenge_enabled)
{
	$arcade->challenge_check($row, $popup);
}

$purge_games_cache = false;

// Handle points if needed and if we are not in a test category
if ($arcade->points['show'] && !$row['cat_test'])
{
	$game_cost = $arcade->get_cost($row);
	$game_reward = $arcade->get_reward($row);
	$game_download_cost = $arcade->get_download_cost($row);

	if ($game_cost != ARCADE_FREE && $game_cost > 0 && !$auth_arcade->acl_get('c_playfree', $row['cat_id']))
	{
		if (!$arcade->set_points('subtract', $user->data['user_id'], $game_cost))
		{
			trigger_error(sprintf($user->lang['ARCADE_NO_POINTS'], $arcade->points['name']) . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
		}

		// The jackpot is only increased if the user actually contributed something to play
		if ($arcade->use_jackpot($row))
		{
			$purge_games_cache = true;
			$game_reward = $arcade->set_jackpot('add', $row);
		}
	}
}

$gidencoded = $arcade_session->begin(true, $row['game_id'], $row['game_type'], $popup);

if (!$gidencoded)
{
	$lang_ary = array(
		'MESSAGE_TITLE'		=> $user->lang['CONFIRM'],
		'MESSAGE_TEXT'		=> sprintf($user->lang['ARCADE_RESTART_GAME_EXPLAIN'], $row['game_name']),
		'CONFIRM_BUTTON1'	=> sprintf($user->lang['ARCADE_RESTART_GAME'], $row['game_name']),
		'PAGE_TITLE'		=> sprintf($user->lang['ARCADE_RESTART_GAME'], $row['game_name'])
	);

	$arcade->confirm($lang_ary, $popup, $row['game_id']);
}

// Increase play count for games that do not submit scores
// only if we are not in a test category, we are basically
// only counting views for non scoring games and the users
// play count and time will not be increased. Currently if
// a NOSCORE_GAME is resynced it will reset the plays back
// to zero because there will be no data inside the plays
// table to accurately resync with.
if ($row['game_type'] == NOSCORE_GAME && !$row['cat_test'])
{
	$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
			SET game_plays = game_plays + 1
			WHERE game_id = ' . (int) $row['game_id'];
	$db->sql_query($sql);

	$purge_games_cache = true;
}

if ($purge_games_cache)
{
	$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
}

$game_width = $game_height = 0;
$arcade->set_game_size($game_width, $game_height, $row['game_width'], $row['game_height'], $row['game_swf']);
$game_file = $file_functions->remove_extension($row['game_swf']);
$game_swf = ($arcade->get_protection($row['game_type'])) ? $arcade->url("swf=" . $game_file) : $arcade->set_path($row['game_swf']);

$game_desc = censor_text($arcade->nl2br_plus((($arcade_config['game_desc_parse_html']) ? htmlspecialchars_decode($row['game_desc']) : $row['game_desc'])));

switch($mode)
{
	case 'play':
		generate_arcade_nav($row, true);

		// Get the favorite games data for the current user...
		$game_fav_data	= $arcade->get_fav_data();

		$template->assign_vars(array(
			'S_ARCADE_FAVORITE'		=> ($auth_arcade->acl_get('u_arcade_favorites')) ? true : false,
			'S_RESOLUTION_SELECT'	=> ($arcade_config['resolution_select'] && $auth_arcade->acl_get('u_arcade_resolution')) ? true : false,
			'S_GAME_DOWNLOAD'		=> ($row['cat_download'] && $row['game_download'] && $auth_arcade->acl_get('u_arcade_download') && $auth_arcade->acl_get('c_download', $row['cat_id'])) ? true : false,
			'S_CAN_REPORT'			=> ($auth_arcade->acl_get('c_report', $row['cat_id'])) ? true : false,
			'S_DISPLAY_DESC'		=> $arcade_config['display_desc'],
			'S_DISPLAY_GAME_TYPE'	=> $arcade_config['display_game_type'],
			'S_FEELING_LUCKY'		=> ($auth_arcade->acl_getc_global('c_play') && sizeof($arcade->games)) ? true : false,

			'U_GAME_DOWNLOAD'		=> $arcade->url("mode=download&amp;g={$row['game_id']}"),
			'U_GAME_REPORT'			=> $arcade->url("mode=report&amp;g={$row['game_id']}") . '#report_game',
			'U_GAME_EDIT'			=> ($auth->acl_get('a_arcade_game') && !empty($user->data['is_registered'])) ? $arcade->url("i=arcade_manage&amp;mode=games&amp;action=edit&amp;g={$row['game_id']}", "adm/index", $user->session_id) : '',
			'U_SCORE_EDIT'			=> ($auth->acl_get('a_arcade_user') && $row['game_plays'] && !empty($user->data['is_registered'])) ? $arcade->url("i=arcade_manage&amp;mode=users&amp;action=show_scores&amp;g={$row['game_id']}", "adm/index", $user->session_id) : '',

			'GAME_TYPE'				=> $arcade->display_game_type($row['game_type']),
			'GAME_FILESIZE'			=> ($row['game_filesize'] > 0 ) ? sprintf($user->lang['ARCADE_GAMES_FILESIZE'], get_formatted_filesize($row['game_filesize'])) : sprintf($user->lang['ARCADE_GAMES_FILESIZE'], get_formatted_filesize($arcade->set_filesize($row['game_id']))),
			'GAME_FAV_IMG'			=> (is_array($game_fav_data)) ? $arcade->set_fav_image($game_fav_data, $row['game_id']) : '',
			'GAME_RATING_IMG'		=> $arcade->set_rating_image($row),
			'SMALL_IMAGE'			=> $arcade->get_image('src', 'img', 'small.png'),
			'BIG_IMAGE'				=> $arcade->get_image('src', 'img', 'big.png'),
			'AUTO_IMAGE'			=> $arcade->get_image('src', 'img', 'automatic.png'),
			'OPEN_CLOSE_IMAGE'		=> $arcade->get_image('src', 'img', 'open_close.png'),
			'L_RESTART_GAME'		=> sprintf($user->lang['ARCADE_RESTART_GAME'], $row['game_name'])
		));

		if (!$user->data['is_bot'] && ($arcade_config['facebook_like_enable'] || $arcade_config['google_like_enable'] || $arcade_config['tweet_like_enable']))
		{
			$anonymous_data = $arcade->userdata('set_auth', ANONYMOUS);

			$auth_arcade2 = new auth_arcade();
			$auth_arcade2->acl($anonymous_data);

			$share_active = true;
			if ($config['board_disable'] || $row['cat_age'] || $row['cat_password'] || !$auth_arcade2->acl_get('u_arcade') || !$auth_arcade2->acl_get('c_play', $row['cat_id']) || ($arcade_config['limit_play'] != LIMIT_PLAY_TYPE_NONE && !$auth_arcade2->acl_get('c_ignorecontrol', $row['cat_id'])) || ($arcade->points['show'] && !$row['cat_test'] && $game_cost != ARCADE_FREE && $game_cost > 0 && !$auth_arcade2->acl_get('c_playfree', $row['cat_id'])))
			{
				$share_active = false;
			}

			unset($auth_arcade2);

			if ($share_active)
			{
				$share_gamename	= sprintf($user->lang['ARCADE_GAME_NAME_FLASH_GAME'], $row['game_name']);
				$game_urlencode	= urlencode(generate_board_url() . "/arcade.{$phpEx}?mode=play&amp;g={$row['game_id']}" . $arcade->gametop);
				$table_bgc		= ($arcade_config['like_background_color']) ? str_replace('#', '', $arcade_config['like_background_color']) : false;
				$share_image	= ($row['game_image'] !== '') ? generate_board_url() . '/arcade.' . $phpEx . '?img=' . $row['game_image'] : false;
				$arcade_meta	= ($arcade_config['facebook_like_enable']) ? '<meta property="og:site_name" content="' . $config['sitename'] . '" />' . "\n" . '<meta property="og:description" content="' . $game_desc . '" />' . "\n" . '<meta property="og:title" content="' . $share_gamename . '" />' . (($share_image) ? "\n" . '<meta property="og:image" content="' . $share_image . '" />' : '') : '';
				$arcade_meta   .= ($arcade_config['google_like_enable']) ? (($arcade_config['facebook_like_enable']) ? "\n" : '') . '<meta itemprop="name" content="' . $share_gamename . '" />' . "\n" . '<meta itemprop="description" content="' . $row['game_desc'] . '" />' . (($share_image) ? "\n" . '<meta itemprop="image" content="' . $share_image . '" />' : '') : '';

				$template->assign_vars(array(
					'ARCADE_META'			=> $arcade_meta,

					'ARCADE_SHARE_HEIGHT'	=> ($arcade_config['like_counter_enable']) ? '72' : '35',
					'ARCADE_SHARE_BGC'		=> $table_bgc,
					'S_ARCADE_FB'			=> ($arcade_config['facebook_like_enable']) ? true : false,
					'S_ARCADE_GOOGLE'		=> ($arcade_config['google_like_enable']) ? true : false,
					'S_ARCADE_TWEET'		=> ($arcade_config['tweet_like_enable']) ? true : false,

					'FACEBOOK_POSITION'		=> ($arcade_config['google_like_enable'] && $arcade_config['tweet_like_enable']) ? 'left:10px;' : (($arcade_config['google_like_enable'] || $arcade_config['tweet_like_enable']) ? 'left: 45px;' : 'left: 0; right: 0;'),
					'GOOGLE_POSITION'		=> ($arcade_config['facebook_like_enable'] && $arcade_config['tweet_like_enable']) ? 'left: 0; right: 0;' : (($arcade_config['facebook_like_enable']) ? 'right: 45px;' : (($arcade_config['tweet_like_enable']) ? 'left: 45px;' : 'left: 0; right: 0;')),
					'TWEET_POSITION'		=> ($arcade_config['facebook_like_enable'] && $arcade_config['google_like_enable']) ? 'right:10px;' : (($arcade_config['facebook_like_enable'] || $arcade_config['google_like_enable']) ? 'right: 45px;' : 'left: 0; right: 0;'),

					'ARCADE_FB_SRC'			=> "http://www.facebook.com/plugins/like.{$phpEx}?href={$game_urlencode}" . (($arcade_config['like_counter_enable']) ? '&amp;layout=box_count' : '') . "&amp;width=90&amp;action=like&amp;colorscheme=light&amp;height=65",
					'ARCADE_FB_DATA'		=> 'allowTransparency="true" scrolling="no" frameborder="0" style="' . ((!$arcade_config['google_like_enable'] && !$arcade_config['tweet_like_enable']) ? 'margin-left: 30px; ' : '') . 'border:none; overflow:hidden; width: ' . (($arcade_config['like_counter_enable']) ? '90' : '65') . 'px; height: 65px;"',
					'ARCADE_GOOGLE_DATA'	=> 'size="tall"' . (($arcade_config['like_counter_enable']) ? '' : ' annotation="none"') . ' href="' . $game_urlencode . '"',
					'ARCADE_TWEET_SRC'		=> "http://platform.twitter.com/widgets/tweet_button.html?url={$game_urlencode}&amp;count=" . (($arcade_config['like_counter_enable']) ? 'vertical' : 'none') . "&amp;text={$share_gamename}&amp;lang={$user->lang['USER_LANG']}",
					'ARCADE_TWEET_DATA'		=> 'style="width: 70px; height: 65px;" frameborder="0" scrolling="no" allowtransparency="true"'
				));

				if ($arcade_config['share_icons_enable'])
				{
					$template->assign_vars(array(
						'S_ARCADE_SHARE_ICON'	=> true,
						'FACEBOOK_ICON'			=> $arcade->get_image('src', 'img', 'facebook.gif'),
						'TWEET_ICON'			=> $arcade->get_image('src', 'img', 'twitter.gif')
					));
				}
			}
		}

		if ($arcade->points['show'] && !$row['cat_test'])
		{
			$template->assign_vars(array(
				'S_SHOW_POINTS'	=> true,
				'S_USE_JACKPOT'	=> ($arcade->use_jackpot($row)) ? true : false,

				'AUSER_POINTS'	=> $arcade->number_format($arcade->points['total']) . ' ' . $arcade->points['name'],
				'POINTS_NAME'	=> $arcade->points['name'],
				'GAME_COST'		=> ($game_cost == ARCADE_FREE || $auth_arcade->acl_get('c_playfree', $row['cat_id'])) ? $user->lang['ARCADE_FREE'] : $arcade->number_format($game_cost) . ' ' . $arcade->points['name'],
				'GAME_REWARD'	=> ($game_reward == ARCADE_FREE) ? $user->lang['ARCADE_NONE'] : $arcade->number_format($game_reward) . ' ' . $arcade->points['name'],
				'DOWNLOAD_COST'	=> ($game_download_cost != ARCADE_FREE && !$auth_arcade->acl_get('c_downloadfree', $row['cat_id'])) ? $arcade->number_format($game_download_cost) . ' ' . $arcade->points['name'] : ''
			));
		}
	break;

	case 'popup':
		$highscore = (!empty($row['username']) && !$arcade->hidden_score($row['game_id'], $row['game_highuser'])) ? $arcade->number_format($row['game_highscore']) : $user->lang['HIDDEN'];
		$popup_title = (!empty($row['username'])) ? sprintf($user->lang['ARCADE_POPUP_HIGHUSER'], $row['game_name'], $row['username'], $highscore) : sprintf($user->lang['ARCADE_POP_NO_HIGHSCORE'], $row['game_name']);

		$template->assign_var('POPUP_TITLE', $popup_title);
	break;
}

$scores_limit = intval($arcade_config['game_scores']);

if (!$scores_limit)
{
	$scores_limit = 3;
	$template->assign_var('S_SCORE_DISABLED', true);
}

$score_columns = ($arcade_config['game_comment']) ? 's.*' : 's.score, s.score_date';
$score_order = ($row['game_scoretype'] == SCORETYPE_HIGH) ? 'DESC' : 'ASC';

$sql_array = array(
	'SELECT'	=> $score_columns . ', p.total_time, p.total_plays, u.username, u.user_id, u.user_colour, u.user_avatar_type, u.user_avatar, u.user_avatar_width, u.user_avatar_height, a.user_arcade_options',
	'FROM'		=> array(
		ARCADE_SCORES_TABLE	=> 's',
	),
	'LEFT_JOIN'	=> array(
		array(
			'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
			'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
		),
		array(
			'FROM'	=> array(USERS_TABLE => 'u'),
			'ON'	=> 's.user_id = u.user_id'
		),
		array(
			'FROM'	=> array(ARCADE_USERS_TABLE => 'a'),
			'ON'	=> 's.user_id = a.user_id'
		),
	),
	'WHERE'		=> 's.game_id = ' . (int) $row['game_id'] . '
		AND s.user_id = u.user_id',
	'ORDER_BY'	=> 's.score ' . $score_order . ', s.score_date ASC',
);

$sql = $db->sql_build_query('SELECT', $sql_array);
$result = $db->sql_query_limit($sql, $scores_limit);
$score_data = $db->sql_fetchrowset($result);
$db->sql_freeresult($result);

if (!empty($score_data))
{
	$top_user = $position = $actual_position = 0;
	$lastscore = '';

	foreach ($score_data as $data)
	{
		$actual_position++;

		if ($user->data['user_id'] != $data['user_id'] && $actual_position > $scores_limit)
		{
			continue;
		}

		// Handle the game champion
		if ($actual_position == 1)
		{
			$template->assign_vars(array(
				'S_GAME_CHAMPION'		=> true,
				'S_CHALLENGE_CHAMPION'	=> ($user->data['is_registered'] && $challenge_enabled && $user->data['user_id'] != $data['user_id'] && $arcade->optionget('challenge', $data['user_arcade_options'], true)) ? true : false,
				'S_USER_ID'				=> $user->data['user_id'],

				'U_CHALLENGE_CHAMPION'	=> $arcade->url('mode=champion_challenge&amp;g=' . $row['game_id'] . '&amp;u=' . $data['user_id']),

				'BEST_USER_NAME'		=> get_arcade_user_name('full', $data['user_id'], $data['username'], $data['user_colour'], 'arcade', 'x'),
				'BEST_COMMENT'			=> ($arcade_config['game_comment'] && $data['comment_text'] !== '') ? generate_text_for_display($data['comment_text'], $data['comment_uid'], $data['comment_bitfield'], $data['comment_options']) : '',
				'BEST_PLAYED'			=> $arcade->number_format($data['total_plays']),
				'BEST_TIME'				=> $arcade->time_format($data['total_time']),
				'BEST_SCORE'			=> (!$arcade->hidden_score($row['game_id'], $data['user_id'], $data['score_date'])) ? $arcade->number_format($data['score']) : $user->lang['HIDDEN'],
				'BEST_DATE_EXPLAIN'		=> sprintf($user->lang['ARCADE_BEST_DATE_EXPLAIN'], $user->format_date($data['score_date'])),
				'FIRST_AVATAR'			=> ($arcade->optionget('view_avatars')) ? get_arcade_user_avatar($data['user_avatar'], $data['user_avatar_type'], $data['user_avatar_width'], $data['user_avatar_height'], 'x', 'arcade', $data['user_id']) : false
			));

			if ($challenge_enabled && $arcade->points['show'])
			{
				$template->assign_vars(array(
					'S_CHALLENGE_BET_FIX'	=> ($arcade_config['challenge_bet_fix']) ? true : false,

					'CHALLENGE_BET_MIN'		=> ($arcade_config['challenge_bet_Minimum']) ? sprintf($user->lang['CHALLENGE_BET_MIN'], $arcade->number_format($arcade_config['challenge_bet_Minimum']) . ' ' . $arcade->points['name']) : false,
					'CHALLENGE_BET_MAX'		=> ($arcade_config['challenge_bet_Maximum']) ? sprintf($user->lang['CHALLENGE_BET_MAX'], $arcade->number_format($arcade_config['challenge_bet_Maximum']) . ' ' . $arcade->points['name']) : false,
					'CHALLENGE_BET'			=> number_format((($arcade_config['challenge_bet_fix']) ? $arcade_config['challenge_bet_fix'] : $arcade_config['challenge_bet_Minimum']), 3)
				));
			}
		}

		if ($lastscore != $data['score'])
		{
			$position = $actual_position;
		}

		$top_user++;
		$lastscore = $data['score'];

		$template->assign_block_vars('scorerow', array(
			'TOP'			=> $top_user,
			'POS'			=> $position,
			'USER_ID'		=> $data['user_id'],
			'USERNAME'		=> $data['username'],
			'SCORE'			=> (!$arcade->hidden_score($row['game_id'], $data['user_id'], $data['score_date'])) ? $arcade->number_format($data['score']) : $user->lang['HIDDEN'],
			'FULL_USERNAME'	=> get_arcade_user_name('full', $data['user_id'], $data['username'], $data['user_colour'], 'arcade', 'x')
		));

		// We break here because we no longer need to loop through the data, we have what we need
		if ($user->data['user_id'] == $data['user_id'] && $actual_position > $scores_limit)
		{
			break;
		}
	}
}

// We will set this cookie to let the arcade know that the user is playing the game
// in a popup window and not on fullscreen. It will later be checked in score.php
$arcade->update_last_play_time($row['game_id'], $row['cat_id']);

$template->assign_vars(array(
	'L_ARCADE_FLASH_VERSION'=> sprintf($user->lang['ARCADE_FLASH_VERSION'], $arcade_config['flash_version']),

	'T_ARCADE_FLASH_UPDATE'	=> (file_exists($phpbb_root_path . 'arcade/swf/flash_player_update.swf')) ? $arcade->url("swf=flash_player_update&amp;p=1") : false,

	'S_ARCADE_GAME_PLAY'	=> true,
	'S_POPUP'				=> ($popup) ? true : false,
	'S_USE_HIGHSCORES'		=> ($row['game_type'] == NOSCORE_GAME) ? false : true,

	'FLASH_VERSION'			=> $arcade_config['flash_version'],
	'ARCADE_SWF'			=> $arcade->url("swf=arcade&p=1"),
	'ARCADE_GAME_SWF'		=> $game_swf,
	'GAME_ID'				=> $row["game_id"],
	'GAME_SID'				=> $arcade_session->game_sid,
	'GAME_GID'				=> $gidencoded,
	'GAME_SCOREVAR'			=> phpbb_hash($row["game_scorevar"]),
	'GAME_NAME'				=> $row['game_name'],
	'GAME_WIDTH'			=> $game_width,
	'GAME_HEIGHT'			=> $game_height,
	'GAME_DESC'				=> $game_desc
));

if ($mode == 'play')
{
	display_arcade_online();
}

page_header($page_title, false);

$html_body = ($mode == 'play') ? 'play_body' : 'popup_body';

$template->set_filenames(array(
	'body' => 'arcade/' . $html_body . '.html')
);

page_footer();

?>