<?php
/**
*
* @package arcade
* @version $Id: score.php 1953 2012-03-10 13:03:18Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

// If you are not logged in the score is never saved...
// Lets show you a nice message and send you on your way.
if (!$user->data['is_registered'])
{
	$arcade->update_last_play_time($game_data['game_id'], $game_data['cat_id']);

	$total_games = $arcade->get_total('games');
	$message = sprintf($user->lang['ARCADE_REGISTER_MESSAGE_SCORE'], $total_games);
	$message .= ($popup) ? $arcade->close_win() : '<br />' . sprintf($user->lang['ARCADE_REGISTER_LOGIN_MESSAGE'], '<a href="' . append_sid("{$phpbb_root_path}ucp.$phpEx", "mode=register") . '">', '</a>', '<a href="' . append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=login&amp;redirect=' . urlencode("arcade.$phpEx")) . '">', '</a>');


	$arcade_session->set_cookie();
	trigger_error($message);
}

$mode = (!empty($mode)) ? $mode : '';

// We can't continue if we don't have a game_id
if (($mode == 'score' || $mode == 'done') && !$game_id)
{
	$arcade_session->set_cookie();
	redirect($arcade->url());
}

if ($arcade_config['game_comment'])
{
	$user->add_lang('posting');
}

// This handles the data from the screen you see immediately after playing a game.
// The handling of the comments are done here.
if ($mode == 'score' && $game_id)
{
	if (!check_form_key('arcade_score'))
	{
		$arcade_session->set_cookie();
		trigger_error($user->lang['FORM_INVALID'] . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
	}

	$cat_id = $arcade->get_game_field($game_id, 'cat_id', true);
	if (!$cat_id || !$auth_arcade->acl_get('c_score', $cat_id))
	{
		$arcade_session->set_cookie();
		trigger_error($user->lang['NO_PERMISSION_ARCADE_SCORE'] . ((!$popup) ? $arcade->back_link() : $arcade->close_win()));
	}

	if ($arcade_config['game_comment'] && $auth_arcade->acl_get('c_comment', $cat_id))
	{
		$comment_data = array(
			'comment_text'		=> utf8_normalize_nfc(request_var('message', '', true)),
			'comment_bitfield'	=> '',
			'comment_options'	=> 0,
			'comment_uid'		=> '',
		);

		$enable_bbcode	= ($arcade_config['parse_bbcode']) ? ((request_var('disable_bbcode', false)) ? false : true) : false;
		$enable_smilies	= ($arcade_config['parse_smilies']) ? ((request_var('disable_smilies', false)) ? false : true) : false;
		$enable_urls	= ($arcade_config['parse_links']) ? ((request_var('disable_magic_url', false)) ? false : true) : false;

		generate_text_for_storage($comment_data['comment_text'], $comment_data['comment_uid'], $comment_data['comment_bitfield'], $comment_data['comment_options'], $enable_bbcode, $enable_urls, $enable_smilies);

		$sql = 'UPDATE ' . ARCADE_SCORES_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $comment_data) . '
				WHERE user_id = ' . (int) $user->data['user_id'] . '
				AND   game_id = ' . (int) $game_id;
		$db->sql_query($sql);
	}

	// Everything is all set so lets display the finished screen...
	redirect($arcade->url("mode=done&amp;g={$game_id}&amp;sp={$popup}") . '#game_done');
}
// This displays the messages and links that are shown after
// the comments have been selected.
else if ($mode == 'done' && $game_id)
{
	$arcade->score_done($game_id, $popup, false);
}

if (!$auth_arcade->acl_get('c_score', $game_data['cat_id']))
{
	$arcade_session->set_cookie();
	trigger_error($user->lang['NO_PERMISSION_ARCADE_SCORE'] . $arcade->return_links($game_data, true, $popup));
}

if (!$popup)
{
	generate_arcade_nav($game_data, true);
}

$error = false;
$lang = '';
if ($arcade_config['game_zero_negative_score'])
{
	if (!isset($score))
	{
		$error	= true;
		$lang	= 'ARCADE_SCORE_ERROR';
	}
}
else
{
	if (!isset($score) || $score <= 0)
	{
		$error	= true;
		$lang	= 'ARCADE_ZERO_NEGATIVE_SCORE';
	}
}

if ($game_data['cat_test'])
{
	$template->assign_var('S_ARCADE_CAT_TEST_MODE', true);
}

if ($error || $game_data['cat_test'])
{
	if ($game_data['cat_test'])
	{
		$message = sprintf($user->lang['ARCADE_TEST_SCORE'], $score) . $arcade->return_links($game_data, true, $popup);
		if ($auth->acl_get('a_') && defined('DEBUG_EXTRA'))
		{
			$message .= '<br /><br />' . arcade_debug($game_data);
		}
	}
	else
	{
		$message = $user->lang[$lang] . $arcade->return_links($game_data, true, $popup);
	}

	$arcade_session->set_cookie();
	trigger_error($message);
}

$comment_columns = ($arcade_config['game_comment']) ? ', comment_text, comment_bitfield, comment_options, comment_uid' : '';

$sql = "SELECT score $comment_columns
		FROM " . ARCADE_SCORES_TABLE . '
		WHERE game_id = ' . (int) $game_data['game_id'] . '
		AND   user_id = ' . (int) $user->data['user_id'];
$result = $db->sql_query($sql);
$saved_score = false;
$saved_highscore = false;
$reward = false;
$high_score_type = ($game_data['game_scoretype'] == SCORETYPE_HIGH) ? true : false;

// If the user has no current score we insert it.
if (!($row = $db->sql_fetchrow($result)))
{
	$sql_ary = array(
		'game_id'		=> (int) $game_data['game_id'],
		'user_id'		=> (int) $user->data['user_id'],
		'score'			=> (float) $score,
		'score_date'	=> (int) $game_data['current_time'],
		'comment_text'	=> '',
	);

	$db->sql_query('INSERT INTO ' . ARCADE_SCORES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));

	$saved_score = true;
}
else
{
	$old_score = $row['score'];
	// So you have an old score, is you new one better?  (Make sure we check the scoretype)
	if (($high_score_type && $old_score < $score) || (!$high_score_type && $old_score > $score))
	{
		// Update score data
		$sql_ary = array(
			'score'			=> $score,
			'score_date'	=> $game_data['current_time']
		);

		$sql = 'UPDATE ' . ARCADE_SCORES_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE game_id = ' . (int) $game_data['game_id'] . '
				AND user_id   = ' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		$saved_score = true;
	}
}

if ($arcade_config['game_comment'])
{
	$comment_data = array(
		'text'			=> $row['comment_text'],
		'allow_bbcode'	=> true,
		'allow_smilies'	=> true,
		'allow_urls'	=> true
	);

	if (isset($row['comment_text']))
	{
		$comment_data = generate_text_for_edit($row['comment_text'], $row['comment_uid'], $row['comment_options']);
	}
}

$db->sql_freeresult($result);

// Update play data
$sql = 'UPDATE ' . ARCADE_PLAYS_TABLE . '
		SET total_plays = total_plays + 1, total_time = total_time + ' . (int) $game_data['total_time'] . '
		WHERE game_id = ' . (int) $game_data['game_id'] . '
		AND user_id   = ' . (int) $user->data['user_id'];
$db->sql_query($sql);

if (!$db->sql_affectedrows())
{
	$sql = 'INSERT INTO ' . ARCADE_PLAYS_TABLE . ' ' . $db->sql_build_array('INSERT', array(
		'game_id'		=> (int) $game_data['game_id'],
		'user_id'		=> (int) $user->data['user_id'],
		'total_time'	=> (int) $game_data['total_time'],
		'total_plays'	=> 1,
	));
	$db->sql_query($sql);
}

$arcade->update_last_play_time($game_data['game_id'], $game_data['cat_id']);
$arcade->set_config('total_plays', intval($arcade_config['total_plays'] + 1), true);
$arcade->set_config('total_plays_time', intval($arcade_config['total_plays_time'] + $game_data['total_time']), true);

// Here we update the category information, this is done to save SQL queries
$sql_ary = array(
	'cat_last_play_game_id'		=> $game_data['game_id'],
	'cat_last_play_game_name'	=> $game_data['game_name'],
	'cat_last_play_user_id'		=> $user->data['user_id'],
	'cat_last_play_score'		=> $score,
	'cat_last_play_time'		=> $game_data['current_time'],
	'cat_last_play_username'	=> $user->data['username'],
	'cat_last_play_user_colour'	=> $user->data['user_colour'],
);

$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
		SET cat_plays = cat_plays + 1, ' .
		$db->sql_build_array('UPDATE', $sql_ary) . '
		WHERE cat_id = ' . (int) $game_data['cat_id'];
$db->sql_query($sql);

// Here we update the total times a game has been played
$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
		SET game_plays = game_plays + 1
		WHERE game_id = ' . (int) $game_data['game_id'];
$db->sql_query($sql);

// This needs to be checked incase the game has no score yet and we are scoring lowest to highest
// If this was not checked there would never be a highscore holder for games that score
// lowest to highest.
$has_highscore = ($game_data['game_highuser'] == 0 && $game_data['game_highdate'] == 0) ? false : true;
$first_score = (!$has_highscore) ? true : false;
$game_data['game_highscore'] = (!$high_score_type && !$has_highscore) ? $score + 1 : $game_data['game_highscore'];

$save_highscore = (($high_score_type && $game_data['game_highscore'] < $score) || (!$high_score_type && $game_data['game_highscore']  > $score) || ($arcade_config['game_zero_negative_score'] && $score <= 0 && !$has_highscore)) ? true : false;
$send_pm = ($save_highscore && $game_data['game_highuser'] > 0 && $arcade_config['send_arcade_pm'] && ($game_data['game_highuser'] != $user->data['user_id']) && !$arcade->hidden_score($game_data['game_id'], $game_data['game_highuser'])) ? true : false;

if (!$game_data['cat_test'] && !$arcade_config['challenge_disable'])
{
	$arcade->challenge($game_data, $score, $send_pm);
}

// Here we check if you are the new highscore holder
if ($save_highscore)
{
	$sql_ary = array(
		'game_highscore'=> $score,
		'game_highuser'	=> $user->data['user_id'],
		'game_highdate'	=> $game_data['current_time'],
	);

	$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
			SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
			WHERE game_id = ' . (int) $game_data['game_id'];
	$db->sql_query($sql);

	$saved_highscore = true;

	if ($arcade->points['show'])
	{
		$reward = $arcade->get_reward($game_data);
		if ($reward == ARCADE_FREE || ($auth_arcade->acl_get('c_playfree', $game_data['cat_id']) && !$arcade_config['playfree_reward']))
		{
			$reward = false;
		}
		else
		{
			// If we are using the jackpot setting we must clear the jackpot
			if ($arcade->use_jackpot($game_data))
			{
				$arcade->set_jackpot('clear', $game_data);
			}
			$arcade->set_points('add', $user->data['user_id'], $reward);
		}
	}

	// We don't need to send a pm if there was never a highscore...
	// We don't need to send a pm if we beat our own score...
	if ($send_pm)
	{
		$other_user = $arcade->userdata('set_auth', $game_data['game_highuser']);

		if ($arcade->optionget('arcade_pm', $other_user['user_arcade_options'], true))
		{
			$auth_arcade2 = new auth_arcade();
			$auth_arcade2->acl($other_user);

			// This sets the score data that is available to use
			// in the pm sent when you lose a highscore.  It
			// is customizable in the ACP
			$score_data = array(
				'PM_SUBJECT'				=> 'ARCADE_PM_SUBJECT',
				'PM_MESSAGE'				=> 'ARCADE_PM_MESSAGE',
				'cat_id'					=> $game_data['cat_id'],
				'game_id'					=> $game_data['game_id'],
				'game_name'					=> $game_data['game_name'],
				'old_score'					=> $arcade->number_format($game_data['game_highscore']),
				'new_score'					=> $arcade->number_format($score),
				'game_image'				=> $game_data['game_image'],

				'game_width'				=> $game_data['game_width'],
				'game_height'				=> $game_data['game_height'],

				'other_user_id'				=> $other_user['user_id'],
				'other_username'			=> $other_user['username'],
				'other_user_lang'			=> $other_user['user_lang'],
				'other_user_arcade_options'	=> $other_user['user_arcade_options']
			);

			unset($other_user);

			if ($auth_arcade2->acl_get('u_arcade_pm'))
			{
				$arcade->send_pm('arcade', $score_data);
			}

			unset($score_data);
			unset($auth_arcade2);
		}
	}
}

if ($save_highscore)
{
	$arcade_cache->destroy('_arcade_leaders');
	$arcade_cache->destroy('_arcade_leaders_all');
}

$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

if (!$saved_score)
{
	$score_desc = sprintf($user->lang['ARCADE_NO_SCORE_SAVED'], $arcade->number_format($score), $arcade->number_format($old_score));
}
else if (!$saved_highscore)
{
	$game_highscore = (!$arcade->hidden_score($game_data['game_id'], $game_data['game_highuser'])) ? $arcade->number_format($game_data['game_highscore']) : $user->lang['HIDDEN'];
	$score_desc = sprintf($user->lang['ARCADE_SCORE_SAVED'], $arcade->number_format($score), $game_highscore);
}
else
{
	if ($first_score)
	{
		$score_desc = sprintf($user->lang['ARCADE_HIGH_SCORE_SAVED_NEW'], $game_data['game_name'], $arcade->number_format($score));
	}
	else
	{
		$score_desc = sprintf($user->lang['ARCADE_HIGH_SCORE_SAVED'], $game_data['game_name'], $arcade->number_format($score), $arcade->number_format($game_data['game_highscore']));
	}
}

if ($arcade->points['show'] && $reward)
{
	$score_desc .= '<br /><br />' . sprintf($user->lang['ARCADE_REWARD_MESSAGE'], $arcade->number_format($reward) . ' ' . $arcade->points['name'], $arcade->points['name'], $arcade->number_format($arcade->points['total']) . ' ' . $arcade->points['name']);
}

// User is not allowed to comment the game so display the final score explanation and redirect to the finished page...
if ($arcade_config['game_comment'] && $auth_arcade->acl_get('c_comment', $game_data['cat_id']))
{
	$s_hidden_fields = build_hidden_fields(array(
		'mode'	=> 'score',
		'g'		=> $game_data['game_id'],
		'c'		=> $game_data['cat_id'],
		'sp'	=> $popup
	));

	add_form_key('arcade_score');

	$enable_bbcode	= ($arcade_config['parse_bbcode']) ? (bool) $arcade->optionget('bbcode') : false;
	$enable_smilies	= ($arcade_config['parse_smilies']) ? (bool) $arcade->optionget('smilies') : false;
	$enable_urls	= ($arcade_config['parse_links']) ? true : false;
	$enable_img		= false;
	$enable_flash	= false;
	$enable_qoute	= false;

	if ($enable_smilies && !$popup)
	{
		// Generate smiley listing
		if (!function_exists('generate_smilies'))
		{
			include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
		}

		generate_smilies('inline', false);
	}

	if ($enable_bbcode)
	{
		// Build custom bbcodes array
		if (!function_exists('display_custom_bbcodes'))
		{
			include($phpbb_root_path . 'includes/functions_display.' . $phpEx);
		}

		display_custom_bbcodes();
	}

	$template->assign_vars(array(
		'S_USE_SIMPLE_HEADER'	=> $popup,
		'S_ACTION'				=> $arcade->url(),
		'S_HIDDEN_FIELDS'		=> $s_hidden_fields,
		'S_BBCODE_ALLOWED'		=> $enable_bbcode,
		'S_SMILIES_ALLOWED'		=> ($enable_smilies) ? true : false,
		'S_SMILIES_POPUP'		=> ($enable_smilies && $popup) ? true : false,
		'S_LINKS_ALLOWED'		=> $enable_urls,

		'S_BBCODE_CHECKED'		=> (!$comment_data['allow_bbcode']) ? ' checked="checked"' : '',
		'S_SMILIES_CHECKED'		=> (!$comment_data['allow_smilies']) ? ' checked="checked"' : '',
		'S_MAGIC_URL_CHECKED'	=> (!$comment_data['allow_urls']) ? ' checked="checked"' : '',

		'S_BBCODE_QUOTE'		=> $enable_qoute,
		'S_BBCODE_FLASH'		=> $enable_flash,
		'S_BBCODE_IMG'			=> $enable_img,

		'U_SMILIES'				=> append_sid("{$phpbb_root_path}posting.$phpEx", 'mode=smilies'),

		'BBCODE_STATUS'			=> sprintf($user->lang['BBCODE_IS_' . (($enable_bbcode) ? 'ON' : 'OFF')], '<a href="' . append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode') . '">', '</a>'),
		'IMG_STATUS'			=> $user->lang['IMAGES_ARE_' . (($enable_img) ? 'ON' : 'OFF')],
		'FLASH_STATUS'			=> $user->lang['FLASH_IS_' . (($enable_flash) ? 'ON' : 'OFF')],
		'SMILIES_STATUS'		=> $user->lang['SMILIES_ARE_' . (($enable_smilies) ? 'ON' : 'OFF')],
		'URL_STATUS'			=> $user->lang['URL_IS_' . (($enable_urls) ? 'ON' : 'OFF')],

		'ARCADE_SCORE_DESC'		=> $score_desc,
		'GAME_NAME'				=> $game_data['game_name'],
		'SCORE_COMMENT'			=> $comment_data['text'],
		'POPUP_HEIGHT'			=> $arcade_config['smilies_popup_height'],
		'POPUP_WIDTH'			=> $arcade_config['smilies_popup_width']
	));

	if (!$popup && $auth_arcade->acl_get('u_arcade_view_whoisplaying'))
	{
		display_arcade_online();
	}

	// Output page
	page_header($user->lang['INDEX'], false);

	$template->set_filenames(array(
		'body' => 'arcade/score_body.html')
	);

	page_footer();
}
else
{
	$arcade->score_done($game_data['game_id'], $popup, $game_data, $score_desc);
}

?>