<?php
/**
*
* @package arcade
* @version $Id: session.php 1912 2012-03-02 17:11:26Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

class arcade_session
{
	var $game_sid	= '';
	var $game_popup	= false;

	function begin($create = false, $game_id = false, $game_type = false, $popup = false)
	{
		global $config, $arcade_config;

		$cookie_sid			= $config['cookie_name'] . '_arcade_sid';
		$cookie_popup		= $config['cookie_name'] . '_arcade_popup';
		$this->game_sid		= (isset($_COOKIE[$cookie_sid])) ? $_COOKIE[$cookie_sid] : ((isset($_POST['game_sid'])) ? request_var('game_sid', '') : '');
		$this->game_popup	= (isset($_COOKIE[$cookie_popup])) ? $_COOKIE[$cookie_popup] : false;

		if ((time() - $arcade_config['gc_time']) > $arcade_config['session_last_gc'])
		{
			$this->delete('gc');
		}

		if ($create)
		{
			global $db, $user;

			$start_time	= time();
			$game_id	= (int) $game_id;
			$user_id	= (int) $user->data['user_id'];

			// Prevent the Flash game to refresh the game playing page score saving before.
			if ($user_id != ANONYMOUS && !isset($_POST['start']) && !empty($this->game_sid))
			{
				$sql = 'SELECT game_id
						FROM ' . ARCADE_SESSIONS_TABLE . "
						WHERE session_id = '" . $db->sql_escape($this->game_sid) . "'
						AND phpbb_session_id = '" . $db->sql_escape($user->session_id) . "'
						AND game_id = $game_id
						AND user_id = $user_id";
				$result = $db->sql_query($sql);
				$gid = (int) $db->sql_fetchfield('game_id');
				$db->sql_freeresult($result);

				if ($gid)
				{
					$create = false;
				}
			}

			if ($create)
			{
				$this->delete('gid', $game_id);
				$this->game_sid = md5(uniqid($user->ip));

				$sql_ary = array(
					'session_id'		=> (string) $this->game_sid,
					'phpbb_session_id'	=> (string) $user->session_id,
					'game_id'			=> $game_id,
					'user_id'			=> $user_id,
					'game_type'			=> $game_type,
					'game_popup'		=> $popup,
					'randchar1'			=> 0,
					'randchar2'			=> 0,
					'randgid1'			=> rand(1, 200),
					'randgid2'			=> rand(1, 200),
					'start_time'		=> $start_time,
				);

				$db->sql_query('INSERT INTO ' . ARCADE_SESSIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));

				$this->set_cookie(true, $popup);
			}

			return ($create) ? (int) $game_id * $sql_ary['randgid1'] ^ $sql_ary['randgid2'] : false;
		}
	}

	function add_game_randchar($session_id = false)
	{
		global $db, $user;

		$sql_ary = array(
			'randchar1'	=> rand(1, 200),
			'randchar2'	=> rand(1, 200),
			'microtime'	=> time(),
		);

		$sql = 'UPDATE ' . ARCADE_SESSIONS_TABLE . ' SET ' . $db->sql_build_array('UPDATE', $sql_ary) . "
				WHERE phpbb_session_id = '" . $db->sql_escape($user->session_id) . "'
				AND user_id = " . (int) $user->data['user_id'] . '
				AND game_type IN (' . PHPBBARCADE_GAME . ', ' . IBPROV3_GAME . ')';

		$sql .= ($session_id) ? " AND session_id = '" . $db->sql_escape($session_id) . "'" : '';
		$db->sql_query($sql);

		return $sql_ary;
	}

	function delete($type = '', $id = false)
	{
		global $db, $user, $arcade, $arcade_config;

		$sql = 'DELETE FROM ' . ARCADE_SESSIONS_TABLE;

		switch ($type)
		{
			case 'gc':
				$arcade->set_config('session_last_gc', time(), true);
				$sql .= ' WHERE start_time < ' . intval(time() - $arcade_config['session_length']);
			break;

			case 'gid':
				if ($id)
				{
					$sql .= ' WHERE game_id = ' . (int) $id . "
								AND phpbb_session_id = '" . $db->sql_escape($user->session_id) . "'
								AND user_id = " . (int) $user->data['user_id'];
				}
			break;

			default:
				return;
			break;
		}

		$db->sql_query($sql);
	}

	/**
	* Set arcade cookie, this is used to track the game id,
	* sid and popup value
	*/
	function set_cookie($create = false, $popup = false, $show = false)
	{
		// The cookies is removed once the arcade_score.php output
		// is displayed.  It just used to check whether of not
		// to use the simple header

		// If the function is called with no parameters and show
		// set to true it will return all cookies formatted to be displayed
		if (empty($this->game_sid) && !$create && !$popup && $show)
		{
			$return = '<pre>';
			$return .= var_export($_COOKIE, true);
			$return .= '</pre>';

			return $return;
		}

		global $user, $arcade_config;

		// If the function is not sent anything it removes all its cookies
		if (!$create && !$popup)
		{
			$set_time		= (time() - intval($arcade_config['session_length'] * 2));
			$user->set_cookie('arcade_sid'	, '', $set_time);
			$user->set_cookie('arcade_popup', '', $set_time);
		}
		else if ($create)
		{
			$set_time = intval(time() + $arcade_config['session_length']);
			$user->set_cookie('arcade_sid'	, $this->game_sid, $set_time);
			$user->set_cookie('arcade_popup', (($popup) ? true : false), $set_time);
		}
	}
}

?>