<?php
/**
*
* @package acp
* @version $Id: acp_arcade_games.php 1936 2012-03-07 20:56:21Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_arcade_games
{
	var $u_action;
	var $tpl_name;
	var $page_title;
	var $parent_id = 0;

	function main($id, $mode)
	{
		global $db, $user, $auth, $config, $template, $phpbb_admin_path, $phpbb_root_path, $phpEx;
		global $arcade, $auth_arcade, $arcade_config, $arcade_cache, $file_functions;

		define('IN_PHPBB_ARCADE', true);
		include($phpbb_root_path . 'arcade/includes/common.' . $phpEx);
		// Initialize arcade class
		$arcade = new arcade_admin();

		$this->tpl_name = 'acp_arcade_games';
		$this->page_title = $user->lang['ACP_ARCADE_' . strtoupper($mode)];

		$start = request_var('start', 0);

		switch ($mode)
		{
			case 'add_games':
				$form_key = 'acp_arcade_games';
				add_form_key($form_key);

				$update = (isset($_POST['update'])) ? true : false;

				if ($update)
				{
					if (!check_form_key($form_key))
					{
						trigger_error($user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
					}

					// Get game to add and makes sures its not blank
					$game_array	= request_var('games', array(''), true);
					if (empty($game_array))
					{
						trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action), E_USER_WARNING);
					}

					// Get category to add game to and make sures its valid
					$cat_data = $arcade->get_cat_info(request_var('cat_id', 0));
					if ($cat_data['cat_type'] != ARCADE_CAT_GAMES)
					{
						trigger_error($user->lang['WRONG_CAT_TYPE'] . adm_back_link($this->u_action), E_USER_WARNING);
					}

					// Add game to database
					$game_name_list = array();
					foreach ($game_array as $game)
					{
						$game_data = $arcade->add_game($game, $cat_data['cat_id']);
						add_log('admin', 'LOG_ARCADE_ADD_GAME', '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $game_data['game_id']) .'">'. $game_data['game_name'] .'</a>');

						if ($arcade_config['announce_game'])
						{
							$arcade->create_announcement($game_data, $arcade_config['announce_forum']);
						}

						$game_name_list[] = $game_data['game_name'];
					}

					if (sizeof($game_name_list) > 1)
					{
						$message = sprintf($user->lang['ARCADE_ADD_GAMES_SUCCESS'], implode(', ', $game_name_list));
					}
					else
					{
						$message = sprintf($user->lang['ARCADE_ADD_GAME_SUCCESS'], implode(', ', $game_name_list));
					}

					$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
					$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
					$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);

					$arcade_cache->destroy('_arcade_cats');
					$arcade_cache->destroy('_arcade_games_filesize');

					trigger_error($message . adm_back_link($this->u_action));
				}

				// Default Games ACP page
				$template->assign_var('S_ARCADE_GAMES', true);
				$parents_list = $arcade->make_cat_select($this->parent_id, false, false, true);
				if (!$parents_list)
				{
					trigger_error('NO_CAT', E_USER_WARNING);
				}
				$dir = $phpbb_root_path . $arcade_config['game_path'];

				if (!is_dir($dir))
				{
					trigger_error($user->lang['NO_GAME_DIRECTORY'] . adm_back_link($this->u_action));
				}

				$games = $games_add = $games_name = array();
				$sql = 'SELECT game_swf, game_name from ' . ARCADE_GAMES_TABLE . '
						ORDER BY game_swf';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$games[$file_functions->remove_extension($row['game_swf'])] = $row['game_swf'];
					$games_name[$row['game_name']] = $row['game_name'];
				}
				$db->sql_freeresult($result);

				$total_games = 0;
				$error = array();
				$s_game_options = '';

				if ($files = scandir($dir))
				{
					foreach ($files as $file)
					{
						if ($file == '.' || $file == '..' || $file == '.svn' || $file == 'index.html' || $file == 'index.htm')
						{
							continue;
						}

						if (isset($games[$file]))
						{
							continue;
						}

						$game_file 		= $dir . $file . '/'. $file .'.swf';
						$install_file 	= $dir . $file . '/'. $file .'.' . $phpEx;
						$install_xml 	= $dir . $file . '/game.xml';
						// Check to make sure a flash file exists before including the game install file
						if (file_exists($game_file))
						{
							if (file_exists($install_file))
							{
								// Convert ibp install file
								$arcade->convert_install_file($install_file);
							}
							else if (file_exists($install_xml))
							{
								// Convert xml install file
								$arcade->convert_install_file($install_xml, true);
							}

							if (file_exists($install_file))
							{
								@include($install_file);

								if (isset($game_data))
								{
									// We have same duplicate gamename
									if (isset($games_name[$game_data['game_name']]) || isset($games_add[$game_data['game_name']]))
									{
										$duplicate_games = 0;
										$temp_name = $game_data['game_name'];
										while (isset($games_name[$game_data['game_name']]) || isset($games_add[$game_data['game_name']]))
										{
											$duplicate_games++;
											$game_data['game_name'] = $temp_name . ' (' . $duplicate_games . ')';
										}
									}

									$games_add[$game_data['game_name']] = $file;
									$total_games++;
									unset($game_data);
								}
								else
								{
									$error[] = $install_file;
								}
							}
						}
					}
				}
				unset($games);

				if ($total_games == 0 && !sizeof($error))
				{
					trigger_error('NO_ADD_GAMES');
				}

				ksort($games_add);
				foreach ($games_add as $key => $value)
				{
					$s_game_options .= '<option value="' . $value . '">' . $key . "</option>\n";
				}
				unset($games_add);

				$template->assign_vars(array(
						'S_ERROR'			=> (sizeof($error)) ? true : false,
						'ERROR_MSG'			=> sprintf($user->lang['ARCADE_BAD_INSTALL_FILE'], implode('<br />', $error)),
						'S_CAT_LIST'		=> $parents_list,
						'S_TOTAL_GAMES'		=> $total_games,
						'S_GAME_OPTIONS'	=> $s_game_options,
						'U_ACTION'			=> $this->u_action)
				);
			break;

			case 'unpack_games':
				$form_key = 'acp_arcade_games';
				add_form_key($form_key);

				$update		= (isset($_POST['update'])) ? true : false;
				$upload_file = (isset($_POST['add_file'])) ? true : false;

				if (!check_form_key($form_key) && ($upload_file || $update))
				{
					trigger_error($user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}

				if ($upload_file)
				{
					$error = array();
					$game_array = $this->upload_game($error);
					if (sizeof($error))
					{
						trigger_error(implode('<br />', $error) . adm_back_link($this->u_action), E_USER_WARNING);
					}

					$update = true;
					$game_array = array($game_array);
				}

				$methods = $arcade->compress_methods();
				if ($update)
				{
					$errors = array();

					// Check permissions dirs writable
					$check_dirs = array('store/', 'games/', 'arcade/gamedata/', $arcade_config['game_path'], $arcade_config['unpack_game_path']);

					foreach ($check_dirs as $dir)
					{
						$path = $phpbb_root_path . $dir;

						if (!file_exists($path))
						{
							$errors[] = sprintf($user->lang['NO_DIRECTORY'], $dir);
						}

						if (file_exists($path) && !phpbb_is_writable($path))
						{
							$errors[] = sprintf($user->lang['ARCADE_ERROR_DIR_WRITABLE'], $dir);
						}
					}

					if (sizeof($errors))
					{
						trigger_error(implode('<br />', $errors) . adm_back_link($this->u_action), E_USER_WARNING);
					}

					include($phpbb_root_path . 'includes/functions_compress.' . $phpEx);
					// Get game to add and makes sures its not blank
					if (!$upload_file)
					{
						$game_array	= request_var('games', array(''), true);
					}

					if (empty($game_array))
					{
						trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action));
					}

					// Add game to database
					$bad_game = $bad_perm_game = array();
					foreach ($game_array as $key => $game)
					{
						$use_method = strrchr($game, '.');
						if ($use_method == '.zip')
						{
							$compress = new compress_zip('r', $phpbb_root_path . $arcade_config['unpack_game_path'] . $game);
						}
						else
						{
							$compress = new compress_tar('r', $phpbb_root_path . $arcade_config['unpack_game_path'] . $game);
						}

						// First we extract the compressed file to a temporary
						// directory in the store folder
						$tmp_path = 'store/' . md5(unique_id()) . '/';
						$compress->extract($phpbb_root_path . $tmp_path);
						$compress->close();

						// Test for games that uses the games/ folder that needs to be at
						// the phpBB forum root
						if (is_dir($phpbb_root_path . $tmp_path . 'games'))
						{
							if ((is_dir($phpbb_root_path . 'games') && !phpbb_is_writable($phpbb_root_path . 'games')) || (!is_dir($phpbb_root_path . 'games') && !phpbb_is_writable($phpbb_root_path)))
							{
								$bad_perm_game[] = $game;
								unset($game_array[$key]);
								$file_functions->delete_dir($phpbb_root_path . $tmp_path);
								continue;
							}
						}

						// As far as I know all ibpro games start with "game_" if this is not the case
						// please let me know as this will have to be changed
						$is_ibpro = $is_xml = $is_old = $game_type = false;
						$game_name = $file_functions->remove_extension($game);

						if (substr($game_name, 0, 5) == 'game_')
						{
							$is_ibpro = true;
							$game_name = str_replace('game_', '', $game_name);
						}

						// Check to make sure a valid config file is present
						$game_config = $phpbb_root_path . $tmp_path . $game_name . '.' . $phpEx;

						if (!file_exists($game_config))
						{
							if (file_exists($phpbb_root_path . $tmp_path . 'game.xml'))
							{
								$is_xml = true;
								$game_config = $phpbb_root_path . $tmp_path . 'game.xml';
							}
							else
							{
								// Let check if this is an older game (before 1.0.RC5)
								if (!file_exists($phpbb_root_path . $tmp_path . 'includes/arcade/games/' . $game_name . '/' . $game_name . '.' . $phpEx))
								{
									$bad_game[] = $game;
									unset($game_array[$key]);
									$file_functions->delete_dir($phpbb_root_path . $tmp_path);
									continue;
								}
								$is_old = true;
							}
						}
						// We do not need the compressed file any more so lets get rid of it
						unlink($phpbb_root_path . $arcade_config['unpack_game_path'] . $game);

						$game_data_path = ($is_old) ? $tmp_path . 'arcade/' : $tmp_path;
						$game_file_path = ($is_old) ? $tmp_path . 'includes/arcade/games/' . $game_name . '/' : $tmp_path;

						// If the gamedata folder exists move it to the correct location
						if (file_exists($phpbb_root_path . $game_data_path . 'gamedata'))
						{
							//Copy and delete
							$file_functions->copy_dir($phpbb_root_path . $game_data_path . 'gamedata', $phpbb_root_path . 'arcade/gamedata');
							$file_functions->delete_dir($phpbb_root_path . $game_data_path . 'gamedata');
						}

						// If the games folder exists move it to the correct location
						if (file_exists($phpbb_root_path . $game_data_path . 'games'))
						{
							//Copy and delete
							$file_functions->copy_dir($phpbb_root_path . $game_data_path . 'games', $phpbb_root_path . 'games');
							$file_functions->delete_dir($phpbb_root_path . $game_data_path . 'games');
						}

						// Convert the ibp or xml install file
						$game_type = $arcade->convert_install_file($game_config, $is_xml);

						$file_functions->copy_dir($phpbb_root_path . $game_file_path, $phpbb_root_path . $arcade_config['game_path'] . $game_name);
						$file_functions->delete_dir($phpbb_root_path . $tmp_path);

						if ($is_ibpro || $is_xml)
						{
							switch ($game_type)
							{
								case AR_GAME:
									add_log('admin', 'LOG_ARCADE_UNPACK_AR_GAME', $game);
								break;

								case IBPRO_GAME:
									add_log('admin', 'LOG_ARCADE_UNPACK_IBPRO_GAME', $game);
								break;

								case IBPROV3_GAME:
									add_log('admin', 'LOG_ARCADE_UNPACK_IBPROV3_GAME', $game);
								break;

								case ARCADELIB_GAME:
									add_log('admin', 'LOG_ARCADE_UNPACK_ARCADELIB_GAME', $game);
								break;

								default:
									add_log('admin', 'LOG_ARCADE_UNPACK_NOSCORE_GAME', $game);
								break;
							}
						}
						else
						{
							add_log('admin', 'LOG_ARCADE_UNPACK_GAME', $game);
						}
					}

					$s_bad_game = sizeof($bad_game);
					$s_bad_perm_game = sizeof($bad_perm_game);
					$s_game_array = sizeof($game_array);

					$message = '';
					$message .= ($upload_file) ? $user->lang['ARCADE_UPLOAD_COMPLETE'] . '<br /><br />' : '';
					if ($s_bad_game)
					{
						$lang = ($s_bad_game > 1) ? $user->lang['ARCADE_UNPACK_GAMES_ERROR'] : $user->lang['ARCADE_UNPACK_GAME_ERROR'];
						$message .= sprintf($lang, implode(', ', $bad_game));
					}

					if ($s_bad_perm_game)
					{
						$lang = ($s_bad_perm_game > 1) ? $user->lang['ARCADE_UNPACK_GAMES_PERM_ERROR'] : $user->lang['ARCADE_UNPACK_GAME_PERM_ERROR'];
						$message .= sprintf($lang, implode(', ', $bad_perm_game));
					}

					if ($s_game_array)
					{
						if ($s_bad_game)
						{
							$message .= '<br /><br />';
						}

						$lang = ($s_game_array > 1) ? $user->lang['ARCADE_UNPACK_GAMES_SUCCESS'] : $user->lang['ARCADE_UNPACK_GAME_SUCCESS'];
						$message .= sprintf($lang, implode(', ', $game_array));
					}

					if ($s_bad_game || $s_bad_perm_game)
					{
						trigger_error($message . adm_back_link($this->u_action), E_USER_WARNING);
					}
					else
					{
						trigger_error($message . adm_back_link($this->u_action));
					}
				}

				// Default Unpack Games ACP page
				$template->assign_var('S_UNPACK_ARCADE_GAMES', true);
				$dir = $phpbb_root_path . $arcade_config['unpack_game_path'];

				if (!is_dir($dir))
				{
					trigger_error($user->lang['NO_UNPACK_GAME_DIRECTORY'] . adm_back_link($this->u_action));
				}

				$total_games = 0;
				$s_game_options = '';
				if ($files = scandir($dir))
				{
					foreach ($files as $file)
					{
						if ($file == '.' || $file == '..' || $file == '.svn' || $file == 'index.html' || $file == 'index.htm')
						{
							continue;
						}

						$found = false;
						foreach ($methods as $type)
						{
							$ext = substr($file, -strlen($type));
							if ($ext == $type)
							{
								$found = true;
								$ext = $type;
								break;
							}
						}

						if ($found)
						{
							$filename = str_replace($ext, '', $file);

							$sql = 'SELECT game_name
									FROM '. ARCADE_GAMES_TABLE . "
									WHERE game_swf = '" . $db->sql_escape($filename . '.swf') . "'";

							$result = $db->sql_query($sql);

							if (!$db->sql_fetchrow($result))
							{
								$s_game_options .= '<option value="' . $file . '">' . $filename . ' (' . $ext . ')' . '</option>';
								$total_games++;
							}
							$db->sql_freeresult($result);
						}

					}
				}

				$can_upload = (file_exists($phpbb_root_path . $arcade_config['unpack_game_path']) && @is_writable($phpbb_root_path . $arcade_config['unpack_game_path']) && (@ini_get('file_uploads') || strtolower(@ini_get('file_uploads')) == 'on')) ? true : false;
				$template->assign_vars(array(
						'S_FORM_ENCTYPE'	=> ($can_upload) ? ' enctype="multipart/form-data"' : '',
						'S_CAN_UPLOAD'		=> $can_upload,
						'S_TOTAL_GAMES'		=> $total_games,
						'S_GAME_OPTIONS'	=> $s_game_options,
						'U_ACTION'			=> $this->u_action
				));
			break;

			case 'backup_games':
				$backup = request_var('backup', false);
				$delete = (isset($_POST['delete'])) ? true : false;

				$limit = $arcade_config['backup_limit'];
				$cat_ids = request_var('cat_ids', array(0));
				$use_method = request_var('use_method', '.tar');
				$methods = $arcade->compress_methods();

				if ($backup && $use_method && !empty($cat_ids))
				{
					$game_ids = array();
					$sql = 'SELECT game_id FROM ' . ARCADE_GAMES_TABLE . '
							WHERE ' . $db->sql_in_set('cat_id', $cat_ids) . '
							ORDER BY game_id ASC';
					$result = $db->sql_query($sql);
					while ($row = $db->sql_fetchrow($result))
					{
						$game_ids[] = $row['game_id'];
					}
					$db->sql_freeresult($result);

					$total_games = sizeof($game_ids);
					if ($total_games)
					{
						if ($start < $total_games || $limit >= $total_games)
						{
							$game_data = $arcade->get_game_data($game_ids, 'g.game_id ASC', $limit, $start);
							foreach ($game_data as $game)
							{
								$arcade->download_game($game, $use_method, $methods, false, false);
							}

							if ($limit < $total_games)
							{
								$get_cat_ids = '';
								foreach ($cat_ids as $value)
								{
									$get_cat_ids .= '&amp;cat_ids[]=' . (int) $value;
								}

								$new_start	= $start + $limit;
								$redirect	= $this->u_action . "&amp;backup=true&amp;use_method=$use_method&amp;start=$new_start$get_cat_ids";
								$redirect	= meta_refresh(3, $redirect);

								$template->assign_vars(array(
									'S_IN_BACKUP_PROCESSING'	=> true,

									'L_TITLE'					=> $user->lang['ACP_ARCADE_' . strtoupper($mode)],

									'ARCADE_BACKUP_PROCESSING'	=> sprintf($user->lang['ARCADE_BACKUP_PROCESSING'], $arcade->get_image('full', 'img', 'loading2.gif', $user->lang['ARCADE_BACKUP'], true), $start + 1, $total_games),
								));

								return;
							}
						}

						$cat_names = '';
						foreach ($cat_ids as $cat_id)
						{
							$cat_names[] = $arcade->get_cat_field($cat_id, 'cat_name');
						}
						$cat_names = implode(', ', $cat_names);

						if (sizeof($cat_ids) > 1)
						{
							add_log('admin', 'LOG_ARCADE_BACKUP_CATS', $cat_names);
							trigger_error($user->lang['ARCADE_BACKUPS_SUCCESS'] . adm_back_link($this->u_action));
						}
						else
						{
							add_log('admin', 'LOG_ARCADE_BACKUP_CAT', $cat_names);
							trigger_error($user->lang['ARCADE_BACKUP_SUCCESS'] . adm_back_link($this->u_action));
						}
					}
					else
					{
						trigger_error($user->lang['ARCADE_BACKUP_NO_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
					}
				}

				$backup_path = $phpbb_root_path . $arcade_config['cat_backup_path'];

				if ($delete)
				{
					if (confirm_box(true))
					{
						if (file_exists($backup_path) && phpbb_is_writable($backup_path))
						{
							if ($file_functions->delete_dir($backup_path, true, 'htaccess', true))
							{
								add_log('admin', 'LOG_ARCADE_BACKUP_EMPTY');
								trigger_error(sprintf($user->lang['ARCADE_BACKUP_EMPTY_SUCCESS'], $arcade_config['cat_backup_path']) . adm_back_link($this->u_action) );
							}
						}

						trigger_error(sprintf($user->lang['ARCADE_BACKUP_EMPTY_ERROR'], $arcade_config['cat_backup_path']) . adm_back_link($this->u_action), E_USER_WARNING);
					}
					else
					{
						$s_hidden_fields = array(
							'delete' => true,
						);
						confirm_box(false, 'ARCADE_BACKUP_EMPTY', build_hidden_fields($s_hidden_fields));
					}
				}

				$radio_buttons = '';
				foreach ($methods as $method)
				{
					$checked = ($method == $use_method) ? ' checked="checked"' : '';
					$radio_buttons .= '<input type="radio"' . ((!$radio_buttons) ? ' id="use_method"' : '') . ' class="radio" value="' . $method . '" name="use_method"' . $checked . ' />&nbsp;' . $method . '&nbsp;';
				}

				$filesize = $file_functions->filesize($backup_path);
				$template->assign_vars(array(
					'S_CAT_OPTIONS' 				=> $arcade->make_cat_select(false, false, false, false, false),
					'S_IN_BACKUP'					=> true,

					'U_ACTION'						=> $this->u_action,

					'ARCADE_BACKUP_DIR_FILESIZE'	=> ($filesize) ? sprintf($user->lang['ARCADE_BACKUP_DIR_FILESIZE'], $arcade_config['cat_backup_path'], get_formatted_filesize($filesize)) : '',
					'RADIO_BUTTONS'					=> $radio_buttons,
				));
			break;

			case 'downloads_games':
				$download_url = utf8_normalize_nfc(request_var('dl_url', ''));
				$sort_cat = request_var('sc', 0);
				$sort_time = request_var('st', 0);
				$sort_key = request_var('sk', 'd');
				$sort_dir = request_var('sd', 'd');
				$hide_found = request_var('hide', 'y');

				if (isset($_POST['purge_dl_cache']))
				{
					$arcade->purge_download_cache();
				}

				if ($download_url != '')
				{
					$download_url = (strpos($download_url, 'http://') === false) ? 'http://' . $download_url : $download_url;
					$download_url = rtrim($download_url, '/');
				}

				$template->assign_vars(array(
					'S_IN_ARCADE_DOWNLOADS'	=> true,

					'U_ACTION'				=> $this->u_action,

					'DOWNLOAD_URL'			=> $download_url,
				));

				$recent_sites = $arcade->set_recent_sites();
				if ($download_url != '')
				{
					$download_data = $arcade->get_remote_data($download_url. "/arcade.$phpEx?mode=download&type=data");
					if (sizeof($download_data) && is_array($download_data))
					{
						$recent_sites = $arcade->set_recent_sites($download_url);

						$template->assign_vars(array(
							'DOWNLOAD_SITENAME'	=> sprintf($user->lang['ARCADE_DOWNLOADS_FROM'], $download_data['sitename']),
							'DOWNLOAD_MESSAGE'	=> (isset($download_data['message'])) ? $download_data['message'] : '',
						));

						$tar = $targz = $tarbz2 = $zip = false;
						$available_methods = array_intersect($arcade->compress_methods(), $download_data['methods']);
						foreach ($available_methods as $method)
						{
							switch ($method)
							{
								case '.zip':
									$zip = true;
								break;

								case '.tar':
									$tar = true;
								break;

								case '.tar.gz':
									$targz = true;
								break;

								case '.tar.bz2':
									$tarbz2 = true;
								break;

								default:
								break;
							}
						}

						$template->assign_vars(array(
							'S_FILE_TYPE'	=> $download_data['file_type'],
							'S_ZIP'			=> $zip,
							'S_TAR'			=> $tar,
							'S_TARGZ'		=> $targz,
							'S_TARBZ2'		=> $tarbz2,
						));

						$s_download_cost = false;
						$total_games = $total_games_real = $total_found = 0;
						$download_list = $arcade->get_remote_data($download_url. "/arcade.php?mode=download&type=list&c=$sort_cat&start=$start&sk=$sort_key&st=$sort_time&sd=$sort_dir&per_page={$arcade_config['download_list_per_page']}");

						if (sizeof($download_list) && is_array($download_list))
						{
							foreach ($download_list['games'] as $row)
							{
								$total_games_real++;
								$game_found = false;

								if (file_exists($arcade->set_path($row['game_swf'], 'install')))
								{
									$total_found++;
									$game_found = true;
									if ($hide_found == 'y')
									{
										continue;
									}
								}

								if ($row['download_cost'] > 0)
								{
									$s_download_cost = true;
								}

								$template->assign_block_vars('games', array(
									'S_GAME_FOUND'		=> $game_found,
									'U_DOWNLOAD'		=> $download_url . '/arcade.php?mode=download&amp;type=acp&amp;g=' . $row['game_id'],
									'GAME_NAME'			=> $row['game_name'],
									'GAME_DESC'			=> $row['game_desc'],
									'DOWNLOAD_COST'		=> ($row['download_cost'] > 0) ? $arcade->number_format($row['download_cost']) : 0,
									'GAME_FILESIZE'		=> get_formatted_filesize($row['game_filesize']),
									'GAME_INSTALLDATE'	=> $user->format_date($row['game_installdate']),
								));
							}
						}

						$template->assign_vars(array(
							'S_ALL_INSTALLED'				=> ($total_games_real - $total_found > 0) ? false : true,
							'S_SHOW_DOWNLOAD_COST'			=> $s_download_cost,

							'L_ARCADE_DOWNLOAD_TOTAL_GAMES'	=> (isset($download_data['categories'][$sort_cat])) ? sprintf($user->lang['ARCADE_DOWNLOAD_TOTAL_GAMES_CAT'], $download_data['categories'][$sort_cat]['cat_name']) : $user->lang['ARCADE_DOWNLOAD_TOTAL_GAMES'],

							'TOTAL_GAMES'					=> $arcade->number_format($download_list['total']),
							'GAMES_FOUND'					=> $total_found,
							'GAMES_NOT_FOUND'				=> $total_games_real - $total_found,
							'SORT_CAT_SELECT'				=> $this->sort_cat_select($sort_cat, $download_data['categories']),
							'SORT_TIME_SELECT'				=> $this->sort_time_select($sort_time),
							'SORT_KEY_SELECT'				=> $this->sort_key_select($sort_key),
							'SORT_DIR_SELECT'				=> $this->sort_dir_select($sort_dir),
							'HIDE_FOUND_SELECT'				=> $this->hide_found_select($hide_found),

							'S_ON_PAGE'						=> on_page($download_list['total'], $arcade_config['download_list_per_page'], $start),
							'PAGINATION'					=> generate_pagination($this->u_action . '&amp;dl_url=' .  urlencode(str_replace('&amp;', '&', $download_url)) . "&amp;sc=$sort_cat&amp;st=$sort_time&amp;sk=$sort_key&amp;sd=$sort_dir&amp;hide=$hide_found", $download_list['total'], $arcade_config['download_list_per_page'], $start, true),
						));
					}
				}

				$template->assign_var('S_RECENT_SITES', ((sizeof($recent_sites)) ? true : false));

				foreach ($recent_sites as $sites)
				{
					$template->assign_block_vars('recent_sites', array(
						'RECENT_SITES' => $sites,
					));
				}
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}

		$template->assign_vars(array(
			'L_TITLE'			=> $user->lang['ACP_ARCADE_' . strtoupper($mode)],
			'L_TITLE_EXPLAIN'	=> $user->lang['ACP_ARCADE_' . strtoupper($mode) . '_EXPLAIN'],
		));
	}

	/**
	* Upload Game - filedata is generated here
	* Uses upload class
	*/
	function upload_game(&$error)
	{
		global $phpbb_root_path, $config, $arcade_config, $db, $user, $phpEx, $arcade;

		$error = array();

		// Init upload class
		$user->add_lang('posting');
		if (!class_exists('fileupload'))
		{
			include($phpbb_root_path . 'includes/functions_upload.' . $phpEx);
		}
		$upload = new fileupload();

		$upload->set_allowed_extensions($arcade->compress_methods(true));

		if (!empty($_FILES['fileupload']['name']))
		{
			$file = $upload->form_upload('fileupload');
		}
		else
		{
			$error[] = $user->lang['NO_UPLOAD_FORM_FOUND'];
			return;
		}

		// Move file and overwrite any existing image
		$file->move_file($arcade_config['unpack_game_path'], true, true);

		if (sizeof($file->error))
		{
			$file->remove();
			$error = array_merge($error, $file->error);
		}

		return $file->realname;
	}

	function sort_time_select($value)
	{
		global $user;

		$limit_time = array(0 => $user->lang['ARCADE_ALL_GAMES'], 1 => $user->lang['1_DAY'], 7 => $user->lang['7_DAYS'], 14 => $user->lang['2_WEEKS'], 30 => $user->lang['1_MONTH'], 90 => $user->lang['3_MONTHS'], 180 => $user->lang['6_MONTHS'], 365 => $user->lang['1_YEAR']);

		$s_limit_time = '';
		foreach ($limit_time as $time => $text)
		{
			$selected = ($value == $time) ? ' selected="selected"' : '';
			$s_limit_time .= '<option value="' . $time . '"' . $selected . '>' . $text . '</option>';
		}
		return $s_limit_time;
	}

	function sort_cat_select($selected, $data)
	{
		global $user;

		$options = '<option value="0"' . ((!$selected) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_ALL_CATEGORIES'] . '</option>';
		foreach ($data as $value)
		{
			$options .= '<option value="' . $value['cat_id'] . '"' . (($value['cat_id'] == $selected) ? ' selected="selected"' : '') . '>' . $value['cat_name'] . '</option>';
		}
		return $options;
	}

	function sort_key_select($value)
	{
		global $user;

		return '<option value="n"'. (($value == 'n') ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAME_NAME'] . '</option><option value="d"' . (($value == 'd') ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_GAMES_SORT_INSTALLDATE'] . '</option>';
	}

	function sort_dir_select($value)
	{
		global $user;

		return '<option value="a"'. (($value == 'a') ? ' selected="selected"' : '') . '>' . $user->lang['ASCENDING'] . '</option><option value="d"' . (($value == 'd') ? ' selected="selected"' : '') . '>' . $user->lang['DESCENDING'] . '</option>';
	}

	function hide_found_select($value)
	{
		global $user;

		return '<option value="y"'. (($value == 'y') ? ' selected="selected"' : '') . '>' . $user->lang['YES'] . '</option><option value="n"' . (($value == 'n') ? ' selected="selected"' : '') . '>' . $user->lang['NO'] . '</option>';
	}
}

?>