<?php
/**
*
* @package acp
* @version $Id: acp_arcade_manage.php 2021 2012-03-19 13:20:35Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_arcade_manage
{
	var $u_action;
	var $tpl_name;
	var $page_title;
	var $parent_id = 0;
	var $new_config = array();
	var $new_game_data = array();

	function main($id, $mode)
	{
		global $db, $user, $auth, $config, $template, $phpbb_admin_path, $phpbb_root_path, $phpEx;
		global $arcade, $arcade_cache, $auth_arcade, $arcade_config, $file_functions;

		define('IN_PHPBB_ARCADE', true);
		include($phpbb_root_path . 'arcade/includes/common.' . $phpEx);
		// Initialize arcade class
		$arcade = new arcade_admin();

		$this->tpl_name = 'acp_arcade_manage';

		switch ($mode)
		{
			case 'category':
				$form_key = 'acp_arcade_category';
				add_form_key($form_key);
				$template->assign_var('S_MANAGE_CAT', true);

				$action = request_var('action', '');
				$confirm_action = array('delete_marked', 'g_delete', 'reset_marked', 'reset_scores_marked', 'move_marked');
				$update = (isset($_POST['update'])) ? true : false;
				$start = request_var('start', 0);

				$cat_data = $errors = array();
				if ($update && !check_form_key($form_key) && !in_array($action, $confirm_action))
				{
					$update = false;
					$errors[] = $user->lang['FORM_INVALID'];
				}

				$this->page_title = 'ACP_ARCADE_MANAGE';

				$cat_id = request_var('c', 0);
				$game_id = request_var('g', 0);
				$this->parent_id = request_var('parent_id', 0);
				$game_ids = (isset($_POST['game_ids'])) ? request_var('game_ids', array(0)) : array();

				// Check additional permissions
				switch ($action)
				{
					case 'progress_bar':
						$this->display_progress_bar();
						exit;
					break;

					case 'delete':
						if (!$auth->acl_get('a_arcade_delete_cat'))
						{
							trigger_error($user->lang['NO_PERMISSION_CAT_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'delete_marked':
						if (!$auth->acl_get('a_arcade_delete_game'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'reset_marked':
					case 'reset_scores_marked':
						if (!$auth->acl_get('a_arcade_user'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_' . (($action == 'reset_marked') ? 'RESET' : 'SCORE_RESET')] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'sync_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							$update = false;
						}
					break;

					case 'add':
						if (!$auth->acl_get('a_arcade_cat'))
						{
							trigger_error($user->lang['NO_PERMISSION_CAT_ADD'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;
				}

				// Major routines
				if ($update)
				{
					switch ($action)
					{
						case 'delete':
							$action_subcats	= request_var('action_subcats', '');
							$subcats_to_id	= request_var('subcats_to_id', 0);
							$action_games	= request_var('action_games', '');
							$games_to_id	= request_var('games_to_id', 0);

							$delete_files = false;
							if (isset($_POST['delete_files']))
							{
								// language frontier
								if (request_var('delete_files', '') === $user->lang['YES'])
								{
									$delete_files = true;
								}
							}

							$sub_delete_files = false;
							if (isset($_POST['sub_delete_files']))
							{
								// language frontier
								if (request_var('sub_delete_files', '') === $user->lang['YES'])
								{
									$sub_delete_files = true;
								}
							}

							$errors = $this->delete_cat($cat_id, $action_games, $delete_files, $action_subcats, $sub_delete_files, $games_to_id, $subcats_to_id);

							if (sizeof($errors))
							{
								break;
							}

							$auth_arcade->acl_clear_prefetch();
							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

							$arcade_cache->destroy('_arcade_cats');
							$arcade_cache->destroy('_arcade_leaders');
							$arcade_cache->destroy('_arcade_leaders_all');

							trigger_error($user->lang['CAT_DELETED'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
						break;

						case 'g_delete':
							if (confirm_box(true))
							{
								$delete_files = false;
								if (isset($_POST['delete_files']))
								{
									// language frontier
									if (request_var('delete_files', '') === $user->lang['YES'])
									{
										$delete_files = true;
									}
								}

								$game_data = $arcade->get_game_data($game_id);
								$errors = $arcade->delete_game($game_id, $game_data['cat_id'], $delete_files);

								if (sizeof($errors))
								{
									break;
								}

								add_log('admin', 'LOG_ARCADE_DELETE_GAME', $game_data['game_name']);

								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);
								$arcade_cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

								$arcade_cache->destroy('_arcade_cats');
								$arcade_cache->destroy('_arcade_leaders');
								$arcade_cache->destroy('_arcade_leaders_all');
								$arcade_cache->destroy('_arcade_games_filesize');
								$arcade_cache->destroy('_arcade_challenge_leaders');
								$arcade_cache->destroy('_arcade_challenge_leaders_all');

								trigger_error(sprintf($user->lang['GAME_DELETED'], $game_data['game_name']) . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
							}
						break;

						case 'move_marked':
							if (confirm_box(true))
							{
								$game_data = $arcade->get_game_data($game_ids);
								$game_data_array = array();
								foreach ($game_data as $key => $value)
								{
									$game_data_array[] = $game_data[$key]['game_name'];
								}
								$game_name_list = implode(', ', $game_data_array);

								$to_cat_id = request_var('to_cat_id', 0);

								$errors = $arcade->move_game($game_ids, $this->parent_id, $to_cat_id);
								if (sizeof($errors))
								{
									break;
								}

								$old_cat_name = $arcade->get_cat_field($this->parent_id, 'cat_name');
								$new_cat_name = $arcade->get_cat_field($to_cat_id, 'cat_name');

								if (sizeof($game_data_array) > 1)
								{
									$message = sprintf($user->lang['GAMES_MOVED'], $game_name_list);
									add_log('admin', 'LOG_ARCADE_MOVE_GAMES', $old_cat_name, $new_cat_name, $game_name_list);
								}
								else
								{
									$message = sprintf($user->lang['GAME_MOVED'], $game_name_list);
									add_log('admin', 'LOG_ARCADE_MOVE_GAME', $old_cat_name, $new_cat_name, $game_name_list);
								}

								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

								$arcade_cache->destroy('_arcade_leaders');
								$arcade_cache->destroy('_arcade_leaders_all');

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
							}
						break;

						case 'sync_marked':
							$arcade->sync('game', $game_ids);
							$arcade->sync('rating', $game_ids);
							$game_data = $arcade->get_game_data($game_ids);

							$game_data_array = array();
							foreach ($game_data as $game)
							{
								$arcade->update_highscore($game);
								$game_data_array[] = $game['game_name'];
							}
							$game_name_list = implode(', ', $game_data_array);

							if (sizeof($game_data_array) > 1)
							{
								$message = sprintf($user->lang['GAMES_RESYNCED'], $game_name_list);
								add_log('admin', 'LOG_ARCADE_SYNC_GAMES', $game_name_list);
							}
							else
							{
								$message = sprintf($user->lang['GAME_RESYNCED'], $game_name_list);
								add_log('admin', 'LOG_ARCADE_SYNC_GAME', $game_name_list);
							}

							$template->assign_var('L_CAT_RESYNCED', $message);
						break;

						case 'reset_marked':
						case 'reset_scores_marked':
							if (confirm_box(true))
							{
								$game_data = $arcade->get_game_data($game_ids);
								$game_data_array = array();
								foreach ($game_data as $key => $value)
								{
									$game_data_array[] = $game_data[$key]['game_name'];
								}
								$game_name_list = implode(', ', $game_data_array);

								$message = sprintf($user->lang['GAME' . ((sizeof($game_data_array) > 1) ? 'S' : '') . '_' . (($action == 'reset_marked') ? 'RESET' : 'SCORES_RESET')], $game_name_list);

								$errors = $arcade->reset_game($action, $game_ids, $this->parent_id);
								if (sizeof($errors))
								{
									break;
								}

								$reset_log = (sizeof($game_data_array) > 1) ? 'LOG_ARCADE_RESET_GAMES' : 'LOG_ARCADE_RESET_GAME';

								add_log('admin', $reset_log . (($action == 'reset_marked') ? '' : '_SCORES'), $game_name_list);
								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

								$arcade_cache->destroy('_arcade_leaders');
								$arcade_cache->destroy('_arcade_leaders_all');
								$arcade_cache->destroy('_arcade_all_games');
								$arcade_cache->destroy('_arcade_latest_highscores');
								$arcade_cache->destroy('_arcade_longest_highscores');
								$arcade_cache->destroy('_arcade_least_popular_games');
								$arcade_cache->destroy('_arcade_most_popular_games');

								$arcade->sync('total_data', 'all');

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
							}
						break;

						case 'delete_marked':
							if (confirm_box(true))
							{
								$delete_files = false;
								if (isset($_POST['delete_files']))
								{
									// language frontier
									if (request_var('delete_files', '') === $user->lang['YES'])
									{
										$delete_files = true;
									}
								}

								$game_data = $arcade->get_game_data($game_ids);
								$errors = $arcade->delete_game($game_ids, $this->parent_id, $delete_files);

								if (sizeof($errors))
								{
									break;
								}

								$game_data_array = array();
								foreach ($game_data as $key => $value)
								{
									$game_data_array[] = $game_data[$key]['game_name'];
								}

								$game_name_list = implode(', ', $game_data_array);

								if (sizeof($game_data_array) > 1)
								{
									$message = sprintf($user->lang['GAMES_DELETED'], $game_name_list);
									add_log('admin', 'LOG_ARCADE_DELETE_GAMES',  $game_name_list);
								}
								else
								{
									$message = sprintf($user->lang['GAME_DELETED'], $game_name_list);
									add_log('admin', 'LOG_ARCADE_DELETE_GAME',  $game_name_list);
								}

								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);
								$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);
								$arcade_cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

								$arcade_cache->destroy('_arcade_cats');
								$arcade_cache->destroy('_arcade_leaders');
								$arcade_cache->destroy('_arcade_leaders_all');
								$arcade_cache->destroy('_arcade_games_filesize');
								$arcade_cache->destroy('_arcade_challenge_leaders');
								$arcade_cache->destroy('_arcade_challenge_leaders_all');

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
							}
						break;

						case 'edit':
							$cat_data = array(
								'cat_id' => $cat_id
							);

						// No break here
						case 'add':
							$cat_data += array(
								'parent_id'				=> request_var('cat_parent_id', $this->parent_id),
								'cat_type'				=> request_var('cat_type', ARCADE_CAT_GAMES),
								'cat_test'				=> request_var('cat_test', false),
								'type_action'			=> request_var('type_action', ''),
								'cat_status'			=> request_var('cat_status', ITEM_UNLOCKED),
								'cat_parents'			=> '',
								'cat_name'				=> utf8_normalize_nfc(request_var('cat_name', '', true)),
								'cat_link'				=> request_var('cat_link', ''),
								'cat_link_track'		=> request_var('cat_link_track', false),
								'cat_desc'				=> utf8_normalize_nfc(request_var('cat_desc', '', true)),
								'cat_age'				=> request_var('cat_age', 0),
								'cat_desc_uid'			=> '',
								'cat_desc_options'		=> 7,
								'cat_desc_bitfield'		=> '',
								'cat_rules'				=> utf8_normalize_nfc(request_var('cat_rules', '', true)),
								'cat_rules_uid'			=> '',
								'cat_rules_options'		=> 7,
								'cat_rules_bitfield'	=> '',
								'cat_rules_link'		=> request_var('cat_rules_link', ''),
								'cat_image'				=> request_var('cat_image', ''),
								'cat_display'			=> request_var('cat_display', 0),
								'cat_style'				=> request_var('cat_style', 0),
								'cat_games_per_page'	=> request_var('cat_games_per_page', 0),
								'cat_download'			=> request_var('cat_download', true),
								'cat_use_jackpot'		=> request_var('cat_use_jackpot', false),
								'cat_download_cost'		=> request_var('cat_download_cost', 0.000),
								'cat_cost'				=> request_var('cat_cost', 0.000),
								'cat_reward'			=> request_var('cat_reward', 0.000),
								'cat_password'			=> request_var('cat_password', ''),
								'cat_password_confirm'	=> request_var('cat_password_confirm', ''),
								'cat_password_unset'	=> request_var('cat_password_unset', false),
								'display_subcat_list'	=> request_var('display_subcat_list', false),
								'display_on_index'		=> request_var('display_on_index', false),
							);

							// Use link_display_on_index setting if forum type is link
							if ($cat_data['cat_type'] == ARCADE_LINK)
							{
								$cat_data['display_on_index'] = request_var('link_display_on_index', false);
							}

							// Linked forums and categories are not able to be locked...
							if ($cat_data['cat_type'] == ARCADE_LINK || $cat_data['cat_type'] == ARCADE_CAT)
							{
								// Linked forums are not able to be locked...
								$cat_data['cat_status'] = ITEM_UNLOCKED;
							}

							// Get data for forum rules if specified...
							if ($cat_data['cat_rules'])
							{
								generate_text_for_storage($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options'], request_var('rules_parse_bbcode', false), request_var('rules_parse_urls', false), request_var('rules_parse_smilies', false));
							}

							// Get data for forum description if specified
							if ($cat_data['cat_desc'])
							{
								generate_text_for_storage($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_bitfield'], $cat_data['cat_desc_options'], request_var('desc_parse_bbcode', false), request_var('desc_parse_urls', false), request_var('desc_parse_smilies', false));
							}

							$errors = $this->update_cat_data($cat_data);

							if (!sizeof($errors))
							{
								$cat_perm_from = request_var('cat_perm_from', 0);

								// Copy permissions?
								if ($cat_perm_from && !empty($cat_perm_from) && $cat_perm_from != $cat_data['cat_id'] &&
									(($action != 'edit') || empty($cat_id) || ($auth->acl_get('a_cauth') && $auth->acl_get('a_authusers') && $auth->acl_get('a_authgroups'))))
								{
									$arcade->copy_category_permissions($cat_perm_from, $cat_data['cat_id'], ($action == 'edit') ? true : false);
								}

								$auth_arcade->acl_clear_prefetch();
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
								$arcade_cache->destroy('_arcade_cats');

								$acl_url = '&amp;mode=setting_category_local&amp;cat_id[]=' . $cat_data['cat_id'];

								$message = ($action == 'add') ? $user->lang['CAT_CREATED'] : $user->lang['CAT_UPDATED'];

								// redirect directly to permission settings screen if authed
								if ($action == 'add' && !$cat_perm_from && $auth->acl_get('a_cauth'))
								{
									$message .= '<br /><br />' . sprintf($user->lang['REDIRECT_ACL'], '<a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_permissions' . $acl_url) . '">', '</a>');

									meta_refresh(4, append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_permissions' . $acl_url));
								}
								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
							}
						break;
					}
				}

				switch ($action)
				{
					case 'move_up':
					case 'move_down':
						if (!$cat_id)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$sql = 'SELECT *
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_id = ' . (int) $cat_id;
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$move_cat_name = $this->move_cat_by($row, $action, 1);

						if ($move_cat_name !== false)
						{
							add_log('admin', 'LOG_ARCADE_' . strtoupper($action), $row['cat_name'], $move_cat_name);
							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
						}
					break;

					case 'g_move_up':
					case 'g_move_down':
						if (!$game_id)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$sql = 'SELECT game_name
								FROM ' . ARCADE_GAMES_TABLE . '
								WHERE game_id = ' . (int) $game_id;
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						//
						// Change order of games  in the DB
						//
						$arcade->move_game_by($game_id, $this->parent_id, $action);
						add_log('admin', 'LOG_ARCADE_' . strtoupper($action), $row['game_name']);
						$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
					break;

					case 'g_delete':
						if (!$auth->acl_get('a_arcade_delete_game'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$s_hidden_fields = array(
							'start'		=> $start,
							'g'			=> $game_id,
							'update'	=> true,
						);
						confirm_box(false, 'DELETE_SELECTED_GAME', build_hidden_fields($s_hidden_fields), 'confirm_body_delete_arcade_games.html');
					break;


					case 'delete_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						if (!$auth->acl_get('a_arcade_delete_game'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$s_hidden_fields = array(
							'start'		=> $start,
							'action'	=> 'delete_marked',
							'update'	=> true,
							'game_ids'	=> $game_ids,
						);
						confirm_box(false, 'DELETE_SELECTED_GAMES', build_hidden_fields($s_hidden_fields), 'confirm_body_delete_arcade_games.html');
					break;

					case 'move_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						$s_hidden_fields = array(
							'action'	=> 'move_marked',
							'update'	=> true,
							'game_ids'	=> $game_ids,
						);

						$template->assign_vars(array(
							'S_CATEGORY_SELECT'		=> $arcade->make_cat_select($this->parent_id, $this->parent_id, true),
						));

						confirm_box(false, 'MOVE_SELECTED_GAMES', build_hidden_fields($s_hidden_fields), 'confirm_body_move_arcade_games.html');
					break;

					case 'reset_marked':
					case 'reset_scores_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						if (!$auth->acl_get('a_arcade_user'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_' . (($action == 'reset_marked') ? 'RESET' : 'SCORE_RESET')] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$s_hidden_fields = array(
							'start'		=> $start,
							'action'	=> $action,
							'update'	=> true,
							'game_ids'	=> $game_ids,
						);

						$confirm = (sizeof($game_ids) == 1) ? 'RESET_SELECTED_GAME' : 'RESET_SELECTED_GAMES';

						confirm_box(false, $confirm . (($action == 'reset_marked') ? '' : '_SCORES'), build_hidden_fields($s_hidden_fields));
					break;

					case 'sync':
						if ($cat_id || $game_id)
						{
							if (!$cat_id && !$game_id)
							{
								trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
							}

							@set_time_limit(0);

							if ($cat_id)
							{
								$cat_name = $arcade->get_cat_field($cat_id, 'cat_name');

								$arcade->set_last_play($cat_id);
								$arcade->sync('category', $cat_id);
								$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);

								add_log('admin', 'LOG_ARCADE_SYNC_CAT', $cat_name);
								$template->assign_var('L_CAT_RESYNCED', sprintf($user->lang['CAT_RESYNCED'], $cat_name));
							}
							else if ($game_id)
							{
								$sql = 'SELECT game_name
										FROM ' . ARCADE_GAMES_TABLE . '
										WHERE game_id = ' . (int) $game_id;
								$result = $db->sql_query($sql);
								$row = $db->sql_fetchrow($result);
								$db->sql_freeresult($result);

								if (!$row)
								{
									trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
								}

								$arcade->sync('game', $game_id);
								$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);

								add_log('admin', 'LOG_ARCADE_SYNC_GAME', $row['game_name']);
								$template->assign_var('L_CAT_RESYNCED', sprintf($user->lang['GAME_RESYNCED'], $row['game_name']));
							}
						}
						else
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'add':
					case 'edit':
						if ($update)
						{
							$cat_data['cat_flags'] = 0;
							$cat_data['cat_flags'] += (request_var('cat_link_track', false)) ? ARCADE_FLAG_LINK_TRACK : 0;
						}

						// Show form to create/modify a forum
						if ($action == 'edit')
						{
							$this->page_title = 'EDIT_CAT';
							$row = $arcade->get_cat_info($cat_id);
							$old_cat_type = $row['cat_type'];

							if (!$update)
							{
								$cat_data = $row;
							}
							else
							{
								$cat_data['left_id'] = $row['left_id'];
								$cat_data['right_id'] = $row['right_id'];
							}

							// Make sure no direct child forums are able to be selected as parents.
							$childs = $arcade->get_cat_branch($cat_id, 'children');

							$exclude_cat = array();
							foreach ($childs as $row)
							{
								$exclude_cat[] = $row['cat_id'];
							}

							$parents_list = $arcade->make_cat_select($cat_data['parent_id'], $exclude_cat, false, false, false);
							$cat_data['cat_password_confirm'] = $cat_data['cat_password'];
						}
						else
						{
							$this->page_title = 'CREATE_CAT';

							$cat_id = $this->parent_id;
							$parents_list = $arcade->make_cat_select($this->parent_id, false, false, false, false);

							// Fill forum data with default values
							if (!$update)
							{
								$cat_data = array(
									'parent_id'				=> $this->parent_id,
									'cat_type'				=> ARCADE_CAT_GAMES,
									'cat_test'				=> false,
									'cat_status'			=> ITEM_UNLOCKED,
									'cat_name'				=> utf8_normalize_nfc(request_var('cat_name', '', true)),
									'cat_age'				=> 0,
									'cat_link'				=> '',
									'cat_link_track'		=> false,
									'cat_desc'				=> '',
									'cat_rules'				=> '',
									'cat_rules_link'		=> '',
									'cat_image'				=> '',
									'cat_display'			=> 0,
									'cat_style'				=> 0,
									'cat_games_per_page'	=> 0,
									'cat_download'			=> true,
									'cat_use_jackpot'		=> false,
									'cat_download_cost'		=> 0,
									'cat_cost'				=> 0,
									'cat_reward'			=> 0,
									'cat_flags'				=> 0,
									'cat_password'			=> '',
									'cat_password_confirm'	=> '',
									'display_subcat_list'	=> true,
									'display_on_index'		=> true,
								);
							}
						}

						$cat_rules_data = array(
							'text'			=> $cat_data['cat_rules'],
							'allow_bbcode'	=> true,
							'allow_smilies'	=> true,
							'allow_urls'	=> true
						);

						$cat_desc_data = array(
							'text'			=> $cat_data['cat_desc'],
							'allow_bbcode'	=> true,
							'allow_smilies'	=> true,
							'allow_urls'	=> true
						);

						$cat_rules_preview = '';

						// Parse rules if specified
						if ($cat_data['cat_rules'])
						{
							if (!isset($cat_data['cat_rules_uid']))
							{
								// Before we are able to display the preview and plane text, we need to parse our request_var()'d value...
								$cat_data['cat_rules_uid'] = '';
								$cat_data['cat_rules_bitfield'] = '';
								$cat_data['cat_rules_options'] = 0;

								generate_text_for_storage($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options'], request_var('rules_allow_bbcode', false), request_var('rules_allow_urls', false), request_var('rules_allow_smiliess', false));
							}

							// Generate preview content
							$cat_rules_preview = generate_text_for_display($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options']);

							// decode...
							$cat_rules_data = generate_text_for_edit($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_options']);
						}

						// Parse desciption if specified
						if ($cat_data['cat_desc'])
						{
							if (!isset($cat_data['cat_desc_uid']))
							{
								// Before we are able to display the preview and plane text, we need to parse our request_var()'d value...
								$cat_data['cat_desc_uid'] = '';
								$cat_data['cat_desc_bitfield'] = '';
								$cat_data['cat_desc_options'] = 0;

								generate_text_for_storage($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_bitfield'], $cat_data['cat_desc_options'], request_var('desc_allow_bbcode', false), request_var('desc_allow_urls', false), request_var('desc_allow_smiliess', false));
							}

							// decode...
							$cat_desc_data = generate_text_for_edit($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_options']);
						}

						$cat_type_options = '';
						$cat_type_ary = array(ARCADE_CAT => 'CAT', ARCADE_CAT_GAMES => 'CAT_GAMES', ARCADE_LINK => 'LINK');

						foreach ($cat_type_ary as $value => $lang)
						{
							$cat_type_options .= '<option value="' . $value . '"' . (($value == $cat_data['cat_type']) ? ' selected="selected"' : '') . '>' . $user->lang['TYPE_' . $lang] . '</option>';
						}

						$styles_list = style_select($cat_data['cat_style'], true);
						$cat_display_list = $this->cat_display_select($cat_data['cat_display']);
						$statuslist = '<option value="' . ITEM_UNLOCKED . '"' . (($cat_data['cat_status'] == ITEM_UNLOCKED) ? ' selected="selected"' : '') . '>' . $user->lang['UNLOCKED'] . '</option><option value="' . ITEM_LOCKED . '"' . (($cat_data['cat_status'] == ITEM_LOCKED) ? ' selected="selected"' : '') . '>' . $user->lang['LOCKED'] . '</option>';

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_type = ' . ARCADE_CAT_GAMES . '
								AND cat_id <> ' . (int) $cat_id;
						$result = $db->sql_query_limit($sql, 1);

						$arcade_cat_games_exists = false;
						if ($db->sql_fetchrow($result))
						{
							$arcade_cat_games_exists = true;
						}
						$db->sql_freeresult($result);

						// Subforum move options
						if ($action == 'edit' && $cat_data['cat_type'] == ARCADE_CAT)
						{
							$subcat_id = array();
							$subcat = $arcade->get_cat_branch($cat_id, 'children');

							foreach ($subcat as $row)
							{
								$subcat_id[] = $row['cat_id'];
							}

							$cat_list = $arcade->make_cat_select($cat_data['parent_id'], $subcat_id);

							if ($arcade_cat_games_exists)
							{
								$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $subcat_id));
							}
							$db->sql_freeresult($result);

							$template->assign_vars(array(
								'S_HAS_SUBCATS'	=> ($cat_data['right_id'] - $cat_data['left_id'] > 1) ? true : false,
								'S_CATS_LIST'	=> $cat_list
							));
						}
						else if ($arcade_cat_games_exists)
						{
							$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $cat_id, false, true, false));
						}

						$s_show_display_on_index = false;

						if ($cat_data['parent_id'] > 0)
						{
							// if this forum is a subforum put the "display on index" checkbox
							if ($parent_info = $arcade->get_cat_info($cat_data['parent_id']))
							{
								if ($parent_info['parent_id'] > 0 || $parent_info['cat_type'] == ARCADE_CAT)
								{
									$s_show_display_on_index = true;
								}
							}
						}

						$filename_list = $arcade->generate_cat_images($cat_data['cat_image']);

						$template->assign_vars(array(
							'S_EDIT_CAT'					=> true,
							'S_ERROR'						=> (sizeof($errors)) ? true : false,
							'S_PARENT_ID'					=> $this->parent_id,
							'S_CAT_PARENT_ID'				=> $cat_data['parent_id'],
							'S_ADD_ACTION'					=> ($action == 'add') ? true : false,

							'U_BACK'						=> $this->u_action . '&amp;parent_id=' . $this->parent_id,
							'U_EDIT_ACTION'					=> $this->u_action . "&amp;parent_id={$this->parent_id}&amp;action=$action&amp;c=$cat_id",

							'L_COPY_PERMISSIONS_EXPLAIN'	=> $user->lang['COPY_PERMISSIONS_' . strtoupper($action) . '_EXPLAIN'],
							'L_TITLE'						=> $user->lang[$this->page_title],
							'ERROR_MSG'						=> (sizeof($errors)) ? implode('<br />', $errors) : '',

							'CAT_NAME'						=> $cat_data['cat_name'],
							'CAT_DATA_LINK'					=> $cat_data['cat_link'],
							'CAT_IMAGE'						=> $cat_data['cat_image'],
							'S_FILENAME_OPTIONS'			=> $filename_list,
							'S_IMAGE_BASEDIR'				=> $phpbb_root_path . $arcade_config['cat_image_path'],
							'CAT_IMAGE_SRC'					=> ($cat_data['cat_image']) ? $phpbb_root_path . $arcade_config['cat_image_path'] . $cat_data['cat_image'] : $phpbb_root_path . 'images/spacer.gif',
							'ARCADE_LINK'					=> ARCADE_LINK,
							'ARCADE_CAT'					=> ARCADE_CAT,
							'ARCADE_CAT_GAMES'				=> ARCADE_CAT_GAMES,
							'CAT_AGE'						=> $cat_data['cat_age'],
							'CAT_GAMES_PER_PAGE'			=> $cat_data['cat_games_per_page'],
							'CAT_DOWNLOAD'					=> $cat_data['cat_download'],
							'CAT_PASSWORD'					=> $cat_data['cat_password'],
							'CAT_PASSWORD_CONFIRM'			=> $cat_data['cat_password_confirm'],
							'CAT_RULES_LINK'				=> $cat_data['cat_rules_link'],
							'CAT_RULES'						=> $cat_data['cat_rules'],
							'CAT_RULES_PREVIEW'				=> $cat_rules_preview,
							'CAT_RULES_PLAIN'				=> $cat_rules_data['text'],
							'CAT_DOWNLOAD_COST'				=> $cat_data['cat_download_cost'],
							'CAT_COST'						=> $cat_data['cat_cost'],
							'CAT_REWARD'					=> $cat_data['cat_reward'],
							'CAT_USE_JACKPOT'				=> $cat_data['cat_use_jackpot'],
							'S_BBCODE_CHECKED'				=> ($cat_rules_data['allow_bbcode']) ? true : false,
							'S_SMILIES_CHECKED'				=> ($cat_rules_data['allow_smilies']) ? true : false,
							'S_URLS_CHECKED'				=> ($cat_rules_data['allow_urls']) ? true : false,
							'S_CAT_PASSWORD_SET'			=> (empty($cat_data['cat_password'])) ? false : true,

							'CAT_DESC'						=> $cat_desc_data['text'],
							'S_DESC_BBCODE_CHECKED'			=> ($cat_desc_data['allow_bbcode']) ? true : false,
							'S_DESC_SMILIES_CHECKED'		=> ($cat_desc_data['allow_smilies']) ? true : false,
							'S_DESC_URLS_CHECKED'			=> ($cat_desc_data['allow_urls']) ? true : false,

							'S_SHOW_POINTS'					=> ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings')) ? true : false,
							'S_CAT_TYPE_OPTIONS'			=> $cat_type_options,
							'S_STATUS_OPTIONS'				=> $statuslist,
							'S_PARENT_OPTIONS'				=> $parents_list,
							'S_STYLES_OPTIONS'				=> $styles_list,
							'S_CAT_DISPLAY_OPTIONS'			=> $cat_display_list,
							'S_CAT_OPTIONS'					=> $arcade->make_cat_select(($action == 'add') ? $cat_data['parent_id'] : false, ($action == 'edit') ? $cat_data['cat_id'] : false, false, false, false),
							'S_SHOW_DISPLAY_ON_INDEX'		=> $s_show_display_on_index,
							'S_DISPLAY_SUBCAT_LIST'			=> ($cat_data['display_subcat_list']) ? true : false,
							'S_DISPLAY_ON_INDEX'			=> ($cat_data['display_on_index']) ? true : false,
							'S_ARCADE_CAT'					=> ($cat_data['cat_type'] == ARCADE_CAT) ? true : false,
							'S_ARCADE_CAT_TEST'				=> ($cat_data['cat_test']) ? true : false,
							'S_CAT_ORIG_CAT_GAMES'			=> (isset($old_cat_type) && $old_cat_type == ARCADE_CAT_GAMES) ? true : false,
							'S_CAT_ORIG_CAT'				=> (isset($old_cat_type) && $old_cat_type == ARCADE_CAT) ? true : false,
							'S_CAT_ORIG_LINK'				=> (isset($old_cat_type) && $old_cat_type == ARCADE_LINK) ? true : false,
							'S_ARCADE_LINK'					=> ($cat_data['cat_type'] == ARCADE_LINK) ? true : false,
							'S_ARCADE_CAT_GAMES'			=> ($cat_data['cat_type'] == ARCADE_CAT_GAMES) ? true : false,
							'S_ARCADE_LINK_TRACK'			=> ($cat_data['cat_flags'] & ARCADE_FLAG_LINK_TRACK) ? true : false,
							'S_CAN_COPY_PERMISSIONS'		=> ($action != 'edit' || empty($cat_id) || ($auth->acl_get('a_cauth') && $auth->acl_get('a_authusers') && $auth->acl_get('a_authgroups'))) ? true : false,
						));
						return;
					break;

					case 'delete':
						if (!$cat_id)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$cat_data = $arcade->get_cat_info($cat_id);

						$subcat_id = array();
						$subcat = $arcade->get_cat_branch($cat_id, 'children');

						foreach ($subcat as $row)
						{
							$subcat_id[] = $row['cat_id'];
						}

						$cat_list = $arcade->make_cat_select($cat_data['parent_id'], $subcat_id);

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_type = ' . ARCADE_CAT_GAMES . "
								AND cat_id <> $cat_id";
						$result = $db->sql_query($sql);

						if ($db->sql_fetchrow($result))
						{
							$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $subcat_id, false, true));
						}
						$db->sql_freeresult($result);

						$parent_id = ($this->parent_id == $cat_id) ? 0 : $this->parent_id;

						$template->assign_vars(array(
							'S_DELETE_CAT'			=> true,
							'U_ACTION'				=> $this->u_action . "&amp;parent_id={$parent_id}&amp;action=delete&amp;c=$cat_id",
							'U_BACK'				=> $this->u_action . '&amp;parent_id=' . $this->parent_id,

							'CAT_NAME'				=> $cat_data['cat_name'],
							'S_ARCADE_CAT_GAMES'	=> ($cat_data['cat_type'] == ARCADE_CAT_GAMES) ? true : false,
							'S_ARCADE_LINK'			=> ($cat_data['cat_type'] == ARCADE_LINK) ? true : false,
							'S_HAS_SUBCATS'			=> ($cat_data['right_id'] - $cat_data['left_id'] > 1) ? true : false,
							'S_CATS_LIST'			=> $cat_list,
							'S_ERROR'				=> (sizeof($errors)) ? true : false,
							'ERROR_MSG'				=> (sizeof($errors)) ? implode('<br />', $errors) : ''
						));
						return;
					break;
				}

				// Default management page
				$parent_cat_type = '';
				if (!$this->parent_id)
				{
					$navigation = $user->lang['ARCADE_INDEX'];
				}
				else
				{
					$navigation = '<a href="' . $this->u_action . '">' . $user->lang['ARCADE_INDEX'] . '</a>';
					$cat_nav = $arcade->get_cat_branch($this->parent_id, 'parents', 'descending');
					foreach ($cat_nav as $row)
					{
						if ($row['cat_id'] == $this->parent_id)
						{
							$parent_cat_type = $row['cat_type'];
							$navigation .= ' -&gt; ' . $row['cat_name'];
						}
						else
						{
							$navigation .= ' -&gt; <a href="' . $this->u_action . '&amp;parent_id=' . $row['cat_id'] . '">' . $row['cat_name'] . '</a>';
						}
					}
				}

				// Jumpbox
				$cat_box = $arcade->make_cat_select($this->parent_id, false, false, false, false);
				if (($action == 'sync' || $action == 'sync_marked') && !sizeof($errors))
				{
					$template->assign_var('S_RESYNCED', true);
				}

				$sql = 'SELECT *
						FROM ' . ARCADE_CATS_TABLE . "
						WHERE parent_id = $this->parent_id
						ORDER BY left_id";
				$result = $db->sql_query($sql);

				if ($row = $db->sql_fetchrow($result))
				{
					do
					{
						$cat_type = $row['cat_type'];

						if ($row['cat_status'] == ITEM_LOCKED)
						{
							$folder_image = '<img src="images/icon_folder_lock.gif" alt="' . $user->lang['LOCKED'] . '" />';
						}
						else
						{
							switch ($cat_type)
							{
								case ARCADE_LINK:
									$folder_image = '<img src="images/icon_folder_link.gif" alt="' . $user->lang['TYPE_LINK'] . '" />';
								break;

								default:
									$folder_image = ($row['left_id'] + 1 != $row['right_id']) ? '<img src="images/icon_subfolder.gif" alt="' . $user->lang['SUBFORUM'] . '" />' : '<img src="images/icon_folder.gif" alt="' . $user->lang['FOLDER'] . '" />';
								break;
							}
						}

						$url = $this->u_action . "&amp;parent_id=$this->parent_id&amp;c={$row['cat_id']}" . (($start) ? "&amp;start=$start" : '');

						$cat_title = ($cat_type != ARCADE_LINK) ? '<a href="' . $this->u_action . '&amp;parent_id=' . $row['cat_id'] . '">' : '';
						$cat_title .= $row['cat_name'];
						$cat_title .= ($cat_type != ARCADE_LINK) ? '</a>' : '';

						$template->assign_block_vars('cats', array(
							'FOLDER_IMAGE'			=> $folder_image,
							'CAT_IMAGE'				=> ($row['cat_image']) ? '<img src="' . $phpbb_root_path . $arcade_config['cat_image_path'] . $row['cat_image'] . '" alt="" />' : '',
							'CAT_IMAGE_SRC'			=> ($row['cat_image']) ? $phpbb_root_path . $row['cat_image'] : '',
							'CAT_NAME'				=> $row['cat_name'],
							'CAT_DESCRIPTION'		=> generate_text_for_display($row['cat_desc'], $row['cat_desc_uid'], $row['cat_desc_bitfield'], $row['cat_desc_options']),
							'CAT_GAMES'				=> $row['cat_games'],
							'CAT_PLAYS'				=> $row['cat_plays'],

							'S_ARCADE_LINK'			=> ($cat_type == ARCADE_LINK) ? true : false,
							'S_ARCADE_CAT_GAMES'	=> ($cat_type == ARCADE_CAT_GAMES) ? true : false,

							'U_CAT'					=> $this->u_action . '&amp;parent_id=' . $row['cat_id'],
							'U_MOVE_UP'				=> $url . '&amp;action=move_up',
							'U_MOVE_DOWN'			=> $url . '&amp;action=move_down',
							'U_EDIT'				=> $url . '&amp;action=edit',
							'U_DELETE'				=> $url . '&amp;action=delete',
							'U_SYNC'				=> $url . '&amp;action=sync'
						));
					}
					while ($row = $db->sql_fetchrow($result));
				}
				else if ($this->parent_id)
				{
					$row = $arcade->get_cat_info($this->parent_id);

					$url = $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;c=' . $row['cat_id'] . (($start) ? "&amp;start=$start" : '');

					$template->assign_vars(array(
						'S_NO_CATS'	=> true,

						'U_EDIT'	=> $url . '&amp;action=edit',
						'U_DELETE'	=> $url . '&amp;action=delete',
						'U_SYNC'	=> $url . '&amp;action=sync'
					));
				}
				$db->sql_freeresult($result);

				$total_games = $arcade->get_total('games', $this->parent_id);

				if ($start >= $total_games)
				{
					$start = $start - $arcade_config['acp_items_per_page'];
					$start = ($start < 0) ? 0 : $start;
				}

				if ($total_games && $this->parent_id)
				{
					$sql = 'SELECT *
							FROM ' . ARCADE_GAMES_TABLE . "
							WHERE cat_id = $this->parent_id
							ORDER BY game_order DESC";
					$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

					if ($row = $db->sql_fetchrow($result))
					{
						do
						{
							$url = $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;g=' . $row['game_id'] . (($start) ? "&amp;start=$start" : '');

							$template->assign_block_vars('games', array(
								'GAME_ID'		=> $row['game_id'],
								'GAME_NAME'		=> $row['game_name'],
								'GAME_DESC'		=> nl2br($row['game_desc']),
								'GAME_PLAYS'	=> $row['game_plays'],
								'GAME_IMAGE'	=> ($row['game_image']) ? '<img src="' . $arcade->url("img={$row['game_image']}") . '" width="50" height="50" alt="" />' : '',

								'U_GAME_PLAY'	=> $arcade->url("mode=play&amp;g={$row['game_id']}"),
								'U_MOVE_UP'		=> $url . '&amp;action=g_move_up',
								'U_MOVE_DOWN'	=> $url . '&amp;action=g_move_down',
								'U_EDIT'		=> append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=games&amp;action=edit&amp;g={$row['game_id']}" . (($start) ? "&amp;start=$start" : '')),
								'U_DELETE'		=> $url . '&amp;action=g_delete',
								'U_SYNC'		=> $url . '&amp;action=sync'
							));
						}
						while ($row = $db->sql_fetchrow($result));
					}
					$db->sql_freeresult($result);

					$mark_options = array('sync_marked', 'reset_scores_marked', 'reset_marked', 'move_marked', 'delete_marked');

					if ((!$auth->acl_get('a_arcade_delete_game') && $key = array_search('delete_marked', $mark_options)) || (!$auth->acl_get('a_arcade_user') && ($key = array_search('reset_marked', $mark_options) || $key = array_search('reset_scores_marked', $mark_options))))
					{
						unset($mark_options[$key]);
					}

					$s_mark_options = '';
					foreach ($mark_options as $mark_option)
					{
						$s_mark_options .= '<option value="' . $mark_option . '">' . $user->lang['ARCADE_' . strtoupper($mark_option)] . '</option>';
					}

					$template->assign_vars(array(
						'S_MARK_OPTIONS'	=> $s_mark_options,
						'S_ON_PAGE'			=> on_page($total_games, $arcade_config['acp_items_per_page'], $start),
						'PAGINATION'		=> generate_pagination($this->u_action . '&amp;parent_id=' . $this->parent_id, $total_games, $arcade_config['acp_items_per_page'], $start, true)
					));


					$sql = 'SELECT *
							FROM ' . ARCADE_GAMES_TABLE . "
							WHERE cat_id = $this->parent_id
							ORDER BY game_name_clean ASC";
					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					// Quick jump of all the games in this category.
					foreach ($row as $game)
					{
						$template->assign_block_vars('quick_jump', array(
							'GAME_ID'	=> $game['game_id'],
							'GAME_NAME'	=> $game['game_name']
						));
					}
					// Quick jump of all the games in this category.
				}

				$template->assign_vars(array(
					'ERROR_MSG'				=> (sizeof($errors)) ? implode('<br />', $errors) : '',
					'NAVIGATION'			=> $navigation,
					'CAT_BOX'				=> $cat_box,
					'S_ARCADE_CAT_GAMES'	=> ($parent_cat_type == ARCADE_CAT_GAMES) ? true : false,
					'U_SEL_ACTION'			=> $this->u_action,
					'U_ACTION'				=> $this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : ''),
					'U_ACTION_DELETE'		=> $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;action=g_delete' . (($start) ? "&amp;start=$start" : ''),

					'U_PROGRESS_BAR'		=> $this->u_action . '&amp;action=progress_bar',
					'UA_PROGRESS_BAR'		=> str_replace('&amp;', '&', $this->u_action) . '&action=progress_bar'
				));
			break;

			case 'games':
				$form_key = 'acp_arcade_games';
				add_form_key($form_key);

				$action				= request_var('action', '');
				$start				= request_var('start', 0);
				$game_id			= request_var('g', 0);
				$cat_id				= request_var('c', 0);
				$game_type			= request_var('gt', 0);
				$gametypev3			= request_var('gametypev3', 0);
				$gametypev32		= request_var('gametypev32', 0);
				$update				= (isset($_POST['update'])) ? true : false;
				$delete_game		= (isset($_POST['delete'])) ? true : false;
				$edit_game			= (isset($_POST['edit'])) ? true : false;
				$cat_games_delete	= (!empty($_POST['cat_games_delete'])) ? true : false;
				$type_games_delete	= (!empty($_POST['type_games_delete'])) ? true : false;
				$error				= array();

				$template->assign_var('S_MANAGE_GAMES', true);

				if ($action == '')
				{
					if ($edit_game)
					{
						$action	= 'edit';
					}
					else if ($delete_game || $cat_games_delete || $type_games_delete)
					{
						$action = 'delete';
					}
				}

				switch ($action)
				{
					case 'edit':
						$old_cat_id = request_var('old_cat_id', 0);
						$old_game_scoretype = request_var('old_game_scoretype', 0);
						$template->assign_var('S_EDIT_GAME', true);

						$display_vars = array(
							'title'					=> 'ARCADE_EDIT_GAME',
							'title_explain'			=> 'ARCADE_EDIT_GAME_EXPLAIN',
							'vars'	=> array(
								'legend1'			=> 'ARCADE_GAME_SETTINGS',
								'game_name'			=> array('lang' => 'ARCADE_GAME_NAME',			'validate' => 'string', 	'type' => 'text:30:255', 	'explain' => true),
								'game_desc'			=> array('lang' => 'ARCADE_GAME_DESC',										'type' => 'textarea:5:25', 	'explain' => true),
								'game_image'		=> array('lang' => 'ARCADE_GAME_IMAGE',			'validate' => 'string', 	'type' => 'text:30:255',	'explain' => true),
								'game_swf'			=> array('lang' => 'ARCADE_GAME_SWF',			'validate' => 'string', 	'type' => 'custom', 		'explain' => true, 	'method' => 'game_swf'),
								'game_width'		=> array('lang' => 'ARCADE_GAME_WIDTH',			'validate' => 'int:50', 	'type' => 'text:3:4', 		'explain' => true),
								'game_height'		=> array('lang' => 'ARCADE_GAME_HEIGHT',		'validate' => 'int:50', 	'type' => 'text:3:4', 		'explain' => true),
								'game_installdate'	=> array('lang' => 'ARCADE_GAME_INSTALLDATE',	'validate' => 'string',		'type' => 'custom', 		'explain' => true, 	'method' => 'game_installdate'),
								'game_filesize'		=> array('lang' => 'ARCADE_GAME_FILESIZE',		'validate' => 'string',		'type' => 'custom', 		'explain' => true, 	'method' => 'game_filesize'),
								'game_download'		=> array('lang' => 'ARCADE_GAME_DOWNLOAD', 		'validate' => 'bool', 		'type' => 'radio:yes_no', 	'explain' => true),
								'cat_id'			=> array('lang' => 'ARCADE_GAME_CATEGORY',		'validate' => 'int', 		'type' => 'select', 		'explain' => true, 	'method' => 'game_cat_select'),

								'legend2'			=> 'ARCADE_SCORE_SETTINGS',
								'game_scorevar'		=> array('lang' => 'ARCADE_GAME_SCOREVAR', 		'validate' => 'string', 	'type' => 'text:30:255', 	'explain' => true),
								'game_type'			=> array('lang' => 'ARCADE_GAME_TYPE',			'validate' => 'int', 		'type' => 'select', 		'explain' => true,	'method' => 'game_type_select'),
								'game_scoretype'	=> array('lang' => 'ARCADE_GAME_SCORETYPE',		'validate' => 'int', 		'type' => 'select', 		'explain' => true,	'method' => 'game_scoretype_select'),
							)
						);

						if ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings'))
						{
							$display_vars['vars'] += array(
								'legend3'			=> 'ARCADE_GAME_POINTS_SETTINGS',
								'game_download_cost'=> array('lang' => 'ARCADE_GAME_DOWNLOAD_COST',	'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_cost'			=> array('lang' => 'ARCADE_GAME_COST', 			'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_reward'		=> array('lang' => 'ARCADE_GAME_REWARD',		'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_use_jackpot'	=> array('lang' => 'ARCADE_GAME_USE_JACKPOT',	'validate' => 'bool', 		'type' => 'radio:yes_no', 		'explain' => true),
								'game_jackpot'		=> array('lang' => 'ARCADE_GAME_JACKPOT',		'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
							);
						}

						$this->new_game_data = $arcade->get_game_cat_data($game_id);
						if (!$this->new_game_data)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_game_data;

						// We validate the complete config if whished
						validate_config_vars($display_vars['vars'], $cfg_array, $error);

						if ($update && !check_form_key($form_key))
						{
							$error[] = $user->lang['FORM_INVALID'];
						}

						$cat_data = $arcade->get_cat_info($cfg_array['cat_id']);
						if ($cat_data['cat_type'] != ARCADE_CAT_GAMES)
						{
							$error[] = $user->lang['WRONG_CAT_TYPE'];
						}

						// Do not write values if there is an error
						if (sizeof($error))
						{
							$update = false;
						}

						// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
						foreach ($display_vars['vars'] as $config_name => $null)
						{
							if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
							{
								continue;
							}

							$config_value = $cfg_array[$config_name];
							$this->new_game_data[$config_name] = $config_value;

							if ($update)
							{
								if (in_array($config_name, array('game_download_cost', 'game_cost', 'game_reward', 'game_jackpot')))
								{
									$negative = ($config_name == 'game_jackpot') ? true : false;
									$arcade->float_num($config_value, $negative);
								}

								// We have to check if the cat_id has changed if it does we need to send the
								// new and old ids
								if ($config_name == 'cat_id')
								{
									$arcade->set_game_data($game_id, $config_name, $config_value, $this->new_game_data['cat_id'], $old_cat_id);
								}
								else if ($config_name == 'game_scoretype')
								{
									$arcade->set_game_data($game_id, $config_name, $config_value, false, false, $this->new_game_data['game_scoretype'], $old_game_scoretype);
								}
								else
								{
									$arcade->set_game_data($game_id, $config_name, $config_value);
								}

								$update_install_file = false;
								if (arcade_game_update_install_file($game_id))
								{
									$update_install_file = true;
								}
							}
						}

						if ($update)
						{
							add_log('admin', 'LOG_ARCADE_EDIT_GAMES', '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $this->new_game_data['game_id']) .'">'. $this->new_game_data['game_name'] .'</a>');
							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);

							$arcade_cache->destroy('_arcade_cats');

							$manage_games_link	= '<br /><br /><a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=games") . '">&laquo; ' . $user->lang['ACP_BACK_MANAGE_GAMES'] . '</a>';
							$manage_cat_link	= $auth->acl_get('a_arcade_cat') ? '<br /><br /><a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=category&amp;parent_id={$old_cat_id}" . (($start) ? "&amp;start=$start" : '')) . '">&laquo; ' . $user->lang['ACP_BACK_MANAGE_CATEGORY'] . '</a>' : '';
							$game_return_links	= '<br /><br />' . sprintf($user->lang['ARCADE_RETURN_LINKS'], '<a href="' . $arcade->url("mode=play&amp;g={$game_id}") . '">&laquo; ', $this->new_game_data['game_name'], '</a>', '<a href="' . $arcade->url('mode=cat&amp;c=' . $this->new_game_data['cat_id']) . '&amp;g=' . $game_id .'#g' . $game_id .'">&laquo; ', $this->new_game_data['cat_name'], '</a>', '<a href="' . $arcade->url() . '">&laquo; ', '</a>');

							if ($update_install_file)
							{
								trigger_error($user->lang['ARCADE_GAME_UPDATED'] . $manage_games_link . $manage_cat_link . $game_return_links);
							}
							else
							{
								trigger_error($user->lang['ARCADE_GAME_UPDATED_ERROR'] . $manage_games_link . $manage_cat_link . $game_return_links, E_USER_WARNING);
							}
						}

						// Output relevant page
						foreach ($display_vars['vars'] as $config_key => $vars)
						{

							if (!is_array($vars) && strpos($config_key, 'legend') === false)
							{
								continue;
							}

							if (strpos($config_key, 'legend') !== false)
							{
								$template->assign_block_vars('options', array(
									'S_LEGEND'	=> true,
									'LEGEND'	=> $user->lang[$vars])
								);

								continue;
							}

							$type = explode(':', $vars['type']);

							if (in_array($config_key, array('game_download_cost', 'game_cost', 'game_reward', 'game_jackpot')))
							{
								$this->new_game_data[$config_key] = $arcade->number_format($this->new_game_data[$config_key], true);
							}

							$template->assign_block_vars('options', array(
								'KEY'			=> (in_array($config_key, array('game_swf', 'game_installdate', 'game_filesize'))) ? false : $config_key,
								'TITLE'			=> $user->lang[$vars['lang']],
								'S_EXPLAIN'		=> $vars['explain'],
								'TITLE_EXPLAIN'	=> ($vars['explain']) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '',
								'CONTENT'		=> build_cfg_template($type, $config_key, $this->new_game_data, $config_key, $vars),
							));

							unset($display_vars['vars'][$config_key]);
						}

						if (isset($display_vars['lang']))
						{
							$user->add_lang($display_vars['lang']);
						}

						$this->page_title = $display_vars['title'];

						$template->assign_vars(array(
								'L_TITLE'				=> $user->lang[$display_vars['title']],
								'L_TITLE_EXPLAIN'		=> $user->lang[$display_vars['title_explain']],

								'S_ERROR'				=> (sizeof($error)) ? true : false,
								'ERROR_MSG'				=> implode('<br />', $error),

								'OLD_CAT_ID'			=> $this->new_game_data['cat_id'],
								'OLD_GAME_SCORETYPE'	=> $this->new_game_data['game_scoretype'],
								'U_ACTION'				=> $this->u_action . '&amp;action=edit&amp;g=' . $game_id . (($start) ? '&amp;start=' . $start : '')
						));

						return;
					break;

					case 'delete':
						if (confirm_box(true))
						{
							$delete_files = false;
							if (isset($_POST['delete_files']))
							{
								// language frontier
								if (request_var('delete_files', '') === $user->lang['YES'])
								{
									$delete_files = true;
								}
							}

							if ($cat_games_delete || $type_games_delete)
							{
								if ($type_games_delete && $game_type == IBPROV3_GAME && !$gametypev3 && !$gametypev32)
								{
									$error[] = $user->lang['ARCADE_SELECT_EMPTY'];
								}
								else
								{
									$type	= ($cat_games_delete) ? 'cat' : 'gametype';
									$id		= ($cat_id) ? $cat_id : $game_type;

									$row = $arcade->get_cat_game_ids($type, $id, $gametypev3, $gametypev32);

									if (!$row)
									{
										$error[] = $user->lang['NOT_FOUND_GAME'];
									}
									else
									{
										$error = $arcade->delete_game($row['game_ids'], $row['cat_ids'], $delete_files);
										$game_names = implode(', ', $row['game_names']);
									}
								}
							}
							else
							{
								$game_data	= $arcade->get_game_data($game_id);
								$error		= $arcade->delete_game($game_id, $game_data['cat_id'], $delete_files);
								$game_names	= $game_data['game_name'];
							}

							if (sizeof($error))
							{
								break;
							}

							$s = ($cat_games_delete || $type_games_delete) ? 'S' : '';

							add_log('admin', 'LOG_ARCADE_DELETE_GAME' . $s, $game_names);

							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);
							$arcade_cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

							$arcade_cache->destroy('_arcade_cats');
							$arcade_cache->destroy('_arcade_leaders');
							$arcade_cache->destroy('_arcade_leaders_all');
							$arcade_cache->destroy('_arcade_games_filesize');
							$arcade_cache->destroy('_arcade_challenge_users');
							$arcade_cache->destroy('_arcade_challenge_leaders');
							$arcade_cache->destroy('_arcade_challenge_leaders_all');

							trigger_error(sprintf($user->lang['GAME' . $s . '_DELETED'], $game_names) . adm_back_link($this->u_action));
						}
						else
						{
							if (!$auth->acl_get('a_arcade_delete_game'))
							{
								trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							if ($cat_games_delete)
							{
								$confirm_lang = 'DELETE_SELECTED_CAT_ALL_GAMES';
								$s_hidden_fields = array(
									'c'					=> $cat_id,
									'cat_games_delete'	=> true
								);
							}
							else if ($type_games_delete)
							{
								$confirm_lang = 'DELETE_SELECTED_GAMETYPE_ALL_GAMES';
								$s_hidden_fields = array(
									'gt'				=> $game_type,
									'type_games_delete'	=> true,
									'gametypev3'		=> $gametypev3,
									'gametypev32'		=> $gametypev32
								);
							}
							else
							{
								$confirm_lang = 'DELETE_SELECTED_GAME';
								$s_hidden_fields = array('g' => $game_id);
							}

							$s_hidden_fields = array_merge($s_hidden_fields, array('action' => 'delete'));

							confirm_box(false, $confirm_lang, build_hidden_fields($s_hidden_fields), 'confirm_body_delete_arcade_games.html');
						}
					break;
				}

				// Default Arcade Games Page
				// Quick jump of all the games in the arcade...
				$sql = 'SELECT game_id, game_name, game_name_clean
					FROM ' . ARCADE_GAMES_TABLE . "
					ORDER BY game_name_clean ASC";
				$result = $db->sql_query($sql);

				while ($row = $db->sql_fetchrow($result))
				{
					$template->assign_block_vars('quick_jump', array(
						'GAME_ID'	=> $row['game_id'],
						'GAME_NAME'	=> $row['game_name'],
					));
				}
				$db->sql_freeresult($result);
				// Quick jump of all the games in the arcade...

				$template->assign_vars(array(
					'L_TITLE'				=> $user->lang['ACP_ARCADE_MANAGE_GAMES'],
					'L_TITLE_EXPLAIN'		=> $user->lang['ACP_ARCADE_MANAGE_GAMES_EXPLAIN'],

					'S_ERROR'				=> (sizeof($error)) ? true : false,
					'ERROR_MSG'				=> (sizeof($error)) ? implode('<br />', $error) : '',
					'S_DEFAULT_EDIT_GAME'	=> true,
					'S_CATEGORY_OPTIONS'	=> $arcade->make_cat_select(false, false, true, false, false),
					'S_GAMETYPE_OPTIONS'	=> $arcade->game_type_select(),
					'S_IBPROV3_GAME'		=> IBPROV3_GAME,

					'U_ACTION'				=> $this->u_action
				));
			break;

			case 'users':
				$form_key = 'acp_arcade_users';
				add_form_key($form_key);
				$template->assign_var('S_MANAGE_USERS', true);

				$start		= request_var('start', 0);
				$user_id	= request_var('u', 0);
				$score		= request_var('score', 0.000);
				$score_date	= request_var('score_date', 0);
				$date_now	= request_var('date_now', false);
				$game_id	= request_var('g', 0);
				$comment	= request_var('comment', '', true);
				$update		= (isset($_POST['update'])) ? true : false;
				$action		= request_var('action', '');

				if ($update && $action == 'edit')
				{
					if (!check_form_key($form_key))
					{
						trigger_error('FORM_INVALID', E_USER_WARNING);
					}

					$score_data = array(
						'score'				=> $score,
						'score_date'		=> ($date_now) ? time(): $score_date,
						'comment_text'		=> $comment,
						'comment_bitfield'	=> '',
						'comment_options'	=> 7,
						'comment_uid'		=> '',
					);
					$game_data = $arcade->get_game_data($game_id);
					$arcade->update_score($score_data, $user_id, $game_id);
					$username = $arcade->userdata('username', $user_id);

					$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
					$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
					$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
					$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

					$arcade_cache->destroy('_arcade_leaders');
					$arcade_cache->destroy('_arcade_leaders_all');

					add_log('admin', 'LOG_ARCADE_EDIT_SCORE', $username, '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $game_data['game_id']) .'">'. $game_data['game_name'] .'</a>');
					trigger_error(sprintf($user->lang['ARCADE_SCORE_UPDATED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
				}

				$action = (isset($_POST['reset_user_scores'])) ? 'reset_user_scores' : $action;
				$action = (isset($_POST['reset_user_all'])) ? 'reset_user_all' : $action;
				$action = (isset($_POST['user_banned'])) ? 'user_banned' : $action;

				switch ($action)
				{
					case 'reset_user_scores':
						if (confirm_box(true))
						{
							$arcade->reset('user_scores', $user_id);
							$username = $arcade->userdata('username', $user_id);

							add_log('admin', 'LOG_ARCADE_RESET_USER_SCORES', $username);
							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

							$arcade_cache->destroy('_arcade_leaders');
							$arcade_cache->destroy('_arcade_leaders_all');

							$arcade->sync('total_data', 'plays');

							trigger_error(sprintf($user->lang['ARCADE_RESET_USER_SCORES_DONE'], $username) . adm_back_link($this->u_action));
						}
						else
						{
							$s_hidden_fields = array(
								'u'					=> $user_id,
								'reset_user_scores'	=> true,
							);
							confirm_box(false, 'RESET_USER_SCORES', build_hidden_fields($s_hidden_fields));
						}
					break;

					case 'reset_user_all':
						if (confirm_box(true))
						{
							$arcade->reset('user', $user_id);
							$username = $arcade->userdata('username', $user_id);

							add_log('admin', 'LOG_ARCADE_RESET_USER_ALL', $username);
							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

							$arcade_cache->destroy('_arcade_leaders');
							$arcade_cache->destroy('_arcade_leaders_all');

							$arcade->sync('total_data', 'plays');

							trigger_error(sprintf($user->lang['ARCADE_RESET_USER_ALL_DONE'], $username) . adm_back_link($this->u_action));
						}
						else
						{
							$s_hidden_fields = array(
								'u'					=> $user_id,
								'reset_user_all'	=> true,
							);
							confirm_box(false, 'RESET_USER_ALL', build_hidden_fields($s_hidden_fields));
						}
					break;

					case 'user_banned':
						if (confirm_box(true))
						{
							$db->sql_return_on_error(true);
							$arcade->banned(ARCADE_BANNED_ADMIN, $user_id);
							$db->sql_return_on_error(false);
							$username = $arcade->userdata('username', $user_id);

							add_log('admin', 'LOG_ARCADE_USER_BANNED', $username);
							trigger_error(sprintf($user->lang['ARCADE_USER_BANNED_DONE'], $username) . adm_back_link($this->u_action));
						}
						else
						{
							$user->add_lang('acp/users');

							if ($user->data['user_id'] == $user_id)
							{
								trigger_error($user->lang['CANNOT_BAN_YOURSELF'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							if ($arcade->userdata('user_type', $user_id) == USER_FOUNDER)
							{
								trigger_error($user->lang['CANNOT_BAN_FOUNDER'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							$s_hidden_fields = array(
								'u'				=> $user_id,
								'user_banned'	=> true,
							);
							confirm_box(false, 'ARCADE_USER_BANNED', build_hidden_fields($s_hidden_fields));
						}
					break;

					case 'delete':
						if (confirm_box(true))
						{
							$game_data = $arcade->get_game_data($game_id);
							$arcade->delete_score($game_data, $user_id);
							$username = $arcade->userdata('username', $user_id);

							$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
							$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
							$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);

							$arcade_cache->destroy('_arcade_leaders');
							$arcade_cache->destroy('_arcade_leaders_all');

							add_log('admin', 'LOG_ARCADE_DELETE_SCORE', $username, $game_data['game_name']);
							trigger_error(sprintf($user->lang['ARCADE_SCORE_DELETED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
						}
						else
						{
							confirm_box(false, 'DELETE_SELECTED_SCORE');
						}
					break;

					case 'progress_bar':
						$this->display_progress_bar();
						exit;
					break;

					case 'sync':
						$game_data = $arcade->get_game_data($game_id);
						$template->assign_vars(array(
							'S_RESYNCED' 	=> true,
							'GAME_RESYNCED' => sprintf($user->lang['GAME_RESYNCED'], $game_data['game_name']),
							)
						);
						$arcade->sync('game', $game_id);
						$arcade->update_highscore($game_data);
						add_log('admin', 'LOG_ARCADE_SYNC_GAME',  $game_data['game_name']);
					break;

					case 'edit':

						if (!$game_id || !$user_id)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}
						$template->assign_var('S_EDIT_SCORES', true);

						$sql_array = array(
							'SELECT'	=> 'g.*, s.*, p.total_time, p.total_plays, u.username, u.user_colour',
							'FROM'		=> array(
								ARCADE_SCORES_TABLE	=> 's',
							),
							'LEFT_JOIN'	=> array(
								array(
									'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
									'ON'	=> 's.game_id = g.game_id'
								),
								array(
									'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
									'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
								),
								array(
									'FROM'	=> array(USERS_TABLE => 'u'),
									'ON'	=> 's.user_id = u.user_id'
								),
							),
							'WHERE'		=> 's.user_id = ' . (int) $user_id . ' AND g.game_id = ' . (int) $game_id,
						);

						$sql = $db->sql_build_query('SELECT', $sql_array);
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$comment_data = generate_text_for_edit($row['comment_text'], $row['comment_uid'], $row['comment_options']);

						$template->assign_vars(array(
							'SCORE_USERNAME'	=> get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'arcade', 'x'),
							'GAME_NAME'			=> $row['game_name'],
							'SCORE'				=> $row['score'],
							'COMMENT'			=> $comment_data['text'],
							'DATE_RAW'			=> $row['score_date'],
							'DATE'				=> $user->format_date($row['score_date']),
							'TOTAL_PLAYS'		=> $row['total_plays'],
							'TOTAL_TIME'		=> $arcade->time_format($row['total_time']),
							'AVG_TIME'			=> $arcade->time_format($row['total_time']/$row['total_plays'])
						));

						$template->assign_vars(array(
							'U_EDIT_ACTION'		=> $this->u_action . "&amp;g=$game_id&amp;u=$user_id&amp;action=$action",

							'L_TITLE_EXPLAIN'	=> $user->lang['ACP_ARCADE_EDIT_SCORES_EDIT_EXPLAIN'],
						));

						return;
					break;

					case 'show_scores':
						$template->assign_var('S_GAME_SCORES', true);

						$game_data = $arcade->get_game_data($game_id);
						$score_order = ($game_data['game_scoretype'] == SCORETYPE_HIGH) ? 'DESC' : 'ASC';

						$sql_array = array(
							'SELECT'	=> 's.*, p.total_time, p.total_plays, u.user_id, u.user_colour, u.username',
							'FROM'		=> array(
								ARCADE_SCORES_TABLE	=> 's',
							),
							'LEFT_JOIN'	=> array(
								array(
									'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
									'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
								),
								array(
									'FROM'	=> array(USERS_TABLE => 'u'),
									'ON'	=> 's.user_id = u.user_id'
								),
							),
							'WHERE'		=> 's.game_id = ' . (int) $game_id,
							'ORDER_BY'	=> 's.score ' . $score_order . ', s.score_date ASC',
						);

						$sql = $db->sql_build_query('SELECT', $sql_array);
						$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

						$position = ($start > 0 ) ? $start : 0;
						$actual_position = ($start > 0 ) ? $start : 0;
						$lastscore = '';
						while ($row = $db->sql_fetchrow($result))
						{
							$actual_position++;

							if ($lastscore != $row['score'])
							{
								$position = $actual_position;
							}
							$lastscore = $row['score'] ;

							$url = $this->u_action . '&amp;g=' . $row['game_id'] . '&amp;u=' . $row['user_id'];

							$template->assign_block_vars('scorerow', array(
								'POS'			=> $position,
								'USERNAME'		=> get_arcade_user_name('full', $row['user_id'], $row['username'], $row['user_colour'], 'arcade', 'x'),
								'SCORE'			=> $arcade->number_format($row['score']),
								'COMMENT'		=> generate_text_for_display($row['comment_text'], $row['comment_uid'], $row['comment_bitfield'], $row['comment_options']),
								'DATE'			=> $user->format_date($row['score_date']),
								'TOTAL_PLAYS'	=> $row['total_plays'],
								'TOTAL_TIME'	=> $arcade->time_format($row['total_time']),
								'U_EDIT'		=> $url . '&amp;action=edit',
								'U_DELETE'		=> $url . '&amp;action=delete',
							));
						}
						$db->sql_freeresult($result);

						$template->assign_vars(array(
							'GAME_TITLE'		=> sprintf($user->lang['ARCADE_GAME_SCORES'], $game_data['game_name']),

							'L_TITLE_EXPLAIN'	=> $user->lang['ACP_ARCADE_EDIT_SCORES_LIST_EXPLAIN'],
						));

						$total_scores = $arcade->get_total('scores', $game_id);
						$template->assign_vars(array(
							'S_ON_PAGE'		=> on_page($total_scores, $arcade_config['acp_items_per_page'], $start),
							'PAGINATION'	=> generate_pagination($this->u_action . '&amp;action=show_scores&amp;g=' . $game_id, $total_scores, $arcade_config['acp_items_per_page'], $start, true),
						));

						return;
					break;
				}

				$template->assign_var('S_USERS_OPTIONS', true);

				$sql_array = array(
					'SELECT'	=> 'g.game_id , g.game_name, g.game_name_clean, g.game_image, g.game_desc, g.game_plays',
					'FROM'		=> array(
						ARCADE_SCORES_TABLE	=> 's',
					),
					'LEFT_JOIN'	=> array(
						array(
							'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
							'ON'	=> 'g.game_id = s.game_id'
						),
					),
					'ORDER_BY'	=> 'g.game_name_clean ASC',
				);

				$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
				$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

				while($row = $db->sql_fetchrow($result))
				{
					$url = $this->u_action . '&amp;g=' . $row['game_id'];

					$template->assign_block_vars('games', array(
						'GAME_ID'		=> $row['game_id'],
						'GAME_NAME'		=> $row['game_name'],
						'GAME_DESC'		=> $row['game_desc'],
						'GAME_IMAGE'	=> ($row['game_image']) ? '<img src="' . $arcade->url("img={$row['game_image']}") . '" width="25" height="25" alt="" />' : '',
						'GAME_PLAYS'	=> $row['game_plays'],

						'U_GAME_PLAY'	=> $arcade->url("mode=play&amp;g={$row['game_id']}"),
						'U_EDIT'		=> $url . '&amp;action=show_scores',
						'U_SYNC'		=> $url . '&amp;action=sync&amp;start=' . $start,
					));
				}
				$db->sql_freeresult($result);

				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrowset($result);
				$db->sql_freeresult($result);

				// Start quick jump of all the games in the arcade with scores...
				foreach ($row as $game)
				{
					$template->assign_block_vars('quick_jump', array(
						'GAME_ID'	=> $game['game_id'],
						'GAME_NAME'	=> $game['game_name']
					));
				}

				$s_hidden_fields = build_hidden_fields(array(
					'action'	=> 'show_scores',
				));
				// End quick jump of all the games in the arcade...

				// This next bit of code is used to grab all the users that have some type of arcade data stored
				$user_ids = array();
				// These tables could all possibly contain user related arcade data
				$tables = array(ARCADE_FAVS_TABLE, ARCADE_PLAYS_TABLE, ARCADE_RATING_TABLE, ARCADE_SCORES_TABLE);
				foreach ($tables as $table)
				{
					$sql = "SELECT user_id from $table ORDER BY user_id";
					$result = $db->sql_query($sql);

					while ($row = $db->sql_fetchrow($result))
					{
						$user_ids[] = $row['user_id'];
					}
					$db->sql_freeresult($result);
				}

				// Order and remove the duplicated user ids
				$user_ids = array_merge(array_unique($user_ids));

				if (sizeof($user_ids))
				{
					// Now lets get the usernames for all the users
					$sql = 'SELECT user_id, username, username_clean
							FROM ' . USERS_TABLE . '
							WHERE ' . $db->sql_in_set('user_id', $user_ids) . '
							ORDER BY username_clean ASC';
					$result = $db->sql_query($sql);

					$user_data = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					foreach ($user_data as $item)
					{
						$template->assign_block_vars('user_jump', array(
							'USER_ID'	=> $item['user_id'],
							'USERNAME'	=> $item['username']
						));
					}
				}

				$template->assign_vars(array(
					'L_TITLE_EXPLAIN'	=> $user->lang['ACP_ARCADE_MANAGE_USERS_EXPLAIN'],

					'S_HIDDEN_FIELDS'	=> $s_hidden_fields,
					'S_SHOW_POINTS'		=> ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings')),

					'U_ACTION'			=> $this->u_action,
					'U_PROGRESS_BAR'	=> $this->u_action . '&amp;action=progress_bar',
					'UA_PROGRESS_BAR'	=> str_replace('&amp;', '&', $this->u_action) . '&action=progress_bar',
				));

				$total_scores = $arcade->get_total('scores');
				$template->assign_vars(array(
					'S_ON_PAGE'			=> on_page($total_scores, $arcade_config['acp_items_per_page'], $start),
					'PAGINATION'		=> generate_pagination($this->u_action, $total_scores, $arcade_config['acp_items_per_page'], $start, true),
				));
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}
	}

	function cat_display_select($value = '')
	{
			global $user;

			return '<option value="' . ARCADE_CAT_DISPLAY_BOTH . '"' . (($value == ARCADE_CAT_DISPLAY_BOTH) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_BOTH'] . '</option><option value="' . ARCADE_CAT_DISPLAY_NAME . '"' . (($value == ARCADE_CAT_DISPLAY_NAME) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_NAME'] . '</option><option value="' . ARCADE_CAT_DISPLAY_IMAGE . '"' . (($value == ARCADE_CAT_DISPLAY_IMAGE) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_IMAGE'] . '</option>';
	}

	/**
	* Update cat data
	*/
	function update_cat_data(&$cat_data)
	{
		global $db, $user, $arcade_cache, $arcade;

		$errors = array();

		if ($cat_data['cat_name'] == '')
		{
			$errors[] = $user->lang['CAT_NAME_EMPTY'];
		}

		if (utf8_strlen($cat_data['cat_desc']) > 4000)
		{
			$errors[] = $user->lang['CAT_DESC_TOO_LONG'];
		}

		if (utf8_strlen($cat_data['cat_rules']) > 4000)
		{
			$errors[] = $user->lang['CAT_RULES_TOO_LONG'];
		}

		if ($cat_data['cat_password'] || $cat_data['cat_password_confirm'])
		{
			if ($cat_data['cat_password'] != $cat_data['cat_password_confirm'])
			{
				$cat_data['cat_password'] = $cat_data['cat_password_confirm'] = '';
				$errors[] = $user->lang['CAT_PASSWORD_MISMATCH'];
			}
		}

		// Set category flags
		// 1 = link tracking
		$cat_data['cat_flags'] = 0;
		$cat_data['cat_flags'] += ($cat_data['cat_link_track']) ? ARCADE_FLAG_LINK_TRACK : 0;

		// Unset data that are not database fields
		$cat_data_sql = $cat_data;

		unset($cat_data_sql['cat_link_track']);
		unset($cat_data_sql['cat_password_confirm']);

		// What are we going to do tonight Brain? The same thing we do everynight,
		// try to take over the world ... or decide whether to continue update
		// and if so, whether it's a new forum/cat/link or an existing one
		if (sizeof($errors))
		{
			return $errors;
		}

		// As we don't know the old password, it's kinda tricky to detect changes
		if ($cat_data_sql['cat_password_unset'])
		{
			$cat_data_sql['cat_password'] = '';
		}
		else if (empty($cat_data_sql['cat_password']))
		{
			unset($cat_data_sql['cat_password']);
		}
		else
		{
			$cat_data_sql['cat_password'] = phpbb_hash($cat_data_sql['cat_password']);
		}
		unset($cat_data_sql['cat_password_unset']);

		if (!isset($cat_data_sql['cat_id']))
		{
			// no cat_id means we're creating a new forum
			unset($cat_data_sql['type_action']);

			if ($cat_data_sql['parent_id'])
			{
				$sql = 'SELECT left_id, right_id
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $cat_data_sql['parent_id'];
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					trigger_error($user->lang['ARCADE_PARENT_NOT_EXIST'] . adm_back_link($this->u_action . '&amp;' . $this->parent_id), E_USER_WARNING);
				}

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET left_id = left_id + 2, right_id = right_id + 2
						WHERE left_id > ' . $row['right_id'];
				$db->sql_query($sql);

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET right_id = right_id + 2
						WHERE ' . $row['left_id'] . ' BETWEEN left_id AND right_id';
				$db->sql_query($sql);

				$cat_data_sql['left_id'] = $row['right_id'];
				$cat_data_sql['right_id'] = $row['right_id'] + 1;
			}
			else
			{
				$sql = 'SELECT MAX(right_id) AS right_id
						FROM ' . ARCADE_CATS_TABLE;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$cat_data_sql['left_id'] = $row['right_id'] + 1;
				$cat_data_sql['right_id'] = $row['right_id'] + 2;
			}

			$sql = 'INSERT INTO ' . ARCADE_CATS_TABLE . ' ' . $db->sql_build_array('INSERT', $cat_data_sql);
			$db->sql_query($sql);

			$cat_data['cat_id'] = $db->sql_nextid();

			add_log('admin', 'LOG_ARCADE_ADD_CAT', $cat_data['cat_name']);
		}
		else
		{
			$row = $arcade->get_cat_info($cat_data_sql['cat_id']);

			if ($row['cat_type'] == ARCADE_CAT_GAMES && $row['cat_type'] != $cat_data_sql['cat_type'])
			{
				// Has subforums and want to change into a link?
				if ($row['right_id'] - $row['left_id'] > 1 && $cat_data_sql['cat_type'] == ARCADE_LINK)
				{
					$errors[] = $user->lang['CAT_WITH_SUBCATS_NOT_TO_LINK'];
					return $errors;
				}

				// we're turning a postable forum into a non-postable forum
				if ($cat_data_sql['type_action'] == 'move')
				{
					$to_cat_id = request_var('to_cat_id', 0);

					if ($to_cat_id)
					{
						$errors = $this->move_cat_content($cat_data_sql['cat_id'], $to_cat_id);
					}
					else
					{
						return array($user->lang['NO_DESTINATION_CAT']);
					}
				}
				else if ($cat_data_sql['type_action'] == 'delete')
				{
					$delete_files = false;
					if (isset($_POST['delete_files']))
					{
						// language frontier
						if (request_var('delete_files', '') === $user->lang['YES'])
						{
							$delete_files = true;
						}
					}

					$errors = $this->delete_cat_content($cat_data_sql['cat_id'], $delete_files);
				}
				else
				{
					return array($user->lang['NO_CAT_ACTION']);
				}

				$cat_data_sql['cat_games'] = $cat_data_sql['cat_plays'] = $cat_data_sql['cat_last_play_game_id'] = $cat_data_sql['cat_last_game_installdate'] = $cat_data_sql['cat_last_play_user_id'] = $cat_data_sql['cat_last_play_score'] = $cat_data_sql['cat_last_play_time'] = 0;
				$cat_data_sql['cat_last_play_game_name'] = $cat_data_sql['cat_last_play_username'] = $cat_data_sql['cat_last_play_user_colour'] = '';
			}
			else if ($row['cat_type'] == ARCADE_CAT && $cat_data_sql['cat_type'] == ARCADE_LINK)
			{
				// Has subforums?
				if ($row['right_id'] - $row['left_id'] > 1)
				{
					// We are turning a category into a link - but need to decide what to do with the subforums.
					$action_subcats = request_var('action_subcats', '');
					$subcats_to_id = request_var('subcats_to_id', 0);

					if ($action_subcats == 'delete')
					{
						$rows = $arcade->get_cat_branch($row['cat_id'], 'children', 'descending', false);

						$delete_files = false;
						if (isset($_POST['sub_delete_files']))
						{
							// language frontier
							if (request_var('sub_delete_files', '') === $user->lang['YES'])
							{
								$sub_delete_files = true;
							}
						}

						foreach ($rows as $_row)
						{
							// Do not remove the forum id we are about to change. ;)
							if ($_row['cat_id'] == $row['cat_id'])
							{
								continue;
							}

							$cat_ids[] = $_row['cat_id'];
							$errors = array_merge($errors, $this->delete_cat_content($_row['cat_id'], $sub_delete_files));
						}

						if (sizeof($errors))
						{
							return $errors;
						}

						if (sizeof($cat_ids))
						{
							$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . '
									WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
							$db->sql_query($sql);
						}
					}
					else if ($action_subcats == 'move')
					{
						if (!$subcats_to_id)
						{
							return array($user->lang['NO_DESTINATION_CAT']);
						}

						$sql = 'SELECT cat_name
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_id = ' . $subcats_to_id;
						$result = $db->sql_query($sql);
						$_row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$_row)
						{
							return array($user->lang['NO_CAT_ID']);
						}

						$subcats_to_name = $_row['cat_name'];

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . "
								WHERE parent_id = {$row['cat_id']}";
						$result = $db->sql_query($sql);

						while ($_row = $db->sql_fetchrow($result))
						{
							$this->move_cat($_row['cat_id'], $subcats_to_id);
						}
						$db->sql_freeresult($result);

						$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
								SET parent_id = $subcats_to_id
								WHERE parent_id = {$row['cat_id']}";
						$db->sql_query($sql);
					}

					// Adjust the left/right id
					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
							SET right_id = left_id + 1
							WHERE cat_id = ' . $row['cat_id'];
					$db->sql_query($sql);
				}
			}
			else if ($row['cat_type'] == ARCADE_CAT && $cat_data_sql['cat_type'] == ARCADE_CAT_GAMES)
			{
				// Changing a category to a forum? Reset the data (you can't post directly in a cat, you must use a forum)
				$cat_data_sql['cat_games'] = 0;
				$cat_data_sql['cat_plays'] = 0;
				$cat_data_sql['cat_last_play_game_id'] = 0;
				$cat_data_sql['cat_last_play_game_name'] = '';
				$cat_data_sql['cat_last_game_installdate'] = 0;
				$cat_data_sql['cat_last_play_user_id'] = 0;
				$cat_data_sql['cat_last_play_score'] = 0;
				$cat_data_sql['cat_last_play_time'] = 0;
				$cat_data_sql['cat_last_play_username'] = '';
				$cat_data_sql['cat_last_play_user_colour'] = '';
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			if ($row['parent_id'] != $cat_data_sql['parent_id'])
			{


				if ($row['cat_id'] != $cat_data_sql['parent_id'])
				{
					$errors = $this->move_cat($cat_data_sql['cat_id'], $cat_data_sql['parent_id']);
				}
				else
				{
					$cat_data_sql['parent_id'] = $row['parent_id'];
				}
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			unset($cat_data_sql['type_action']);

			if ($row['cat_name'] != $cat_data_sql['cat_name'])
			{
				// the forum name has changed, clear the parents list of all forums (for safety)
				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
						SET cat_parents = ''";
				$db->sql_query($sql);
			}

			// Setting the forum id to the forum id is not really received well by some dbs. ;)
			$cat_id = $cat_data_sql['cat_id'];
			unset($cat_data_sql['cat_id']);

			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $cat_data_sql) . '
					WHERE cat_id = ' . $cat_id;
			$db->sql_query($sql);

			// Add it back
			$cat_data['cat_id'] = $cat_id;

			add_log('admin', 'LOG_ARCADE_CAT_EDIT', $cat_data['cat_name']);
		}

		return $errors;
	}

	/**
	* Move forum
	*/
	function move_cat($from_id, $to_id)
	{
		global $db, $arcade;

		$to_data = $moved_ids = $errors = array();
		// Check if we want to move to a parent with link type
		if ($to_id > 0)
		{
			$to_data = $arcade->get_cat_info($to_id);

			if ($to_data['cat_type'] == ARCADE_LINK)
			{
				$errors[] = $user->lang['PARENT_IS_LINK_ARCADE'];
				return $errors;
			}
		}

		$moved_forums = $arcade->get_cat_branch($from_id, 'children', 'descending');
		$from_data = $moved_forums[0];
		$diff = sizeof($moved_forums) * 2;

		$moved_ids = array();
		for ($i = 0; $i < sizeof($moved_forums); ++$i)
		{
			$moved_ids[] = $moved_forums[$i]['cat_id'];
		}

		// Resync parents
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET right_id = right_id - $diff, cat_parents = ''
				WHERE left_id < " . $from_data['right_id'] . "
				AND right_id > " . $from_data['right_id'];
		$db->sql_query($sql);

		// Resync righthand side of tree
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id - $diff, right_id = right_id - $diff, cat_parents = ''
				WHERE left_id > " . $from_data['right_id'];
		$db->sql_query($sql);

		if ($to_id > 0)
		{
			$to_data = $arcade->get_cat_info($to_id);

			// Resync new parents
			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
					SET right_id = right_id + $diff, cat_parents = ''
					WHERE " . $to_data['right_id'] . ' BETWEEN left_id AND right_id
					AND ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$db->sql_query($sql);

			// Resync the righthand side of the tree
			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
					SET left_id = left_id + $diff, right_id = right_id + $diff, cat_parents = ''
					WHERE left_id > " . $to_data['right_id'] . '
					AND ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$db->sql_query($sql);

			// Resync moved branch
			$to_data['right_id'] += $diff;

			if ($to_data['right_id'] > $from_data['right_id'])
			{
				$diff = '+ ' . ($to_data['right_id'] - $from_data['right_id'] - 1);
			}
			else
			{
				$diff = '- ' . abs($to_data['right_id'] - $from_data['right_id'] - 1);
			}
		}
		else
		{
			$sql = 'SELECT MAX(right_id) AS right_id
					FROM ' . ARCADE_CATS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$diff = '+ ' . ($row['right_id'] - $from_data['left_id'] + 1);
		}

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id $diff, right_id = right_id $diff, cat_parents = ''
				WHERE " . $db->sql_in_set('cat_id', $moved_ids);
		$db->sql_query($sql);
	}

	/**
	* Move category content from one to another forum
	*/
	function move_cat_content($from_id, $to_id, $sync = true)
	{
		global $db, $arcade;

		$table_ary = array(ARCADE_GAMES_TABLE);

		foreach ($table_ary as $table)
		{
			$sql = "UPDATE $table
					SET cat_id = $to_id
					WHERE cat_id = $from_id";
			$db->sql_query($sql);
		}
		unset($table_ary);

		if ($sync)
		{
			// Sync total games and plays for the categories...
			$arcade->sync('category');
		}
		return array();
	}

	/**
	* Remove complete category
	*/
	function delete_cat($cat_id, $action_games = 'delete', $delete_files = false, $action_subcats = 'delete', $sub_delete_files = false, $games_to_id = 0, $subcats_to_id = 0)
	{
		global $db, $user, $arcade_cache, $arcade;

		$cat_data = $arcade->get_cat_info($cat_id);

		$errors = array();
		$log_action_games = $log_action_cats = $games_to_name = $subcats_to_name = '';
		$cat_ids = array($cat_id);

		if ($action_games == 'delete')
		{
			$log_action_games = 'GAMES';
			$errors = array_merge($errors, $this->delete_cat_content($cat_id, $delete_files));
		}
		else if ($action_games == 'move')
		{
			if (!$games_to_id)
			{
				$errors[] = $user->lang['NO_DESTINATION_CAT'];
			}
			else
			{
				$log_action_games = 'MOVE_GAMES';

				$sql = 'SELECT cat_name
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $games_to_id;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					$errors[] = $user->lang['NO_CAT_ID'];
				}
				else
				{
					$posts_to_name = $row['cat_name'];
					$errors = array_merge($errors, $this->move_cat_content($cat_id, $games_to_id));
				}
			}
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		if ($action_subcats == 'delete')
		{
			$log_action_cats = 'CATS';
			$rows = $arcade->get_cat_branch($cat_id, 'children', 'descending', false);

			foreach ($rows as $row)
			{
				$cat_ids[] = $row['cat_id'];
				$errors = array_merge($errors, $this->delete_cat_content($row['cat_id'], $sub_delete_files));
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			$diff = sizeof($cat_ids) * 2;

			$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
			$db->sql_query($sql);
		}
		else if ($action_subcats == 'move')
		{
			if (!$subcats_to_id)
			{
				$errors[] = $user->lang['NO_DESTINATION_CAT'];
			}
			else
			{
				$log_action_cats = 'MOVE_CATS';

				$sql = 'SELECT cat_name
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $subcats_to_id;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					$errors[] = $user->lang['NO_CAT_ID'];
				}
				else
				{
					$subcats_to_name = $row['cat_name'];

					$sql = 'SELECT cat_id
							FROM ' . ARCADE_CATS_TABLE . "
							WHERE parent_id = $cat_id";
					$result = $db->sql_query($sql);

					while ($row = $db->sql_fetchrow($result))
					{
						$this->move_cat($row['cat_id'], $subcats_to_id);
					}
					$db->sql_freeresult($result);

					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
							SET parent_id = $subcats_to_id
							WHERE parent_id = $cat_id";
					$db->sql_query($sql);

					$cat_data = $arcade->get_cat_info($cat_id);

					$diff = 2;
					$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . "
							WHERE cat_id = $cat_id";
					$db->sql_query($sql);
				}
			}

			if (sizeof($errors))
			{
				return $errors;
			}
		}
		else
		{
			$diff = 2;
			$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . "
					WHERE cat_id = $cat_id";
			$db->sql_query($sql);
		}

		// Resync tree
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET right_id = right_id - $diff
				WHERE left_id < {$cat_data['right_id']} AND right_id > {$cat_data['right_id']}";
		$db->sql_query($sql);

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id - $diff, right_id = right_id - $diff
				WHERE left_id > {$cat_data['right_id']}";
		$db->sql_query($sql);

		$log_action = implode('_', array($log_action_games, $log_action_cats));

		switch ($log_action)
		{
			case 'MOVE_GAMES_MOVE_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES_MOVE_CATS', $posts_to_name, $subcats_to_name, $cat_data['cat_name']);
			break;

			case 'MOVE_GAMES_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES_CATS', $posts_to_name, $cat_data['cat_name']);
			break;

			case 'GAMES_MOVE_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_GAMES_MOVE_CATS', $subcats_to_name, $cat_data['cat_name']);
			break;

			case '_MOVE_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_MOVE_CATS', $subcats_to_name, $cat_data['cat_name']);
			break;

			case 'MOVE_GAMES_':
				add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES', $posts_to_name, $cat_data['cat_name']);
			break;

			case 'GAMES_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_GAMES_CATS', $cat_data['cat_name']);
			break;

			case '_CATS':
				add_log('admin', 'LOG_ARCADE_DEL_CATS', $cat_data['cat_name']);
			break;

			case 'GAMES_':
				add_log('admin', 'LOG_ARCADE_DEL_GAMES', $cat_data['cat_name']);
			break;

			default:
				add_log('admin', 'LOG_ARCADE_DEL_CAT', $cat_data['cat_name']);
			break;
		}

		return $errors;
	}

	/**
	* Delete category content
	*/
	function delete_cat_content($cat_id, $delete_files = false)
	{
		global $db, $user, $arcade_cache, $arcade;

		$errors = array();
		if (!$cat_id)
		{
			$errors[] = $user->lang['NO_CAT_ID'];
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		$sql = 'SELECT game_id FROM ' . ARCADE_GAMES_TABLE . '
				WHERE cat_id = ' . $cat_id;
		$result = $db->sql_query($sql);

		$delete_games = array();

		while ($row = $db->sql_fetchrow($result))
		{
			$delete_games[$cat_id][] = $row['game_id'];
		}
		$db->sql_freeresult($result);

		if (sizeof($delete_games))
		{
			foreach ($delete_games as $cat_id => $game_ids)
			{
				$errors = $arcade->delete_game($game_ids, $cat_id, $delete_files, false);
			}
		}

		$arcade_cache->destroy('sql', ARCADE_CATS_TABLE);
		$arcade_cache->destroy('sql', ARCADE_GAMES_TABLE);
		$arcade_cache->destroy('sql', ARCADE_SCORES_TABLE);
		$arcade_cache->destroy('sql', ARCADE_PLAYS_TABLE);
		$arcade_cache->destroy('sql', ARCADE_CHALLENGE_TABLE);
		$arcade_cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

		$arcade_cache->destroy('_arcade_cats');
		$arcade_cache->destroy('_arcade_leaders');
		$arcade_cache->destroy('_arcade_leaders_all');
		$arcade_cache->destroy('_arcade_games_filesize');
		$arcade_cache->destroy('_arcade_challenge_leaders');
		$arcade_cache->destroy('_arcade_challenge_leaders_all');

		$arcade->sync('total_data', 'all');
		$arcade->set_last_play($cat_id);
		$arcade->sync('category', $cat_id);

		return $errors;
	}

	/**
	* Move forum position by $steps up/down
	*/
	function move_cat_by($cat_row, $action = 'move_up', $steps = 1)
	{
		global $db;

		/**
		* Fetch all the siblings between the module's current spot
		* and where we want to move it to. If there are less than $steps
		* siblings between the current spot and the target then the
		* module will move as far as possible
		*/
		$sql = 'SELECT cat_id, cat_name, left_id, right_id
				FROM ' . ARCADE_CATS_TABLE . "
				WHERE parent_id = {$cat_row['parent_id']}
				AND " . (($action == 'move_up') ? "right_id < {$cat_row['right_id']} ORDER BY right_id DESC" : "left_id > {$cat_row['left_id']} ORDER BY left_id ASC");
		$result = $db->sql_query_limit($sql, $steps);

		$target = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$target = $row;
		}
		$db->sql_freeresult($result);

		if (!sizeof($target))
		{
			// The forum is already on top or bottom
			return false;
		}

		/**
		* $left_id and $right_id define the scope of the nodes that are affected by the move.
		* $diff_up and $diff_down are the values to substract or add to each node's left_id
		* and right_id in order to move them up or down.
		* $move_up_left and $move_up_right define the scope of the nodes that are moving
		* up. Other nodes in the scope of ($left_id, $right_id) are considered to move down.
		*/
		if ($action == 'move_up')
		{
			$left_id = $target['left_id'];
			$right_id = $cat_row['right_id'];

			$diff_up = $cat_row['left_id'] - $target['left_id'];
			$diff_down = $cat_row['right_id'] + 1 - $cat_row['left_id'];

			$move_up_left = $cat_row['left_id'];
			$move_up_right = $cat_row['right_id'];
		}
		else
		{
			$left_id = $cat_row['left_id'];
			$right_id = $target['right_id'];

			$diff_up = $cat_row['right_id'] + 1 - $cat_row['left_id'];
			$diff_down = $target['right_id'] - $cat_row['right_id'];

			$move_up_left = $cat_row['right_id'] + 1;
			$move_up_right = $target['right_id'];
		}

		// Now do the dirty job
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id + CASE
				WHEN left_id BETWEEN {$move_up_left} AND {$move_up_right} THEN -{$diff_up}
				ELSE {$diff_down}
				END,
				right_id = right_id + CASE
				WHEN right_id BETWEEN {$move_up_left} AND {$move_up_right} THEN -{$diff_up}
				ELSE {$diff_down}
				END,
				cat_parents = ''
				WHERE
				left_id BETWEEN {$left_id} AND {$right_id}
				AND right_id BETWEEN {$left_id} AND {$right_id}";
		$db->sql_query($sql);

		return $target['cat_name'];
	}

	/**
	* Display progress bar for syncinc forums
	*/
	function display_progress_bar()
	{
		global $template, $user;

		adm_page_header($user->lang['SYNC_IN_PROGRESS']);

		$template->set_filenames(array(
			'body' => 'progress_bar.html')
		);

		$template->assign_vars(array(
			'L_PROGRESS'			=> $user->lang['SYNC_IN_PROGRESS'],
			'L_PROGRESS_EXPLAIN'	=> $user->lang['SYNC_IN_PROGRESS_EXPLAIN'])
		);

		adm_page_footer();
	}

	function game_swf()
	{
		return $this->new_game_data['game_swf'];
	}

	function game_installdate()
	{
		global $user;

		return $user->format_date($this->new_game_data['game_installdate']);
	}

	function game_filesize()
	{
		return get_formatted_filesize($this->new_game_data['game_filesize']);
	}

	function game_type_select($value, $key = '')
	{
		global $arcade;

		return $arcade->game_type_select($value);
	}

	function game_scoretype_select($value, $key = '')
	{
		global $arcade;

		return $arcade->game_scoretype_select($value);
	}

	function game_cat_select($value, $key = '')
	{
		global $arcade;
		return $arcade->make_cat_select($value, false, false, true);
	}
}

?>