<?php
/**
*
* @package install
* @version $Id: install_install.php 2022 2012-03-20 16:20:21Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
*/
if (!defined('IN_INSTALL') || !defined('IN_PHPBB'))
{
	exit;
}

if (!empty($setmodules))
{
	if ($this->installed_version)
	{
		return;
	}

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'INSTALL',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 10,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'REQUIREMENTS', 'INSTALL'),
		'module_reqs'		=> ''
	);
}

/**
* Installation
* @package install
*/
class install_install extends module
{
	function install_install(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $user, $template, $phpbb_root_path;

		switch ($sub)
		{
			case 'intro':
				$this->page_title = $user->lang['SUB_INTRO'];

				$template->assign_vars(array(
					'TITLE'			=> $user->lang['INSTALL_INTRO'],
					'BODY'			=> $user->lang['INSTALL_INTRO_BODY'],
					'L_SUBMIT'		=> $user->lang['NEXT_STEP'],
					'U_ACTION'		=> $this->p_master->module_url . "?mode=$mode&amp;sub=requirements",
				));

			break;

			case 'requirements':
				$this->page_title = $user->lang['STAGE_REQUIREMENTS'];

				$this->p_master->requirements($mode, $sub);

			break;

			case 'install':
				$this->install($mode, $sub);

			break;
		}

		$this->tpl_name = 'install_install';
	}

	/**
	* Obtain the information required to connect to the database
	*/
	function install($mode, $sub)
	{
		global $table_prefix, $user, $template, $cache, $phpEx, $phpbb_root_path, $phpbb_db_tools, $arcade_mod_config;

		$this->page_title = $user->lang['STAGE_INSTALL_ARCADE'];

		foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
		{
			$phpbb_db_tools->sql_create_table($table_name, $table_data);
		}

		// Load all the arcade data
		if (!empty($arcade_mod_config['data_file']['add']))
		{
			$this->p_master->load_data($arcade_mod_config['data_file']['add']);
		}

		$this->p_master->arcade_set_config($arcade_mod_config['config_data']);

		// Alter some existing tables
		if (!empty($arcade_mod_config['schema_changes']))
		{
			$phpbb_db_tools->perform_schema_changes($arcade_mod_config['schema_changes']);
		}

		$this->p_master->add_permissions($arcade_mod_config['permission_options']['arcade'], 'arcade');
		$this->p_master->add_roles($arcade_mod_config['roles']['arcade'], 'arcade');

		// Add the admin permissions for the arcade acp modules
		$this->p_master->add_permissions($arcade_mod_config['permission_options']['phpbb']);
		// Add the admin permissions for the arcade acp modules to the correct roles
		$this->p_master->update_roles(array('ROLE_ADMIN_STANDARD', 'ROLE_ADMIN_FULL'), $arcade_mod_config['permission_options']['phpbb']['global']);

		// Add the modules
		foreach ($arcade_mod_config['modules'] as $modules)
		{
			$this->p_master->create_modules($modules['parent_module_data'], $modules['module_data']);
		}

		$this->p_master->add_modules('ACP_CAT_USERS', 'acp', $arcade_mod_config['phpbb_users_modules']);
		$this->p_master->phpbb_set_config($arcade_mod_config['phpbb_config']);

		// Purge the cache
		$this->p_master->cache_purge(array('template', 'theme', 'imageset', 'auth', ''));

		add_log('admin', 'LOG_ARCADE_INSTALL', $arcade_mod_config['version']['current']);

		$perm_msg = '</p><div class="errorbox"><h3>' . $user->lang['WARNING'] . '</h3><p>' . $user->lang['PERMISSIONS_WARNING'] . '</p></div>';
		$donate_button = '<br /><a onclick="window.open(this.href); return false;" href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&amp;hosted_button_id=UA2VMFQD6GKFW"><img src="http://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif" alt="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" title="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" /></a>';
		$donate = '<div class="successbox"><h3>' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '</h3><p>' . $user->lang['PHPBB_ARCADE_DONATE'] . '</p><p>' . $donate_button . '</p></div>';

		$template->assign_vars(array(
			'TITLE'		=> $user->lang['INSTALL_CONGRATS'],
			'BODY'		=> $user->lang['STAGE_INSTALL_ARCADE_EXPLAIN'] . '<br /><br />' . $perm_msg . $donate . sprintf($user->lang['INSTALL_CONGRATS_EXPLAIN'], $arcade_mod_config['version']['current']),
			'L_SUBMIT'	=> $user->lang['CAT_VERIFY'],
			'U_ACTION'	=> $this->p_master->module_url . '?mode=verify&amp;sub=verify',
		));
	}
}

?>