<?php
/**
*
* @package install
* @version $Id: install_update.php 2027 2012-03-20 18:45:39Z killbill $
* @copyright (c) 2008-2012 http://www.phpbbarcade.com
* @copyright (c) 2011-2012 http://jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
*/
if (!defined('IN_INSTALL') || !defined('IN_PHPBB'))
{
	exit;
}

if (!empty($setmodules))
{
	if (!$this->installed_version || (phpbb_version_compare($this->installed_version, $arcade_mod_config['version']['current'], '>=')))
	{
		return;
	}

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'UPDATE',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 20,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'REQUIREMENTS', 'UPDATE'),
		'module_reqs'		=> ''
	);
}

/**
* Installation
* @package install
*/
class install_update extends module
{
	function install_update(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $user, $template, $phpbb_root_path, $phpEx;

		switch ($sub)
		{
			case 'intro':
				$this->page_title = $user->lang['SUB_INTRO'];

				// challenge mod check
				$challenge_instructions = '';
				if (!empty($this->p_master->mod_config['ul_challenge_version']) && $content = get_file_contents("{$phpbb_root_path}includes/constants.{$phpEx}"))
				{
					if (strpos($content, 'includes/challenge/constants') !== false)
					{
						$challenge_instructions .= '</p><div class="errorbox"><h3>' . $user->lang['WARNING'] . '</h3><p>' . $user->lang['CHALLENGE_INSTRUCTIONS'] . '</p></div><p>';
					}
				}

				$template->assign_vars(array(
					'TITLE'		=> $user->lang['UPDATE_INTRO'],
					'BODY'		=> $user->lang['UPDATE_INTRO_BODY'] . $challenge_instructions,
					'L_SUBMIT'	=> ($challenge_instructions == '') ? $user->lang['NEXT_STEP'] : '',
					'U_ACTION'	=> $this->p_master->module_url . "?mode=$mode" . (($challenge_instructions == '') ? "&amp;sub=requirements" : ''),
				));

			break;

			case 'requirements':
				$this->page_title = $user->lang['STAGE_REQUIREMENTS'];
				$this->p_master->requirements($mode, $sub);
			break;

			case 'update':
				$this->update($mode, $sub);
			break;
		}

		$this->tpl_name = 'install_install';
	}

	/**
	* Obtain the information required to connect to the database
	*/
	function update($mode, $sub)
	{
		global $user, $template, $cache, $phpEx, $phpbb_root_path, $file_functions, $phpbb_db_tools, $db, $arcade_mod_config;

		$this->page_title = $user->lang['STAGE_UPDATE_ARCADE'];

		switch ($this->p_master->installed_version)
		{
			case '1.0.0':
				// The last thing we do is update the game install files.  This is only necessary to stop the
				// error message in the acp create install file module about the install file changing.  The
				// old install files will still install in the new arcade (1.1.x or higher).  This could also
				// take some time of boards with large amounts of games, so it may have to be tweaked.
				$this->p_master->convert_install_file();

				$sql = 'SELECT game_installdate
						FROM ' . ARCADE_GAMES_TABLE . '
						ORDER BY game_installdate ASC';
				$result = $db->sql_query_limit($sql, 1);
				$startdate = (int) $db->sql_fetchfield('game_installdate');
				$db->sql_freeresult($result);

				$startdate = ($startdate) ? $startdate : time();

				$sql = 'SELECT SUM(total_plays) AS games_played, SUM(total_time) AS games_time
						FROM ' . ARCADE_PLAYS_TABLE;
				$result = $db->sql_query($sql);
				$play_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$sql = 'SELECT SUM(total) as downloads
						FROM ' . ARCADE_DOWNLOAD_TABLE;
				$result = $db->sql_query($sql);
				$total_downloads = (int) $db->sql_fetchfield('downloads');
				$db->sql_freeresult($result);

				if ($db->sql_layer === 'sqlite')
				{
					$sql = 'SELECT COUNT(user_id) as total_users_played
							FROM (
								SELECT DISTINCT user_id
								FROM ' . ARCADE_PLAYS_TABLE . '
							)';
				}
				else
				{
					$sql = 'SELECT COUNT(DISTINCT user_id) as total_users_played
							FROM ' . ARCADE_PLAYS_TABLE;
				}

				$result = $db->sql_query($sql);
				$total_users_played = (int) $db->sql_fetchfield('total_users_played');
				$db->sql_freeresult($result);

				$sql = 'SELECT COUNT(report_id) as reports_open
						FROM ' . ARCADE_REPORTS_TABLE . '
						WHERE report_closed = ' . ARCADE_REPORT_OPEN;
				$result = $db->sql_query($sql);
				$reports_open = (int) $db->sql_fetchfield('reports_open');
				$db->sql_freeresult($result);

				$total_plays = (int) $play_data['games_played'];
				$total_times = (int) $play_data['games_time'];

				// Now create the table
				$phpbb_db_tools->sql_create_table(ARCADE_USERS_TABLE, $arcade_mod_config['schema_data'][ARCADE_USERS_TABLE]);

				// Lets move the old data over...
				$db->sql_return_on_error(true);
				$sql = 'SELECT user_id, user_arcade_permissions, user_arcade_perm_from, user_arcade_pm, games_per_page, games_sort_dir, games_sort_order
						FROM ' . USERS_TABLE . '
						ORDER BY user_id';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					// We will only move the data over if the user changed the default values.
					// The other users will be created when they first visit the arcade.
					if ($row['user_arcade_pm'] != true || $row['games_per_page'] != 0 || $row['games_sort_dir'] != 'a' || $row['games_sort_order'] != 'n')
					{
						$db->sql_query('INSERT INTO ' . ARCADE_USERS_TABLE . ' ' . $db->sql_build_array('INSERT', array(
								'user_id'						=> $row['user_id'],
								'user_arcade_permissions'		=> $row['user_arcade_permissions'],
								'user_arcade_perm_from'			=> $row['user_arcade_perm_from'],
								'user_arcade_pm'				=> $row['user_arcade_pm'],
								'games_per_page'				=> $row['games_per_page'],
								'games_sort_dir'				=> $row['games_sort_dir'],
								'games_sort_order'				=> $row['games_sort_order'],
							)
						));
					}
				}
				$db->sql_freeresult($result);
				$db->sql_return_on_error(false);

				switch ($db->sql_layer)
				{
					case 'sqlite':
					case 'firebird':
						$db->sql_query('DELETE FROM ' . ARCADE_SESSIONS_TABLE);
					break;

					default:
						$db->sql_query('TRUNCATE TABLE ' . ARCADE_SESSIONS_TABLE);
					break;
				}

				// Process schema changes
				$phpbb_db_tools->perform_schema_changes($arcade_mod_config['schema_changes']['update']['2.0.RC1']);

				$arcade_configs = array(array('arcade_startdate', $startdate),
										array('arcade_resetdate', '0'),
										array('backup_limit', '50'),
										array('cat_backup_path', 'arcade/backup/'),
										array('download_anonymous_interval', '0'),
										array('download_interval', '0'),
										array('download_on_demand', '1'),
										array('facebook_font_type', 'arial'),
										array('founder_exempt', '0'),
										array('game_popup_icon_enabled', '1'),
										array('load_list', '0'),
										array('playfree_reward', '0'),
										array('play_anonymous_interval', '0'),
										array('play_interval', '0'),
										array('reports_open', $reports_open),
										array('search_anonymous_interval', '0'),
										array('search_filter_age', '1'),
										array('search_filter_password', '1'),
										array('search_interval', '0'),
										array('session_length', '72000'),
										array('total_downloads', $total_downloads, true),
										array('total_plays', $total_plays, true),
										array('total_plays_time', $total_times, true),
										array('flash_version', '10.0.0.0'),
				);

				$this->p_master->arcade_set_config($arcade_configs);

				$delete_arcade_configs = array(
					array('pm_subject'),
					array('pm_message'),
				);
				// Remove unneeded config values
				$this->p_master->arcade_remove_config($delete_arcade_configs);

				// Add new permission options
				$this->p_master->add_permissions($arcade_mod_config['permission_options']['update']['2.0.RC1']['phpbb']);
				$this->p_master->add_permissions($arcade_mod_config['permission_options']['update']['2.0.RC1']['arcade'], 'arcade');

				// Add new permission options to correct roles
				$this->p_master->update_roles(array('ROLE_ARCADE_STANDARD'), array('c_re_rate'), 'arcade');
				$this->p_master->update_roles(array('ROLE_ARCADE_FULL'), $arcade_mod_config['permission_options']['update']['2.0.RC1']['arcade']['local'], 'arcade');

				// Add new roles
				$roles_ary = array();
				foreach ($arcade_mod_config['roles']['arcade'] as $role)
				{
					if ($role['role_type'] != 'c_')
					{
						$roles_ary[] = $role;
					}
				}

				$this->p_master->add_roles($roles_ary, 'arcade');

				// Remove and then re-add the modules
				$this->p_master->remove_modules($arcade_mod_config['modules_remove']);
				foreach ($arcade_mod_config['modules'] as $modules)
				{
					$this->p_master->create_modules($modules['parent_module_data'], $modules['module_data']);
				}
			case '2.0.RC1':
				if ($this->p_master->installed_version == '2.0.RC1')
				{
					$this->p_master->convert_install_file(ARCADELIB_GAME);
				}

				// added new tables
				foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
				{
					if (!in_array($table_name, array(ARCADE_USERS_BANNED_TABLE, ARCADE_ANNOUNCE_TABLE)))
					{
						continue;
					}

					$phpbb_db_tools->sql_create_table($table_name, $table_data);
				}

				// update challenge
				if (!empty($this->p_master->mod_config['ul_challenge_version']))
				{
					$sql = 'SELECT SUM(champ_challenger_time + champ_opponent_time) AS total_times, COUNT(champ_id) AS total_plays
							FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
							WHERE champ_close = ' . CHALLENGE_CLOSE;
					$result = $db->sql_query($sql);
					$total = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);

					$arcade_configs = array(
						array('challenge_acc_exp_date'		, $this->p_master->mod_config['challenge_time_delete']),
						array('challenge_disable'			, (($this->p_master->mod_config['challenge_enable']) ? 0 : 1)),
						array('challenge_latest_winners'	, $this->p_master->mod_config['challenge_first_user']),
						array('challenge_links_index'		, $this->p_master->mod_config['challenge_index_link']),
						array('challenge_links_stats'		, $this->p_master->mod_config['challenge_stat_link']),
						array('challenge_ong_exp_date'		, $this->p_master->mod_config['challenge_time_delete_ong']),
						array('challenge_bet_Minimum'		, $this->p_master->mod_config['challenge_points_min']),
						array('challenge_bet_Maximum'		, $this->p_master->mod_config['challenge_points_max']),
						array('challenge_bet_fix'			, $this->p_master->mod_config['challenge_points_fix']),
						array('challenge_users_select'		, $this->p_master->mod_config['challenge_user_select']),
						array('challenge_top_games_header'	, $this->p_master->mod_config['challenge_top_games']),
						array('challenge_resetdate'			, (($this->p_master->mod_config['challenge_reset_time']) ? $this->p_master->mod_config['challenge_reset_time'] : $this->p_master->mod_config['challenge_install_time'])),
						array('challenge_total_plays'		, (int) $total['total_plays'], true),
						array('challenge_total_plays_time'	, (int) $total['total_times'], true),
						array('challenge_welcome_index'		, $this->p_master->mod_config['challenge_index_top']),
						array('challenge_welcome_stats'		, $this->p_master->mod_config['challenge_stat_top']),
					);

					$delete_arcade_configs = array(
						array('challenge_index_search'),
						array('challenge_stat_search'),
						array('challenge_statement_search'),
						array('challenge_par_reset_user'),
						array('challenge_v_good'),
						array('challenge_flash_menu'),
						array('challenge_flash_width'),
						array('challenge_flash_height'),
						array('challenge_copyright'),
						array('ul_challenge_version'),
						array('challenge_flash_navig'),
						array('challenge_index_user_play'),
						array('challenge_stat_user_play'),
						array('challenge_statement_user_play'),
						array('challenge_install_time'),
						array('challenge_update_time'),
						array('challenge_reset_user'),
						array('challenge_par_reset_time'),
						array('challenge_random_enable'),
						array('challenge_anonym'),
						array('challenge_cstt_enable'),
						array('challenge_astt_enable'),
						array('challenge_stat_enable'),
						array('challenge_points_enable'),
						array('challenge_stat_user'),
						array('challenge_stat_row'),
						array('challenge_index_link'),
						array('challenge_stat_top'),
						array('challenge_index_top'),
						array('challenge_stat_link'),
						array('challenge_statement_top'),
						array('challenge_statement_link'),
						array('challenge_first_user'),
						array('challenge_points_min'),
						array('challenge_points_max'),
						array('challenge_points_fix'),
						array('challenge_reset_time'),
						array('challenge_enable'),
						array('challenge_time_delete'),
						array('challenge_time_delete_ong'),
						array('challenge_favorite_enable'),
						array('challenge_friends_enable'),
						array('challenge_top_games')
					);

					$this->p_master->arcade_remove_config($delete_arcade_configs);

					$old_challenge_permissions_arcade = array(
						'local'		=> array('c_challenge_opcio','c_challenge_play','c_challenge_play_treat','c_challenge_report','c_challenge_perm', 'c_popup', 'c_resolution'),
						'global'	=> array(),
					);

					$old_challenge_permissions_phpbb = array(
						'local'		=> array(),
						'global'	=> array('a_challenge_config', 'a_challenge_reset', 'a_challenge_report'),
					);

					$this->p_master->remove_permissions($old_challenge_permissions_arcade, 'arcade');
					$this->p_master->remove_permissions($old_challenge_permissions_phpbb);

					$delete_challenge_modules = array(
						'modules' => array('arcade_challenge'),
						'parents' => array('ACP_ARCADE_CHALLENGE', 'UCP_ARCADE_CHALLENGE'),
					);

					$this->p_master->remove_modules($delete_challenge_modules);

					global $table_prefix;

					$phpbb_db_tools->sql_table_drop("{$table_prefix}arcade_challenge_report");
					$phpbb_db_tools->sql_table_drop("{$table_prefix}arcade_challenge_stats");

					$challenge_update_db = array(
						'drop_columns'			=> array(
							USERS_TABLE						=> array('user_challenge_on', 'user_challenge_pm_on', 'user_challenge_stat'),
						),
						'add_columns'						=> array(
							ARCADE_CHALLENGE_TABLE			=> array(
								'challenger_game_cost'		=> array('PDECIMAL:25', 0.000),
								'opponent_game_cost'		=> array('PDECIMAL:25', 0.000),
							),
							ARCADE_CHALLENGE_CHAMP_TABLE	=> array(
								'game_id'					=> array('UINT', NULL),
							),
						),
						'add_index'					=> array(
							ARCADE_CHALLENGE_TABLE	=> array(
								'ca'		=> array('challenge_accept'),
								'cid'		=> array('challenger_id'),
								'oid'		=> array('opponent_id'),
								'gid'		=> array('game_id'),
							),
							ARCADE_CHALLENGE_CHAMP_TABLE	=> array(
								'ci'		=> array('champ_challenger_id'),
								'oi'		=> array('champ_opponent_id'),
								'cg'		=> array('game_id'),
								'cw'		=> array('champ_winner'),
								'cc'		=> array('champ_close'),
							),
						),
					);

					$phpbb_db_tools->perform_schema_changes($challenge_update_db);

					$sql = 'SELECT champ_game_id
							FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE;
					$result = $db->sql_query($sql);
					while ($row = $db->sql_fetchrow($result))
					{
						$sql = 'UPDATE ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
								SET game_id = ' . (int) $row['champ_game_id'] . '
								WHERE champ_game_id = ' . (int) $row['champ_game_id'];
						$db->sql_query($sql);
					}
					$db->sql_freeresult($result);

					$phpbb_db_tools->perform_schema_changes(array(
						'drop_columns'					=> array(
						ARCADE_CHALLENGE_CHAMP_TABLE	=> array('champ_game_id', 'champ_points'),
					)));
				}
				else
				{
					foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
					{
						if (strpos($table_name, 'challenge') === false)
						{
							continue;
						}

						$phpbb_db_tools->sql_create_table($table_name, $table_data);
					}

					$arcade_configs = array(
						array('challenge_acc_exp_date', '604800'),
						array('challenge_disable', '0'),
						array('challenge_latest_winners', '5'),
						array('challenge_links_index', '1'),
						array('challenge_links_stats', '1'),
						array('challenge_ong_exp_date', '1209600'),
						array('challenge_bet_Minimum', '0'),
						array('challenge_bet_Maximum', '0'),
						array('challenge_bet_fix', '0'),
						array('challenge_users_select', '1'),
						array('challenge_resetdate', time()),
						array('challenge_top_games_header', '10'),
						array('challenge_total_plays', '0', true),
						array('challenge_total_plays_time', '0', true),
						array('challenge_welcome_index', '1'),
						array('challenge_welcome_stats', '1'),
					);
				}

				$delete_arcade_configs = array(
						array('jackpot_maximum'),
						array('jackpot_minimum'),
						array('game_popup_icon_enabled'),
						array('facebook_table_height'),
						array('facebook_table_width'),
						array('facebook_enable_like'),
						array('facebook_show_faces'),
						array('facebook_layout_style'),
						array('facebook_color_scheme'),
						array('facebook_background_color'),
						array('facebook_font_type')
				);
				// Remove unneeded config values
				$this->p_master->arcade_remove_config($delete_arcade_configs);

				$arcade_configs = array_merge($arcade_configs, array(
					array('arcade_flash_intro', '1'),
					array('cat_leaders', '5'),
					array('cat_least_downloaded', '5'),
					array('cat_least_popular', '5'),
					array('cat_longest_held_scores', '5'),
					array('cat_most_downloaded', '5'),
					array('cat_most_popular', '5'),
					array('cat_least_played_users', '5'),
					array('cat_most_played_users', '5'),
					array('challenge_exp_last_gc', time(), true),
					array('challenge_least_played_users', '5'),
					array('challenge_disable_msg', ''),
					array('challenge_most_played_users', '5'),
					array('challenge_leaders', '5'),
					array('challenge_leaders_header', '3'),
					array('challenge_least_popular', '5'),
					array('challenge_most_popular', '5'),
					array('corrupt_score_banned', '0'),
					array('display_game_image', '1'),
					array('display_user_avatar', '1'),
					array('display_game_popup_icon', ((isset($this->p_master->mod_config['game_popup_icon_enabled'])) ? $this->p_master->mod_config['game_popup_icon_enabled'] : '1')),
					array('download_file_type', ''),
					array('enable_image_link', '1'),
					array('enable_user_avatar_link', '1'),
					array('facebook_like_enable', ((isset($this->p_master->mod_config['facebook_enable_like'])) ? $this->p_master->mod_config['facebook_enable_like'] : '0')),
					array('gamename_maxchars', '12'),
					array('google_like_enable', '0'),
					array('game_comment', '1'),
					array('game_desc_parse_html', '0'),
					array('game_download_cost', '0'),
					array('global_announce_enable', '0'),
					array('gc_time', '3600'),
					array('jackpot_Maximum', ((isset($this->p_master->mod_config['jackpot_maximum'])) ? $this->p_master->mod_config['jackpot_maximum'] : '0')),
					array('jackpot_Minimum', ((isset($this->p_master->mod_config['jackpot_minimum'])) ? $this->p_master->mod_config['jackpot_minimum'] : '0')),
					array('least_played_users', '5'),
					array('like_background_color', ''),
					array('like_counter_enable', '1'),
					array('least_jackpot', '10'),			// *****
					array('least_users_playtime', '10'),	// *****
					array('most_played_users', '5'),
					array('most_jackpot', '10'),			// *****
					array('most_users_playtime', '10'),		// *****
					array('protect_phpbbarcade', '1'),
					array('record_playing_date', time(), true),
					array('record_playing_users', '0', true),
					array('score_security_level', '2'),
					array('session_last_gc', time(), true),
					array('share_icons_enable', '0'),
					array('smilies_popup_height', 350),
					array('smilies_popup_width', 500),
					array('tweet_like_enable', '0'),
					array('copyright', $arcade_mod_config['copyright']),
				));

				$this->p_master->arcade_set_config($arcade_configs);

				$options = array(
					array('u_favorites'			, 'u_arcade_favorites'),
					array('u_ignoreflood_search', 'u_arcade_ignoreflood_search'),
					array('u_view_whoisplaying'	, 'u_arcade_view_whoisplaying'),
					array('u_viewstats'			, 'u_arcade_viewstats')
				);

				$this->p_master->rename_arcade_permissions($options);

				$this->p_master->remove_permissions(array('local'	=> array('c_popup', 'c_resolution')), 'arcade');
				$this->p_master->remove_permissions(array('global'	=> array('a_arcade_scores')));

				$this->p_master->add_permissions($arcade_mod_config['permission_options']['update']['2.0.RC2']['arcade'], 'arcade');
				$this->p_master->add_permissions($arcade_mod_config['permission_options']['update']['2.0.RC2']['phpbb']);

				$this->p_master->update_roles(array('ROLE_ARCADE_FULL'), $arcade_mod_config['permission_options']['update']['2.0.RC2']['arcade']['local'], 'arcade');
				$this->p_master->update_roles(array('ROLE_ARCADE_USER_FULL', 'ROLE_ARCADE_USER_STANDARD'), $arcade_mod_config['permission_options']['update']['2.0.RC2']['arcade']['global'], 'arcade');
				$this->p_master->update_roles(array('ROLE_ARCADE_USER_LIMITED'), array('u_arcade_challenge', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution'), 'arcade');
				$this->p_master->update_roles(array('ROLE_ADMIN_FULL', 'ROLE_ADMIN_STANDARD'), $arcade_mod_config['permission_options']['update']['2.0.RC2']['phpbb']['global']);

				switch ($db->sql_layer)
				{
					case 'sqlite':
					case 'firebird':
						$db->sql_query('DELETE FROM ' . ARCADE_ERRORS_TABLE);
					break;

					default:
						$db->sql_query('TRUNCATE TABLE ' . ARCADE_ERRORS_TABLE);
					break;
				}

				// Process schema changes
				$phpbb_db_tools->perform_schema_changes($arcade_mod_config['schema_changes']['update']['2.0.RC2']);

				// Remove and then re-add the modules
				$this->p_master->remove_modules($arcade_mod_config['modules_remove']);
				foreach ($arcade_mod_config['modules'] as $modules)
				{
					$this->p_master->create_modules($modules['parent_module_data'], $modules['module_data']);
				}

				$this->p_master->add_modules('ACP_CAT_USERS', 'acp', $arcade_mod_config['phpbb_users_modules']);

				$sql_array = array(
					'user_arcade_options'			=> 220,
					'games_per_page'				=> 0,
					'games_sort_dir'				=> 'd',
					'games_sort_order'				=> 'd'
				);

				$sql = 'UPDATE ' . ARCADE_USERS_TABLE . ' SET ' . $db->sql_build_array('UPDATE', $sql_array) . '
						WHERE user_id = ' . ANONYMOUS;
				$db->sql_query($sql);

				$this->p_master->table_optimize($arcade_mod_config['verify']['tables']);
			case '2.0.RC2':
				$this->p_master->phpbb_set_config($arcade_mod_config['phpbb_config']);
			break;

			default:
			break;
		}

		// Set arcade version config value to latest version
		$this->p_master->arcade_set_config('version', $arcade_mod_config['version']['current']);

		// Purge the cache
		$this->p_master->cache_purge(array('template', 'theme', 'imageset', 'auth', ''));

		add_log('admin', 'LOG_ARCADE_UPDATE', $this->p_master->installed_version, $arcade_mod_config['version']['current']);

		if ($this->p_master->installed_version != '2.0.RC2')
		{
			$perm_msg = '</p><div class="errorbox"><h3>' . $user->lang['WARNING'] . '</h3><p>' . $user->lang['PERMISSIONS_WARNING'] . '</p></div>';
		}
		else
		{
			$perm_msg = '</p>';
		}

		$donate_button = '<br /><a onclick="window.open(this.href); return false;" href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&amp;hosted_button_id=UA2VMFQD6GKFW"><img src="http://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif" alt="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" title="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" /></a>';
		$donate = '<div class="successbox"><h3>' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '</h3><p>' . $user->lang['PHPBB_ARCADE_DONATE'] . '</p><p>' . $donate_button . '</p></div>';

		$template->assign_vars(array(
			'TITLE'		=> $user->lang['INSTALL_CONGRATS'],
			'BODY'		=> $user->lang['STAGE_UPDATE_ARCADE_EXPLAIN'] . '<br /><br />' . $perm_msg . $donate . sprintf($user->lang['UPDATE_CONGRATS_EXPLAIN'], $arcade_mod_config['version']['current']),
			'L_SUBMIT'	=> $user->lang['CAT_VERIFY'],
			'U_ACTION'	=> $this->p_master->module_url . '?mode=verify&amp;sub=verify&amp;type=update',
		));
	}
}

?>